@echo off

SETLOCAL

SET "sdmx_file="
SET sdmx_file=%1

REM Check if a filename is provided
IF NOT DEFINED sdmx_file (
	ECHO Missing input file name. Exiting...
	GOTO end
)

REM Handle special characters
SETLOCAL ENABLEDELAYEDEXPANSION
FOR /F "tokens=1,2 delims=*<>|&?^" %%a IN ("a!sdmx_file!a") DO (
	SET var=%%b
	IF DEFINED var (
		ECHO Input file name can't contain the following characters: ^* ^< ^> ^| ^& ^? ^^. Exiting...
		GOTO end
	)
)
ENDLOCAL

REM Handle quotes
SET sdmx_file=%sdmx_file:"=%
SET sdmx_file="%sdmx_file%"

REM Check if file exists
IF NOT EXIST %sdmx_file% (
	ECHO Input file doesn't exist. Exiting...
	GOTO end
)

REM Check if file is empty (or a folder was provided)
FOR %%a IN (%sdmx_file%) DO (
	IF %%~za==0 (
		ECHO The input is a folder or an empty file. Exiting...
		GOTO end
	)
)

REM Check if file is readable by for /F (needed to find out the file format)
2>nul ( 
	FOR /F "usebackq delims=" %%a IN (%sdmx_file%) DO (
		GOTO fileOpened
	)
)

echo The input file cannot be opened. Exiting...
GOTO end

:fileOpened

REM Find out the file format
FOR /F "usebackq delims=" %%a IN (%sdmx_file%) DO (
	SET temp=%%a
	SETLOCAL ENABLEDELAYEDEXPANSION
	SET temp=!temp:"=!
	SET temp=!temp:?=!
	SET temp=!temp: =:!
	SET temp=!temp:/= !
	SET temp=!temp:+= !
	FOR %%b IN (!temp!) DO (
		IF %%b==v2_0 SET format=sdmx20c
		IF %%b==v2_1 SET format=sdmx21
		IF %%b==GESMES SET format=gesmes
		IF DEFINED format GOTO acceptedFormat
	)
	ENDLOCAL
)

ECHO The file provided is not in an accepted format.
GOTO end

:acceptedFormat

REM Build the JAVA command
IF %format%==sdmx20c (
	SET config_file=checker_sdmx20c.cnf
	SET reader_string=-DInt.ecb.gesmescb.xml.XMLReader.dataReader.source=%sdmx_file%
)
IF %format%==sdmx21 (
	SET config_file=checker_sdmx21.cnf
	SET reader_string=-DInt.ecb.gesmescb.xml.XMLReader.dataReader.source=%sdmx_file%
)
IF %format%==gesmes (
	SET config_file=checker_ges.cnf
	SET reader_string=-DInt.ecb.gesmescb.edifact.EDIFACTReader.source=%sdmx_file%
)

REM Run the JAVA command
JAVA -Xms16m -cp "%~dp0JarFiles\gesmescb.jar;%~dp0JarFiles\xercesImpl.jar;%~dp0JarFiles\xml-apis.jar;%~dp0JarFiles\xalan.jar;%~dp0JarFiles\xercesSamples.jar"^
 -DInt.ecb.logging.config.class=Int.ecb.gesmescb.util.LoggingPropertiesReader^
 -DInt.ecb.logging.config.file=%~dp0CheckerConfig\%config_file%^
 -DInt.ecb.prefs.config.file=%~dp0CheckerConfig\%config_file%^
 %reader_string%^
 Int.ecb.gesmescb.application.BatchTextUI %~dp0StructuralDefinitions\*^
 %sdmx_file%

REM JAVA error handling 
IF errorlevel 3 GOTO panic
IF errorlevel 2 GOTO error
IF errorlevel 1 GOTO unexpected
GOTO ok

:panic
ECHO General problem encountered, maybe incorrect installation.
GOTO end

:error
ECHO Errors found
GOTO end

:unexpected
ECHO Unexpected objects found
GOTO end

:ok
ECHO No problems

:end
ENDLOCAL