@echo off

SETLOCAL

SET "target_extension="
SET "overwrite_output="
SET "source_format="
SET "message="
SET "source_file="
SET "target_format="
SET "overwrite_output="

SET source_file=%1
SET target_format=%2
SET overwrite_output=%3

REM Check if a filename is provided
IF NOT DEFINED source_file (
	ECHO Missing input file name. Exiting...
	GOTO end
)

REM Handle special characters
SETLOCAL ENABLEDELAYEDEXPANSION
FOR /F "tokens=1,2 delims=*<>|&?^" %%a IN ("a!source_file!a") DO (
	SET var=%%b
	IF DEFINED var (
		ECHO Input file name can't contain the following characters: ^* ^< ^> ^| ^& ^? ^^. Exiting...
		GOTO end
	)
)
ENDLOCAL

REM Handle quotes
SET source_file=%source_file:"=%
SET log_file=%source_file%.log
SET source_file="%source_file%"

REM Check if file exists
IF NOT EXIST %source_file% (
	ECHO Input file doesn't exist or you don't have permission to read it. Exiting...
	GOTO end
)

REM Check if file is empty
FOR %%a IN (%source_file%) DO (
	IF %%~za==0 (
		ECHO The input is a folder or an empty file. Exiting...
		GOTO end
	)
)

REM Check if second input (target format) is correct
SETLOCAL ENABLEDELAYEDEXPANSION
IF "!target_format!"=="sdmx20c" SET target_extension=sdmx20c.xml
IF "!target_format!"=="sdmx21" SET target_extension=sdmx21.xml
IF "!target_format!"=="gesmes" SET target_extension=ges
IF NOT DEFINED target_extension (
	ECHO Unknown target format provided. Exiting...
	GOTO end
)


REM Check if third input (overwrite flag) is correct
IF /i "!overwrite_output!"=="y" GOTO correctInput3
IF /i "!overwrite_output!"=="n" GOTO correctInput3
IF "!overwrite_output!"=="" (
	SET overwrite_output=n
	GOTO correctInput3
)
ENDLOCAL

ECHO The third parameter must be either y (yes), n (no) or an empty string, which defaults to n.
GOTO end

:correctInput3

REM Check if file is readable by for /F (needed to find out the file format)
2>nul ( 
	FOR /F "usebackq delims=" %%a IN (%source_file%) DO (
		GOTO fileOpened
	)
)

echo The input file cannot be opened. Exiting...
GOTO end

:fileOpened

REM Find out the file format
FOR /F "usebackq delims=" %%a IN (%source_file%) DO (
	SET temp=%%a
	SETLOCAL ENABLEDELAYEDEXPANSION
	SET temp=!temp:"=!
	SET temp=!temp:?=!
	SET temp=!temp: =:!
	SET temp=!temp:/= !
	SET temp=!temp:+= !
	FOR %%b IN (!temp!) DO (
		IF %%b==v2_0 SET source_format=sdmx20c
		IF %%b==v2_1 SET source_format=sdmx21
		IF %%b==GESMES SET source_format=gesmes
		IF DEFINED source_format GOTO acceptedFormat
	)
	ENDLOCAL
)

ECHO The input file is not in an accepted format. Exiting...
GOTO end

:acceptedFormat

REM Check if  input and output file formats are different.
IF %target_format%==%source_format% (
	ECHO Target format and source format are equal. Nothing to do. Exiting...
	GOTO end
)

SET source_file=%source_file:"=%
SET target_file=%source_file%.%target_extension%

REM Check if you can overwrite (or save) output file
IF EXIST "%target_file%" SET message=The output file already exists.

2>nul ( >>"%target_file%" (CALL ) ) && (
	IF NOT DEFINED message GOTO executeCommand	
	IF /i "%overwrite_output%"=="y" GOTO executeCommand
	ECHO %message% If you want to overwrite it run converter.bat with "y" as third parameter. Exiting...
	GOTO end
)

ECHO You can't overwrite or save the output file. This can be due to a lock or a permission issue. Exiting...
GOTO end

:executeCommand

REM Build the JAVA command
IF %target_format%==gesmes (
	SET writer_string=-DInt.ecb.gesmescb.edifact.EDIFACTWriter.destination="%target_file%"
) ELSE (
	SET writer_string=-DInt.ecb.gesmescb.xml.SDMXMLWriter.destination="%target_file%" -DInt.ecb.gesmescb.xml.SDMXMLWriter.allowMultipleMessages=true
)

REM Run the JAVA command
JAVA -Xmx1024m -cp "%~dp0JarFiles\gesmescb.jar;%~dp0JarFiles\xercesImpl.jar;%~dp0JarFiles\xml-apis.jar;%~dp0JarFiles\xalan.jar;%~dp0JarFiles\xercesSamples.jar"^
 -DInt.ecb.logging.config.class=Int.ecb.gesmescb.util.LoggingPropertiesReader^
 -DInt.ecb.logging.config.file=%~dp0ConverterConfig\converter_%source_format%2%target_format%.cnf^
 -DInt.ecb.prefs.config.file=%~dp0ConverterConfig\converter_%source_format%2%target_format%.cnf^
 -DInt.ecb.logging.FileHandler.pattern="%log_file%"^
 -DInt.ecb.gesmescb.xml.XMLReader.dataReader.source="%source_file%"^
 %writer_string%^
 Int.ecb.gesmescb.application.BatchTextUI %~dp0StructuralDefinitions\*^
 "%source_file%"

REM JAVA error handling 
IF errorlevel 3 GOTO panic
IF errorlevel 2 GOTO error
IF errorlevel 1 GOTO unexpected
GOTO ok

:panic
ECHO General problem encountered, maybe incorrect installation.
GOTO end

:error
ECHO Errors found
GOTO end

:unexpected
ECHO Unexpected objects found
GOTO end

:ok
ECHO No problems

:end
ENDLOCAL