//
// Copyright (c) 2001, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb;

import java.util.Iterator;
import java.util.Arrays;
import java.util.Date;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/* ***************************** Class ****************************** */

/**
 * <P>Tests the implementation of the class {@link CodeList}.</P>
 *
 * @author
 * <A HREF="mailto:Sami.Airo@ECB.int">Sami Airo</A>
 * @author
 * <A HREF="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</A>
 *
 * @version @(#)$Id: CodeListTest.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */

public class CodeListTest extends TestCase {

	/* ========================= Fields ========================= */

	/* ====================== Constructors ====================== */

	/**
	 * Constructs a PreferencesTes with the specified name.
	 *
	 * @param name Test case name
	 */
	public CodeListTest ( String name ) {
		super(name);
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite
	 */
	public static Test suite () {
		TestSuite suite = new TestSuite(CodeListTest.class);
		return suite;
	}

	/**
	 * Runs the test case
	 */
	public static void main ( String args[] ) {
		String[] testCaseName = { CodeListTest.class.getName() };
		junit.textui.TestRunner.main(testCaseName);
	}

	/* ---------------------- Test Methods ---------------------- */

	/**
	 * Test the two constructors.
	 */
	public void testConstruct () {
		Date date = new Date();
		String id = "CL_TEST";
		String[] codes = new String[]{"A", "B"};
		String[] descs = new String[]{"AD", "BD"};
		String[] codes1 = new String[]{"A"};
		String[] descs1 = new String[]{"AD"};

		CodeList c1 = new CodeList(
			id, "name", "desc", date, codes, descs
		);
		CodeList c2 = new CodeList(
			id, "name", "desc", date,
			Arrays.asList( codes ), Arrays.asList( descs )
		);

		assertTrue("c1 equals c2", c1.equals(c2));
		assertTrue("c1 hashcode == c2.hashcode", c1.hashCode() == c2.hashCode());
		assertTrue("c1 != c2", c1 != c2);

		id = "CL_TEST1";
		assertTrue("c1.id not equals id", !c1.equals(id));
		assertTrue("c2.id not equals id", !c2.equals(id));

		// Check that no Exceptions are thrown.
		CodeList c3 = new CodeList(
			id, "name", "desc", date,
			codes1, descs
		);
		// useless test so as to make use of c3
		assertTrue("c3.id not equals id", !c3.equals(id));
		
		CodeList c4 = new CodeList(
			id, "name", "desc", date,
			codes, descs1
		);
		assertTrue("c4.id not equals id", !c4.equals(id));
		
		CodeList c5 = new CodeList(
			id, "name", "desc", date,
			Arrays.asList( codes1 ), Arrays.asList( descs )
		);
		assertTrue("c5.id not equals id", !c5.equals(id));
		
		CodeList c6 = new CodeList(
			id, "name", "desc", date,
			Arrays.asList( codes ), Arrays.asList( descs1 )
		);
		assertTrue("c6.id not equals id", !c6.equals(id));
		
	}

	/**
	 * Test the equals and hashcode methods method.
	 */
	public void testEquals () {
		Date date = new Date();
		CodeList c1 = new CodeList(
			"CL_TEST", "name", "desc", date,
			new String[]{"A", "B"}, new String[]{"AD", "BD"}
		);
		CodeList c2 = new CodeList(
			"CL_TEST", "name", "desc", date, 
			new String[]{"A", "B"}, new String[]{"AD", "BD"}
		);
		CodeList c3 = new CodeList(
			"CL_TEST1", "name1", "desc1", date,
			new String[]{"A1", "B1"}, new String[]{"AD1", "BD1"}
		);

		assertTrue("c1 equals c2", c1.equals(c2));
		assertTrue("c1 not equals c3", !c1.equals(c3));
		assertTrue("c1.hashcode == c2.hashcode", c1.hashCode() == c2.hashCode());
		assertTrue("c1.hashcode != c3.hashcode", c1.hashCode() != c3.hashCode());
		assertTrue("c1.toString == c2.toString", c1.toString().equals( c2.toString() ));
		assertTrue("c1.toString != c3.toString", !c1.toString().equals( c3.toString() ));
	}

	/**
	 * Test the equals and hashcode methods method.
	 */
	public void testSimpleProperties () {
		Date date = new Date();
		CodeList c1 = new CodeList(
			"CL_TEST", "name", "desc", date,
			new String[]{"A", "B"}, new String[]{"AD", "BD"}
		);

		assertEquals("CL_TEST", c1.getId());
		assertEquals("name", c1.getName());
		assertEquals("desc", c1.getDescription());
	}

	/**
	 * Test the array based properties.
	 */
	public void testArrayProperties () {
		Date date = new Date();
		String[] v = new String[]{"A", "B"};
		String[] d = new String[]{"AD", "BD"};

		CodeList c1 = new CodeList("CL_TEST", "name", "desc", date, v, d);

		assertEquals(v.length, c1.getCodeValueCount());
		assertEquals("AD", c1.getCodeDescription("A"));
		assertEquals("BD", c1.getCodeDescription("B"));
		assertNull(c1.getCodeDescription("C"));
		assertTrue(c1.containsCodeValue("A"));
		assertTrue(c1.containsCodeValue("B"));
		assertTrue(!c1.containsCodeValue("C"));

		Iterator vi = c1.codeValueIterator();
		Iterator di = c1.codeDescriptionIterator();

		for( int n = 0; n < v.length; n++) {
			assertEquals(v[n], (String)vi.next());
			assertEquals(d[n], (String)di.next());
		}

		try {
			vi.remove();
			fail("UnsupportedOperationException not thrown on remove");
		} catch (UnsupportedOperationException e) {
		}
	}

	/**
	 * Tests serialization.
	 */
	public void testSerialization () {
		Date date = new Date();
		final CodeList c1 = new CodeList(
			"CL_TEST", "name", "desc", date,
			new String[]{"A", "B"}, new String[]{"AD", "BD"}
		);
		final CodeList c2
		= (CodeList) AttributeTest.transserialize( c1 );
		assertEquals( c1, c2 );
		assertTrue( c2.containsCodeValue( "A" ) );
		assertEquals( "AD", c2.getCodeDescription( "A" ) );
	}
}
