// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/* ***************************** Class ****************************** */

/**
 * <P>Tests the implementation of {@link ContactChannel}.</P>
 *
 * @author
 * <A HREF="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</A>
 *
 * @version @(#)$Id: ContactChannelTest.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */

public class ContactChannelTest extends TestCase {

	/* ======================= Constants ======================== */

	public static final ContactChannel contactChannel = new ContactChannel(
		ContactChannel.TELEPHONE, "number"
	);

	/* ====================== Constructors ====================== */

	/**
	 * Constructs a <CODE>ContactChannelTest</CODE> with the specified name.
	 */
	public ContactChannelTest ( final String name ) {
		super( name );
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite.
	 */
	public static Test suite () {
		TestSuite suite = new TestSuite( ContactChannelTest.class );
		return suite;
	}

	/**
	 * Runs the test case.
	 */
	public static void main ( String args[] ) {
		String[] testCaseName = { ContactChannelTest.class.getName() };
		junit.textui.TestRunner.main( testCaseName );
	}

	/* ---------------------- Test Methods ---------------------- */

	/** Test equals and hashCode. */
	public void testEquals () {
		final ContactChannel other = new ContactChannel(
			new String( contactChannel.getType() ),
			new String( contactChannel.getNumber() )
		);
		assertEquals( "equals", contactChannel, other );
		assertEquals( "hashCode", contactChannel.hashCode(), other.hashCode() );
		assertEquals( "toString", contactChannel.toString(), other.toString() );

		final ContactChannel[] channels = new ContactChannel[2];
		for ( int i = 0; i < channels.length; ++i ) {
			channels[i] = new ContactChannel(
				i == 0 ? ContactChannel.EMAIL : contactChannel.getType(),
				i == 1 ? "slumber" : contactChannel.getNumber()
			);
		}
		for ( int i = 0; i < channels.length; ++i ) {
			assertTrue( "not equal " + i, !contactChannel.equals( channels[i] ) );
			assertTrue(
				"not hashCode " + i,
				contactChannel.hashCode() != channels[i].hashCode()
			);
			assertTrue(
				"not toString " + i,
				!contactChannel.toString().equals( channels[i].toString() )
			);
		}
	}

	/**
	 * Tests the normal fields.
	 */
	public void testNormalFields () {
		assertEquals( ContactChannel.TELEPHONE, contactChannel.getType() );
		assertEquals( "number", contactChannel.getNumber() );
	}

	/**
	 * Tests serialization.
	 */
	public void testSerialization () {
		final ContactChannel transserialized
		= (ContactChannel) AttributeTest.transserialize( contactChannel );
		assertTrue( equals( contactChannel, transserialized ) );
		assertEquals( contactChannel.hashCode(), transserialized.hashCode() );
	}

	public static boolean equals (
		final ContactChannel cc1, final ContactChannel cc2
	) {
		return
			cc1.getType().equals( cc2.getType() )
			&&
			cc1.getNumber().equals( cc2.getNumber() );
	}
}
