// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb;

import java.util.Collection;
import java.util.ArrayList;
import java.util.Iterator;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/* ***************************** Class ****************************** */

/**
 * <P>Tests the implementation of {@link Contact}.</P>
 *
 * @author
 * <A HREF="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</A>
 *
 * @version @(#)$Id: ContactTest.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */

public class ContactTest extends TestCase {

	/* ======================= Constants ======================== */

	private static final ContactChannel[] channels = new ContactChannel[] {
		ContactChannelTest.contactChannel
	};

	public static final Contact contact = new Contact(
		Contact.DATA_PROCESSING, "SIS", "Yurij", channels
	);

	/* ====================== Constructors ====================== */

	/**
	 * Constructs a <CODE>ContactTest</CODE> with the specified name.
	 */
	public ContactTest ( final String name ) {
		super( name );
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite.
	 */
	public static Test suite () {
		TestSuite suite = new TestSuite( ContactTest.class );
		return suite;
	}

	/**
	 * Runs the test case.
	 */
	public static void main ( String args[] ) {
		String[] testCaseName = { ContactTest.class.getName() };
		junit.textui.TestRunner.main( testCaseName );
	}

	/* ---------------------- Test Methods ---------------------- */

	/** Test constructor equivalence. */
	public void testConstructor () {
		final Collection channels = new ArrayList();
		channels.add( ContactChannelTest.contactChannel );
		Contact other = new Contact(
			contact.getFunction(),
			contact.getIdentity(),
			contact.getName(),
			channels
		);
		assertEquals( contact, other );
	}

	/** Test equals and hashCode. */
	public void testEquals () {
		final Contact other = new Contact(
			new String( contact.getFunction() ),
			new String( contact.getIdentity() ),
			new String( contact.getName() ),
			new ContactChannel[] {
				new ContactChannel(
					ContactChannelTest.contactChannel.getType(),
					ContactChannelTest.contactChannel.getNumber()
				)
			}
		);
		assertEquals( "equals", contact, other );
		assertEquals( "hashCode", contact.hashCode(), other.hashCode() );
		assertEquals( "toString", contact.toString(), other.toString() );

		final Contact[] contacts = new Contact[4];
		for ( int i = 0; i < contacts.length; ++i ) {
			contacts[i] = new Contact(
				i == 0 ? Contact.INFORMATION_PRODUCTION : contact.getFunction(),
				i == 1 ? "hump" : contact.getIdentity(),
				i == 2 ? "dump" : contact.getName(),
				i == 3 ? new ContactChannel[] {} : channels
			);
		}
		for ( int i = 0; i < contacts.length; ++i ) {
			assertTrue( "not equal " + i, !contact.equals( contacts[i] ) );
			assertTrue(
				"not hashCode " + i,
				contact.hashCode() != contacts[i].hashCode()
			);
			assertTrue(
				"not toString " + i,
				!contact.toString().equals( contacts[i].toString() )
			);
		}
	}

	/** Test the fields. */
	public void testFields () {
		assertEquals( Contact.DATA_PROCESSING, contact.getFunction() );
		assertEquals( "SIS", contact.getIdentity() );
		assertEquals( "Yurij", contact.getName() );
		final Iterator iterator = contact.channelIterator();
		assertSame( ContactChannelTest.contactChannel, iterator.next() );
		assertTrue( "Only one", !iterator.hasNext() );
	}

	/** Test serialization. */
	public void testSerialization () {
		final Contact transserialized
		= (Contact) AttributeTest.transserialize( contact );
		assertEquals( contact, transserialized );
		assertEquals( contact.hashCode(), transserialized.hashCode() );
	}
}
