// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import Int.ecb.gesmescb.InterchangeBegin;
import Int.ecb.gesmescb.InterchangeEnd;

/* ***************************** Class ****************************** */

/**
 * <P>Tests the implementation of the class {@link InterchangeEnd}.</P>
 *
 * @author
 * <A HREF="mailto:Sami.Airo@ECB.int">Sami Airo</A>
 * @author
 * <A HREF="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</A>
 *
 * @version @(#)$Id: InterchangeEndTest.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */

public class InterchangeEndTest extends TestCase {

	/* ========================= Fields ========================= */

	private InterchangeEnd ie1, ie2, ie3, ie4;

	/* ====================== Constructors ====================== */

	/**
	 * Constructs a InterchangeEndTest with the specified name.
	 *
	 * @param name Test case name
	 */
	public InterchangeEndTest ( String name ) {
		super( name );
	}

	/* ================== Protected Methods ===================== */

	/**
	 * Sets up the test fixture
	 */
	protected void setUp () {
		ie1 = new InterchangeEnd( "IREF123456" );
		ie2 = new InterchangeEnd( new String( "IREF123456" ) );
		ie3 = new InterchangeEnd( "IREF" );
		ie4 = new InterchangeEnd( "IRF" );
	}

	/**
	 * Tears down the test fixture
	 */
	protected void tearDown () {
		ie1 = ie2 = ie3 = ie4 = null;
	}


	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite
	 */
	public static Test suite () {
		TestSuite suite = new TestSuite( InterchangeEndTest.class );
		return suite;
	}

	/**
	 * Runs the test case.
	 */
	public static void main ( String args[] ) {
		String[] testCaseName = { InterchangeEndTest.class.getName() };
		junit.textui.TestRunner.main( testCaseName );
	}

	/* ---------------------- Test Methods ---------------------- */

	/**
	 * Tests the equals and hashcode methods method
	 */
	public void testEquals () {
		assertEquals( "ie1 equals ie2", ie1, ie2 );
		assertEquals(
			"ie1.hashCode == ie2.hashCode",
			ie1.hashCode(),
			ie2.hashCode()
		);
		assertEquals(
			"ie1.toString equals ie2.toString",
			ie1.toString(),
			ie2.toString()
		);

		assertTrue( "ie1 unequal ie3", !ie1.equals( ie3 ) );
		// Inequality of the hashCode is not really contract.
		// But the probability of eventual equal hashCode is
		// far less than that of a buggy
		// hashCode-implementation.
		// Therefore check.
		assertTrue(
			"ie1.hashCode unequal",
			ie1.hashCode() != ie3.hashCode()
		);
		assertTrue(
			"ie1.toString unequal",
			!ie1.toString().equals( ie3.toString() )
		);
	}

	/**
	 * Tests the fields.
	 */
	public void testFields () {
		assertEquals( "IREF123456", ie1.getIref() );
		assertEquals( "irefNo", 123456, ie1.getIrefNo() );
		assertEquals(
			"illegal irefNo3", InterchangeEnd.INVALID_IREF, ie3.getIrefNo()
		);
		assertEquals(
			"illegal irefNo4", InterchangeEnd.INVALID_IREF, ie4.getIrefNo()
		);
	}

	/**
	 * Test correspondence to InterchangeBegin.
	 */
	public void testCorrespond () {
		final InterchangeBegin ib1 = new InterchangeBegin(
			new String( "IREF123456" ),
			"CIA", "SIS", "200010231414", true
		);
		final InterchangeBegin ib2 = new InterchangeBegin(
			"IREF113456", "CIA", "SIS", "200010231414", true
		);

		assertTrue( "ends", ie1.isEndFor( ib1 ) );
		assertTrue( "ends not", !ie1.isEndFor( ib2 ) );
	}

	/**
	 * Tests serialization.
	 */
	public void testSerialization () {
		final InterchangeEnd iex
		= (InterchangeEnd) AttributeTest.transserialize( ie1 );
		assertEquals( ie1, iex );
		assertEquals( ie1.hashCode(), iex.hashCode() );
		assertEquals( ie1.getIrefNo(), iex.getIrefNo() );
	}
}
