// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/* ***************************** Class ****************************** */

/**
 * <P>Tests the implementation of the Int.ecb.gesmesdb.KeyFamilyComponent class.</P>
 *
 * @author
 * <A HREF="mailto:Sami.Airo@ECB.int">Sami Airo</A>
 *
 * @version @(#)$Id: KeyFamilyComponentTest.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */

public class KeyFamilyComponentTest extends TestCase {

	/* ====================== Constructors ====================== */

	/**
	 * Constructs a KeyFamilyComponentTest with the specified name.
	 *
	 * @param name Test case name
	 */
	public KeyFamilyComponentTest ( final String name ) {
		super( name );
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite
	 */
	public static Test suite () {
		return new TestSuite( KeyFamilyComponentTest.class );
	}

	/**
	 * Runs the test case.
	 */
	public static void main ( final String args[] ) {
		String[] testCaseName = { KeyFamilyComponentTest.class.getName() };
		junit.textui.TestRunner.main( testCaseName );
	}

	/* --------------------- Test Methods --------------------- */

	/**
	 * Tests the equals and hashcode methods method
	 */
	public void testEquals () {
		KeyFamilyComponent k1 = new KeyFamilyComponent("CONCEPT1", "Z09", "5", "AN5", "2", "5", "CL_TEST" );
		KeyFamilyComponent k2 = new KeyFamilyComponent("CONCEPT1", "Z09", "5", "AN5", "2", "5", "CL_TEST" );
		KeyFamilyComponent k3 = new KeyFamilyComponent("CONCEPT2", "Z09", "5", "AN5", "2", "5", "CL_TEST" );

		assertEquals( "k1 equals k2", k1, k2 );
		assertTrue( "k1 not equals k3", !k1.equals( k3 ) );
		assertEquals(
			"k1 hashcode == k2.hashcode",
			k1.hashCode(), k2.hashCode()
		);
		assertTrue(
			"k1 hashcode != k3.hashcode",
			k1.hashCode() != k3.hashCode()
		);
	}

	/**
	 * Tests the compareTo method
	 */
	public void testCompareTo () {
		final KeyFamilyComponent[] component = new KeyFamilyComponent[]{
			new KeyFamilyComponent(
				"FREQ", KeyFamilyComponent.KEY_STRUCTURE_DIMENSION,
				"1", "AN5", "", "", "CL_TEST"
			),
			new KeyFamilyComponent(
				"AREA", KeyFamilyComponent.KEY_STRUCTURE_DIMENSION,
				"2", "AN5", "", "", "CL_TEST"
			),
			new KeyFamilyComponent(
				"TIME", KeyFamilyComponent.TIME_DIMENSION,
				"3", "AN5", "", "", ""
			),
			new KeyFamilyComponent(
				"VALUE", KeyFamilyComponent.ARRAY_CELL,
				"4", "AN5", "", "", ""
			),
			new KeyFamilyComponent(
				"COMMENT1", KeyFamilyComponent.ATTRIBUTE,
				"", "AN5", KeyFamilyComponent.CONDITIONAL,
				KeyFamilyComponent.ATTACHED_TO_OBSERVATION, ""
			),
			new KeyFamilyComponent(
				"COMMENT1b", KeyFamilyComponent.ATTRIBUTE,
				"", "AN5", KeyFamilyComponent.CONDITIONAL,
				KeyFamilyComponent.ATTACHED_TO_OBSERVATION, ""
			),
			new KeyFamilyComponent(
				"COMMENT2", KeyFamilyComponent.ATTRIBUTE,
				"", "AN5", KeyFamilyComponent.CONDITIONAL,
				KeyFamilyComponent.ATTACHED_TO_TIME_SERIES, ""
			),
			new KeyFamilyComponent(
				"COMMENT2b", KeyFamilyComponent.ATTRIBUTE,
				"", "AN5", KeyFamilyComponent.CONDITIONAL,
				KeyFamilyComponent.ATTACHED_TO_TIME_SERIES, ""
			),
			new KeyFamilyComponent(
				"COMMENT3", KeyFamilyComponent.ATTRIBUTE,
				"", "AN5", KeyFamilyComponent.CONDITIONAL,
				KeyFamilyComponent.ATTACHED_TO_SIBLING_GROUP, ""
			),
			new KeyFamilyComponent(
				"COMMENT4", KeyFamilyComponent.ATTRIBUTE,
				"", "AN5", KeyFamilyComponent.CONDITIONAL,
				KeyFamilyComponent.ATTACHED_TO_DATA_SET, ""
			)
		};

		for ( int i = 0; i < component.length; ++i ) {
			for ( int j = 0; j < component.length; ++j ) {
				if ( i < j ) {
					assertTrue(
						"component[" + i + "] :: component[" + j + "]",
						component[i].compareTo( component[j] ) < 0
					);
				} else if ( i > j ) {
					assertTrue(
						"component[" + i + "] :: component[" + j + "]",
						component[i].compareTo( component[j] ) > 0
					);
				} else {
					assertEquals(
						"component[" + i + "] :: component[" + j + "]",
						0,
						component[i].compareTo( component[j] )
					);
				}
			}
		}
	}

	/**
	 * Tests the normal fields
	 */
	public void testNormalFields () {
		KeyFamilyComponent k1 = new KeyFamilyComponent("CONCEPT1", "Z09", "1", "AN5", "2", "5", "CL_TEST");

		assertEquals("CONCEPT1", k1.getConceptId());
		assertEquals("Z09", k1.getConceptType());
		assertEquals("1", k1.getPosition());
		assertEquals("AN5", k1.getFormat());
		assertEquals("2", k1.getUsageStatus());
		assertEquals("5", k1.getAttachmentLevel());
		assertEquals("CL_TEST", k1.getCodeListId());
	}

	/**
	 * Tests the computed fields
	 */
	public void testComputedFields () {
		{
			final KeyFamilyComponent k1 = new KeyFamilyComponent(
				"CONCEPT1", KeyFamilyComponent.ATTRIBUTE, "1", "AN..10",
				KeyFamilyComponent.MANDATORY,
				KeyFamilyComponent.ATTACHED_TO_OBSERVATION,
				"CL_TEST"
			);
			assertTrue( "k1 non frequency", !k1.isFrequencyDimension() );
			assertTrue( "k1 non keystructure", !k1.isKeyStructureDimension() );
			assertTrue( "k1 attribute", k1.isAttribute() );
			assertTrue( "k1 coded", k1.isCoded() );
			assertEquals( "k1 positionNo", 1, k1.getPositionNo() );
			assertTrue( "k1 mandatory", k1.isMandatory() );
			assertTrue( "k1 not numeric", !k1.isNumeric() );
			assertTrue( "k1 not fixed length", !k1.isFixedLength() );
			assertTrue( "k1 attached to obs", k1.isAttachedToObservation() );
			assertEquals(
				"k1 text format",
				TextFormat.create( TextFormat.ANY, 10, true ),
				k1.getTextFormat()
			);
		}
		{
			final KeyFamilyComponent k2 = new KeyFamilyComponent(
				"FREQ", KeyFamilyComponent.KEY_STRUCTURE_DIMENSION, "1", "A1",
				"", "", "CL_TEST"
			);

			assertTrue( "k2 frequency", k2.isFrequencyDimension() );
			assertTrue( "k2 keystructure", k2.isKeyStructureDimension() );
			assertEquals(
				TextFormat.create( TextFormat.ALPHABETIC, 1, false ),
				k2.getTextFormat()
			);
		}
		{
			final KeyFamilyComponent k3 = new KeyFamilyComponent(
				"FREQUENCIA", KeyFamilyComponent.FREQUENCY_DIMENSION, "8", "A1",
				"", "", "CL_TEST", true
			);
			assertTrue( "k3 frequency", k3.isFrequencyDimension() );
		}
		{
			final KeyFamilyComponent k4 = new KeyFamilyComponent(
				"BAD", "", "", "", "", "", ""
			);
			assertNull( k4.getTextFormat() );
		}
		{
			final KeyFamilyComponent k5 = new KeyFamilyComponent(
				"FREQ", KeyFamilyComponent.KEY_STRUCTURE_DIMENSION, "1", "A1",
				"", "", "CL_TEST", true
			);
			assertTrue( "k5 non frequency", !k5.isFrequencyDimension() );
			assertTrue( "k5 keystructure", k5.isKeyStructureDimension() );
		}
	}

	/**
	 * Tests serialization
	 */
	public void testSerialization () {
		KeyFamilyComponent k1 = new KeyFamilyComponent("CONCEPT1", "Z09", "1", "AN5", "2", "5", "CL_TEST");
		final KeyFamilyComponent k2
		= (KeyFamilyComponent) AttributeTest.transserialize( k1 );
		assertEquals( k1, k2 );
	}
}
