// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, 2004, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb;

import java.util.*;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/* ***************************** Class ****************************** */

/**
 * <P>Tests the implementation of {@link KeyFamily}.</P>
 *
 * @author
 * <A HREF="mailto:Sami.Airo@ECB.int">Sami Airo</A>
 *
 * @version @(#)$Id: KeyFamilyTest.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */

public class KeyFamilyTest extends TestCase {

	/* ========================= Fields ========================= */

	private static final KeyFamilyComponent
		DIM1 = new KeyFamilyComponent(
			"CONCEPT1",
			KeyFamilyComponent.KEY_STRUCTURE_DIMENSION,
			"1",
			"AN5",
			KeyFamilyComponent.MANDATORY,
			"",
			"CL_TEST",
			true
		),
		DIM3 = new KeyFamilyComponent(
			"CONCEPT5",
			KeyFamilyComponent.KEY_STRUCTURE_DIMENSION,
			"3",
			"AN5",
			KeyFamilyComponent.MANDATORY,
			"",
			"CL_TEST",
			true
		),
		DIM2 = new KeyFamilyComponent(
			"CONCEPTX",
			KeyFamilyComponent.KEY_STRUCTURE_DIMENSION,
			"2",
			"AN5",
			KeyFamilyComponent.MANDATORY,
			"",
			"CL_TEST",
			true
		),
		DIM4 = new KeyFamilyComponent(
			"FREQ",
			KeyFamilyComponent.FREQUENCY_DIMENSION,
			"4",
			"AN5",
			KeyFamilyComponent.MANDATORY,
			"",
			"CL_TEST",
			true
		),
		ATT2 = new KeyFamilyComponent(
			"CONCEPT4",
			KeyFamilyComponent.ATTRIBUTE,
			"",
			"AN5",
			KeyFamilyComponent.CONDITIONAL,
			KeyFamilyComponent.ATTACHED_TO_OBSERVATION,
			"CL_TEST"
		),
		ATT1 = new KeyFamilyComponent(
			"CONCEPT3",
			KeyFamilyComponent.ATTRIBUTE,
			"",
			"AN5",
			KeyFamilyComponent.CONDITIONAL,
			KeyFamilyComponent.ATTACHED_TO_OBSERVATION,
			"CL_TEST"
		),
		ATT3 = new KeyFamilyComponent(
			"ATT3",
			KeyFamilyComponent.ATTRIBUTE,
			"",
			"A5",
			KeyFamilyComponent.MANDATORY,
			KeyFamilyComponent.ATTACHED_TO_TIME_SERIES,
			"CL_TEST"
		),
		ATT4 = new KeyFamilyComponent(
			"ATT4",
			KeyFamilyComponent.ATTRIBUTE,
			"",
			"N25",
			KeyFamilyComponent.CONDITIONAL,
			KeyFamilyComponent.ATTACHED_TO_SIBLING_GROUP,
			"CL_TEST"
		),
		ATT5 = new KeyFamilyComponent(
			"ATT5",
			KeyFamilyComponent.ATTRIBUTE,
			"",
			"N..2048",
			KeyFamilyComponent.CONDITIONAL,
			KeyFamilyComponent.ATTACHED_TO_DATA_SET,
			"CL_TEST"
		),
		TIME2 = new KeyFamilyComponent(
			"TIME2",
			KeyFamilyComponent.TIME_DIMENSION,
			"6",
			"AN5",
			KeyFamilyComponent.MANDATORY,
			"",
			"CL_TEST"
		),
		TIME1 = new KeyFamilyComponent(
			"TIME1",
			KeyFamilyComponent.TIME_DIMENSION,
			"5",
			"AN5",
			KeyFamilyComponent.MANDATORY,
			"",
			"CL_TEST"
		),
		OBS_CONF = new KeyFamilyComponent(
			"OBS_CONF",
			KeyFamilyComponent.ARRAY_CELL,
			"9",
			"AN..15",
			KeyFamilyComponent.MANDATORY,
			"",
			""
		),
		OBS_PRE_BREAK = new KeyFamilyComponent(
			"OBS_PRE_BREAK",
			KeyFamilyComponent.ARRAY_CELL,
			"10",
			"AN..15",
			KeyFamilyComponent.MANDATORY,
			"",
			""
		),
		OBS_VALUE = new KeyFamilyComponent(
			"OBS_VALUE",
			KeyFamilyComponent.ARRAY_CELL,
			"7",
			"AN..15",
			KeyFamilyComponent.MANDATORY,
			"",
			""
		),
		OBS_STATUS = new KeyFamilyComponent(
			"OBS_STATUS",
			KeyFamilyComponent.ARRAY_CELL,
			"8",
			"AN..15",
			KeyFamilyComponent.MANDATORY,
			"",
			""
		);

	private static final KeyFamilyComponent[]
		dimensions = new KeyFamilyComponent[]{
			DIM1, DIM3, DIM2, DIM4
		},
		times = new KeyFamilyComponent[]{
			TIME2, TIME1
		},
		arrayCells = new KeyFamilyComponent[]{
			OBS_CONF, OBS_PRE_BREAK, OBS_VALUE, OBS_STATUS
		},
		attributes = new KeyFamilyComponent[]{
			ATT5, ATT4, ATT3, ATT2, ATT1,
		},
		obsAttributes = new KeyFamilyComponent[]{
			ATT1, ATT2
		},
		tsAttributes = new KeyFamilyComponent[]{
			ATT3
		},
		sgAttributes = new KeyFamilyComponent[]{
			ATT4
		},
		dsAttributes = new KeyFamilyComponent[]{
			ATT5
		},
		components = new KeyFamilyComponent[
			dimensions.length + times.length
			+ arrayCells.length + attributes.length
		];

	static {
		int index = 0;
		System.arraycopy(
			arrayCells, 0, components, index, arrayCells.length
		);
		index += arrayCells.length;
		System.arraycopy(
			attributes, 0, components, index, attributes.length
		);
		index += attributes.length;
		System.arraycopy(
			dimensions, 0, components, index, dimensions.length
		);
		index += dimensions.length;
		System.arraycopy(
			times, 0, components, index, times.length
		);
		index += times.length;
	}

	private static final Date versionDate1 = new Date();
	private static final KeyFamily k1 = new KeyFamily(
		"KF1", "name", "desc", versionDate1, components
	);

	/* ====================== Constructors ====================== */

	/**
	 * Constructs a KeyFamilyTest with the specified name.
	 *
	 * @param name Test case name
	 */
	public KeyFamilyTest ( final String name ) {
		super( name );
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite
	 */
	public static Test suite () {
		final TestSuite suite = new TestSuite( KeyFamilyTest.class );
		return suite;
	}

	/**
	 * Runs the test case
	 */
	public static void main ( final String args[] ) {
		final String[] testCaseName = { KeyFamilyTest.class.getName() };
		junit.textui.TestRunner.main( testCaseName );
	}

	/* --------------------- Test Methods --------------------- */

	/**
	 * Test the equals and hashcode methods.
	 */
	public void testEquals () {
		final KeyFamilyComponent[] othercomponents
		= (KeyFamilyComponent[]) components.clone();

		KeyFamily k2 = new KeyFamily(
			new String( "KF1" ),
			new String( "name" ),
			new String( "desc" ),
			versionDate1,
			(KeyFamilyComponent[]) othercomponents
		);

		othercomponents[0] = new KeyFamilyComponent(
			"CONCEPT1",
			KeyFamilyComponent.KEY_STRUCTURE_DIMENSION,
			"1",
			"AN6",
			KeyFamilyComponent.MANDATORY,
			"",
			"CL_TEST"
		);
		KeyFamily k3 = new KeyFamily(
			"KF1", "name", "desc", versionDate1, othercomponents
		);

		assertEquals( "k1 equals k2", k1, k2 );
		assertTrue( "k1 not equals k3", !k1.equals( k3 ) );
		assertEquals( "k1 hashcode k2", k1.hashCode(), k2.hashCode() );
		assertTrue( "k1 not hashcode k3", k1.hashCode() != k3.hashCode() );
		assertEquals( "k1 toString k2", k1.toString(), k2.toString() );
		assertTrue( "k1 not toString k3", k1.toString() != k3.toString() );
	}

	/**
	 * Tests the array access methods
	 */
	public void testArrayAccess () {
		for ( int i = 0; i < components.length; ++i ) {
			assertSame(
				components[i],
				k1.getComponent( new String(
					components[i].getConceptId()
				) )
			);
		}
		assertNull(k1.getComponent("NOCONCEPT"));
	}

	/** Test the special concept accessors. */
	public void testSpecialConcepts () {
		assertSame( "FREQ", dimensions[3], k1.getFrequency() );
		assertSame( "TIME_PERIOD", times[1], k1.getTimePeriod() );
		assertSame( "TIME_FORMAT", times[0], k1.getTimeFormat() );
		assertSame( "OBS_VALUE", arrayCells[2], k1.getObsValue() );
		assertSame( "OBS_STATUS", arrayCells[3], k1.getObsStatus() );
		assertSame( "OBS_CONF", arrayCells[0], k1.getObsConf() );
		assertSame( "OBS_PRE_BREAK", arrayCells[1], k1.getObsPreBreak() );
	}

	/**
	 * Test some transients.
	 */
	public void testTransients () {
		assertionsOnTransients( k1 );
	}

	/** Test computeSiblingGroupKey and computeFrequencyValue. */
	public void testComputeSiblingGroupKeyAndFrequencyValue () {
		assertEquals(
			"k1: CO1.COX.CO5.M",
			"CO1.COX.CO5",
			k1.computeSiblingGroupKey( "CO1.COX.CO5.M" )
		);
		assertEquals(
			"k1: CO1.COX.CO5.M (frequency)",
			"M",
			k1.computeFrequencyValue( "CO1.COX.CO5.M" )
		);
		assertEquals(
			"k1: CO1.COX.CO5. (frequency)",
			"",
			k1.computeFrequencyValue( "CO1.COX.CO5." )
		);
		assertEquals(
			"k1: CO1.COX.CO5 (frequency)",
			"",
			k1.computeFrequencyValue( "CO1.COX.CO5" )
		);
		final KeyFamily kb = new KeyFamily(
			"KFB", "name", "desc", versionDate1,
			new KeyFamilyComponent[]{
				new KeyFamilyComponent(
					"FREQ",
					KeyFamilyComponent.FREQUENCY_DIMENSION,
					"1",
					"AN1",
					KeyFamilyComponent.MANDATORY,
					"",
					"CL_TEST",
					true
				),
				DIM2,
				DIM3,
				new KeyFamilyComponent(
					"CONCEPTY",
					KeyFamilyComponent.KEY_STRUCTURE_DIMENSION,
					"4",
					"AN1",
					KeyFamilyComponent.MANDATORY,
					"",
					"CL_TEST",
					true
				),
				TIME1,
				TIME2,
				OBS_VALUE,
				OBS_STATUS
			}
		);
		assertEquals(
			"kb: M.CO1.COX.COY",
			"CO1.COX.COY",
			kb.computeSiblingGroupKey( "M.CO1.COX.COY" )
		);
		assertEquals(
			"kb: M.CO1.COX.COY (frequency)",
			"M",
			kb.computeFrequencyValue( "M.CO1.COX.COY" )
		);
		assertEquals(
			"kb: .CO1.COX.COY (frequency)",
			"",
			kb.computeFrequencyValue( ".CO1.COX.COY" )
		);
		final KeyFamily km = new KeyFamily(
			"KFM", "name", "desc", versionDate1,
			new KeyFamilyComponent[]{
				DIM1,
				new KeyFamilyComponent(
					"FREQ",
					KeyFamilyComponent.FREQUENCY_DIMENSION,
					"2",
					"AN1",
					KeyFamilyComponent.MANDATORY,
					"",
					"CL_TEST",
					true
				),
				DIM3,
				new KeyFamilyComponent(
					"CONCEPTY",
					KeyFamilyComponent.KEY_STRUCTURE_DIMENSION,
					"4",
					"AN1",
					KeyFamilyComponent.MANDATORY,
					"",
					"CL_TEST",
					true
				),
				new KeyFamilyComponent(
					"TIME2",
					KeyFamilyComponent.TIME_DIMENSION,
					"3",
					"AN5",
					KeyFamilyComponent.MANDATORY,
					"",
					"CL_TEST"
				),
				new KeyFamilyComponent(
					"TIME1",
					KeyFamilyComponent.TIME_DIMENSION,
					"2",
					"AN5",
					KeyFamilyComponent.MANDATORY,
					"",
					"CL_TEST"
				),
				new KeyFamilyComponent(
					"OBS_VALUE",
					KeyFamilyComponent.ARRAY_CELL,
					"7",
					"AN..15",
					KeyFamilyComponent.MANDATORY,
					"",
					""
				),
				new KeyFamilyComponent(
					"OBS_STATUS",
					KeyFamilyComponent.ARRAY_CELL,
					"8",
					"AN..15",
					KeyFamilyComponent.MANDATORY,
					"",
					""
				)
			}
		);
		assertEquals(
			"km: CO1.M.COX.COY",
			"CO1.COX.COY",
			km.computeSiblingGroupKey( "CO1.M.COX.COY" )
		);
		assertEquals(
			"km: CO1.M.COX.COY (frequency)",
			"M",
			km.computeFrequencyValue( "CO1.M.COX.COY" )
		);
		assertEquals(
			"km: CO1..COX.COY (frequency)",
			"",
			km.computeFrequencyValue( "CO1..COX.COY" )
		);
		final KeyFamily ks = new KeyFamily(
			"KFS", "name", "desc", versionDate1,
			new KeyFamilyComponent[]{
				new KeyFamilyComponent(
					"FREQ",
					KeyFamilyComponent.FREQUENCY_DIMENSION,
					"1",
					"AN1",
					KeyFamilyComponent.MANDATORY,
					"",
					"CL_TEST",
					true
				),
				TIME1,
				TIME2,
				OBS_VALUE,
				OBS_STATUS
			}
		);
		assertEquals(
			"ks: M",
			"",
			ks.computeSiblingGroupKey( "M" )
		);
		assertEquals(
			"ks: M (frequency)",
			"M",
			ks.computeFrequencyValue( "M" )
		);
		assertEquals(
			"ks: \"\" (frequency)",
			"",
			ks.computeFrequencyValue( "" )
		);
	}

	/**
	 * Tests serialization.
	 */
	public void testSerialization () {
		final KeyFamily k2
		= (KeyFamily) AttributeTest.transserialize( k1 );
		assertEquals( k1, k2 );
		assertionsOnTransients( k2 );
	}

	private void assertionsOnTransients ( final KeyFamily kf ) {
		assertEquals(
			"ComponentCount", components.length, kf.getComponentCount()
		);
		assertEquals(
			"KeyDimensionCount", dimensions.length, kf.getKeyDimensionCount()
		);
		assertEquals(
			"TimeDimensionCount", times.length, kf.getTimeDimensionCount()
		);
		assertEquals(
			"ArrayCellCount", arrayCells.length, kf.getArrayCellCount()
		);
		assertEquals(
			"FrequencyPosition", 4, kf.getFrequencyDimensionPosition()
		);

		KeyFamilyComponent[] array = (KeyFamilyComponent[])components.clone();
		Arrays.sort( array );
		assertionsOnIterator ( kf.componentIterator(), array );

		array = (KeyFamilyComponent[])dimensions.clone();
		Arrays.sort( array );
		assertionsOnIterator ( kf.keyStructureIterator(), array );

		array = (KeyFamilyComponent[])times.clone();
		Arrays.sort( array );
		assertionsOnIterator ( kf.timeDimensionIterator(), array );

		array = (KeyFamilyComponent[])arrayCells.clone();
		Arrays.sort( array );
		assertionsOnIterator ( kf.arrayCellIterator(), array );

		array = (KeyFamilyComponent[])attributes.clone();
		Arrays.sort( array );
		assertionsOnIterator( kf.attributeIterator(), array );

		array = (KeyFamilyComponent[]) obsAttributes.clone();
		Arrays.sort( array );
		assertionsOnIterator( kf.attributeIterator(
			KeyFamilyComponent.ATTACHED_TO_OBSERVATION
		), array );

		array = (KeyFamilyComponent[]) tsAttributes.clone();
		Arrays.sort( array );
		assertionsOnIterator( kf.attributeIterator(
			KeyFamilyComponent.ATTACHED_TO_TIME_SERIES
		), array );

		array = (KeyFamilyComponent[]) sgAttributes.clone();
		Arrays.sort( array );
		assertionsOnIterator( kf.attributeIterator(
			KeyFamilyComponent.ATTACHED_TO_SIBLING_GROUP
		), array );

		array = (KeyFamilyComponent[]) dsAttributes.clone();
		Arrays.sort( array );
		assertionsOnIterator( kf.attributeIterator(
			KeyFamilyComponent.ATTACHED_TO_DATA_SET
		), array );
	}

	private void assertionsOnIterator (
		final Iterator iterator, final KeyFamilyComponent[] array
	) {
		int i = 0;
		while ( iterator.hasNext() ) {
			final KeyFamilyComponent kfc = (KeyFamilyComponent) iterator.next();
			assertEquals(
				"iterating key",
				array[i],
				kfc
			);
			i++;
		}

		assertEquals( "Iterated through exactly all", array.length, i );

		try {
			iterator.remove();
			fail( "UnsupportedOperationException not thrown on remove" );
		} catch ( UnsupportedOperationException e ) {
		}
	}
}
