// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import Int.ecb.gesmescb.MessageBegin;
import Int.ecb.gesmescb.MessageEnd;

/* ***************************** Class ****************************** */

/**
 * <P>Tests the implementation of the class {@link MessageEnd}.</P>
 *
 * @author
 * <A HREF="mailto:Sami.Airo@ECB.int">Sami Airo</A>
 * @author
 * <A HREF="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</A>
 *
 * @version @(#)$Id: MessageEndTest.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */

public class MessageEndTest extends TestCase {

	/* ========================= Fields ========================= */

	private MessageEnd me1, me2, me3, me4;

	/* ====================== Constructors ====================== */

	/**
	 * Constructs a MessageEndTest with the specified name.
	 *
	 * @param name Test case name
	 */
	public MessageEndTest ( String name ) {
		super( name );
	}

	/* ================== Protected Methods ===================== */

	/**
	 * Sets up the test fixture
	 */
	protected void setUp () {
		me1 = new MessageEnd( "MREF123456" );
		me2 = new MessageEnd( new String( "MREF123456" ) );
		me3 = new MessageEnd( "MREF" );
		me4 = new MessageEnd( "MRF" );
	}

	/**
	 * Tears down the test fixture
	 */
	protected void tearDown () {
		me1 = me2 = me3 = me4 = null;
	}


	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite
	 */
	public static Test suite () {
		TestSuite suite = new TestSuite( MessageEndTest.class );
		return suite;
	}

	/**
	 * Runs the test case.
	 */
	public static void main ( String args[] ) {
		String[] testCaseName = { MessageEndTest.class.getName() };
		junit.textui.TestRunner.main( testCaseName );
	}

	/* ---------------------- Test Methods ---------------------- */

	/**
	 * Tests the equals and hashcode methods method
	 */
	public void testEquals () {
		assertEquals( "me1 equals me2", me1, me2 );
		assertEquals(
			"me1.hashCode == me2.hashCode",
			me1.hashCode(),
			me2.hashCode()
		);
		assertEquals(
			"me1.toString equals me2.toString",
			me1.toString(),
			me2.toString()
		);

		assertTrue( "me1 unequal me3", !me1.equals( me3 ) );
		// Inequality of the hashCode is not really contract.
		// But the probability of eventual equal hashCode is
		// far less than that of a buggy
		// hashCode-implementation.
		// Therefore check.
		assertTrue(
			"me1.hashCode unequal",
			me1.hashCode() != me3.hashCode()
		);
		assertTrue(
			"me1.toString unequal",
			!me1.toString().equals( me3.toString() )
		);
	}

	/**
	 * Tests the fields.
	 */
	public void testFields () {
		assertEquals( "MREF123456", me1.getMref() );
		assertEquals( "mrefNo", 123456, me1.getMrefNo() );
		assertEquals(
			"illegal mrefNo3", MessageEnd.INVALID_MREF, me3.getMrefNo()
		);
		assertEquals(
			"illegal mrefNo4", MessageEnd.INVALID_MREF, me4.getMrefNo()
		);
	}

	/**
	 * Test correspondence to MessageBegin.
	 */
	public void testCorrespond () {
		final MessageBegin mb1 = new MessageBegin(
			new String( "MREF123456" ),
			"CIA", "SIS", "ECB", MessageBegin.STATISTICAL_DATA,
			"subject", new Contact[] {}
		);
		final MessageBegin mb2 = new MessageBegin(
			"MREF113456", "CIA", "SIS", "ECB", MessageBegin.STATISTICAL_DATA,
			"subject", new Contact[] {}
		);

		assertTrue( "ends", me1.isEndFor( mb1 ) );
		assertTrue( "ends not", !me1.isEndFor( mb2 ) );
	}

	/**
	 * Tests serialization.
	 */
	public void testSerialization () {
		final MessageEnd transserialized
		= (MessageEnd) AttributeTest.transserialize( me1 );
		assertEquals( me1, transserialized );
		assertEquals( me1.hashCode(), transserialized.hashCode() );
		assertEquals( me1.getMrefNo(), transserialized.getMrefNo() );
	}
}
