// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb;

import java.util.Date;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/* ***************************** Class ****************************** */

/**
 * <P>Tests the implementation of {@link StatisticalConcept}.</P>
 *
 * @author
 * <A HREF="mailto:Sami.Airo@ECB.int">Sami Airo</A>
 * @author
 * <A HREF="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</A>
 *
 * @version @(#)$Id: StatisticalConceptTest.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */

public class StatisticalConceptTest extends TestCase {

	/* ====================== Constructors ====================== */

	/**
	 * Constructs a StatisticalConceptTest with the specified name.
	 *
	 * @param name Test case name
	 */
	public StatisticalConceptTest ( final String name ) {
		super( name );
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite
	 */
	public static Test suite () {
		return new TestSuite( StatisticalConceptTest.class );
	}

	/**
	 * Runs the test case.
	 */
	public static void main ( String args[] ) {
		final String[] testCaseName = {
			StatisticalConceptTest.class.getName()
		};
		junit.textui.TestRunner.main( testCaseName );
	}

	/* --------------------- Test Methods --------------------- */

	/**
	 * Tests the equals and hashcode methods method
	 */
	
	public void testEquals () {
		Date date  = new Date( ); 
		StatisticalConcept c1 = new StatisticalConcept("CONCEPT1", "Concept 1", "1abc", date );
		StatisticalConcept c2 = new StatisticalConcept("CONCEPT1", "Concept 1", "1abc", date );
		StatisticalConcept c3 = new StatisticalConcept("CONCEPT2", "Concept 1", "1abc", date );

		assertTrue("c1 equals c2", c1.equals(c2));
		assertTrue("c1 not equals c3", !c1.equals(c3));
		assertTrue("c1 hashcode == c2.hashcode", c1.hashCode() == c2.hashCode());
		assertTrue("c1 hashcode != c3.hashcode", c1.hashCode() != c3.hashCode());
	}

	/**
	 * Tests the normal fields
	 */
	public void testNormalFields () {
		Date date  = new Date( ); 
		StatisticalConcept c1 = new StatisticalConcept("CONCEPT1", "Concept 1", "1abc", date );

		assertEquals("CONCEPT1", c1.getId());
		assertEquals("Concept 1", c1.getName());
		assertEquals("1abc", c1.getDescription());
		assertEquals( date, c1.getVersionDate() );
	}

	/**
	 * Tests serialization.
	 */
	public void testSerialization () {
		Date date  = new Date( ); 
		final StatisticalConcept c1
		= new StatisticalConcept( "CONCEPT1", "Concept 1", "1abc", date );
		final StatisticalConcept c2
		= (StatisticalConcept) AttributeTest.transserialize( c1 );
		assertEquals( c1, c2 );
	}
}
