// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.conversion;

import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;
import java.util.StringTokenizer;
import java.util.NoSuchElementException;

import Int.ecb.logging.Level;

import Int.ecb.gesmescb.*;
import Int.ecb.gesmescb.event.*;

/* **************************** Class ***************************** */

/**
 * Converts receiver according to preference settings.
 *
 * @author
 * <A HREF="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</A>
 * @author
 * <A HREF="mailto:Sami.Airo@ECB.int">Sami Airo</A>
 *
 * @version @(#)$Id: PreferenceConfiguredReceiverConverter.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */

public class PreferenceConfiguredReceiverConverter extends GESMESCBEventBroker {

	/* ======================= Constants ======================== */

	/* ========================= Fields ========================= */

	/* ====================== Constructors ====================== */

	public PreferenceConfiguredReceiverConverter () {
		this( "" );
	}

	public PreferenceConfiguredReceiverConverter( final String name ) {
		super( name, true );
	}

	/* ===================== Public Methods ===================== */

	public void handle ( final InterchangeBegin interchangeBegin ) {
		final String[] conversion = getConversion(interchangeBegin.getSender(), interchangeBegin.getReceiver());
		
		if (conversion == null) return;

		if (conversion[0] == null) {
			dispatch(interchangeBegin);
			return;
		}

		dispatch(
			new InterchangeBegin(
					interchangeBegin.getIref(),
					conversion[0],
					conversion[1],
					interchangeBegin.getPreparationDateTime(),
					interchangeBegin.isTest() 
			)
		);
	}


	public void handle ( final MessageBegin messageBegin ) {
		final String[] conversion = getConversion( messageBegin.getSender(), messageBegin.getReceiver());

		if (conversion == null) return;

		if (conversion[0] == null) {
			dispatch(messageBegin);
			return;
		}

		final List contacts = new ArrayList();
		for (
			final Iterator iterator = messageBegin.contactIterator();
			iterator.hasNext();
		) {
			contacts.add( iterator.next() );
		}

		dispatch(
			new MessageBegin(
				messageBegin.getMref(),
				conversion[0],
				conversion[1],
				messageBegin.getMaintenanceAgencyId(),
				messageBegin.getFunction(),
				messageBegin.getIdentity(),
				contacts,
				messageBegin.getDataSetId(),
				messageBegin.getAction(),
				messageBegin.getExtractionDateTime(),
				messageBegin.getReportingPeriod(),
				messageBegin.getReportingPeriodFormat(),
				messageBegin.getKeyFamilyId(),
				messageBegin.getOriginalDataSetId()
			)
		);
	}
	

	protected void logAndDispatchSevereError (
		final String msg,
		final String procedureName,
		final Exception e
	) {
		componentHelper.logger.log(
			Level.SEVERE,
			componentHelper.logName,
			procedureName,
			msg,
			e
		);
		
		dispatch(
			new ErrorEvent( msg, ErrorEvent.ERROR )
		);
	}
	
	
	public String[] getConversion ( 
		final String sender, 
		final String receiver 
	) {
		final String address = sender + "." + receiver;
		final String conversion = componentHelper.getPreference("convert." + address, null);
		final String[] values = new String[2];

		if (conversion == null) return values;

		StringTokenizer tokenizer = new StringTokenizer(conversion, ".");		

		try {
			values[0] = tokenizer.nextToken();
			values[1] = tokenizer.nextToken();
		} catch (final NoSuchElementException e) {
			logAndDispatchSevereError(
				"Error in parsing preference value \"" + conversion + "\"",
				"getConversion",
				e
			);
			return null;
		}

		componentHelper.logger.log(
			Level.CONFIG, componentHelper.logName, "getConversion",
			"Converted sender/receiver from " + address + " to " + conversion
		);
		
		return values;
	}
}
