// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.conversion;

import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;

import Int.ecb.logging.Level;

import Int.ecb.gesmescb.MessageBegin;
import Int.ecb.gesmescb.event.GESMESCBEventBroker;

/* **************************** Class ***************************** */

/**
 * Converts data set identifiers according to preference settings.
 *
 * @author
 * <A HREF="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</A>
 * @author
 * <A HREF="mailto:Sami.Airo@ECB.int">Sami Airo</A>
 *
 * @version @(#)$Id: SimpleDSIConverter.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */

public class SimpleDSIConverter extends GESMESCBEventBroker {

	/* ======================= Constants ======================== */

	/* ========================= Fields ========================= */

	/* ====================== Constructors ====================== */

	public SimpleDSIConverter () {
		this( "" );
	}

	public SimpleDSIConverter( final String name ) {
		super( name, true );
	}

	/* ===================== Public Methods ===================== */

	public void handle ( final MessageBegin messageBegin ) {
		final boolean reportConversionInMessageIdentity
		= componentHelper.getPreference(
			"reportConversionInMessageIdentity", "false"
		).equalsIgnoreCase("true");
														
		final String internalDatasetName
		= componentHelper.getPreference(
			"internalName." + messageBegin.getDataSetId(), 
			messageBegin.getDataSetId()
		);

		String identity = messageBegin.getIdentity();

		if (! internalDatasetName.equals( messageBegin.getDataSetId() ) ) {
			componentHelper.logger.log(
				Level.INFO,
				componentHelper.logName,
				"handle",
				"Data set identifier " + messageBegin.getDataSetId()
				+ " converted to " + internalDatasetName + "."
			);
			
			if (reportConversionInMessageIdentity) {
				identity += "\nOriginal dataflow "
					+ messageBegin.getSender() + "." + messageBegin.getReceiver() 
					+ "." + messageBegin.getMaintenanceAgencyId()
					+ "." + messageBegin.getKeyFamilyId()
					+ "." + messageBegin.getDataSetId()
					+ " converted to "
					+ messageBegin.getSender() + "." + messageBegin.getReceiver() 
					+ "." + messageBegin.getMaintenanceAgencyId()
					+ "." + messageBegin.getKeyFamilyId()
					+ "." + internalDatasetName;
			}
		}

		final List contacts = new ArrayList();
		for (
			final Iterator iterator = messageBegin.contactIterator();
			iterator.hasNext();
		) {
			contacts.add( iterator.next() );
		}

		dispatch(
			new MessageBegin(
				messageBegin.getMref(),
				messageBegin.getSender(),
				messageBegin.getReceiver(),
				messageBegin.getMaintenanceAgencyId(),
				messageBegin.getFunction(),
				identity,
				contacts,
				internalDatasetName,
				messageBegin.getAction(),
				messageBegin.getExtractionDateTimeAsDate(),
				messageBegin.getReportingPeriod(),
				messageBegin.getReportingPeriodFormat(),
				messageBegin.getKeyFamilyId(),
				messageBegin.getOriginalDataSetId()
			)
		);
	}

}
