// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001-2005, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.conversion;

import java.util.Properties;
import java.util.Collections;

import java.io.IOException;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import Int.ecb.gesmescb.*;
import Int.ecb.gesmescb.event.*;

/* ***************************** Class ****************************** */

/**
 * Tests the implementation of the class {@link PreferenceConfiguredKeyConverter}.
 *
 * @author
 * <A HREF="mailto:Sami.Airo@ECB.int">Sami Airo</A>
 * <a href="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</A>
 *
 * @version @(#)$Id: SimpleDSIConverterTest.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */

public class SimpleDSIConverterTest extends TestCase {

	/* ======================= Constants ======================== */

	protected static final MessageBegin DATA_MESSAGE_BEGIN_MATCHING = new MessageBegin(
		"MREF000001", "LT2", "4F0", "ECB",
		MessageBegin.STATISTICAL_DATA,
		"Identity", Collections.EMPTY_SET,
		"ECB_QFG",
		MessageBegin.UPDATE_OR_REPLACE_ACTION,
		"20040804:1112", "20002003", "702",
		"ECB_MUFA"
	);

	protected static final MessageBegin DATA_MESSAGE_BEGIN_MISMATCHING = new MessageBegin(
		"MREF000001", "LU2", "4F0", "ECB",
		MessageBegin.STATISTICAL_DATA,
		"Identity", Collections.EMPTY_SET,
		"ECB_MUFA",
		MessageBegin.UPDATE_OR_REPLACE_ACTION,
		"20040804:1112", "20002003", "702",
		"ECB_MUFA"
	);

	/* ========================= Fields ========================= */

	/* ====================== Constructors ====================== */

	/**
	 * Constructs a SimpleDSIConverterTest with the specified name.
	 *
	 * @param name Test case name
	 */
	public SimpleDSIConverterTest ( final String name ) {
		super( name );
	}

	/* ================== Protected Methods ===================== */

	/**
	 * Sets up the test fixture.
	 */
	protected void setUp () throws IOException {
		System.setProperty( ".level", "ALL" );
		System.setProperty( "handlers", "Int.ecb.logging.ConsoleHandler" );
		System.setProperty(
			"Int.ecb.logging.ConsoleHandler.formatter",
			"Int.ecb.gesmescb.util.NormalLogFormatter"
		);

		final String className = SimpleDSIConverter.class.getName();
		System.setProperty(
			className + ".internalName.ECB_QFG", "GST"
		);
		System.setProperty(
			className + ".reportConversionInMessageIdentity",
			"false"
		);
	}

	/**
	 * Tears down the test fixture.
	 */
	protected void tearDown () {
		final String className = SimpleDSIConverterTest.class.getName();
		final Properties properties = System.getProperties();
		properties.remove(
			className + ".internalName.ECB_QFG"
		);
		properties.remove(
			className + ".reportConversionInMessageIdentity"
		);
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite.
	 */
	public static Test suite () {
		return new TestSuite( SimpleDSIConverterTest.class );
	}

	/**
	 * Runs the test case.
	 */
	public static void main ( final String args[] ) {
		final String[] testCaseName = {
			SimpleDSIConverterTest.class.getName()
		};
		junit.textui.TestRunner.main( testCaseName );
	}

	/* --------------------- Test Methods --------------------- */

	public void testConvertMessage () {
		final SimpleDSIConverter converter = new SimpleDSIConverter();
		final TestEventListener listener = new TestEventListener();
		converter.addListener( listener, true );

		converter.handle( DATA_MESSAGE_BEGIN_MATCHING );
		assertEquals( "MessageBegin size 1", 1, listener.size() );
		assertEquals(
			"MessageBegin converted OK",
			new MessageBegin(
				"MREF000001", "LT2", "4F0", "ECB",
				MessageBegin.STATISTICAL_DATA,
				"Identity", Collections.EMPTY_SET,
				"GST",
				MessageBegin.UPDATE_OR_REPLACE_ACTION,
				"20040804:1112", "20002003", "702",
				"ECB_MUFA",
				"ECB_QFG"
			),
			listener.getLastObject()
		);
	}

	public void testNotConvertMessage () {
		final SimpleDSIConverter converter = new SimpleDSIConverter();
		final TestEventListener listener = new TestEventListener();
		converter.addListener( listener, true );

		converter.handle( DATA_MESSAGE_BEGIN_MISMATCHING );
		assertEquals( "MessageBegin size 1", 1, listener.size() );
		assertEquals(
			"MessageBegin converted OK",
			DATA_MESSAGE_BEGIN_MISMATCHING,
			listener.getLastObject()
		);
	}

	public void testConvertReportMessage () {
		final String className = SimpleDSIConverter.class.getName();
		System.setProperty(
			className + ".reportConversionInMessageIdentity",
			"true"
		);

		final SimpleDSIConverter converter = new SimpleDSIConverter();
		final TestEventListener listener = new TestEventListener();
		converter.addListener( listener, true );

		converter.handle( DATA_MESSAGE_BEGIN_MATCHING );
		assertEquals( "MessageBegin size 1", 1, listener.size() );
		assertEquals(
			"MessageBegin converted OK",
			new MessageBegin(
				"MREF000001", "LT2", "4F0", "ECB",
				MessageBegin.STATISTICAL_DATA,
				DATA_MESSAGE_BEGIN_MATCHING.getIdentity() + 
				"\nOriginal dataflow LT2.4F0.ECB.ECB_MUFA.ECB_QFG converted to LT2.4F0.ECB.ECB_MUFA.GST", 
				Collections.EMPTY_SET,
				"GST",
				MessageBegin.UPDATE_OR_REPLACE_ACTION,
				"20040804:1112", "20002003", "702",
				"ECB_MUFA", "ECB_QFG"
			),
			listener.getLastObject()
		);
	}
}
