// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2003, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.csv;

import java.util.NoSuchElementException;

import java.io.Reader;
import java.io.StringReader;
import java.io.IOException;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/* ***************************** Class ****************************** */

/**
 * Implements a few tests for the {@link CSVTokenizer}.
 *
 * @author
 * <A HREF="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</A>
 * @author
 * <A HREF="mailto:Ana.Padrao@ECB.int">Ana Garcia Padrao</A>
 *
 * @version @(#)$Id: CSVTokenizerTest.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */
public class CSVTokenizerTest extends TestCase {

	/* ====================== Constructors ====================== */

	public CSVTokenizerTest ( final String name ) {
		super( name );
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite
	 */
	public static Test suite () {
		return new TestSuite( CSVTokenizerTest.class );
	}

	/**
	 * @param arg the array of argument {@link String}s
	 */
	public static void main ( final String[] arg ) {
		final String[] testCaseName = {
			CSVTokenizerTest.class.getName()
		};
		junit.textui.TestRunner.main( testCaseName );
	}

	/* ---------------------- Test Methods ---------------------- */

	/** Test {@link CSVTokenizer#run}. */
	public void test ()
		throws IOException
	{
		final Reader reader = new StringReader(
			"abc,ced,\nbb,,,\n,d\n"
		);
		final CSVTokenizer tokenizer = new CSVTokenizer( reader );

		assertTrue( "1st line exists", tokenizer.hasMoreLines() );
		final CSVTokenizer.Line line1 = tokenizer.nextLine();
		assertEquals( "Is line 1", 1, line1.getLineNo() );
		assertEquals( "Is cell 0", 0, line1.getCellNo() );
		assertTrue( "1st cell exists", line1.hasMoreCells() );
		assertEquals( "1st cell", "abc", line1.nextCell() );
		assertEquals( "Is cell 1", 1, line1.getCellNo() );
		assertTrue( "2nd cell exists", line1.hasMoreCells() );
		assertEquals( "2nd cell", "ced", line1.nextCell() );
		assertTrue( "nothing more in line 1", !line1.hasMoreCells() );

		assertTrue( "2nd line exists", tokenizer.hasMoreLines() );
		final CSVTokenizer.Line line2 = tokenizer.nextLine();
		assertTrue( "2,1 cell exists", line2.hasMoreCells() );
		assertEquals( "bb", line2.nextCell() );
		assertTrue( "nothing more in line 2", !line2.hasMoreCells() );
		try {
			line2.nextCell();
			fail( "'No line remainder' should throw exception." );
		}
		catch ( final NoSuchElementException ne ) {}

		assertTrue( "3nd line exists", tokenizer.hasMoreLines() );
		final CSVTokenizer.Line line3 = tokenizer.nextLine();
		assertTrue( "3,1 cell exists", line3.hasMoreCells() );
		assertEquals( "", line3.nextCell() );
		assertTrue( "3,2 cell exists", line3.hasMoreCells() );
		assertEquals( "d", line3.nextCell() );
		assertTrue( "nothing more in line 3", !line3.hasMoreCells() );

		assertTrue( "4nd line doesn't exist", !tokenizer.hasMoreLines() );
		try {
			tokenizer.nextLine();
			fail( "'No file remainder' should throw exception." );
		}
		catch ( final NoSuchElementException ne ) {}

		reader.close();
	}
}
