// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2007, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.csv;

import java.io.IOException;

/* ***************************** Class ****************************** */

/**
 * <p>Writes {@link TimeSeries} to a EAACS project style empty output template
 * for Matlab.</p>
 *
 * @see EaacsLabWriter for configuration
 * @see EaacsLabConfig for shared configuration
 *
 * @author
 * <a href="mailto:sis.external@ecb.int">Bernhard Bodenstorfer</a>
 *
 * @version @(#)$Id: EaacsTemplateWriter.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */
public class EaacsTemplateWriter extends EaacsLabWriter {

	/* ====================== Constructors ====================== */

	/**
	 * Construct a <code>EaacsTemplateWriter</code> to write to a file.
	 *
	 * @param name used for logging and preferences.
	 * @param filename where to write to.
	 *
	 * @throws NullPointerException if <code>filename==null</code>
	 * and the preference variable
	 * <code>destination</code> is undefined.
	 * @throws IOException if the file cannot be opened for writing.
	 */
	public EaacsTemplateWriter ( final String name, final String filename )
		throws IOException
	{
		super( name, filename, EMPTY_STRATEGY );
	}

	/**
	 * Construct a <code>EaacsTemplateWriter</code>
	 * writing to a file according to preferences.
	 *
	 * @param name used for logging and preferences.
	 *
	 * @throws NullPointerException if the preference variable
	 * <code>destination</code> is undefined.
	 * @throws IOException if the file cannot be opened for writing.
	 */
	public EaacsTemplateWriter ( final String name )
		throws IOException
	{
		this( name, null );
	}
}
