// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2003, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.csv;

import java.util.Iterator;
import java.util.Date;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.text.ParseException;

import java.io.File;
import java.io.Reader;
import java.io.StringReader;
import java.io.Writer;
import java.io.FileWriter;
import java.io.BufferedWriter;
import java.io.IOException;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import Int.ecb.gesmescb.*;

import Int.ecb.gesmescb.event.ErrorEvent;
import Int.ecb.gesmescb.event.TestEventListener;

/* ***************************** Class ****************************** */

/**
 * Implements a few tests for the {@link SPFReader}.
 *
 * @author
 * <A HREF="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</A>
 * @author
 * <A HREF="mailto:Ana.Padrao@ECB.int">Ana Garcia Padrao</A>
 *
 * @version @(#)$Id: SPFReaderTest.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */
public class SPFReaderTest extends TestCase {

	/* ======================= Constants ======================== */

	/** The input to the {@link SPFReader} under Test. */
	private static final String testInput
	= "SPF,Oct-2002,"
	+ "HICP,,,,,,,,,"
	+ "GDP,,,,,,,,,"
	+ "Unemployment,,,,,,,,,"
	+ "Assumptions,,,,,Assumptions,,\n"
	+ "Period,Forecaster,"
	+ ",<0.0,0.0-0.4,>0.4,,Sum,,Point,,"
	+ ",<1.0,1.0-1.4,>1.4,,Sum,,Point,,"
	+ ",<5.0,5.0-10.0,>11.0,,Sum,,Point,,"
	+ ",ir,oil,usd,,,lab,\n"
	+ "0,ECB,"
	+ "Sep-2002,,,,,,,2.31%,,"
	+ "2002Q2,,,,,,,0.9%,,"
	+ "Aug-2002,,,,,,,8.4%,,"
	+ ",,,,,,,X\n"
	+ "1,1,"
	+ "2002,0.00%,4.00%,3.00%,,7.00%,,2.1%,,"
	+ "2002,13.60%,15.00%,60.00%,,97.00%,,0.8%,,"
	+ "2002,1.00%,0.00%,0.00%,,100.00%,,8.4%,,"
	+ "2002Q4,3.250%,28.000,0.980,,2002,1.1,X\n"
	+ "1,2,"
	+ "2002,0.00%,0.00%,0.00%,,0.00%,,2.2%,,"
	+ "2002,0.0%,0.0%,90.0%,,90.00%,,0.7%,,"
	+ "2002,100.00%,0.00%,0.00%,,100.00%,,8.3%,,"
	+ "2002Q4,,,,,2002,1.2,X\n"
	+ "2,1,"
	+ "2003,0.00%,4.40%,21.00%,,100.00%,,2.2%,,"
	+ "2003,0.00%,0.00%,7.00%,,0.0%,,4.2%,,"
	+ "2003,0.00%,0.1%,0.00%,,40.00%,,1.7%,,"
	+ "2003Q1,3.450%,27.000,0.999,,2003,2.1,X\n"
	+ "3,1,"
	+ "2004,25.00%,11.11%,1.00%,,111.00%,,2.3%,,"
	+ "2004,30.00%,27.00%,1.00%,,330.00%,,2.2%,,"
	+ "2004,2%,1.%,4.00%,,00.00%,,9.1%,,"
	+ "2003Q2,3.4%,25.0,1,,2004,3.1,X\n"
	+ "4,1,"
	+ "Sep-2003,12.00%,8.00%,1.00%,,13.40%,,4.6%,,"
	+ "2003Q2,0.0%,8.30%,7.90%,,42.00%,,2.4%,,"
	+ "Aug-2003,0%,1.01%,00%,,070%,,2.3%,,"
	+ "2003Q3,3.350%,24,01.20,,2005,4.1,X\n"
	+ "5,1,"
	+ "Sep-2004,0.00%,0.00%,6.00%,,100.00%,,2.6%,,"
	+ "2004Q2,3.00%,3.00%,0.00%,,10.00%,,2.1%,,"
	+ "Aug-2004,1.00%,5.00%,.02%,,100.00%,,6.8%,,"
	+ "2003Q4,30%,26,1.2,,,,X\n"
	+ "6,1,"
	+ "2007,10.00%,10.00%,10.00%,,30.00%,,1.3%,,"
	+ "2007,45.0%,23.0%,3.0%,,100.00%,,1.3%,,"
	+ "2007,5.7%,7%,3.3%,,100.00%,,1.1%,,"
	+ ",,,,,,,X\n"
	+ "7,1,"
	+ "Dec-2004,1.00%,2.00%,6.00%,,100.00%,,2.4%,,"
	+ "2004Q4,7.00%,13.00%,14.00%,,130.00%,,3.1%,,"
	+ "Nov-2004,19.00%,50.00%,30%,,100.00%,,8.8%,,"
	+ ",,,,,,,X\n"
	+ "9,100,"
	+ "X,X,X,X,X,X,X,X,X,"
	+ "X,X,X,X,X,X,X,X,X,"
	+ "X,X,X,X,X,X,X,X,X,"
	+ "X,X,X,X,X,X,X,X\n";

	/**
	 * The objects into which
	 * the {@link SPFReader} should translate
	 * {@link #testInput} when called in the correct way.
	 */
	private static final Object[] testOutput = {
		new InterchangeBegin(
			"IREF000035", "4F0", "4F0", "001201:1410", true
		),
		new MessageBegin(
			"MREF000001", "4F0", "4F0", "ECB",
			MessageBegin.STATISTICAL_DATA, "csv_test.csv",
			new Contact[]{},
			"ECB_SPF", MessageBegin.UPDATE_OR_REPLACE_ACTION,
			"200203140015", "", "", "ECB_FCT1"
		),
		// 0,1
		new TimeSeries(
			"M.U2.HICP.T0_0.M0M.Q.ECB", "200209", "610",
			new Observation[] { new Observation( "0", "A", "F", "" ) }
		),
		new Attribute(
			".U2.HICP.T0_0.M0M.Q.ECB", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"M.U2.HICP.F0_0T0_4.M0M.Q.ECB", "200209", "610",
			new Observation[] { new Observation( "0", "A", "F", "" ) }
		),
		new Attribute(
			".U2.HICP.F0_0T0_4.M0M.Q.ECB", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"M.U2.HICP.F0_4.M0M.Q.ECB", "200209", "610",
			new Observation[] { new Observation( "0", "A", "F", "" ) }
		),
		new Attribute(
			".U2.HICP.F0_4.M0M.Q.ECB", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"M.U2.HICP.SUM.M0M.Q.ECB", "200209", "610",
			new Observation[] { new Observation( "0", "A", "F", "" ) }
		),
		new Attribute(
			".U2.HICP.SUM.M0M.Q.ECB", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"M.U2.HICP.POINT.M0M.Q.ECB", "200209", "610",
			new Observation[] { new Observation( "2.31", "A", "F", "" ) }
		),
		new Attribute(
			".U2.HICP.POINT.M0M.Q.ECB", "", "", "UNIT", "PCPA"
		),
		new TimeSeries(
			"Q.U2.RGDP.T1_0.M3M.Q.ECB", "20022", "608",
			new Observation[] { new Observation( "0", "A", "F", "" ) }
		),
		new Attribute(
			".U2.RGDP.T1_0.M3M.Q.ECB", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"Q.U2.RGDP.F1_0T1_4.M3M.Q.ECB", "20022", "608",
			new Observation[] { new Observation( "0", "A", "F", "" ) }
		),
		new Attribute(
			".U2.RGDP.F1_0T1_4.M3M.Q.ECB", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"Q.U2.RGDP.F1_4.M3M.Q.ECB", "20022", "608",
			new Observation[] { new Observation( "0", "A", "F", "" ) }
		),
		new Attribute(
			".U2.RGDP.F1_4.M3M.Q.ECB", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"Q.U2.RGDP.SUM.M3M.Q.ECB", "20022", "608",
			new Observation[] { new Observation( "0", "A", "F", "" ) }
		),
		new Attribute(
			".U2.RGDP.SUM.M3M.Q.ECB", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"Q.U2.RGDP.POINT.M3M.Q.ECB", "20022", "608",
			new Observation[] { new Observation( "0.9", "A", "F", "" ) }
		),
		new Attribute(
			".U2.RGDP.POINT.M3M.Q.ECB", "", "", "UNIT", "PCPA"
		),
		new TimeSeries(
			"M.U2.UNEM.T5_0.M0M.Q.ECB", "200208", "610",
			new Observation[] { new Observation( "0", "A", "F", "" ) }
		),
		new Attribute(
			".U2.UNEM.T5_0.M0M.Q.ECB", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"M.U2.UNEM.F5_0T10_0.M0M.Q.ECB", "200208", "610",
			new Observation[] { new Observation( "0", "A", "F", "" ) }
		),
		new Attribute(
			".U2.UNEM.F5_0T10_0.M0M.Q.ECB", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"M.U2.UNEM.F11_0.M0M.Q.ECB", "200208", "610",
			new Observation[] { new Observation( "0", "A", "F", "" ) }
		),
		new Attribute(
			".U2.UNEM.F11_0.M0M.Q.ECB", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"M.U2.UNEM.SUM.M0M.Q.ECB", "200208", "610",
			new Observation[] { new Observation( "0", "A", "F", "" ) }
		),
		new Attribute(
			".U2.UNEM.SUM.M0M.Q.ECB", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"M.U2.UNEM.POINT.M0M.Q.ECB", "200208", "610",
			new Observation[] { new Observation( "8.4", "A", "F", "" ) }
		),
		new Attribute(
			".U2.UNEM.POINT.M0M.Q.ECB", "", "", "UNIT", "PC"
		),
		// 1,1
		new TimeSeries(
			"A.U2.HICP.T0_0.P3M.Q.001", "2002", "602",
			new Observation[] { new Observation( "0.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.HICP.T0_0.P3M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.HICP.F0_0T0_4.P3M.Q.001", "2002", "602",
			new Observation[] { new Observation( "4.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.HICP.F0_0T0_4.P3M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.HICP.F0_4.P3M.Q.001", "2002", "602",
			new Observation[] { new Observation( "3.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.HICP.F0_4.P3M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.HICP.SUM.P3M.Q.001", "2002", "602",
			new Observation[] { new Observation( "7.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.HICP.SUM.P3M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.HICP.POINT.P3M.Q.001", "2002", "602",
			new Observation[] { new Observation( "2.1", "F", "C", "" ) }
		),
		new Attribute(
			".U2.HICP.POINT.P3M.Q.001", "", "", "UNIT", "PCPA"
		),
		new TimeSeries(
			"A.U2.RGDP.T1_0.P3M.Q.001", "2002", "602",
			new Observation[] { new Observation( "13.60", "F", "C", "" ) }
		),
		new Attribute(
			".U2.RGDP.T1_0.P3M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.RGDP.F1_0T1_4.P3M.Q.001", "2002", "602",
			new Observation[] { new Observation( "15.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.RGDP.F1_0T1_4.P3M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.RGDP.F1_4.P3M.Q.001", "2002", "602",
			new Observation[] { new Observation( "60.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.RGDP.F1_4.P3M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.RGDP.SUM.P3M.Q.001", "2002", "602",
			new Observation[] { new Observation( "97.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.RGDP.SUM.P3M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.RGDP.POINT.P3M.Q.001", "2002", "602",
			new Observation[] { new Observation( "0.8", "F", "C", "" ) }
		),
		new Attribute(
			".U2.RGDP.POINT.P3M.Q.001", "", "", "UNIT", "PCPA"
		),
		new TimeSeries(
			"A.U2.UNEM.T5_0.P3M.Q.001", "2002", "602",
			new Observation[] { new Observation( "1.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.UNEM.T5_0.P3M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.UNEM.F5_0T10_0.P3M.Q.001", "2002", "602",
			new Observation[] { new Observation( "0.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.UNEM.F5_0T10_0.P3M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.UNEM.F11_0.P3M.Q.001", "2002", "602",
			new Observation[] { new Observation( "0.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.UNEM.F11_0.P3M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.UNEM.SUM.P3M.Q.001", "2002", "602",
			new Observation[] { new Observation( "100.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.UNEM.SUM.P3M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.UNEM.POINT.P3M.Q.001", "2002", "602",
			new Observation[] { new Observation( "8.4", "F", "C", "" ) }
		),
		new Attribute(
			".U2.UNEM.POINT.P3M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"Q.U2.ASSU.IR.P3M.Q.001", "20024", "608",
			new Observation[] { new Observation( "3.250", "F", "C", "" ) }
		),
		new Attribute(
			".U2.ASSU.IR.P3M.Q.001", "", "", "UNIT", "PCPA"
		),
		new TimeSeries(
			"Q.U2.ASSU.OIL.P3M.Q.001", "20024", "608",
			new Observation[] { new Observation( "28.000", "F", "C", "" ) }
		),
		new Attribute(
			".U2.ASSU.OIL.P3M.Q.001", "", "", "UNIT", "USD"
		),
		new TimeSeries(
			"Q.U2.ASSU.USD.P3M.Q.001", "20024", "608",
			new Observation[] { new Observation( "0.980", "F", "C", "" ) }
		),
		new Attribute(
			".U2.ASSU.USD.P3M.Q.001", "", "", "UNIT", "USD"
		),
		new TimeSeries(
			"A.U2.ASSU.LAB.P3M.Q.001", "2002", "602",
			new Observation[] { new Observation( "1.1", "F", "C", "" ) }
		),
		new Attribute(
			".U2.ASSU.LAB.P3M.Q.001", "", "", "UNIT", "PCPA"
		),
		// 1,2
		new TimeSeries(
			"A.U2.HICP.T0_0.P3M.Q.002", "2002", "602",
			new Observation[] { new Observation( "0.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.HICP.T0_0.P3M.Q.002", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.HICP.F0_0T0_4.P3M.Q.002", "2002", "602",
			new Observation[] { new Observation( "0.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.HICP.F0_0T0_4.P3M.Q.002", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.HICP.F0_4.P3M.Q.002", "2002", "602",
			new Observation[] { new Observation( "0.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.HICP.F0_4.P3M.Q.002", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.HICP.SUM.P3M.Q.002", "2002", "602",
			new Observation[] { new Observation( "0.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.HICP.SUM.P3M.Q.002", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.HICP.POINT.P3M.Q.002", "2002", "602",
			new Observation[] { new Observation( "2.2", "F", "C", "" ) }
		),
		new Attribute(
			".U2.HICP.POINT.P3M.Q.002", "", "", "UNIT", "PCPA"
		),
		new TimeSeries(
			"A.U2.RGDP.T1_0.P3M.Q.002", "2002", "602",
			new Observation[] { new Observation( "0.0", "F", "C", "" ) }
		),
		new Attribute(
			".U2.RGDP.T1_0.P3M.Q.002", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.RGDP.F1_0T1_4.P3M.Q.002", "2002", "602",
			new Observation[] { new Observation( "0.0", "F", "C", "" ) }
		),
		new Attribute(
			".U2.RGDP.F1_0T1_4.P3M.Q.002", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.RGDP.F1_4.P3M.Q.002", "2002", "602",
			new Observation[] { new Observation( "90.0", "F", "C", "" ) }
		),
		new Attribute(
			".U2.RGDP.F1_4.P3M.Q.002", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.RGDP.SUM.P3M.Q.002", "2002", "602",
			new Observation[] { new Observation( "90.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.RGDP.SUM.P3M.Q.002", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.RGDP.POINT.P3M.Q.002", "2002", "602",
			new Observation[] { new Observation( "0.7", "F", "C", "" ) }
		),
		new Attribute(
			".U2.RGDP.POINT.P3M.Q.002", "", "", "UNIT", "PCPA"
		),
		new TimeSeries(
			"A.U2.UNEM.T5_0.P3M.Q.002", "2002", "602",
			new Observation[] { new Observation( "100.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.UNEM.T5_0.P3M.Q.002", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.UNEM.F5_0T10_0.P3M.Q.002", "2002", "602",
			new Observation[] { new Observation( "0.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.UNEM.F5_0T10_0.P3M.Q.002", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.UNEM.F11_0.P3M.Q.002", "2002", "602",
			new Observation[] { new Observation( "0.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.UNEM.F11_0.P3M.Q.002", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.UNEM.SUM.P3M.Q.002", "2002", "602",
			new Observation[] { new Observation( "100.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.UNEM.SUM.P3M.Q.002", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.UNEM.POINT.P3M.Q.002", "2002", "602",
			new Observation[] { new Observation( "8.3", "F", "C", "" ) }
		),
		new Attribute(
			".U2.UNEM.POINT.P3M.Q.002", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"Q.U2.ASSU.IR.P3M.Q.002", "20024", "608",
			new Observation[] { new Observation( "-", "M", "C", "" ) }
		),
		new Attribute(
			".U2.ASSU.IR.P3M.Q.002", "", "", "UNIT", "PCPA"
		),
		new TimeSeries(
			"Q.U2.ASSU.OIL.P3M.Q.002", "20024", "608",
			new Observation[] { new Observation( "-", "M", "C", "" ) }
		),
		new Attribute(
			".U2.ASSU.OIL.P3M.Q.002", "", "", "UNIT", "USD"
		),
		new TimeSeries(
			"Q.U2.ASSU.USD.P3M.Q.002", "20024", "608",
			new Observation[] { new Observation( "-", "M", "C", "" ) }
		),
		new Attribute(
			".U2.ASSU.USD.P3M.Q.002", "", "", "UNIT", "USD"
		),
		new TimeSeries(
			"A.U2.ASSU.LAB.P3M.Q.002", "2002", "602",
			new Observation[] { new Observation( "1.2", "F", "C", "" ) }
		),
		new Attribute(
			".U2.ASSU.LAB.P3M.Q.002", "", "", "UNIT", "PCPA"
		),
		// 2,1
		new TimeSeries(
			"A.U2.HICP.T0_0.P15M.Q.001", "2003", "602",
			new Observation[] { new Observation( "0.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.HICP.T0_0.P15M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.HICP.F0_0T0_4.P15M.Q.001", "2003", "602",
			new Observation[] { new Observation( "4.40", "F", "C", "" ) }
		),
		new Attribute(
			".U2.HICP.F0_0T0_4.P15M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.HICP.F0_4.P15M.Q.001", "2003", "602",
			new Observation[] { new Observation( "21.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.HICP.F0_4.P15M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.HICP.SUM.P15M.Q.001", "2003", "602",
			new Observation[] { new Observation( "100.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.HICP.SUM.P15M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.HICP.POINT.P15M.Q.001", "2003", "602",
			new Observation[] { new Observation( "2.2", "F", "C", "" ) }
		),
		new Attribute(
			".U2.HICP.POINT.P15M.Q.001", "", "", "UNIT", "PCPA"
		),
		new TimeSeries(
			"A.U2.RGDP.T1_0.P15M.Q.001", "2003", "602",
			new Observation[] { new Observation( "0.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.RGDP.T1_0.P15M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.RGDP.F1_0T1_4.P15M.Q.001", "2003", "602",
			new Observation[] { new Observation( "0.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.RGDP.F1_0T1_4.P15M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.RGDP.F1_4.P15M.Q.001", "2003", "602",
			new Observation[] { new Observation( "7.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.RGDP.F1_4.P15M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.RGDP.SUM.P15M.Q.001", "2003", "602",
			new Observation[] { new Observation( "0.0", "F", "C", "" ) }
		),
		new Attribute(
			".U2.RGDP.SUM.P15M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.RGDP.POINT.P15M.Q.001", "2003", "602",
			new Observation[] { new Observation( "4.2", "F", "C", "" ) }
		),
		new Attribute(
			".U2.RGDP.POINT.P15M.Q.001", "", "", "UNIT", "PCPA"
		),
		new TimeSeries(
			"A.U2.UNEM.T5_0.P15M.Q.001", "2003", "602",
			new Observation[] { new Observation( "0.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.UNEM.T5_0.P15M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.UNEM.F5_0T10_0.P15M.Q.001", "2003", "602",
			new Observation[] { new Observation( "0.1", "F", "C", "" ) }
		),
		new Attribute(
			".U2.UNEM.F5_0T10_0.P15M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.UNEM.F11_0.P15M.Q.001", "2003", "602",
			new Observation[] { new Observation( "0.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.UNEM.F11_0.P15M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.UNEM.SUM.P15M.Q.001", "2003", "602",
			new Observation[] { new Observation( "40.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.UNEM.SUM.P15M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.UNEM.POINT.P15M.Q.001", "2003", "602",
			new Observation[] { new Observation( "1.7", "F", "C", "" ) }
		),
		new Attribute(
			".U2.UNEM.POINT.P15M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"Q.U2.ASSU.IR.P6M.Q.001", "20031", "608",
			new Observation[] { new Observation( "3.450", "F", "C", "" ) }
		),
		new Attribute(
			".U2.ASSU.IR.P6M.Q.001", "", "", "UNIT", "PCPA"
		),
		new TimeSeries(
			"Q.U2.ASSU.OIL.P6M.Q.001", "20031", "608",
			new Observation[] { new Observation( "27.000", "F", "C", "" ) }
		),
		new Attribute(
			".U2.ASSU.OIL.P6M.Q.001", "", "", "UNIT", "USD"
		),
		new TimeSeries(
			"Q.U2.ASSU.USD.P6M.Q.001", "20031", "608",
			new Observation[] { new Observation( "0.999", "F", "C", "" ) }
		),
		new Attribute(
			".U2.ASSU.USD.P6M.Q.001", "", "", "UNIT", "USD"
		),
		new TimeSeries(
			"A.U2.ASSU.LAB.P15M.Q.001", "2003", "602",
			new Observation[] { new Observation( "2.1", "F", "C", "" ) }
		),
		new Attribute(
			".U2.ASSU.LAB.P15M.Q.001", "", "", "UNIT", "PCPA"
		),
		// 3,1
		new TimeSeries(
			"A.U2.HICP.T0_0.P27M.Q.001", "2004", "602",
			new Observation[] { new Observation( "25.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.HICP.T0_0.P27M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.HICP.F0_0T0_4.P27M.Q.001", "2004", "602",
			new Observation[] { new Observation( "11.11", "F", "C", "" ) }
		),
		new Attribute(
			".U2.HICP.F0_0T0_4.P27M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.HICP.F0_4.P27M.Q.001", "2004", "602",
			new Observation[] { new Observation( "1.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.HICP.F0_4.P27M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.HICP.SUM.P27M.Q.001", "2004", "602",
			new Observation[] { new Observation( "111.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.HICP.SUM.P27M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.HICP.POINT.P27M.Q.001", "2004", "602",
			new Observation[] { new Observation( "2.3", "F", "C", "" ) }
		),
		new Attribute(
			".U2.HICP.POINT.P27M.Q.001", "", "", "UNIT", "PCPA"
		),
		new TimeSeries(
			"A.U2.RGDP.T1_0.P27M.Q.001", "2004", "602",
			new Observation[] { new Observation( "30.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.RGDP.T1_0.P27M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.RGDP.F1_0T1_4.P27M.Q.001", "2004", "602",
			new Observation[] { new Observation( "27.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.RGDP.F1_0T1_4.P27M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.RGDP.F1_4.P27M.Q.001", "2004", "602",
			new Observation[] { new Observation( "1.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.RGDP.F1_4.P27M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.RGDP.SUM.P27M.Q.001", "2004", "602",
			new Observation[] { new Observation( "330.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.RGDP.SUM.P27M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.RGDP.POINT.P27M.Q.001", "2004", "602",
			new Observation[] { new Observation( "2.2", "F", "C", "" ) }
		),
		new Attribute(
			".U2.RGDP.POINT.P27M.Q.001", "", "", "UNIT", "PCPA"
		),
		new TimeSeries(
			"A.U2.UNEM.T5_0.P27M.Q.001", "2004", "602",
			new Observation[] { new Observation( "2", "F", "C", "" ) }
		),
		new Attribute(
			".U2.UNEM.T5_0.P27M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.UNEM.F5_0T10_0.P27M.Q.001", "2004", "602",
			new Observation[] { new Observation( "1.", "F", "C", "" ) }
		),
		new Attribute(
			".U2.UNEM.F5_0T10_0.P27M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.UNEM.F11_0.P27M.Q.001", "2004", "602",
			new Observation[] { new Observation( "4.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.UNEM.F11_0.P27M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.UNEM.SUM.P27M.Q.001", "2004", "602",
			new Observation[] { new Observation( "00.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.UNEM.SUM.P27M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.UNEM.POINT.P27M.Q.001", "2004", "602",
			new Observation[] { new Observation( "9.1", "F", "C", "" ) }
		),
		new Attribute(
			".U2.UNEM.POINT.P27M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"Q.U2.ASSU.IR.P9M.Q.001", "20032", "608",
			new Observation[] { new Observation( "3.4", "F", "C", "" ) }
		),
		new Attribute(
			".U2.ASSU.IR.P9M.Q.001", "", "", "UNIT", "PCPA"
		),
		new TimeSeries(
			"Q.U2.ASSU.OIL.P9M.Q.001", "20032", "608",
			new Observation[] { new Observation( "25.0", "F", "C", "" ) }
		),
		new Attribute(
			".U2.ASSU.OIL.P9M.Q.001", "", "", "UNIT", "USD"
		),
		new TimeSeries(
			"Q.U2.ASSU.USD.P9M.Q.001", "20032", "608",
			new Observation[] { new Observation( "1", "F", "C", "" ) }
		),
		new Attribute(
			".U2.ASSU.USD.P9M.Q.001", "", "", "UNIT", "USD"
		),
		new TimeSeries(
			"A.U2.ASSU.LAB.P27M.Q.001", "2004", "602",
			new Observation[] { new Observation( "3.1", "F", "C", "" ) }
		),
		new Attribute(
			".U2.ASSU.LAB.P27M.Q.001", "", "", "UNIT", "PCPA"
		),
		// 4,1
		new TimeSeries(
			"M.U2.HICP.T0_0.P12M.Q.001", "200309", "610",
			new Observation[] { new Observation( "12.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.HICP.T0_0.P12M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"M.U2.HICP.F0_0T0_4.P12M.Q.001", "200309", "610",
			new Observation[] { new Observation( "8.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.HICP.F0_0T0_4.P12M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"M.U2.HICP.F0_4.P12M.Q.001", "200309", "610",
			new Observation[] { new Observation( "1.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.HICP.F0_4.P12M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"M.U2.HICP.SUM.P12M.Q.001", "200309", "610",
			new Observation[] { new Observation( "13.40", "F", "C", "" ) }
		),
		new Attribute(
			".U2.HICP.SUM.P12M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"M.U2.HICP.POINT.P12M.Q.001", "200309", "610",
			new Observation[] { new Observation( "4.6", "F", "C", "" ) }
		),
		new Attribute(
			".U2.HICP.POINT.P12M.Q.001", "", "", "UNIT", "PCPA"
		),
		new TimeSeries(
			"Q.U2.RGDP.T1_0.P9M.Q.001", "20032", "608",
			new Observation[] { new Observation( "0.0", "F", "C", "" ) }
		),
		new Attribute(
			".U2.RGDP.T1_0.P9M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"Q.U2.RGDP.F1_0T1_4.P9M.Q.001", "20032", "608",
			new Observation[] { new Observation( "8.30", "F", "C", "" ) }
		),
		new Attribute(
			".U2.RGDP.F1_0T1_4.P9M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"Q.U2.RGDP.F1_4.P9M.Q.001", "20032", "608",
			new Observation[] { new Observation( "7.90", "F", "C", "" ) }
		),
		new Attribute(
			".U2.RGDP.F1_4.P9M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"Q.U2.RGDP.SUM.P9M.Q.001", "20032", "608",
			new Observation[] { new Observation( "42.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.RGDP.SUM.P9M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"Q.U2.RGDP.POINT.P9M.Q.001", "20032", "608",
			new Observation[] { new Observation( "2.4", "F", "C", "" ) }
		),
		new Attribute(
			".U2.RGDP.POINT.P9M.Q.001", "", "", "UNIT", "PCPA"
		),
		new TimeSeries(
			"M.U2.UNEM.T5_0.P12M.Q.001", "200308", "610",
			new Observation[] { new Observation( "0", "F", "C", "" ) }
		),
		new Attribute(
			".U2.UNEM.T5_0.P12M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"M.U2.UNEM.F5_0T10_0.P12M.Q.001", "200308", "610",
			new Observation[] { new Observation( "1.01", "F", "C", "" ) }
		),
		new Attribute(
			".U2.UNEM.F5_0T10_0.P12M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"M.U2.UNEM.F11_0.P12M.Q.001", "200308", "610",
			new Observation[] { new Observation( "00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.UNEM.F11_0.P12M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"M.U2.UNEM.SUM.P12M.Q.001", "200308", "610",
			new Observation[] { new Observation( "070", "F", "C", "" ) }
		),
		new Attribute(
			".U2.UNEM.SUM.P12M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"M.U2.UNEM.POINT.P12M.Q.001", "200308", "610",
			new Observation[] { new Observation( "2.3", "F", "C", "" ) }
		),
		new Attribute(
			".U2.UNEM.POINT.P12M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"Q.U2.ASSU.IR.P12M.Q.001", "20033", "608",
			new Observation[] { new Observation( "3.350", "F", "C", "" ) }
		),
		new Attribute(
			".U2.ASSU.IR.P12M.Q.001", "", "", "UNIT", "PCPA"
		),
		new TimeSeries(
			"Q.U2.ASSU.OIL.P12M.Q.001", "20033", "608",
			new Observation[] { new Observation( "24", "F", "C", "" ) }
		),
		new Attribute(
			".U2.ASSU.OIL.P12M.Q.001", "", "", "UNIT", "USD"
		),
		new TimeSeries(
			"Q.U2.ASSU.USD.P12M.Q.001", "20033", "608",
			new Observation[] { new Observation( "01.20", "F", "C", "" ) }
		),
		new Attribute(
			".U2.ASSU.USD.P12M.Q.001", "", "", "UNIT", "USD"
		),
		new TimeSeries(
			"A.U2.ASSU.LAB.P39M.Q.001", "2005", "602",
			new Observation[] { new Observation( "4.1", "F", "C", "" ) }
		),
		new Attribute(
			".U2.ASSU.LAB.P39M.Q.001", "", "", "UNIT", "PCPA"
		),
		// 5,1
		new TimeSeries(
			"M.U2.HICP.T0_0.P24M.Q.001", "200409", "610",
			new Observation[] { new Observation( "0.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.HICP.T0_0.P24M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"M.U2.HICP.F0_0T0_4.P24M.Q.001", "200409", "610",
			new Observation[] { new Observation( "0.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.HICP.F0_0T0_4.P24M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"M.U2.HICP.F0_4.P24M.Q.001", "200409", "610",
			new Observation[] { new Observation( "6.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.HICP.F0_4.P24M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"M.U2.HICP.SUM.P24M.Q.001", "200409", "610",
			new Observation[] { new Observation( "100.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.HICP.SUM.P24M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"M.U2.HICP.POINT.P24M.Q.001", "200409", "610",
			new Observation[] { new Observation( "2.6", "F", "C", "" ) }
		),
		new Attribute(
			".U2.HICP.POINT.P24M.Q.001", "", "", "UNIT", "PCPA"
		),
		new TimeSeries(
			"Q.U2.RGDP.T1_0.P21M.Q.001", "20042", "608",
			new Observation[] { new Observation( "3.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.RGDP.T1_0.P21M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"Q.U2.RGDP.F1_0T1_4.P21M.Q.001", "20042", "608",
			new Observation[] { new Observation( "3.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.RGDP.F1_0T1_4.P21M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"Q.U2.RGDP.F1_4.P21M.Q.001", "20042", "608",
			new Observation[] { new Observation( "0.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.RGDP.F1_4.P21M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"Q.U2.RGDP.SUM.P21M.Q.001", "20042", "608",
			new Observation[] { new Observation( "10.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.RGDP.SUM.P21M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"Q.U2.RGDP.POINT.P21M.Q.001", "20042", "608",
			new Observation[] { new Observation( "2.1", "F", "C", "" ) }
		),
		new Attribute(
			".U2.RGDP.POINT.P21M.Q.001", "", "", "UNIT", "PCPA"
		),
		new TimeSeries(
			"M.U2.UNEM.T5_0.P24M.Q.001", "200408", "610",
			new Observation[] { new Observation( "1.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.UNEM.T5_0.P24M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"M.U2.UNEM.F5_0T10_0.P24M.Q.001", "200408", "610",
			new Observation[] { new Observation( "5.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.UNEM.F5_0T10_0.P24M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"M.U2.UNEM.F11_0.P24M.Q.001", "200408", "610",
			new Observation[] { new Observation( ".02", "F", "C", "" ) }
		),
		new Attribute(
			".U2.UNEM.F11_0.P24M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"M.U2.UNEM.SUM.P24M.Q.001", "200408", "610",
			new Observation[] { new Observation( "100.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.UNEM.SUM.P24M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"M.U2.UNEM.POINT.P24M.Q.001", "200408", "610",
			new Observation[] { new Observation( "6.8", "F", "C", "" ) }
		),
		new Attribute(
			".U2.UNEM.POINT.P24M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"Q.U2.ASSU.IR.P15M.Q.001", "20034", "608",
			new Observation[] { new Observation( "30", "F", "C", "" ) }
		),
		new Attribute(
			".U2.ASSU.IR.P15M.Q.001", "", "", "UNIT", "PCPA"
		),
		new TimeSeries(
			"Q.U2.ASSU.OIL.P15M.Q.001", "20034", "608",
			new Observation[] { new Observation( "26", "F", "C", "" ) }
		),
		new Attribute(
			".U2.ASSU.OIL.P15M.Q.001", "", "", "UNIT", "USD"
		),
		new TimeSeries(
			"Q.U2.ASSU.USD.P15M.Q.001", "20034", "608",
			new Observation[] { new Observation( "1.2", "F", "C", "" ) }
		),
		new Attribute(
			".U2.ASSU.USD.P15M.Q.001", "", "", "UNIT", "USD"
		),
		// 6,1
		new TimeSeries(
			"A.U2.HICP.T0_0.P63M.Q.001", "2007", "602",
			new Observation[] { new Observation( "10.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.HICP.T0_0.P63M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.HICP.F0_0T0_4.P63M.Q.001", "2007", "602",
			new Observation[] { new Observation( "10.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.HICP.F0_0T0_4.P63M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.HICP.F0_4.P63M.Q.001", "2007", "602",
			new Observation[] { new Observation( "10.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.HICP.F0_4.P63M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.HICP.SUM.P63M.Q.001", "2007", "602",
			new Observation[] { new Observation( "30.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.HICP.SUM.P63M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.HICP.POINT.P63M.Q.001", "2007", "602",
			new Observation[] { new Observation( "1.3", "F", "C", "" ) }
		),
		new Attribute(
			".U2.HICP.POINT.P63M.Q.001", "", "", "UNIT", "PCPA"
		),
		new TimeSeries(
			"A.U2.RGDP.T1_0.P63M.Q.001", "2007", "602",
			new Observation[] { new Observation( "45.0", "F", "C", "" ) }
		),
		new Attribute(
			".U2.RGDP.T1_0.P63M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.RGDP.F1_0T1_4.P63M.Q.001", "2007", "602",
			new Observation[] { new Observation( "23.0", "F", "C", "" ) }
		),
		new Attribute(
			".U2.RGDP.F1_0T1_4.P63M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.RGDP.F1_4.P63M.Q.001", "2007", "602",
			new Observation[] { new Observation( "3.0", "F", "C", "" ) }
		),
		new Attribute(
			".U2.RGDP.F1_4.P63M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.RGDP.SUM.P63M.Q.001", "2007", "602",
			new Observation[] { new Observation( "100.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.RGDP.SUM.P63M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.RGDP.POINT.P63M.Q.001", "2007", "602",
			new Observation[] { new Observation( "1.3", "F", "C", "" ) }
		),
		new Attribute(
			".U2.RGDP.POINT.P63M.Q.001", "", "", "UNIT", "PCPA"
		),
		new TimeSeries(
			"A.U2.UNEM.T5_0.P63M.Q.001", "2007", "602",
			new Observation[] { new Observation( "5.7", "F", "C", "" ) }
		),
		new Attribute(
			".U2.UNEM.T5_0.P63M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.UNEM.F5_0T10_0.P63M.Q.001", "2007", "602",
			new Observation[] { new Observation( "7", "F", "C", "" ) }
		),
		new Attribute(
			".U2.UNEM.F5_0T10_0.P63M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.UNEM.F11_0.P63M.Q.001", "2007", "602",
			new Observation[] { new Observation( "3.3", "F", "C", "" ) }
		),
		new Attribute(
			".U2.UNEM.F11_0.P63M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.UNEM.SUM.P63M.Q.001", "2007", "602",
			new Observation[] { new Observation( "100.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.UNEM.SUM.P63M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"A.U2.UNEM.POINT.P63M.Q.001", "2007", "602",
			new Observation[] { new Observation( "1.1", "F", "C", "" ) }
		),
		new Attribute(
			".U2.UNEM.POINT.P63M.Q.001", "", "", "UNIT", "PC"
		),
		// 7,1
		new TimeSeries(
			"M.U2.HICP.T0_0.P27M.Q.001", "200412", "610",
			new Observation[] { new Observation( "1.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.HICP.T0_0.P27M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"M.U2.HICP.F0_0T0_4.P27M.Q.001", "200412", "610",
			new Observation[] { new Observation( "2.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.HICP.F0_0T0_4.P27M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"M.U2.HICP.F0_4.P27M.Q.001", "200412", "610",
			new Observation[] { new Observation( "6.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.HICP.F0_4.P27M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"M.U2.HICP.SUM.P27M.Q.001", "200412", "610",
			new Observation[] { new Observation( "100.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.HICP.SUM.P27M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"M.U2.HICP.POINT.P27M.Q.001", "200412", "610",
			new Observation[] { new Observation( "2.4", "F", "C", "" ) }
		),
		new Attribute(
			".U2.HICP.POINT.P27M.Q.001", "", "", "UNIT", "PCPA"
		),
		new TimeSeries(
			"Q.U2.RGDP.T1_0.P27M.Q.001", "20044", "608",
			new Observation[] { new Observation( "7.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.RGDP.T1_0.P27M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"Q.U2.RGDP.F1_0T1_4.P27M.Q.001", "20044", "608",
			new Observation[] { new Observation( "13.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.RGDP.F1_0T1_4.P27M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"Q.U2.RGDP.F1_4.P27M.Q.001", "20044", "608",
			new Observation[] { new Observation( "14.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.RGDP.F1_4.P27M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"Q.U2.RGDP.SUM.P27M.Q.001", "20044", "608",
			new Observation[] { new Observation( "130.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.RGDP.SUM.P27M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"Q.U2.RGDP.POINT.P27M.Q.001", "20044", "608",
			new Observation[] { new Observation( "3.1", "F", "C", "" ) }
		),
		new Attribute(
			".U2.RGDP.POINT.P27M.Q.001", "", "", "UNIT", "PCPA"
		),
		new TimeSeries(
			"M.U2.UNEM.T5_0.P27M.Q.001", "200411", "610",
			new Observation[] { new Observation( "19.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.UNEM.T5_0.P27M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"M.U2.UNEM.F5_0T10_0.P27M.Q.001", "200411", "610",
			new Observation[] { new Observation( "50.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.UNEM.F5_0T10_0.P27M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"M.U2.UNEM.F11_0.P27M.Q.001", "200411", "610",
			new Observation[] { new Observation( "30", "F", "C", "" ) }
		),
		new Attribute(
			".U2.UNEM.F11_0.P27M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"M.U2.UNEM.SUM.P27M.Q.001", "200411", "610",
			new Observation[] { new Observation( "100.00", "F", "C", "" ) }
		),
		new Attribute(
			".U2.UNEM.SUM.P27M.Q.001", "", "", "UNIT", "PC"
		),
		new TimeSeries(
			"M.U2.UNEM.POINT.P27M.Q.001", "200411", "610",
			new Observation[] { new Observation( "8.8", "F", "C", "" ) }
		),
		new Attribute(
			".U2.UNEM.POINT.P27M.Q.001", "", "", "UNIT", "PC"
		),
		new MessageEnd(
			"MREF000001"
		),
		new InterchangeEnd(
			"IREF000035"
		)
	};

	/** Input to the {@link SPFReader}
	 * with mismatch between reference and survey periods.
	 */
	private static final String testInput2
	= "SPF,Sep-2002,"
	+ "HICP,,,,,,,,,"
	+ "GDP,,,,,,,,,"
	+ "Unemployment,,,,,,,,,"
	+ "Assumptions,,,,,Assumptions,,\n"
	+ "Period,Forecaster,"
	+ ",<0.0,0.0-0.4,>0.4,,Sum,,Point,,"
	+ ",<1.0,1.0-1.4,>1.4,,Sum,,Point,,"
	+ ",<5.0,5.0-10.0,>11.0,,Sum,,Point,,"
	+ ",ir,oil,usd,,,lab,\n"
	+ "0,ECB,"
	+ "Sep-2002,,,,,,,2.31%,,"
	+ "2002Q2,,,,,,,0.9%,,"
	+ "Aug-2002,,,,,,,8.4%,,"
	+ ",,,,,,,X\n";

	/**
	 * The objects into which
	 * the {@link SPFReader} should translate
	 * {@link #testInput} when called in the correct way.
	 */
	private static final Object[] testOutput2 = {
		new InterchangeBegin(
			"IREF000035", "4F0", "4F0", "001201:1410", true
		),
		new MessageBegin(
			"MREF000001", "4F0", "4F0", "ECB",
			MessageBegin.STATISTICAL_DATA, "csv_test.csv",
			new Contact[]{},
			"ECB_SPF", MessageBegin.UPDATE_OR_REPLACE_ACTION,
			"200203140015", "", "", "ECB_FCT1"
		),
		// 0,1
		new ErrorEvent(
			"Survey month Sep-2002 does not fit HICP reference period Sep-2002.",
			ErrorEvent.ERROR
		),
		new TimeSeries(
			"M.U2.HICP.T0_0.P3M.Q.ECB", "200209", "610",
			new Observation[] { new Observation( "0", "F", "F", "" ) }
		),
		new Attribute(
			".U2.HICP.T0_0.P3M.Q.ECB", "", "", "UNIT", "PC"
		),
		new ErrorEvent(
			"Survey month Sep-2002 does not fit HICP reference period Sep-2002.",
			ErrorEvent.ERROR
		),
		new TimeSeries(
			"M.U2.HICP.F0_0T0_4.P3M.Q.ECB", "200209", "610",
			new Observation[] { new Observation( "0", "F", "F", "" ) }
		),
		new Attribute(
			".U2.HICP.F0_0T0_4.P3M.Q.ECB", "", "", "UNIT", "PC"
		),
		new ErrorEvent(
			"Survey month Sep-2002 does not fit HICP reference period Sep-2002.",
			ErrorEvent.ERROR
		),
		new TimeSeries(
			"M.U2.HICP.F0_4.P3M.Q.ECB", "200209", "610",
			new Observation[] { new Observation( "0", "F", "F", "" ) }
		),
		new Attribute(
			".U2.HICP.F0_4.P3M.Q.ECB", "", "", "UNIT", "PC"
		),
		new ErrorEvent(
			"Survey month Sep-2002 does not fit HICP reference period Sep-2002.",
			ErrorEvent.ERROR
		),
		new TimeSeries(
			"M.U2.HICP.SUM.P3M.Q.ECB", "200209", "610",
			new Observation[] { new Observation( "0", "F", "F", "" ) }
		),
		new Attribute(
			".U2.HICP.SUM.P3M.Q.ECB", "", "", "UNIT", "PC"
		),
		new ErrorEvent(
			"Survey month Sep-2002 does not fit HICP reference period Sep-2002.",
			ErrorEvent.ERROR
		),
		new TimeSeries(
			"M.U2.HICP.POINT.P3M.Q.ECB", "200209", "610",
			new Observation[] { new Observation( "2.31", "F", "F", "" ) }
		),
		new Attribute(
			".U2.HICP.POINT.P3M.Q.ECB", "", "", "UNIT", "PCPA"
		),
		new ErrorEvent(
			"Survey month Sep-2002 does not fit RGDP reference period 2002Q2.",
			ErrorEvent.ERROR
		),
		new TimeSeries(
			"Q.U2.RGDP.T1_0.M0M.Q.ECB", "20022", "608",
			new Observation[] { new Observation( "0", "A", "F", "" ) }
		),
		new Attribute(
			".U2.RGDP.T1_0.M0M.Q.ECB", "", "", "UNIT", "PC"
		),
		new ErrorEvent(
			"Survey month Sep-2002 does not fit RGDP reference period 2002Q2.",
			ErrorEvent.ERROR
		),
		new TimeSeries(
			"Q.U2.RGDP.F1_0T1_4.M0M.Q.ECB", "20022", "608",
			new Observation[] { new Observation( "0", "A", "F", "" ) }
		),
		new Attribute(
			".U2.RGDP.F1_0T1_4.M0M.Q.ECB", "", "", "UNIT", "PC"
		),
		new ErrorEvent(
			"Survey month Sep-2002 does not fit RGDP reference period 2002Q2.",
			ErrorEvent.ERROR
		),
		new TimeSeries(
			"Q.U2.RGDP.F1_4.M0M.Q.ECB", "20022", "608",
			new Observation[] { new Observation( "0", "A", "F", "" ) }
		),
		new Attribute(
			".U2.RGDP.F1_4.M0M.Q.ECB", "", "", "UNIT", "PC"
		),
		new ErrorEvent(
			"Survey month Sep-2002 does not fit RGDP reference period 2002Q2.",
			ErrorEvent.ERROR
		),
		new TimeSeries(
			"Q.U2.RGDP.SUM.M0M.Q.ECB", "20022", "608",
			new Observation[] { new Observation( "0", "A", "F", "" ) }
		),
		new Attribute(
			".U2.RGDP.SUM.M0M.Q.ECB", "", "", "UNIT", "PC"
		),
		new ErrorEvent(
			"Survey month Sep-2002 does not fit RGDP reference period 2002Q2.",
			ErrorEvent.ERROR
		),
		new TimeSeries(
			"Q.U2.RGDP.POINT.M0M.Q.ECB", "20022", "608",
			new Observation[] { new Observation( "0.9", "A", "F", "" ) }
		),
		new Attribute(
			".U2.RGDP.POINT.M0M.Q.ECB", "", "", "UNIT", "PCPA"
		),
		new ErrorEvent(
			"Survey month Sep-2002 does not fit UNEM reference period Aug-2002.",
			ErrorEvent.ERROR
		),
		new TimeSeries(
			"M.U2.UNEM.T5_0.P3M.Q.ECB", "200208", "610",
			new Observation[] { new Observation( "0", "F", "F", "" ) }
		),
		new Attribute(
			".U2.UNEM.T5_0.P3M.Q.ECB", "", "", "UNIT", "PC"
		),
		new ErrorEvent(
			"Survey month Sep-2002 does not fit UNEM reference period Aug-2002.",
			ErrorEvent.ERROR
		),
		new TimeSeries(
			"M.U2.UNEM.F5_0T10_0.P3M.Q.ECB", "200208", "610",
			new Observation[] { new Observation( "0", "F", "F", "" ) }
		),
		new Attribute(
			".U2.UNEM.F5_0T10_0.P3M.Q.ECB", "", "", "UNIT", "PC"
		),
		new ErrorEvent(
			"Survey month Sep-2002 does not fit UNEM reference period Aug-2002.",
			ErrorEvent.ERROR
		),
		new TimeSeries(
			"M.U2.UNEM.F11_0.P3M.Q.ECB", "200208", "610",
			new Observation[] { new Observation( "0", "F", "F", "" ) }
		),
		new Attribute(
			".U2.UNEM.F11_0.P3M.Q.ECB", "", "", "UNIT", "PC"
		),
		new ErrorEvent(
			"Survey month Sep-2002 does not fit UNEM reference period Aug-2002.",
			ErrorEvent.ERROR
		),
		new TimeSeries(
			"M.U2.UNEM.SUM.P3M.Q.ECB", "200208", "610",
			new Observation[] { new Observation( "0", "F", "F", "" ) }
		),
		new Attribute(
			".U2.UNEM.SUM.P3M.Q.ECB", "", "", "UNIT", "PC"
		),
		new ErrorEvent(
			"Survey month Sep-2002 does not fit UNEM reference period Aug-2002.",
			ErrorEvent.ERROR
		),
		new TimeSeries(
			"M.U2.UNEM.POINT.P3M.Q.ECB", "200208", "610",
			new Observation[] { new Observation( "8.4", "F", "F", "" ) }
		),
		new Attribute(
			".U2.UNEM.POINT.P3M.Q.ECB", "", "", "UNIT", "PC"
		),
		new MessageEnd(
			"MREF000001"
		),
		new InterchangeEnd(
			"IREF000035"
		)
	};

	/* ========================= Fields ========================= */

	/** The file used for the location of test data. */
	private File testFile;

	/* ====================== Constructors ====================== */

	public SPFReaderTest ( final String name ) {
		super( name );
	}

	/* ================== Protected Methods ===================== */

	/** Sets up the test fixture. */
	protected void setUp () {
		try {
			testFile = File.createTempFile( "spf_testinput.", null );
			testFile.deleteOnExit();

			System.setProperty(
				SPFReader.class.getName() + ".Roberta.source",
				testFile.getAbsolutePath()
			);
			System.setProperty(
				SPFReader.class.getName() + ".Roberta.iref", "IREF000035"
			);
			System.setProperty(
				SPFReader.class.getName() + ".Roberta.test", "TRUE"
			);
			System.setProperty(
				SPFReader.class.getName() + ".Roberta.generateAttributes", "true"
			);
		}
		catch ( final IOException ioe ) {
			fail(
				"Write testFile failed: " + ioe
			);
		}
	}

	/** Tears down the test fixture. */
	protected void tearDown () {
		System.getProperties().remove(
			SPFReader.class.getName() + ".Roberta.source"
		);
		System.getProperties().remove(
			SPFReader.class.getName() + ".Roberta.iref"
		);
		System.getProperties().remove(
			SPFReader.class.getName() + ".Roberta.test"
		);
		System.getProperties().remove(
			SPFReader.class.getName() + ".Roberta.generateAttributes"
		);
		testFile.delete();
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite
	 */
	public static Test suite () {
		return new TestSuite( SPFReaderTest.class );
	}

	/**
	 * @param arg the array of argument {@link String}s
	 */
	public static void main ( final String[] arg ) {
		final String[] testCaseName = {
			SPFReaderTest.class.getName()
		};
		junit.textui.TestRunner.main( testCaseName );
	}

	/* ---------------------- Test Methods ---------------------- */

	/** Test {@link SPFReader#run}. */
	public void testRunFromFile ()
		throws IOException
	{
		final SPFReader reader = new SPFReader( "Roberta" );

		writeTestData( testInput );

		// Run the test.
		doTestRun( reader, testOutput );

		// Close the reader.
		reader.close();
	}

	/** Test {@link SPFReader#run} with mistaken survey period. */
	public void testSurveyPeriodMistake ()
		throws IOException
	{
		final SPFReader reader = new SPFReader( "Roberta" );

		writeTestData( testInput2 );

		// Run the test.
		doTestRun( reader, testOutput2 );

		// Close the reader.
		reader.close();
	}

	/* ==================== Private Methods ===================== */

	private void writeTestData ( final String input )
		throws IOException
	{
		final Reader reader = new StringReader( input );
		final Writer data = new BufferedWriter(
			new FileWriter( testFile )
		);
		int c = 0;
		while ( ( c = reader.read() ) != -1 ) {
			data.write( c );
		}
		data.close();
		reader.close();
	}

	private void doTestRun (
		final SPFReader reader,
		final Object[] output
	)
		throws IOException
	{
		final TestEventListener listener
		= new TestEventListener( 2 * output.length + 100 );
		reader.addListener( listener, true );

		reader.run();

		// Compare the event contents.
		{
			final Iterator iterator = listener.iterator();
			int i = 0;
			while ( iterator.hasNext() && i < output.length ) {
				Object expected = output[i++];
				final Object received = iterator.next();
				if (
					received instanceof InterchangeBegin
					&&
					expected instanceof InterchangeBegin
				) {
					final InterchangeBegin expectedInt
					= (InterchangeBegin) expected;
					final InterchangeBegin receivedInt
					= (InterchangeBegin) received;
					assertEquals(
						"iref",
						expectedInt.getIref(),
						receivedInt.getIref()
					);
					assertEquals(
						"sender",
						expectedInt.getSender(),
						receivedInt.getSender()
					);
					assertEquals(
						"receiver",
						expectedInt.getReceiver(),
						receivedInt.getReceiver()
					);
					final DateFormat format
					= new SimpleDateFormat( "yyMMdd:HHmm" );
					try {
						format.parse(
							receivedInt.getPreparationDateTime()
						);
					}
					catch ( final ParseException pe ) {
						fail( "prep dateTime " + pe );
					}
					assertEquals(
						"test",
						expectedInt.isTest(),
						receivedInt.isTest()
					);
					continue;
				}
				if (
					received instanceof MessageBegin
					&&
					expected instanceof MessageBegin
				) {
					final MessageBegin expectedMes = (MessageBegin) expected;
					expected = new MessageBegin(
						expectedMes.getMref(),
						expectedMes.getSender(),
						expectedMes.getReceiver(),
						expectedMes.getMaintenanceAgencyId(),
						expectedMes.getFunction(),
						testFile.getName(),
						new Contact[]{},
						expectedMes.getDataSetId(),
						expectedMes.getAction(),
						new SimpleDateFormat( "yyyyMMddHHmm" ).format(
							new Date( testFile.lastModified() )
						),
						expectedMes.getReportingPeriod(),
						expectedMes.getReportingPeriodFormat(),
						expectedMes.getKeyFamilyId()
					);
				}
				if (
					received instanceof ErrorEvent
					&&
					expected instanceof ErrorEvent
				) {
					final ErrorEvent
						expectedEvent = (ErrorEvent) expected,
						receivedEvent = (ErrorEvent) received;
					assertEquals(
						"Error level",
						expectedEvent.getSeverity(),
						receivedEvent.getSeverity()
					);
					assertTrue(
						"Error message contained: \""
						+ expectedEvent.getMessage()
						+ "\" in \"" + receivedEvent.getMessage() + "\"",
						receivedEvent.getMessage().indexOf(
							expectedEvent.getMessage()
						) >= 0
					);
					continue;
				}
				if ( !expected.equals( received ) ) {
					System.out.println( "expected: " + expected );
					System.out.println( "received: " + received );
				}
				assertEquals( "Output", expected, received );
			}
			assertTrue(
				"Not too many events",
				!iterator.hasNext()
			);
			assertTrue(
				"Not too few events",
				i == output.length
			);
		}
	}
}
