// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (C) 2005, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.database;

import java.util.Iterator;

/**
 * Describe the common functionalities between {@link Cube} and
 * {@link CubeImplementation}.
 *
 * @author
 * <a href="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</a>
 * @author
 * <a href="mailto:Xavier.Sosnowska@ECB.int">Xavier Sosnowska</a>
 *
 * @version @(#)$Id: Cuboid.java,v 1.1 2017/01/19 16:14:03 trainot Exp $
 */
public interface Cuboid {
	/**
	 * Tell whether this <code>cuboid</code> is empty.
	 *
	 * @return true if no attributes nor other implementing class specific
	 * content is available (e.g. subcubes).
	 */
	boolean isEmpty ();

	/** Retrieve all attributes.
	 * @return an iterator over {@link Java.util.Map.Entry} objects.
	 */
	Iterator iterateAttributes ();

	/** Get the attribute value for a given concept. */
	String getAttribute ( String conceptId );

	/** Set the attribute value for a given concept. */
	void setAttribute ( String conceptId, String value );

	/** Remove the attribute value for a given concept. */
	void removeAttribute ( String conceptId );
}
