// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (C) 2005, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.database;

import java.util.Iterator;
import java.util.Map;

import junit.framework.TestCase;

/* ***************************** Class ****************************** */

/**
 * <p>Basic testing of {@link Cuboid} implementations.</P>
 *
 * @author
 * <a href="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</a>
 * @author
 * <a href="mailto:Xavier.Sosnowska@ECB.int">Xavier Sosnowska</a>
 *
 * @version @(#)$Id: CuboidTest.java,v 1.1 2017/01/19 16:14:03 trainot Exp $
 */
public abstract class CuboidTest extends TestCase {

	/* ====================== Constructors ====================== */

	public CuboidTest ( final String name ) {
		super( name );
	}

	/* =================== Protected Methods ==================== */

	protected abstract Cuboid createCuboid ();

	/* ===================== Public Methods ===================== */

	public void testAttributes () {
		final Cuboid cuboid = createCuboid();
		assertTrue( "Cuboid empty", cuboid.isEmpty() );
		assertTrue( "No attributes yet", ! cuboid.iterateAttributes().hasNext() );

		cuboid.setAttribute( "COMMENT", "Hi, there!" );
		assertTrue( "Cuboid no longer empty", !cuboid.isEmpty() );
		assertEquals( "One attribute 1", "Hi, there!", cuboid.getAttribute( "COMMENT" ) );
		final Iterator iterator = cuboid.iterateAttributes();
		assertTrue( "Some attributes now 1", iterator.hasNext() );
		final Map.Entry entry = (Map.Entry) iterator.next();
		assertEquals( "The attribute key 1", "COMMENT", entry.getKey() );
		assertEquals( "The attribute value 1", "Hi, there!", entry.getValue() );
		assertTrue( "No more attributes 1", ! iterator.hasNext() );

		cuboid.setAttribute( "COMMENT", "Hey, you!" );
		assertEquals( "One attribute 2", "Hey, you!", cuboid.getAttribute( "COMMENT" ) );
		final Iterator iterator2 = cuboid.iterateAttributes();
		assertTrue( "Some attributes now 2", iterator2.hasNext() );
		final Map.Entry entry2 = (Map.Entry) iterator2.next();
		assertEquals( "The attribute key 2", "COMMENT", entry2.getKey() );
		assertEquals( "The attribute value 2", "Hey, you!", entry2.getValue() );
		assertTrue( "No more attributes 2", !iterator2.hasNext() );

		cuboid.removeAttribute( "COMMENT" );
		assertTrue( "Cuboid empty again", cuboid.isEmpty() );
		assertTrue( "All attributes removed", ! cuboid.iterateAttributes().hasNext() );
	}
}
