// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (C) 2005, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.database;

import java.util.Iterator;

/**
 * Implements a cube with implicit storage management in a hierarchy.
 * In contrast to the {@link HierarchicalCubeImplementation}, there is no notion
 * of existence of a subcube. In positive words: The user of <code>HierarchicalCube</code>
 * classes does not have to worry about storage allocation and deallocation.
 * Subcubes become known as soon as they
 * or one of their descendant subcubes are filled with attributes
 * and disappear as soon as the last attribute is removed from the
 * descendant hierarchy.
 *
 * @author
 * <a href="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</a>
 * @author
 * <a href="mailto:Xavier.Sosnowska@ECB.int">Xavier Sosnowska</a>
 *
 * @version @(#)$Id: HierarchicalCube.java,v 1.1 2017/01/19 16:14:03 trainot Exp $
 */
public interface HierarchicalCube extends Cuboid {

	/** Retrieve all keys of known direct subcubes.
	 * @return an iterator over {@link Object}s.
	 */
	Iterator iterateChildKeys ();

	/** Tells whether there is any direct subcube with the given key. */
	boolean knowsChild ( String key );

	/** Retrieve the direct subcube with the given key.
	 * If no such subcube is yet known, it will be newly instantiated.
	 */
	HierarchicalCube getChild ( String key );
}
