// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (C) 2005, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.database;

import java.util.Iterator;

/* ***************************** Class ****************************** */

/**
 * <p>Basic testing of {@link HierarchicalCubeImplementation} implementations.</P>
 *
 * @author
 * <a href="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</a>
 * @author
 * <a href="mailto:Xavier.Sosnowska@ECB.int">Xavier Sosnowska</a>
 *
 * @version @(#)$Id: HierarchicalCubeImplementationTest.java,v 1.1 2017/01/19 16:14:03 trainot Exp $
 */
public abstract class HierarchicalCubeImplementationTest extends CuboidTest {

	/* ====================== Constructors ====================== */

	public HierarchicalCubeImplementationTest ( final String name ) {
		super( name );
	}

	/* =================== Protected Methods ==================== */

	protected abstract HierarchicalCubeImplementation createCubeImplementation ();

	protected Cuboid createCuboid () {
		return createCubeImplementation();
	}

	/* ===================== Public Methods ===================== */

	public void testChild () {
		final HierarchicalCubeImplementation root = createCubeImplementation();
		assertTrue( "No Child", !root.iterateChildKeys().hasNext() );
		assertNull( "No child (null)", root.getChild( "child" ) );
		assertTrue( "Child is nonexistent", !root.existsChild( "child" ) );

		final HierarchicalCubeImplementation child = root.createChild( "child" );
		assertTrue( "Child exists", root.existsChild( "child" ) );
		assertEquals( "One child 1", child, root.getChild( "child" ) );
		final Iterator iterator = root.iterateChildKeys();
		assertTrue( "Some children", iterator.hasNext() );
		assertEquals( "The child's key", "child", iterator.next() );
		assertTrue( "No more children", !iterator.hasNext() );
		root.removeChild( "child" );
		assertTrue( "Child has been deleted", !root.existsChild( "child" ) );
		assertTrue( "No Child, because removed", !root.iterateChildKeys().hasNext() );
	}

	public void testTwoWayAccess () {
		final HierarchicalCubeImplementation root = createCubeImplementation();
		root.createChild( "child" );
		try {
			root.createChild( "child" );
			fail ( "Double creation not detected" );
		} catch ( final IllegalStateException e ) {}
	}

	public void testEmptyConcerningChildren () {
		final HierarchicalCubeImplementation root = createCubeImplementation();
		assertTrue( "Initially empty", root.isEmpty() );
		assertTrue( "Child initially empty", root.createChild( "child" ).isEmpty() );
		assertTrue( "Root no longer empty", !root.isEmpty() );
		root.removeChild( "child" );
		assertTrue( "Again empty", root.isEmpty() );

		root.setAttribute( "COMMENT", "REMARK" );
		assertTrue( "Root is full", !root.isEmpty() );
		root.removeAttribute( "COMMENT" );
		assertTrue( "Once again empty", root.isEmpty() );
	}
}
