// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (C) 2005, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.database;

import junit.framework.Test;
import junit.framework.TestSuite;

import Int.ecb.gesmescb.database.memory.MemoryCubeImplementation;

/* ***************************** Class ****************************** */

/**
 * <p>Testing of {@link HierarchicalCubifier} implementation.</P>
 *
 * @author
 * <a href="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</a>
 * @author
 * <a href="mailto:Xavier.Sosnowska@ECB.int">Xavier Sosnowska</a>
 *
 * @version @(#)$Id: HierarchicalCubifierTest.java,v 1.1 2017/01/19 16:14:03 trainot Exp $
 */
public class HierarchicalCubifierTest extends HierarchicalCubeTest {

	/* ====================== Constructors ====================== */

	public HierarchicalCubifierTest ( final String name ) {
		super( name );
	}

	/* =================== Protected Methods ==================== */

	protected HierarchicalCube createCube () {
		return HierarchicalCubifier.newCubifier( new MemoryCubeImplementation() );
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite
	 */
	public static Test suite () {
		return new TestSuite( HierarchicalCubifierTest.class );
	}

	/**
	 * Runs the test case.
	 */
	public static void main ( final String args[] ) {
		String[] testCaseName = { HierarchicalCubifierTest.class.getName() };
		junit.textui.TestRunner.main( testCaseName );
	}

	/* ---------------------- Test methods ---------------------- */

	public void testProtectedMethods () {
		final HierarchicalCubifier root = (HierarchicalCubifier) createCube();
		assertNotNull( "get root not null", root.getCubeImplementation() );
		assertNotNull( "create root not null", root.createOrGetCubeImplementation() );
		root.possiblyRemoveCubeImplementation();
		assertNotNull( "get root not removed", root.getCubeImplementation() );

		final HierarchicalCubifier child = (HierarchicalCubifier) root.getChild( "child" );
		assertNull( "get child null", child.getCubeImplementation() );
		assertNotNull( "create child not null", child.createOrGetCubeImplementation() );
		assertNotNull( "get child not null", child.getCubeImplementation() );
		child.possiblyRemoveCubeImplementation();
		assertNull( "removed child", child.getCubeImplementation() );
	}
}
