// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (C) 2004, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.database;

import java.util.Collection;
import java.util.LinkedList;
import java.util.Iterator;

import Int.ecb.gesmescb.KeyFamily;
import Int.ecb.gesmescb.TimeSeries;
import Int.ecb.gesmescb.Attribute;
import Int.ecb.gesmescb.Observation;

import Int.ecb.gesmescb.time.RegularInterval;

/* ***************************** Class ****************************** */

/**
 * Implements {@link AbstractDataSet#iterateData()}
 * and {@link AbstractDataSet#close()}
 * to make it run.
 *
 * This class is meant to test the {@link AbstractDataSet}.
 * It is not written for efficiency, but to use all available code.
 *
 * @see AbstractDataSetTest
 *
 * @author
 * <a href="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</a>
 *
 * @version @(#)$Id: TestDataStore.java,v 1.1 2017/01/19 16:14:03 trainot Exp $
 */

public class TestDataStore extends AbstractDataSet implements DataStore {

	/* ========================= Fields ========================= */

	/** Holds the object in the data store. */
	private final Collection content;

	/* ====================== Constructors ====================== */

	/**
	 * Construct a new <code>TestDataStore</code>.
	 *
	 * @param name to be used for logging and preferences.
	 * @param keyFamily determining the structure of the data set.
	 */
	public TestDataStore (
		final String name,
		final KeyFamily keyFamily
	) {
		super( name, keyFamily );
		content = new LinkedList();
	}

	/* ===================== Public Methods ===================== */

	/* ------------------------ DataSet ------------------------- */

	public Iterator iterateData () {
		return content.iterator();
	}

	public void close () {}

	/* ----------------------- DataStore ------------------------ */

	public void store ( final TimeSeries timeSeries ) {
		for (
			final Iterator observations = timeSeries.observationIterator();
			observations.hasNext();
		) {
			final Observation observation = (Observation) observations.next();
			if ( ! observation.isEmpty() ) {
				content.add( timeSeries );
				break;
			}
		}
	}

	public void store ( final Attribute attribute ) {
		content.add( attribute );
	}

	public void remove (
		final String key,
		final String period,
		final String periodFormat
	) {
		throw new UnsupportedOperationException( "TestDataStore.remove" );
	}

	public void remove (
		final String key,
		final RegularInterval interval
	) {
		throw new UnsupportedOperationException( "TestDataStore.remove" );
	}

	public void remove (
		final String key,
		final String period,
		final String periodFormat,
		final String conceptId
	) {
		throw new UnsupportedOperationException( "TestDataStore.remove" );
	}

	public void remove (
		final String key,
		final RegularInterval interval,
		final String conceptId
	) {
		throw new UnsupportedOperationException( "TestDataStore.remove" );
	}
}
