// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (C) 2005, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.database.memory;

import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;

import Int.ecb.gesmescb.database.HierarchicalCubeImplementation;

/**
 * Implements a {@link HierarchicalCubeImplementation} in memory using {@link HashMap}s.
 *
 * @author
 * <a href="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</a>
 * @author
 * <a href="mailto:Xavier.Sosnowska@ECB.int">Xavier Sosnowska</a>
 *
 * @version @(#)$Id: MemoryCubeImplementation.java,v 1.1 2017/01/19 16:14:05 trainot Exp $
 */
public class MemoryCubeImplementation implements HierarchicalCubeImplementation {

	/* ===================== Private Fields ===================== */

	private final Map attributes, children;

	/* ====================== Constructors ====================== */

	public MemoryCubeImplementation () {
		attributes = new HashMap();
		children = new HashMap();
	}

	/* ===================== Public Methods ===================== */

	/* --------------- CubeImplementation Methods --------------- */

	public Iterator iterateAttributes () {
		return attributes.entrySet().iterator();
	}

	public String getAttribute ( final String conceptId ) {
		return (String) attributes.get( conceptId );
	}

	public void setAttribute ( final String conceptId, final String value ) {
		attributes.put( conceptId, value );
	}

	public void removeAttribute ( final String conceptId ) {
		attributes.remove( conceptId );
	}

	public HierarchicalCubeImplementation createChild ( final String key ) {
		final HierarchicalCubeImplementation child = new MemoryCubeImplementation();
		final Object old = children.put( key, child );
		if ( old != null ) {
			children.put( key, old );
			throw new IllegalStateException(
				"Child cube with key \"" + key + "\" already exists"
			);
		}
		return child;
	}

	public Iterator iterateChildKeys () {
		return children.keySet().iterator();
	}

	public boolean existsChild ( final String key ) {
		return children.containsKey( key );
	}

	public HierarchicalCubeImplementation getChild ( final String key ) {
		return (HierarchicalCubeImplementation) children.get( key );
	}

	public void removeChild ( final String key ) {
		children.remove( key );
	}

	public boolean isEmpty () {
		return attributes.isEmpty() && children.isEmpty();
	}
}
