// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.error;

import Int.ecb.gesmescb.InterchangeBegin;
import Int.ecb.gesmescb.InterchangeEnd;
import Int.ecb.gesmescb.MessageBegin;
import Int.ecb.gesmescb.MessageEnd;

import Int.ecb.gesmescb.event.ErrorEvent;
import Int.ecb.gesmescb.event.GESMESCBEventListenerAdapter;

import Int.ecb.gesmescb.error.ErrorWatcher;

/* **************************** Class ***************************** */

/**
 * Remembers the first {@link ErrorEvent}
 * encountered.
 *
 * @author
 * <A HREF="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</A>
 *
 * @version @(#)$Id: ErrorEventListener.java,v 1.1 2017/01/19 16:14:05 trainot Exp $
 */
public class ErrorEventListener extends GESMESCBEventListenerAdapter
	implements ErrorWatcher
{

	/* ========================= Fields ========================= */

	/**
	 * Reference to the first {@link ErrorEvent}
	 * encountered.
	 */
	private ErrorEvent event;

	/* ====================== Constructors ====================== */

	/**
	 * Construct a clean and fresh ErrorEventListener.
	 */
	public ErrorEventListener () {
		clear();
	}

	/* ===================== Public Methods ===================== */

	public synchronized int getSeverity () {
		return event != null ? event.getSeverity() : OK;
	}

	public synchronized String getMessage () {
		return event != null
			? event.toString()
			: "No problem detected.";
	}

	public synchronized void clear () {
		event = null;
	}

	/**
	 * Returns the <CODE>ErrorEvent</CODE> related to a reported problem
	 * or <CODE>null</CODE> if no problem encountered.
	 */
	public synchronized ErrorEvent getErrorEvent () {
		return event;
	}

	/** Ignore. */
	public void handle ( final InterchangeBegin interchangeBegin ) {}

	/** Ignore. */
	public void handle ( final InterchangeEnd interchangeEnd ) {}

	/** Ignore. */
	public void handle ( final MessageBegin messageBegin ) {}

	/** Ignore. */
	public void handle ( final MessageEnd messageEnd ) {}

	/**
	 * The most severe {@link ErrorEvent} is remembered.
	 */
	public synchronized void handle ( final ErrorEvent event ) {
		if ( event.getSeverity() > getSeverity() ) {
			this.event = event;
		}
	}
}
