// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.error;

import java.util.ArrayList;

import Int.ecb.logging.Handler;
import Int.ecb.logging.Level;
import Int.ecb.logging.LogRecord;

/* **************************** Class ***************************** */

/**
 * Remembers the first highest {@link Int.ecb.logging.LogRecord}
 * encountered.
 *
 * @author
 * <A HREF="mailto:Sami.Airo@ECB.int">Sami Airo</A>
 *
 * @version @(#)$Id: ErrorHandler.java,v 1.1 2017/01/19 16:14:05 trainot Exp $
 */
public class ErrorHandler extends Handler	implements ErrorWatcher
{

	/* ========================= Fields ========================= */

	/**
	 * Reference to the first {@link Int.ecb.gesmescb.event.ErrorEvent}
	 * encountered.
	 */
	private LogRecord firstHighestLogRecord;
	
	private ArrayList logRecord;

	/* ====================== Constructors ====================== */

	/**
	 * Construct a clean and fresh ErrorEventHandler.
	 */
	public ErrorHandler () {
		logRecord = new ArrayList();
		clear();
	}

	/* ===================== Public Methods ===================== */
	/**
	 * 
	 * An error record for the last 100 errors/warnings is stored.
	 */
	public void publish ( final LogRecord record ) {
		if ( isLoggable( record ) ) {
			synchronized( this ) {
				if (
					firstHighestLogRecord == null
					||
					firstHighestLogRecord.getLevel().intValue()
					< record.getLevel().intValue()
				) {
					firstHighestLogRecord = record;
				}
				
				//Current log record needs to be at least a warning to be stored
				if((record.getLevel().intValue() >= Level.WARNING.intValue()) &&
					this.logRecord.size() < 100){
					
					this.logRecord.add(record);
				}
				
				
			}
		}
	}

	public synchronized int getSeverity () {
		if ( firstHighestLogRecord == null ) return OK;

		if (
			firstHighestLogRecord.getLevel().intValue()
			==
			Level.SEVERE.intValue()
		) {
			return FAILURE;
		}

		if (
			firstHighestLogRecord.getLevel().intValue()
			==
			Level.WARNING.intValue()
		) {
			return WARNING;
		}

		return OK;
	}

	public synchronized String getMessage () {
		return firstHighestLogRecord != null
			? firstHighestLogRecord.getMessage()
			: "No problem encountered.";
	}
	
	public synchronized ArrayList getAllMessages(){
		return logRecord;
	}

	public synchronized void clear () {
		firstHighestLogRecord = null;
		logRecord.clear();
	}

	public void close () {
		clear();
	}

}
