// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.error;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import Int.ecb.logging.LogRecord;
import Int.ecb.logging.Level;
import Int.ecb.gesmescb.error.ErrorHandler;
import Int.ecb.gesmescb.error.ErrorWatcher;


/* ***************************** Class ****************************** */

/**
 * <P>Tests the implementation of ErrorHandler class in the
 * Int.ecb.gesmescb.error package.</P>
 *
 * @author
 * <A HREF="mailto:Sami.Airo@ECB.int">Sami Airo</A>
 *
 * @version @(#)$Id: ErrorHandlerTest.java,v 1.1 2017/01/19 16:14:05 trainot Exp $
 */

public class ErrorHandlerTest extends TestCase {

	/* ========================= Fields ========================= */

	private ErrorHandler _handler;

	/* ====================== Constructors ====================== */

	/**
	 * Constructs a ErrorHandlerTest with the specified name.
	 *
	 * @param name Test case name
	 */
	public ErrorHandlerTest(String name) {
		super(name);
	}

	/* ================== Protected Methods ===================== */

	/**
	 * Sets up the test fixture
	 */
	protected void setUp() {
		_handler = new ErrorHandler();
	}

	/**
	 * Tears sown the test fixture
	 */
	protected void tearDown() {
		_handler = null;
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite
	 */
	public static Test suite() {
		TestSuite suite = new TestSuite(ErrorHandlerTest.class);
		return suite;
	}

	/**
	 * Runs the test case
	 */
	public static void main(String args[]) {
		String[] testCaseName = {ErrorHandlerTest.class.getName()};
		junit.textui.TestRunner.main(testCaseName);
	}

	/* --------------------- Test Methods --------------------- */

	/**
	 * Tests setting the log level
	 */
	public void testPublish() {
		LogRecord r1 = new LogRecord( Level.WARNING, "test1" );
		LogRecord r2 = new LogRecord( Level.SEVERE, "test2" );
		LogRecord r3 = new LogRecord( Level.SEVERE, "test3" );

		_handler.setLevel(Level.INFO);

		_handler.publish(r1);
		_handler.publish(r2);
		_handler.publish(r3);

		assertEquals(r2.getMessage(), _handler.getMessage());
		assertEquals(ErrorWatcher.FAILURE, _handler.getSeverity());
	}
	

	public void testNumberofRecords() {
		
		LogRecord r1 = new LogRecord( Level.WARNING, "test1" );

		_handler.setLevel(Level.WARNING);
		
		for(int i = 0;i <= 200;i++){
			_handler.publish(r1);
		}

		assertEquals(_handler.getAllMessages().size(), 100);

	}
}
