// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, (c) 2003 European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.event;

/* *************************** Interface **************************** */

/**
 * GESMESCBEventSource is the base interface for GESMES/TS event sources to
 * add and remove event listeners.
 *
 * @author
 * <A HREF="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</A>
 *
 * @version @(#)$Id: GESMESCBEventSource.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */

public interface GESMESCBEventSource {

	/**
	 * <P>If the given listener is not already registered,
	 * register it. Otherwise unspecified action.</P>
	 *
	 * <P>The specified importance of a listener is used
	 * to guide automatic end of service.
	 * When no important listeners are left, the operation(s)
	 * like {@link GESMESCBEventSourceAdapter#dispatch}
	 * return <CODE>false</CODE> to inform about this lack of
	 * interest. A sensible source can decide from this
	 * to stop its activity.
	 * This is useful to be able to stop execution even if the
	 * {@link Int.ecb.gesmescb.metabase.Metabase} is still
	 * listening.</P>
	 *
	 * @param listener to receive events.
	 * @param isImportant whether the listener counts in the
	 * evaluation of interest.
	 */
	public void addListener (
		final GESMESCBEventListener listener,
		final boolean isImportant
	);

	/**
	 * Remove all listeners registered from the distribution list.
	 * The listeners should simply be removed
	 * without notification that they are.
	 * Therefore this removal does not automatically
	 * make the listeners close business.
	 */
	public void removeAllListeners ();
}
