// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.event;

import java.lang.Runnable;

import Int.ecb.gesmescb.event.GESMESCBEventSource;

/* *************************** Interface **************************** */

/**
 * <p>Reads input and translates it to
 * {@link Int.ecb.gesmescb.GESMESCBObject}s.</p>
 *
 * <P>Typically input is read from a permanent destination.</P>
 *
 * <P>To support multithreading, the interface extends
 * <CODE>java.lang.Runnable</CODE>.
 * No complete internal synchronisation is required by
 * classes implementing <CODE>GESMESCBReader</CODE>.
 * However, the following synchronisation strategy
 * must be guaranteed to work:</P>
 *
 * <UL><LI>Thread <I>A</I> is responsible for creating the
 * <CODE>GESMESCBReader</CODE>,
 * registering the listeners, and finally calling {@link #close}.</LI>
 * <LI>Thread <I>B</I> executes {@link #run}.
 * <I>B</I> must either equal <I>A</I>
 * or be created after all listeners have been registered.</LI></UL>
 *
 * @author
 * <A HREF="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</A>
 *
 * @version @(#)$Id: GESMESCBReader.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */

public interface GESMESCBReader
	extends GESMESCBEventSource, Runnable
{

	/**
	 * <P>Start reading all available input.</P>
	 */
	public void run ();

	/**
	 * <P>Close the read destination.
	 * This method should be called when the data transfer is
	 * complete so that the GESMESCBReader
	 * can free any external resources like open files
	 * or database connections.</P>
	 *
	 * <P>TODO: Allow exception in case of errors?</P>
	 */
	public void close ();
}
