// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, (c) 2003, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.event;

/* ***************************** Class ****************************** */

/**
 * <P>Dispatches objects to all {@link GESMESCBEventListener}s.</P>
 *
 * <P>This class is intended to be used for testing only.
 * For production, use the {@link GESMESCBEventSourceHelper} instead.
 * That has no generic <CODE>dispatch</CODE> methods and therefore enforces
 * better compile-time type checking.
 * In production code, there seems to be no need for a generic
 * <CODE>dispatch</CODE>.</P>
 *
 * <P>The generic <CODE>dispatch</CODE> methods
 * {@link #dispatch(Object} and {@link #dispatch(Object[])}
 * shield testing code against modifications of the event model used in
 * {@link Int.ecb.gesmescb.event}.</p>
 *
 * @see TestEventListener
 *
 * @author
 * <A HREF="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</A>
 *
 * @version @(#)$Id: TestEventSource.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */
public final class TestEventSource
	extends GESMESCBEventSourceAdapter
{
	/**
	 * <P>Distribute an object to all listeners.</P>
	 *
	 * <P>This makes it possible to dispatch mixed information
	 * independent of the available signatures for <CODE>dispatch</CODE>.
	 * The price is a possible <CODE>ClassCastException</CODE>.</P>
	 *
	 * @param object to be distributed.
	 *
	 * @return true if at least one important {@link GESMESCBEventListener}
	 * was listening.
	 *
	 * @throws ClassCastException if the object is
	 * not one that can be dispatched.
	 */
	public boolean dispatch ( final Object object ) {
		return super.dispatch( object );
	}

	/**
	 * Calls {@link #dispatch(Object)} for all entries in <CODE>objects</CODE>.
	 *
	 * @param objects to be distributed.
	 *
	 * @throws ClassCastException if one of the objects is
	 * not one that can be dispatched.
	 */
	public void dispatch ( final Object[] objects ) {
		for ( int i = 0; i < objects.length; ++i ) {
			dispatch( objects[i] );
		}
	}
}
