// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, (c) 2003, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.event;

import java.lang.Object;

import java.util.Iterator;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import Int.ecb.gesmescb.InterchangeBegin;
import Int.ecb.gesmescb.InterchangeEnd;
import Int.ecb.gesmescb.Contact;
import Int.ecb.gesmescb.MessageBegin;
import Int.ecb.gesmescb.MessageEnd;

import Int.ecb.gesmescb.Attribute;

import Int.ecb.gesmescb.event.TestEventListener;

import Int.ecb.gesmescb.event.ThreadPump;

/* ***************************** Class ****************************** */

/**
 * <P>Tests the implementation of the
 * class {@link ThreadPump}.</P>
 *
 * @author
 * <A HREF="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</A>
 *
 * @version @(#)$Id: ThreadPumpTest.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */

public final class ThreadPumpTest extends TestCase {

	/* ====================== Constructors ====================== */

	/**
	 * Constructs a ThreadPumpTest with the specified name.
	 *
	 * @param name Test case name
	 */
	public ThreadPumpTest ( String name ) {
		super( name );
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite
	 */
	public static Test suite () {
		TestSuite suite = new TestSuite( ThreadPumpTest.class );
		return suite;
	}

	/**
	 * Runs the test case
	 */
	public static void main ( String args[] ) {
		String[] testCaseName = { ThreadPumpTest.class.getName() };
		junit.textui.TestRunner.main( testCaseName );
	}

	/* --------------------- Test Methods --------------------- */

	/**
	 * Tests whether all that goes in comes out.
	 */
	public void testPumping () {
		final InterchangeBegin ib = new InterchangeBegin(
			"IREF", "Sender", "Reveiver", "prep", true
		);
		final MessageBegin mb = new MessageBegin(
			"mref", "se", "re", "ma", "func", "subject", new Contact[] {}
		);
		final Attribute at = new Attribute(
			"1", "2", "3", "4", "5"
		);
		final ErrorEvent er = new ErrorEvent(
			"Test", ErrorEvent.INFO
		);
		final MessageEnd me = new MessageEnd( "mref" );
		final InterchangeEnd ie = new InterchangeEnd( "IREF" );
		final Object[] objects = { ib, mb, at, er, me, ie };

		final TestEventSource source = new TestEventSource();
		final TestEventListener listener
		= new TestEventListener( objects.length + 1 );

		final ThreadPump pump = new ThreadPump( 2 );

		source.addListener( pump, true );
		pump.addListener( listener, true );

		source.dispatch( objects );

		pump.close();

		final Iterator iterator = listener.iterator();

		for ( int i = 0; i < objects.length; ++i ) {
			final Object object = iterator.next();
			assertSame( objects[i], object );
		}

		assertTrue( "No excessive objects", !iterator.hasNext() );
	}
}
