// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.fame;

import com.fame.timeiq.*;
import com.fame.timeiq.dates.*;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.File;
import java.util.Properties;
import java.util.Date;

import Int.ecb.gesmescb.*;
import Int.ecb.gesmescb.event.*;
import Int.ecb.gesmescb.metabase.Metabase;
import Int.ecb.gesmescb.util.ComponentHelper;

/* ***************************** Class ****************************** */

/**
 * <P>
 * Tests the implementation of ESDBDeleter class in the Int.ecb.gesmescb.fame
 * package.
 * </P>
 * 
 * @author <A HREF="mailto:Sami.Airo@ECB.int">Sami Airo</A>
 * 
 * @version @(#)$Id: ESDBDeleterTest.java,v 1.7 2008/02/25 09:47:07 hoeltzl Exp
 *          $
 */
public class ESDBDeleterFAME10Test extends TestCase {

    /* ========================= Fields ========================= */

    private FAMEDb fameDb;
    private ESDBUpdater updater;
    private ESDBDeleter deleter;
    private String fileName = "/tmp/ESDBUpdaterTest.db";

    private static String formatDbVersion = "4";

    /* ====================== Constructors ====================== */

    /**
     * Constructs a ESDBDeleterTest with the specified name.
     * 
     * @param name
     *            Test case name
     */
    public ESDBDeleterFAME10Test(final String name) {
        super(name);
    }

    /* ================== Protected Methods ===================== */

    /**
     * Sets up the test fixture.
     */
    protected void setUp() {
     
            // body
            System.setProperty("Int.ecb.logging.config.class",
                    "Int.ecb.gesmescb.util.LoggingPropertiesReader");
            System.setProperty(".level", "ALL");
            System.setProperty("handlers", "Int.ecb.logging.ConsoleHandler");
            System.setProperty("Int.ecb.logging.ConsoleHandler.level", "INFO");

            TestEventSource source = new TestEventSource();
            source.addListener(Metabase.getMetabase(), true);

            source
                    .dispatch(new Object[]{
                            new InterchangeBegin("IREF0001", "4F0", "ZZZ", "",
                                    false),
                            new MessageBegin("MREF0001", "4F0", "ZZZ", "ECB",
                                    MessageBegin.STATISTICAL_DEFINITIONS, "",
                                    new Contact[]{}),
                            new CodeList("CL_FREQ", "", "", new Date(),
                                    new String[]{"M", "Q"},
                                    new String[]{"", ""}),
                            new KeyFamily(
                                    "ECB_BOP1",
                                    "",
                                    "",
                                    new Date(),
                                    new KeyFamilyComponent[]{
                                            new KeyFamilyComponent(
                                                    "TEST",
                                                    KeyFamilyComponent.ATTRIBUTE,
                                                    "",
                                                    "",
                                                    "",
                                                    KeyFamilyComponent.ATTACHED_TO_OBSERVATION,
                                                    ""),
                                            new KeyFamilyComponent(
                                                    "TEST2",
                                                    KeyFamilyComponent.ATTRIBUTE,
                                                    "",
                                                    "",
                                                    "",
                                                    KeyFamilyComponent.ATTACHED_TO_TIME_SERIES,
                                                    ""),
                                            new KeyFamilyComponent(
                                                    "TEST3",
                                                    KeyFamilyComponent.ATTRIBUTE,
                                                    "",
                                                    "",
                                                    "",
                                                    KeyFamilyComponent.ATTACHED_TO_SIBLING_GROUP,
                                                    ""),
                                            new KeyFamilyComponent(
                                                    "OBS_PRE_BREAK",
                                                    KeyFamilyComponent.ARRAY_CELL_DIMENSION,
                                                    "", "", "", "", "")}),
                            new MessageEnd("MREF0001"),
                            new InterchangeEnd("IREF0001")});

            fameDb = new FAMEDb("");

            Properties properties = new Properties();
            properties.put("mode", "CREATE");
            properties.put("dbVersion", formatDbVersion);

            // if (!fameDb.open( fileName, properties ) ) {
            if (!fameDb.open(fileName, properties)) {
                fail("cannot open Fame database");
            }

            MessageBegin messageBegin = new MessageBegin("MREF00001", "DE2",
                    "4F0", "ECB", MessageBegin.STATISTICAL_DATA, "test",
                    new Contact[]{}, "BOP",
                    MessageBegin.UPDATE_OR_REPLACE_ACTION, "", "", "",
                    "ECB_BOP1");

            updater = new ESDBUpdater(new ComponentHelper("", this), fameDb,
                    messageBegin);
            deleter = new ESDBDeleter(new ComponentHelper("", this), fameDb,
                    messageBegin);
      
    }

    /**
     * Tears down the test fixture.
     */
    protected void tearDown() {
        fameDb.close(false);
        try {
            File file = new File(fileName);
            file.delete();
        } catch (Exception e) {
            fail(e.getMessage());
        }
    }

    /* ===================== Public Methods ===================== */

    /**
     * Assembles and returns a test suite for all the test methods of this test
     * case.
     * 
     * @return a non-null test suite
     */
    public static Test suite() {
        TestSuite suite = new TestSuite(ESDBDeleterFAME10Test.class);
        return suite;
    }

    /**
     * Runs the test case
     */
    public static void main(String args[]) {
 
        String[] testCaseName = {ESDBDeleterFAME10Test.class.getName()};
        junit.textui.TestRunner.main(testCaseName);
    }

    /* --------------------- Test Methods --------------------- */

    public void testHandleTimeSeries() {
        TimeSeries timeSeries = new TimeSeries("M.TEST1", "200012200102",
                "710", new Int.ecb.gesmescb.Observation[]{
                        new Int.ecb.gesmescb.Observation(
                                Int.ecb.gesmescb.Observation.MISSING_VALUE,
                                "S", "N", ""),
                        new Int.ecb.gesmescb.Observation("2", "A", "",
                                Int.ecb.gesmescb.Observation.MISSING_VALUE),
                        new Int.ecb.gesmescb.Observation("3", "A", "C", "13")});

        assertTrue(updater.handle(timeSeries));

        Series series = (Series) fameDb.getObject("BOP.M.TEST1");
        assertNotNull(series);

        timeSeries = new TimeSeries("M.TEST1", "", "",
                new Int.ecb.gesmescb.Observation[]{});

        assertTrue(deleter.handle(timeSeries));

        series = (Series) fameDb.getObject("BOP.M.TEST1");
        assertNull(series);
    }

    public void testHandleObservation() {
        TimeSeries timeSeries = new TimeSeries("M.TEST1", "200012200102",
                "710", new Int.ecb.gesmescb.Observation[]{
                        new Int.ecb.gesmescb.Observation(
                                Int.ecb.gesmescb.Observation.MISSING_VALUE,
                                "S", "N", ""),
                        new Int.ecb.gesmescb.Observation("2", "A", "",
                                Int.ecb.gesmescb.Observation.MISSING_VALUE),
                        new Int.ecb.gesmescb.Observation("3", "A", "C", "13")});

        assertTrue(updater.handle(timeSeries));

        Series series = (Series) fameDb.getObject("BOP.M.TEST1");
        assertNotNull(series);

        timeSeries = new TimeSeries("M.TEST1", "200012", "610",
                new Int.ecb.gesmescb.Observation[]{});

        assertTrue(deleter.handle(timeSeries));

        series = (Series) fameDb.getObject("BOP.M.TEST1");
        assertNotNull(series);
        assertEquals(2001, DateHelper.indexToYear(series.getMinIndex()));
        assertEquals(1, DateHelper.indexToMonth(series.getMinIndex()));
        assertEquals(2001, DateHelper.indexToYear(series.getMaxIndex()));
        assertEquals(2, DateHelper.indexToMonth(series.getMaxIndex()));
    }

    public void testHandleSiblingSet() {
        TimeSeries timeSeries = new TimeSeries("M.TEST1", "200012200102",
                "710", new Int.ecb.gesmescb.Observation[]{
                        new Int.ecb.gesmescb.Observation(
                                Int.ecb.gesmescb.Observation.MISSING_VALUE,
                                "S", "N", ""),
                        new Int.ecb.gesmescb.Observation("2", "A", "",
                                Int.ecb.gesmescb.Observation.MISSING_VALUE),
                        new Int.ecb.gesmescb.Observation("3", "A", "C", "13")});

        assertTrue(updater.handle(timeSeries));

        Series series = (Series) fameDb.getObject("BOP.M.TEST1");
        assertNotNull(series);

        timeSeries = new TimeSeries("Q.TEST1", "2001120012", "708",
                new Int.ecb.gesmescb.Observation[]{
                        new Int.ecb.gesmescb.Observation(
                                Int.ecb.gesmescb.Observation.MISSING_VALUE,
                                "S", "N", ""),
                        new Int.ecb.gesmescb.Observation("2", "A", "",
                                Int.ecb.gesmescb.Observation.MISSING_VALUE)});

        assertTrue(updater.handle(timeSeries));

        series = (Series) fameDb.getObject("BOP.Q.TEST1");
        assertNotNull(series);

        Attribute attribute = new Attribute(".TEST1", "", "", "TEST3", "G");

        assertTrue(updater.handle(attribute));

        Scalar scalar = (Scalar) fameDb.getObject("BOP.TEST1.TEST3");
        assertNotNull(scalar);

        timeSeries = new TimeSeries(".TEST1", "", "",
                new Int.ecb.gesmescb.Observation[]{});

        assertTrue(deleter.handle(timeSeries));

        series = (Series) fameDb.getObject("BOP.M.TEST1");
        assertNull(series);
        series = (Series) fameDb.getObject("BOP.Q.TEST1");
        assertNull(series);
        scalar = (Scalar) fameDb.getObject("BOP.TEST1.TEST3");
        assertNull(scalar);
    }

    public void testHandleAttributeObservation() {
        Attribute attribute = new Attribute("M.TEST1", "200108", "610", "TEST",
                "G");

        assertTrue(updater.handle(attribute));

        attribute = new Attribute("M.TEST1", "200109", "610", "TEST", "H");

        assertTrue(updater.handle(attribute));

        Series series = (Series) fameDb.getObject("BOP.M.TEST1.TEST");
        assertNotNull(series);
        assertEquals(2001, DateHelper.indexToYear(series.getMinIndex()));
        assertEquals(8, DateHelper.indexToMonth(series.getMinIndex()));
        assertEquals(2001, DateHelper.indexToYear(series.getMaxIndex()));
        assertEquals(9, DateHelper.indexToMonth(series.getMaxIndex()));

        attribute = new Attribute("M.TEST1", "200109", "610", "TEST", "");

        assertTrue(deleter.handle(attribute));

        series = (Series) fameDb.getObject("BOP.M.TEST1.TEST");
        assertNotNull(series);
        assertEquals(2001, DateHelper.indexToYear(series.getMinIndex()));
        assertEquals(8, DateHelper.indexToMonth(series.getMinIndex()));
        assertEquals(2001, DateHelper.indexToYear(series.getMaxIndex()));
        assertEquals(8, DateHelper.indexToMonth(series.getMaxIndex()));

    }

    public void testdeleteHandleAttribute() {
        Attribute attribute = new Attribute("M.TEST1", "", "", "TEST2", "R");

        assertTrue(updater.handle(attribute));

        Scalar scalar = (Scalar) fameDb.getObject("BOP.M.TEST1.TEST2");
        assertNotNull(scalar);

        attribute = new Attribute("M.TEST1", "", "", "TEST2", "");

        assertTrue(deleter.handle(attribute));

        scalar = (Scalar) fameDb.getObject("BOP.M.TEST1.TEST2");
        assertNull(scalar);
    }

}
