// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.fame;

import com.fame.timeiq.*;
import com.fame.timeiq.data.ObservationList;
import com.fame.timeiq.dates.*;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.File;
import java.util.Properties;
import java.util.Date;

import Int.ecb.gesmescb.*;
import Int.ecb.gesmescb.event.*;
import Int.ecb.gesmescb.metabase.Metabase;
import Int.ecb.gesmescb.util.ComponentHelper;
import Int.ecb.gesmescb.fame.FAMEDb;

/* ***************************** Class ****************************** */

/**
 * <P>
 * Tests the implementation of ESDBUpdater class in the Int.ecb.gesmescb.fame
 * package.
 * </P>
 * 
 * @author <A HREF="mailto:Sami.Airo@ECB.int">Sami Airo</A>
 * 
 * @version @(#)$Id: ESDBUpdaterTest.java,v 1.9 2010/09/21 13:33:12 horvatg Exp
 *          $
 */

public class ESDBUpdatertFAME10Test extends TestCase {

    /* ========================= Fields ========================= */

    FAMEDb fameDb;
    File file;
    ESDBUpdater updater;
    String fileName = "/tmp/ESDBUpdaterTest.db";

    private static String formatDbVersion = "4";

    /* ====================== Constructors ====================== */

    /**
     * Constructs a ESDBUpdaterTest with the specified name.
     * 
     * @param name
     *            Test case name
     */
    public ESDBUpdatertFAME10Test(final String name) {
        super(name);
    }

    /* ================== Protected Methods ===================== */

    /**
     * Sets up the test fixture.
     */
    protected void setUp() {
            System.setProperty("Int.ecb.logging.config.class",
                    "Int.ecb.gesmescb.util.LoggingPropertiesReader");
            System.setProperty(".level", "ALL");
            System.setProperty("handlers", "Int.ecb.logging.ConsoleHandler");
            System.setProperty("Int.ecb.logging.ConsoleHandler.level", "INFO");

            TestEventSource source = new TestEventSource();
            source.addListener(Metabase.getMetabase(), true);

            source
                    .dispatch(new Object[]{
                            new InterchangeBegin("IREF0001", "4F0", "ZZZ", "",
                                    false),
                            new MessageBegin("MREF0001", "4F0", "ZZZ", "ECB",
                                    MessageBegin.STATISTICAL_DEFINITIONS, "",
                                    new Contact[]{}),
                            new CodeList("CL_FREQ", "", "", new Date(),
                                    new String[]{"M", "Q"},
                                    new String[]{"", ""}),
                            new KeyFamily(
                                    "ECB_BOP1",
                                    "",
                                    "",
                                    new Date(),
                                    new KeyFamilyComponent[]{
                                            new KeyFamilyComponent(
                                                    "TEST",
                                                    KeyFamilyComponent.ATTRIBUTE,
                                                    "",
                                                    "",
                                                    "",
                                                    KeyFamilyComponent.ATTACHED_TO_OBSERVATION,
                                                    ""),
                                            new KeyFamilyComponent(
                                                    "TEST2",
                                                    KeyFamilyComponent.ATTRIBUTE,
                                                    "",
                                                    "",
                                                    "",
                                                    KeyFamilyComponent.ATTACHED_TO_TIME_SERIES,
                                                    ""),
                                            new KeyFamilyComponent(
                                                    "COLLECTION",
                                                    KeyFamilyComponent.ATTRIBUTE,
                                                    "",
                                                    "",
                                                    "",
                                                    KeyFamilyComponent.ATTACHED_TO_SIBLING_GROUP,
                                                    "")}),
                            new MessageEnd("MREF0001"),
                            new InterchangeEnd("IREF0001")});

            fameDb = new FAMEDb("");

            Properties properties = new Properties();
            properties.put("mode", "CREATE");

            properties.put("dbVersion", formatDbVersion);

            // if (!fameDb.open( fileName, properties ) ) {
            if (!fameDb.open(fileName, properties)) {
                fail("cannot open Fame database");
            }

            MessageBegin messageBegin = new MessageBegin("MREF00001", "DE2",
                    "4F0", "ECB", MessageBegin.STATISTICAL_DATA, "test",
                    new Contact[]{}, "BOP",
                    MessageBegin.UPDATE_OR_REPLACE_ACTION, "", "", "",
                    "ECB_BOP1");

            updater = new ESDBUpdater(new ComponentHelper("", this), fameDb,
                    messageBegin);
    }

    /**
     * Tears down the test fixture.
     */
    protected void tearDown() {
        fameDb.close(false);
        try {
            File file = new File(fileName);
            file.delete();
        } catch (Exception e) {
            fail(e.getMessage());
        }
    }

    /* ===================== Public Methods ===================== */

    /**
     * Assembles and returns a test suite for all the test methods of this test
     * case.
     * 
     * @return a non-null test suite
     */
    public static Test suite() {
        TestSuite suite = new TestSuite(ESDBUpdatertFAME10Test.class);
        return suite;
    }

    /**
     * Runs the test case
     */
    public static void main(String args[]) {
        String[] testCaseName = {ESDBUpdatertFAME10Test.class.getName()};
        junit.textui.TestRunner.main(testCaseName);
    }

    /* --------------------- Test Methods --------------------- */

    public void testHandleTimeSeriesInsert() {
        TimeSeries timeSeries = new TimeSeries("M.TEST1", "200012200102",
                "710", new Int.ecb.gesmescb.Observation[]{
                        new Int.ecb.gesmescb.Observation(
                                Int.ecb.gesmescb.Observation.MISSING_VALUE,
                                "S", "N", ""),
                        new Int.ecb.gesmescb.Observation("2", "A", "",
                                Int.ecb.gesmescb.Observation.MISSING_VALUE),
                        new Int.ecb.gesmescb.Observation("3", "A", "C", "13")});

        assertTrue(updater.handle(timeSeries));

        Series series = (Series) fameDb.getObject("BOP.M.TEST1");

        ObservationList observationList = series.getObservations();
        long[] indexesAsLong = observationList.getIndexes();

        assertNotNull(series);
        assertEquals(TiqType.DOUBLE, series.getTiqType());
        assertEquals(TiqFrequency.MONTHLY, series.getFrequency());
        assertEquals(2000, DateHelper.indexToYear(series.getMinIndex()));
        assertEquals(12, DateHelper.indexToMonth(series.getMinIndex()));
        assertEquals(2001, DateHelper.indexToYear(series.getMaxIndex()));
        assertEquals(2, DateHelper.indexToMonth(series.getMaxIndex()));

        assertEquals(TiqConstants.STATUS_NC, observationList.getObservation(
                indexesAsLong[0]).getStatus());
        assertEquals(2, observationList.getObservation(indexesAsLong[1])
                .getIntValue());
        assertEquals(TiqConstants.STATUS_OK, observationList.getObservation(
                indexesAsLong[1]).getStatus());
        assertEquals(3, observationList.getObservation(indexesAsLong[2])
                .getIntValue());
        assertEquals(TiqConstants.STATUS_OK, observationList.getObservation(
                indexesAsLong[2]).getStatus());

        series = (Series) fameDb.getObject("BOP.M.TEST1.OBS_STATUS");
        observationList = series.getObservations();
        indexesAsLong = observationList.getIndexes();

        assertNotNull(series);
        assertEquals(TiqType.STRING, series.getTiqType());
        assertEquals(TiqFrequency.MONTHLY, series.getFrequency());
        assertEquals(2000, DateHelper.indexToYear(series.getMinIndex()));
        assertEquals(12, DateHelper.indexToMonth(series.getMinIndex()));
        assertEquals(2001, DateHelper.indexToYear(series.getMaxIndex()));
        assertEquals(2, DateHelper.indexToMonth(series.getMaxIndex()));

        assertEquals("S", observationList.getObservation(indexesAsLong[0])
                .getStringValue());
        assertEquals(TiqConstants.STATUS_OK, observationList.getObservation(
                indexesAsLong[0]).getStatus());
        assertEquals("A", observationList.getObservation(indexesAsLong[1])
                .getStringValue());
        assertEquals(TiqConstants.STATUS_OK, observationList.getObservation(
                indexesAsLong[1]).getStatus());
        assertEquals("A", observationList.getObservation(indexesAsLong[2])
                .getStringValue());
        assertEquals(TiqConstants.STATUS_OK, observationList.getObservation(
                indexesAsLong[2]).getStatus());

        series = (Series) fameDb.getObject("BOP.M.TEST1.OBS_CONF");
        observationList = series.getObservations();
        indexesAsLong = observationList.getIndexes();

        assertNotNull(series);
        assertEquals(TiqType.STRING, series.getTiqType());
        assertEquals(TiqFrequency.MONTHLY, series.getFrequency());
        assertEquals(2000, DateHelper.indexToYear(series.getMinIndex()));
        assertEquals(12, DateHelper.indexToMonth(series.getMinIndex()));
        assertEquals(2001, DateHelper.indexToYear(series.getMaxIndex()));
        assertEquals(2, DateHelper.indexToMonth(series.getMaxIndex()));

        assertEquals("N", observationList.getObservation(indexesAsLong[0])
                .getStringValue());
        assertEquals(TiqConstants.STATUS_OK, observationList.getObservation(
                indexesAsLong[0]).getStatus());
        assertEquals(TiqConstants.STATUS_ND, observationList.getObservation(
                indexesAsLong[1]).getStatus());
        assertEquals("C", observationList.getObservation(indexesAsLong[2])
                .getStringValue());
        assertEquals(TiqConstants.STATUS_OK, observationList.getObservation(
                indexesAsLong[2]).getStatus());

        series = (Series) fameDb.getObject("BOP.M.TEST1.BREAK");
        observationList = series.getObservations();
        indexesAsLong = observationList.getIndexes();

        assertNotNull(series);
        assertEquals(TiqType.DOUBLE, series.getTiqType());
        assertEquals(TiqFrequency.MONTHLY, series.getFrequency());
        assertEquals(2001, DateHelper.indexToYear(series.getMinIndex()));
        assertEquals(1, DateHelper.indexToMonth(series.getMinIndex()));
        assertEquals(2001, DateHelper.indexToYear(series.getMaxIndex()));
        assertEquals(2, DateHelper.indexToMonth(series.getMaxIndex()));

        assertEquals(TiqConstants.STATUS_NC, observationList.getObservation(
                indexesAsLong[0]).getStatus());
        assertEquals(13, observationList.getObservation(indexesAsLong[1])
                .getIntValue());
        assertEquals(TiqConstants.STATUS_OK, observationList.getObservation(
                indexesAsLong[1]).getStatus());
    }

    public void testHandleTimeSeriesUpdate() {
        TimeSeries timeSeries = new TimeSeries("M.TEST1", "200012200102",
                "710", new Int.ecb.gesmescb.Observation[]{
                        new Int.ecb.gesmescb.Observation("1", "S", "N", ""),
                        new Int.ecb.gesmescb.Observation("2", "A", "C", "22"),
                        new Int.ecb.gesmescb.Observation("3", "A", "C", "13")});

        assertTrue(updater.handle(timeSeries));

        timeSeries = new TimeSeries("M.TEST1", "200101200103", "710",
                new Int.ecb.gesmescb.Observation[]{
                        new Int.ecb.gesmescb.Observation("11", "G", "", ""),
                        Int.ecb.gesmescb.Observation.EMPTY_OBSERVATION,
                        new Int.ecb.gesmescb.Observation("4", "T", "L", "44")});

        assertTrue(updater.handle(timeSeries));

        Series series = (Series) fameDb.getObject("BOP.M.TEST1");
        ObservationList observationList = series.getObservations();
        long[] indexesAsLong = observationList.getIndexes();

        assertNotNull(series);
        assertEquals(TiqType.DOUBLE, series.getTiqType());
        assertEquals(TiqFrequency.MONTHLY, series.getFrequency());
        assertEquals(2000, DateHelper.indexToYear(series.getMinIndex()));
        assertEquals(12, DateHelper.indexToMonth(series.getMinIndex()));
        assertEquals(2001, DateHelper.indexToYear(series.getMaxIndex()));
        assertEquals(3, DateHelper.indexToMonth(series.getMaxIndex()));

        assertEquals(1, observationList.getObservation(indexesAsLong[0])
                .getIntValue());
        assertEquals(TiqConstants.STATUS_OK, observationList.getObservation(
                indexesAsLong[0]).getStatus());
        assertEquals(11, observationList.getObservation(indexesAsLong[1])
                .getIntValue());
        assertEquals(TiqConstants.STATUS_OK, observationList.getObservation(
                indexesAsLong[1]).getStatus());
        assertEquals(3, observationList.getObservation(indexesAsLong[2])
                .getIntValue());
        assertEquals(TiqConstants.STATUS_OK, observationList.getObservation(
                indexesAsLong[2]).getStatus());
        assertEquals(4, observationList.getObservation(indexesAsLong[3])
                .getIntValue());
        assertEquals(TiqConstants.STATUS_OK, observationList.getObservation(
                indexesAsLong[3]).getStatus());

        series = (Series) fameDb.getObject("BOP.M.TEST1.OBS_STATUS");
        observationList = series.getObservations();
        indexesAsLong = observationList.getIndexes();

        assertNotNull(series);
        assertEquals(TiqType.STRING, series.getTiqType());
        assertEquals(TiqFrequency.MONTHLY, series.getFrequency());
        assertEquals(2000, DateHelper.indexToYear(series.getMinIndex()));
        assertEquals(12, DateHelper.indexToMonth(series.getMinIndex()));
        assertEquals(2001, DateHelper.indexToYear(series.getMaxIndex()));
        assertEquals(3, DateHelper.indexToMonth(series.getMaxIndex()));

        assertEquals("S", observationList.getObservation(indexesAsLong[0])
                .getStringValue());
        assertEquals(TiqConstants.STATUS_OK, observationList.getObservation(
                indexesAsLong[0]).getStatus());
        assertEquals("G", observationList.getObservation(indexesAsLong[1])
                .getStringValue());
        assertEquals(TiqConstants.STATUS_OK, observationList.getObservation(
                indexesAsLong[1]).getStatus());
        assertEquals("A", observationList.getObservation(indexesAsLong[2])
                .getStringValue());
        assertEquals(TiqConstants.STATUS_OK, observationList.getObservation(
                indexesAsLong[2]).getStatus());
        assertEquals("T", observationList.getObservation(indexesAsLong[3])
                .getStringValue());
        assertEquals(TiqConstants.STATUS_OK, observationList.getObservation(
                indexesAsLong[3]).getStatus());

        series = (Series) fameDb.getObject("BOP.M.TEST1.OBS_CONF");
        observationList = series.getObservations();
        indexesAsLong = observationList.getIndexes();

        assertNotNull(series);
        assertEquals(TiqType.STRING, series.getTiqType());
        assertEquals(TiqFrequency.MONTHLY, series.getFrequency());
        assertEquals(2000, DateHelper.indexToYear(series.getMinIndex()));
        assertEquals(12, DateHelper.indexToMonth(series.getMinIndex()));
        assertEquals(2001, DateHelper.indexToYear(series.getMaxIndex()));
        assertEquals(3, DateHelper.indexToMonth(series.getMaxIndex()));

        assertEquals("N", observationList.getObservation(indexesAsLong[0])
                .getStringValue());
        assertEquals(TiqConstants.STATUS_OK, observationList.getObservation(
                indexesAsLong[0]).getStatus());
        assertEquals(TiqConstants.STATUS_ND, observationList.getObservation(
                indexesAsLong[1]).getStatus());
        assertEquals("C", observationList.getObservation(indexesAsLong[2])
                .getStringValue());
        assertEquals(TiqConstants.STATUS_OK, observationList.getObservation(
                indexesAsLong[2]).getStatus());
        assertEquals("L", observationList.getObservation(indexesAsLong[3])
                .getStringValue());
        assertEquals(TiqConstants.STATUS_OK, observationList.getObservation(
                indexesAsLong[3]).getStatus());

        series = (Series) fameDb.getObject("BOP.M.TEST1.BREAK");
        observationList = series.getObservations();
        indexesAsLong = observationList.getIndexes();

        assertNotNull(series);
        assertEquals(TiqType.DOUBLE, series.getTiqType());
        assertEquals(TiqFrequency.MONTHLY, series.getFrequency());
        assertEquals(2001, DateHelper.indexToYear(series.getMinIndex()));
        assertEquals(2, DateHelper.indexToMonth(series.getMinIndex()));
        assertEquals(2001, DateHelper.indexToYear(series.getMaxIndex()));
        assertEquals(3, DateHelper.indexToMonth(series.getMaxIndex()));

        assertEquals(13, observationList.getObservation(indexesAsLong[0])
                .getIntValue());
        assertEquals(TiqConstants.STATUS_OK, observationList.getObservation(
                indexesAsLong[0]).getStatus());
        assertEquals(44, observationList.getObservation(indexesAsLong[1])
                .getIntValue());
        assertEquals(TiqConstants.STATUS_OK, observationList.getObservation(
                indexesAsLong[1]).getStatus());
    }

    public void testHandleAttributeObservation() {
        Attribute attribute = new Attribute("M.TEST1", "200108", "610", "TEST",
                "G");

        assertTrue(updater.handle(attribute));

        Series series = (Series) fameDb.getObject("BOP.M.TEST1.TEST");

        assertNotNull(series);
        assertEquals(TiqType.STRING, series.getTiqType());
        assertEquals(TiqFrequency.MONTHLY, series.getFrequency());
        assertEquals(2001, DateHelper.indexToYear(series.getMinIndex()));
        assertEquals(8, DateHelper.indexToMonth(series.getMinIndex()));
        assertEquals(2001, DateHelper.indexToYear(series.getMaxIndex()));
        assertEquals(8, DateHelper.indexToMonth(series.getMaxIndex()));

        ObservationList observationList = series.getObservations();
        long[] indexesAsLong = observationList.getIndexes();

        assertEquals(TiqConstants.STATUS_OK, observationList.getObservation(
                indexesAsLong[0]).getStatus());
        assertEquals("G", observationList.getObservation(indexesAsLong[0])
                .getStringValue());

    }

    public void testdeleteHandleAttribute() {
        Attribute attribute = new Attribute("M.TEST1", "", "", "TEST2", "R");

        assertTrue(updater.handle(attribute));

        Scalar scalar = (Scalar) fameDb.getObject("BOP.M.TEST1.TEST2");

        assertNotNull(scalar);
        assertEquals(TiqType.STRING, scalar.getTiqType());
        assertEquals(TiqConstants.STATUS_OK, scalar.getTiqValue().getStatus());
        assertEquals("R", scalar.getTiqValue().getStringValue());
    }

    public void testHandleObservedAttribute() {
        TimeSeries timeSeries = new TimeSeries(
                "M.TEST1",
                "200101",
                "610",
                new Int.ecb.gesmescb.Observation[]{new Int.ecb.gesmescb.Observation(
                        "100", "S", "N", ""),});

        assertTrue(updater.handle(timeSeries));
        Series series = (Series) fameDb.getObject("BOP.M.TEST1");

        assertNotNull(series);
        assertEquals(TiqObserved.AVERAGED, series.getObserved());
        assertEquals(TiqType.DOUBLE, series.getTiqType());
        assertEquals(TiqFrequency.MONTHLY, series.getFrequency());
        assertEquals(2001, DateHelper.indexToYear(series.getMinIndex()));
        assertEquals(1, DateHelper.indexToMonth(series.getMinIndex()));
        assertEquals(2001, DateHelper.indexToYear(series.getMaxIndex()));
        assertEquals(1, DateHelper.indexToMonth(series.getMaxIndex()));

        ObservationList observationList = series.getObservations();
        long[] indexesAsLong = observationList.getIndexes();
        assertEquals(TiqConstants.STATUS_OK, series.getObservations()
                .getObservation(indexesAsLong[0]).getStatus());
        assertEquals(TiqConstants.STATUS_OK, series.getObservations()
                .getObservation(indexesAsLong[0]).getStatus());
        assertEquals(100, series.getObservations().getObservation(
                indexesAsLong[0]).getIntValue());

        Attribute attribute = new Attribute(".TEST1", "", "", "COLLECTION", "S");

        assertTrue(updater.handle(attribute));
        series = (Series) fameDb.getObject("BOP.M.TEST1");

        assertNotNull(series);
        assertEquals(TiqObserved.SUMMED, series.getObserved());
        assertEquals(TiqType.DOUBLE, series.getTiqType());
        assertEquals(TiqFrequency.MONTHLY, series.getFrequency());
        assertEquals(2001, DateHelper.indexToYear(series.getMinIndex()));
        assertEquals(1, DateHelper.indexToMonth(series.getMinIndex()));
        assertEquals(2001, DateHelper.indexToYear(series.getMaxIndex()));
        assertEquals(1, DateHelper.indexToMonth(series.getMaxIndex()));

        observationList = series.getObservations();
        indexesAsLong = observationList.getIndexes();

        assertEquals(TiqConstants.STATUS_OK, observationList.getObservation(
                indexesAsLong[0]).getStatus());
        assertEquals(100, observationList.getObservation(indexesAsLong[0])
                .getIntValue());
    }

}
