package Int.ecb.gesmescb.fame;

import java.io.File;
import java.io.IOException;

import Int.ecb.gesmescb.CodeList;
import Int.ecb.gesmescb.Contact;
import Int.ecb.gesmescb.InterchangeBegin;
import Int.ecb.gesmescb.InterchangeEnd;
import Int.ecb.gesmescb.MessageBegin;
import Int.ecb.gesmescb.MessageEnd;
import Int.ecb.gesmescb.event.TestEventSource;
import Int.ecb.gesmescb.metabase.Metabase;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import com.fame.timeiq.Scalar;

public class FAMEIDBReaderTest extends TestCase {
	/* ========================= Fields ========================= */
	String dbName = "/tmp/2002041614131";
	String fullDBName = dbName + "_4F0_ZZZ_IREF987654_MREF00001_BOP.db";

	    
	/* ====================== Constructors ====================== */
	/**
	 * Constructs a FAMEIDBReaderTest with the specified name.
	 *
	 * @param name Test case name
	 */	
	public FAMEIDBReaderTest( final String name ) {
		super( name );
	}
	/* ================== Protected Methods ===================== */

	/**
	 * Sets up the test fixture.
	 */	
	protected void setUp() {
		TestEventSource source = new TestEventSource();
		source.addListener( Metabase.getMetabase(), true );

		source.dispatch( new Object[]{
			new InterchangeBegin(
				"IREF987654", "4F0", "ZZZ", "", false
			),
			new MessageBegin(
				"MREF0001", "4F0", "ZZZ", "ECB", MessageBegin.STATISTICAL_DEFINITIONS, "", new Contact[]{}
			),
			new CodeList(
				"CL_FREQ", "", "", new String[]{"M", "Q"}, new String[]{"",""}
			),
			new MessageEnd(
				"MREF0001"
			),
			new InterchangeEnd(
				"IREF987654"
			)
		} );
		System.setProperty(
			"Int.ecb.gesmescb.fame.FAMEIDBReader.reader.internalKeyFamilyName",
			"ESAT"
		);
		System.setProperty(
			"Int.ecb.gesmescb.fame.FAMEIDBReader.reader.ignoreDecimalsAttributes",
			"true"
		);
		System.setProperty("Int.ecb.gesmescb.fame.SDPEWriter.destination", dbName);
		SDPEWriter writer = new SDPEWriter();
		FAMEDb fameDB = writer.getFAMEDb();
		final InterchangeBegin testInterchange = 	new InterchangeBegin(
				"IREF987654", "4F0", "ZZZ", "", false
		);
		writer.handle(testInterchange);
		final MessageBegin testMessage = new MessageBegin(
			"MREF00001",
			"DE2",
			"4F0",
			"ECB",
			MessageBegin.STATISTICAL_DATA,
			"test",
			new Contact[]{},
			"BOP",
			MessageBegin.DELETE_ACTION,
			"", "", "",
			"ECB_BOP1",
			"BLURP"
		);
		//fameDB.createObject(
		//	"ENVELOPE.ORIGINAL_DATASET",
		//	new Scalar("BLURP")
		//);
		writer.handle(testMessage);
		writer.handle(
			new MessageEnd(
				"MREF0001"
			)
		);
		writer.handle(
			new InterchangeEnd(
				"IREF987654"
			)
		);
	}
	
	/**
	 * Tears down the test fixture.
	 */
	protected void tearDown() {
		try {
			File file = new File(fullDBName);
			file.delete();
		} catch (Exception e) {
			fail( e.getMessage() );
		}
	}
	
	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite
	 */
	public static Test suite() {
		TestSuite suite = new TestSuite( FAMEIDBReaderTest.class );
		return suite;
	}

	/**
	 * Runs the test case
	 */
	public static void main(String args[]) {
		String[] testCaseName = {
			FAMEIDBReaderTest.class.getName()
		};
		junit.textui.TestRunner.main(testCaseName);
	}

	/* --------------------- Test Methods --------------------- */
	
	public void testFAMEIDBReader() {
		FAMEIDBReader reader=null;
		try {	
			reader = new FAMEIDBReader(
				"reader",
				fullDBName
			);
		} catch ( IOException e ) {
			System.out.println("constructor break!");
		}
		assertTrue( "No reader!", null != reader );
		String iref = reader.getMandatoryPreference( "IREF" );
		assertEquals( "iref", "IREF987654", iref);
		String sender = reader.getMandatoryPreference( "sender" );
		assertEquals( "sender", "DE2", sender );
		String receiver = reader.getMandatoryPreference( "receiver" );
		assertEquals( "receiver", "4F0", receiver );
		String internalKeyFamilyName = reader.getMandatoryPreference( "internalKeyFamilyName" );
		//String internalKeyFamilyName = reader.getEnvelopeProperty( "internalKeyFamilyName" );
		assertEquals( "internalKeyFamilyName", "ESAT", internalKeyFamilyName );
		String originalDataset = reader.getMandatoryPreference( "originalDataset");
		assertEquals( "originalDataset", "BLURP", originalDataset );
		String[] expected = new String[]{ "3.1415926" };
		double[] values = new double[]{ 3.1415926 };
		assertEquals( "ignoreDecimals not set to true?", expected[0], reader.arrayToStringArray( values, 3)[0]);
	}
}

