// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.fame;

import com.fame.timeiq.*;
import com.fame.timeiq.data.DoubleList;
import com.fame.timeiq.data.StringList;
import com.fame.timeiq.dates.*;

import Int.ecb.logging.Level;

import Int.ecb.gesmescb.*;

import Int.ecb.gesmescb.util.ComponentHelper;

/* **************************** Class ***************************** */

/**
 * Writes deletions to FAME databases.
 *
 * @author
 * <A HREF="mailto:klo@FAME.com">Kam Lo</A>
 * @author
 * <A HREF="mailto:Sami.Airo@ECB.int">Sami Airo</A>
 *
 * @version @(#)$Id: SDPEDeleter.java,v 1.1 2017/01/19 16:14:03 trainot Exp $
 */

public class SDPEDeleter extends FAMEDeleter {

	/* ======================= Constants ======================== */

	/* ========================= Fields ========================= */

	/* ====================== Constructors ====================== */

	public SDPEDeleter(
		final ComponentHelper componentHelper,
		final FAMEDb fameDb,
		final MessageBegin messageBegin
	) {
		super(componentHelper, fameDb, messageBegin);
	}

	/* ===================== Protected Methods ===================== */

	protected boolean deleteObservations (TimeSeries timeSeries) {
		getComponentHelper().logger.log(
			Level.FINER,
			getComponentHelper().logName,
			"deleteObservations",
			"Deleting observations from " + timeSeries
		);

		IndexValueRange range = getFameRange(timeSeries.getInterval());

		if (range == null) {
			return false;
		}

		int obsCount = range.size();

		double[] doubleValues = new double[obsCount];
		String[] stringValues = new String[obsCount];
		byte[] statusValues = new byte[obsCount];

		for (int i = 0; i < obsCount; i++) {
			doubleValues[i] = 0;
			stringValues[i] = "";
			statusValues[i] = TiqConstants.STATUS_NC;
		}

		RegularSeries doubleSeries = new RegularSeries(range, new DoubleList(doubleValues), statusValues);
		RegularSeries stringSeries = new RegularSeries(range, new StringList(stringValues), statusValues);

		String fameName = getFameName(timeSeries);
		return (		getFAMEDb().createUpdateTimeSeries(fameName, doubleSeries)
					&&	getFAMEDb().createUpdateTimeSeries(fameName + KEY_SEPARATOR + "OBS_STATUS", stringSeries)
					&&	getFAMEDb().createUpdateTimeSeries(fameName + KEY_SEPARATOR + "OBS_CONF", stringSeries)
					&&	getFAMEDb().createUpdateTimeSeries(fameName + KEY_SEPARATOR + "BREAK", doubleSeries) );
	}

	protected boolean deleteAttributeObservation (Attribute attribute, KeyFamilyComponent component) {
		getComponentHelper().logger.log(
			Level.FINER,
			getComponentHelper().logName,
			"deleteAttributeObservation",
			"Deleting observation level attribute value from " + attribute
		);

		IndexValueRange range = getFameRange(attribute.getInterval());

		if (range == null) {
			return false;
		}

		// Size of range
		int rangeSize = range.size();

		RegularSeries regSeries;

// All attributes are currently stored as strings...

			// Create lists of NCs for range
/*		if (component.isNumeric()) {
			double[] doubleValues = new double[rangeSize];
			byte[] statusValues = new byte[rangeSize];

			for (int i = 0; i < rangeSize; i++) {
				doubleValues[i] = 0;
				statusValues[i] = TiqConstants.STATUS_NC;
			}

			regSeries = new RegularSeries(range, new DoubleList(doubleValues), statusValues);
		} else {
*/			String[] stringValues = new String[rangeSize];
			byte[] statusValues = new byte[rangeSize];

			for (int i = 0; i < rangeSize; i++) {
				stringValues[i] = "";
				statusValues[i] = TiqConstants.STATUS_NC;
			}

			regSeries = new RegularSeries(range, new StringList(stringValues), statusValues);
//		}

		return getFAMEDb().createUpdateTimeSeries(getFameName(attribute, component), regSeries);
	}

	protected boolean deleteAttribute (Attribute attribute, KeyFamilyComponent component) {
		getComponentHelper().logger.log(
			Level.FINER,
			getComponentHelper().logName,
			"deleteAttribute",
			"Deleting attribute " + attribute
		);

		Scalar deleteObj;

// All attributes are currently stored as strings...

/*		if (component.isNumeric()) {
			deleteObj = new Scalar(new Double(0), TiqConstants.STATUS_NC);
		} else {
*/			deleteObj = new Scalar(TiqType.STRING, TiqConstants.STATUS_NC);
//		}

		return getFAMEDb().overwriteObject( getFameName(attribute, component), deleteObj );
	}
}
