// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.fame;

import com.fame.timeiq.*;
import com.fame.timeiq.data.ObservationList;
import com.fame.timeiq.dates.*;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.File;
import java.util.Properties;

import Int.ecb.gesmescb.*;
import Int.ecb.gesmescb.event.*;
import Int.ecb.gesmescb.metabase.Metabase;
import Int.ecb.gesmescb.util.ComponentHelper;
import Int.ecb.gesmescb.fame.FAMEDb;
import Int.ecb.gesmescb.fame.SDPEDeleter;

/* ***************************** Class ****************************** */

/**
 * <P>
 * Tests the implementation of SDPEDeleter class in the Int.ecb.gesmescb.fame
 * package.
 * </P>
 * 
 * @author <A HREF="mailto:Sami.Airo@ECB.int">Sami Airo</A>
 * 
 * @version @(#)$Id: SDPEDeleterTest.java,v 1.8 2010/09/21 13:01:19 trainot Exp
 *          $
 */

public class SDPEDeleterFAME10Test extends TestCase {

    /* ========================= Fields ========================= */

    FAMEDb fameDb;
    File file;
    SDPEDeleter deleter;
    String fileName = "/tmp/SDPEDeleterTest.db";
    private static String formatDbVersion = "4";

    /* ====================== Constructors ====================== */

    /**
     * Constructs a SDPEDeleterTest with the specified name.
     * 
     * @param name
     *            Test case name
     */
    public SDPEDeleterFAME10Test(final String name) {
        super(name);
    }

    /* ================== Protected Methods ===================== */

    /**
     * Sets up the test fixture.
     */
    protected void setUp() {
            System.setProperty("Int.ecb.logging.config.class",
                    "Int.ecb.gesmescb.util.LoggingPropertiesReader");
            System.setProperty(".level", "ALL");
            System.setProperty("handlers", "Int.ecb.logging.ConsoleHandler");
            System.setProperty("Int.ecb.logging.ConsoleHandler.level", "INFO");

            TestEventSource source = new TestEventSource();
            source.addListener(Metabase.getMetabase(), true);

            source.dispatch(new Object[]{
                    new InterchangeBegin("IREF0001", "4F0", "ZZZ", "", false),
                    new MessageBegin("MREF0001", "4F0", "ZZZ", "ECB",
                            MessageBegin.STATISTICAL_DEFINITIONS, "",
                            new Contact[]{}),
                    new CodeList("CL_FREQ", "", "", new String[]{"M", "Q"},
                            new String[]{"", ""}), new MessageEnd("MREF0001"),
                    new InterchangeEnd("IREF0001")});

            fameDb = new FAMEDb("");

            Properties properties = new Properties();
            properties.put("mode", "CREATE");
            properties.put("dbVersion", formatDbVersion);
            fameDb.open(fileName, properties);

            MessageBegin messageBegin = new MessageBegin("MREF00001", "DE2",
                    "4F0", "ECB", MessageBegin.STATISTICAL_DATA, "test",
                    new Contact[]{}, "BOP", MessageBegin.DELETE_ACTION, "", "",
                    "", "ECB_BOP1");

            deleter = new SDPEDeleter(new ComponentHelper("", this), fameDb,
                    messageBegin);
    }

    /**
     * Tears down the test fixture.
     */
    protected void tearDown() {
        fameDb.close(false);
        try {
            File file = new File(fileName);
            file.delete();
        } catch (Exception e) {
            fail(e.getMessage());
        }
    }

    /* ===================== Public Methods ===================== */

    /**
     * Assembles and returns a test suite for all the test methods of this test
     * case.
     * 
     * @return a non-null test suite
     */
    public static Test suite() {
        TestSuite suite = new TestSuite(SDPEDeleterFAME10Test.class);
        return suite;
    }

    /**
     * Runs the test case
     */
    public static void main(String args[]) {
        String[] testCaseName = {SDPEDeleterFAME10Test.class.getName()};
        junit.textui.TestRunner.main(testCaseName);
    }

    /* --------------------- Test Methods --------------------- */

    public void testDeleteObservations() {
        TimeSeries timeSeries = new TimeSeries("M.TEST1", "200012200102",
                "710", new Int.ecb.gesmescb.Observation[]{
                        Int.ecb.gesmescb.Observation.EMPTY_OBSERVATION,
                        Int.ecb.gesmescb.Observation.EMPTY_OBSERVATION,
                        Int.ecb.gesmescb.Observation.EMPTY_OBSERVATION});

        assertTrue(deleter.deleteObservations(timeSeries));

        Series series = (Series) fameDb.getObject("BOP.M.TEST1");

        assertNotNull(series);
        assertEquals(TiqType.DOUBLE, series.getTiqType());
        assertEquals(TiqFrequency.MONTHLY, series.getFrequency());
        assertEquals(2000, DateHelper.indexToYear(series.getMinIndex()));
        assertEquals(12, DateHelper.indexToMonth(series.getMinIndex()));
        assertEquals(2001, DateHelper.indexToYear(series.getMaxIndex()));
        assertEquals(2, DateHelper.indexToMonth(series.getMaxIndex()));

        ObservationList observationList = series.getObservations();
        long[] indexesAsDates = observationList.getIndexes();

        assertEquals(TiqConstants.STATUS_NC, observationList.getObservation(
                indexesAsDates[0]).getStatus());
        assertEquals(TiqConstants.STATUS_NC, observationList.getObservation(
                indexesAsDates[1]).getStatus());
        assertEquals(TiqConstants.STATUS_NC, observationList.getObservation(
                indexesAsDates[2]).getStatus());

        timeSeries = new TimeSeries("Q.TEST1", "19801", "608",
                new Int.ecb.gesmescb.Observation[]{});

        assertTrue(deleter.deleteObservations(timeSeries));

        series = (Series) fameDb.getObject("BOP.Q.TEST1");

        assertNotNull(series);
        assertEquals(TiqType.DOUBLE, series.getTiqType());
        assertEquals(TiqFrequency.QUARTERLY, series.getFrequency());
        assertEquals(1980, DateHelper.indexToYear(series.getMinIndex()));
        assertEquals(1, DateHelper.indexToMonth(series.getMinIndex()));
        assertEquals(1980, DateHelper.indexToYear(series.getMaxIndex()));
        assertEquals(1, DateHelper.indexToMonth(series.getMaxIndex()));

        observationList = series.getObservations();
        indexesAsDates = observationList.getIndexes();

        assertEquals(TiqConstants.STATUS_NC, observationList.getObservation(
                indexesAsDates[0]).getStatus());
    }

    public void testDeleteAttributeObservation() {
        Attribute attribute = new Attribute("M.TEST1", "200108", "610", "TEST",
                "");

        KeyFamilyComponent component = new KeyFamilyComponent("TEST",
                KeyFamilyComponent.ATTRIBUTE, "", "", "",
                KeyFamilyComponent.ATTACHED_TO_OBSERVATION, "");

        assertTrue(deleter.deleteAttributeObservation(attribute, component));

        Series series = (Series) fameDb.getObject("BOP.M.TEST1.TEST");

        assertNotNull(series);
        assertEquals(TiqType.STRING, series.getTiqType());
        assertEquals(TiqFrequency.MONTHLY, series.getFrequency());
        assertEquals(2001, DateHelper.indexToYear(series.getMinIndex()));
        assertEquals(8, DateHelper.indexToMonth(series.getMinIndex()));
        assertEquals(2001, DateHelper.indexToYear(series.getMaxIndex()));
        assertEquals(8, DateHelper.indexToMonth(series.getMaxIndex()));

        ObservationList observationList = series.getObservations();
        long[] indexesAsDates = observationList.getIndexes();

        assertEquals(TiqConstants.STATUS_NC, observationList.getObservation(
                indexesAsDates[0]).getStatus());
    }

    public void testdeleteAttribute() {
        Attribute attribute = new Attribute("M.TEST1", "", "", "TEST", "");
        KeyFamilyComponent component = new KeyFamilyComponent("TEST",
                KeyFamilyComponent.ATTRIBUTE, "", "", "",
                KeyFamilyComponent.ATTACHED_TO_TIME_SERIES, "");

        assertTrue(deleter.deleteAttribute(attribute, component));

        Scalar scalar = (Scalar) fameDb.getObject("BOP.M.TEST1.TEST");

        assertNotNull(scalar);
        assertEquals(TiqType.STRING, scalar.getTiqType());
        assertEquals(TiqConstants.STATUS_NC, scalar.getTiqValue().getStatus());
    }
}
