// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.fame;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.File;

import Int.ecb.gesmescb.*;
import Int.ecb.gesmescb.event.*;
import Int.ecb.gesmescb.metabase.Metabase;

import com.fame.timeiq.*;

/* ***************************** Class ****************************** */

/**
 * <P>Tests the implementation of SDPEWriter class in the
 * Int.ecb.gesmescb.fame package.</P>
 *
 * @author
 * <A HREF="mailto:Sami.Airo@ECB.int">Sami Airo</A>
 *
 * @version @(#)$Id: SDPEWriterTest.java,v 1.1 2017/01/19 16:14:03 trainot Exp $
 */

public class SDPEWriterTest extends TestCase {

	/* ========================= Fields ========================= */

	String dbName = "/tmp/2002041614131";
	String fullDBName = dbName + "_4F0_ZZZ_IREF0001_MREF00001_BOP.db";


	/* ====================== Constructors ====================== */

	/**
	 * Constructs a SDPEWriterTest with the specified name.
	 *
	 * @param name Test case name
	 */
	public SDPEWriterTest( final String name ) {
		super( name );
	}

	/* ================== Protected Methods ===================== */

	/**
	 * Sets up the test fixture.
	 */
	protected void setUp() {
		System.setProperty("Int.ecb.logging.config.class","Int.ecb.gesmescb.util.LoggingPropertiesReader");
		System.setProperty(".level","ALL");
		System.setProperty("handlers", "Int.ecb.logging.ConsoleHandler");
		System.setProperty("Int.ecb.logging.ConsoleHandler.level","INFO");
		System.setProperty("Int.ecb.gesmescb.fame.SDPEWriter.destination", dbName);

		TestEventSource source = new TestEventSource();
		source.addListener( Metabase.getMetabase(), true );

		source.dispatch( new Object[]{
			new InterchangeBegin(
				"IREF0001", "4F0", "ZZZ", "", false
			),
			new MessageBegin(
				"MREF0001", "4F0", "ZZZ", "ECB", MessageBegin.STATISTICAL_DEFINITIONS, "", new Contact[]{}
			),
			new CodeList(
				"CL_FREQ", "", "", new String[]{"M", "Q"}, new String[]{"",""}
			),
			new MessageEnd(
				"MREF0001"
			),
			new InterchangeEnd(
				"IREF0001"
			)
		} );
	}

	/**
	 * Tears down the test fixture.
	 */
	protected void tearDown() {
		try {
			File file = new File(fullDBName);
			file.delete();
		} catch (Exception e) {
			fail( e.getMessage() );
		}
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite
	 */
	public static Test suite() {
		TestSuite suite = new TestSuite( SDPEWriterTest.class );
		return suite;
	}

	/**
	 * Runs the test case
	 */
	public static void main(String args[]) {
		String[] testCaseName = {
			SDPEWriterTest.class.getName()
		};
		junit.textui.TestRunner.main(testCaseName);
	}

	/* --------------------- Test Methods --------------------- */

	public void testHandleInterchange() {
		SDPEWriter writer = new SDPEWriter();
		FAMEDb fameDB = writer.getFAMEDb();

		assertTrue( ! fameDB.isOpen() );

		final InterchangeBegin testInterchange = 	new InterchangeBegin(
				"IREF0001", "4F0", "ZZZ", "", false
		);

		writer.handle(testInterchange);

		final MessageBegin testMessage = new MessageBegin(
			"MREF00001",
			"DE2",
			"4F0",
			"ECB",
			MessageBegin.STATISTICAL_DATA,
			"test",
			new Contact[]{},
			"BOP",
			MessageBegin.DELETE_ACTION,
			"", "", "",
			"ECB_BOP1"
		);

		writer.handle(testMessage);

		assertTrue( fameDB.isOpen() );
		assertEquals( fullDBName, fameDB.toString());

		assertNotNull( 
			"IREF exists", 
			fameDB.getObject(testMessage.getDataSetId() + ".IREF") 
		);
		assertEquals(
			"IREF value", 
			testInterchange.getIref(), 
			((Scalar)fameDB.getObject(testMessage.getDataSetId() + ".IREF")).getTiqValue().getStringValue() 
		);

		assertNotNull( 
			"IREF exists 2", 
			fameDB.getObject("ENVELOPE.IREF") 
		);
		assertEquals( 
			"IREF value 2", 
			testInterchange.getIref(), 
			((Scalar)fameDB.getObject("ENVELOPE.IREF")).getTiqValue().getStringValue() 
		);

		assertNotNull( 
			"MREF exists", 
			fameDB.getObject("ENVELOPE.MREF") 
		);
		assertEquals( 
			"MREF value", 
			testMessage.getMref(), 
			((Scalar)fameDB.getObject("ENVELOPE.MREF")).getTiqValue().getStringValue() 
		);

		assertNotNull( 
			"SENDER exists", 
			fameDB.getObject("ENVELOPE.SENDER") 
		);
		assertEquals(
			"SENDER value", 
			testMessage.getSender(), 
			((Scalar)fameDB.getObject("ENVELOPE.SENDER")).getTiqValue().getStringValue() 
		);

		assertNotNull( 
			"RECEIVER exists", 
			fameDB.getObject("ENVELOPE.RECEIVER") 
		);
		assertEquals(
			"RECEIVER value", 
			testMessage.getReceiver(), 
			((Scalar)fameDB.getObject("ENVELOPE.RECEIVER")).getTiqValue().getStringValue() 
		);

		assertNotNull( 
			"AGENCY exists", 
			fameDB.getObject("ENVELOPE.AGENCY") 
		);
		assertEquals( 
			"AGENCY value", 
			testMessage.getMaintenanceAgencyId(), 
			((Scalar)fameDB.getObject("ENVELOPE.AGENCY")).getTiqValue().getStringValue() 
		);

		assertNotNull( 
			"KEY_FAMILY exists", 
			fameDB.getObject("ENVELOPE.KEY_FAMILY") 
		);
		assertEquals( 
			"KEY_FAMILY value", 
			testMessage.getKeyFamilyId(), 
			((Scalar)fameDB.getObject("ENVELOPE.KEY_FAMILY")).getTiqValue().getStringValue() 
		);

		assertNotNull( 
			"DATASET exists", 
			fameDB.getObject("ENVELOPE.DATASET") 
		);
		assertEquals( 
			"DATASET value", 
			testMessage.getDataSetId(), 
			((Scalar)fameDB.getObject("ENVELOPE.DATASET")).getTiqValue().getStringValue() 
		);

		assertNotNull( 
			"Action exists", 
			fameDB.getObject("ENVELOPE.ACTION") 
		);
		assertEquals( 
			"Action value", 
			testMessage.getAction(), 
			((Scalar)fameDB.getObject("ENVELOPE.ACTION")).getTiqValue().getStringValue() 
		);

		assertNotNull( 
			"EXTRACTION_TIME exists", 
			fameDB.getObject("ENVELOPE.EXTRACTION_TIME") 
		);
		assertEquals( 
			"EXTRACTION_TIME value", 
			testMessage.getExtractionDateTime(), 
			((Scalar)fameDB.getObject("ENVELOPE.EXTRACTION_TIME")).getTiqValue().getStringValue() 
		);

		assertNotNull( 
			"REPORTING_PERIOD exists", 
			fameDB.getObject("ENVELOPE.REPORTING_PERIOD") 
		);
		assertEquals( 
			"REPORTING_PERIOD value", 
			testMessage.getReportingPeriod(), 
			((Scalar)fameDB.getObject("ENVELOPE.REPORTING_PERIOD")).getTiqValue().getStringValue() 
		);

		assertNotNull( 
			"IDENTITY exists", 
			fameDB.getObject("ENVELOPE.IDENTITY") 
		);
		assertEquals( 
			"IDENTITY value", 
			testMessage.getIdentity(), 
			((Scalar)fameDB.getObject("ENVELOPE.IDENTITY")).getTiqValue().getStringValue() 
		);

		writer.handle(
			new MessageEnd(
				"MREF0001"
			)
		);

		assertTrue( ! fameDB.isOpen() );

		writer.handle(
			new InterchangeEnd(
				"IREF0001"
			)
		);

	}

}
