// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.fame;

import Int.ecb.logging.Level;

import Int.ecb.gesmescb.*;

import java.io.File;

/* **************************** Class ***************************** */

/**
 * Writes data to FAME databases using ESDBUpdater and ESDBDeleter. Open
 * one database per interchange.
 *
 * @author
 * <A HREF="mailto:Sami.Airo@ECB.int">Sami Airo</A>
 *
 * @version @(#)$Id: SimpleESDBWriter.java,v 1.1 2017/01/19 16:14:03 trainot Exp $
 */

public class SimpleESDBWriter extends FAMEWriter {

	/* ======================= Constants ======================== */

	/* ========================= Fields ========================= */

	private FAMERevisor revisor;

	/* ====================== Constructors ====================== */

	public SimpleESDBWriter () {
		this( "" );
	}

	public SimpleESDBWriter(final String name) {
		super( name );
	}

	/* ===================== Protected Methods ===================== */

	public void handle ( InterchangeBegin interchangeBegin ) {
		String databaseName = getComponentHelper().getPreference("destination", "");

		if (databaseName.equals("")) {
			getComponentHelper().logger.log(
				Level.SEVERE,
				getComponentHelper().logName,
				"handle",
				"Cannot read FAME database name from preference \"destination\"."
			);

			errorDisconnect();
			return;
		}

		getComponentHelper().logger.log(
			Level.CONFIG,
			getComponentHelper().logName,
			"handle",
			"destination: " + databaseName
		);

		openDatabase( new File(databaseName) );
		super.handle( interchangeBegin );
	}

	public void handle ( InterchangeEnd interchangeEnd ) {
		super.handle( interchangeEnd );
		closeDatabase(true);
	}

	public void handle ( MessageBegin messageBegin ) {
		revisor = null;

		if (messageBegin.isUpdateOrReplaceAction()) {
			revisor = new ESDBUpdater( getComponentHelper(), getFAMEDb(), messageBegin );
		}

		if (messageBegin.isDeleteAction()) {
			revisor = new ESDBDeleter( getComponentHelper(), getFAMEDb(), messageBegin );
		}

		super.handle( messageBegin );
	}

	protected FAMERevisor getRevisor () {
		return revisor;
	}

}
