// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.fame;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.File;

import Int.ecb.gesmescb.*;
import Int.ecb.gesmescb.event.*;
import Int.ecb.gesmescb.metabase.Metabase;

/* ***************************** Class ****************************** */

/**
 * <P>Tests the implementation of SimpleESDBWriter class in the
 * Int.ecb.gesmescb.fame package.</P>
 *
 * @author
 * <A HREF="mailto:Sami.Airo@ECB.int">Sami Airo</A>
 *
 * @version @(#)$Id: SimpleESDBWriterTest.java,v 1.1 2017/01/19 16:14:03 trainot Exp $
 */

public class SimpleESDBWriterTest extends TestCase {

	/* ========================= Fields ========================= */

	String dbName = "/tmp/SimpleESDBWriterTest.db";

  
	/* ====================== Constructors ====================== */

	/**
	 * Constructs a SimpleESDBWriterTest with the specified name.
	 *
	 * @param name Test case name
	 */
	public SimpleESDBWriterTest( final String name ) {
		super( name );
	}

	/* ================== Protected Methods ===================== */

	/**
	 * Sets up the test fixture.
	 */
	protected void setUp() {
		System.setProperty("Int.ecb.logging.config.class","Int.ecb.gesmescb.util.LoggingPropertiesReader");
		System.setProperty(".level","ALL");
		System.setProperty("handlers", "Int.ecb.logging.ConsoleHandler");
		System.setProperty("Int.ecb.logging.ConsoleHandler.level","INFO");
		System.setProperty("Int.ecb.gesmescb.fame.SimpleESDBWriter.destination", dbName);

		TestEventSource source = new TestEventSource();
		source.addListener( Metabase.getMetabase(), true );

		source.dispatch( new Object[]{
			new InterchangeBegin(
				"IREF0001", "4F0", "ZZZ", "", false
			),
			new MessageBegin(
				"MREF0001", "4F0", "ZZZ", "ECB", MessageBegin.STATISTICAL_DEFINITIONS, "", new Contact[]{}
			),
			new CodeList(
				"CL_FREQ", "", "", new String[]{"M", "Q"}, new String[]{"",""}
			),
			new MessageEnd(
				"MREF0001"
			),
			new InterchangeEnd(
				"IREF0001"
			)
		} );
	}

	/**
	 * Tears down the test fixture.
	 */
	protected void tearDown() {
		try {
			File file = new File(dbName);
			file.delete();
		} catch (Exception e) {
			fail( e.getMessage() );
		}
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite
	 */
	public static Test suite() {
		TestSuite suite = new TestSuite( SimpleESDBWriterTest.class );
		return suite;
	}

	/**
	 * Runs the test case
	 */
	public static void main(String args[]) {
		String[] testCaseName = {
			SimpleESDBWriterTest.class.getName()
		};
		junit.textui.TestRunner.main(testCaseName);
	}

	/* --------------------- Test Methods --------------------- */

	public void testHandleInterchange() {
		SimpleESDBWriter writer = new SimpleESDBWriter();
		FAMEDb fameDB = writer.getFAMEDb();

		assertTrue( ! fameDB.isOpen() );

		writer.handle(
			new InterchangeBegin(
				"IREF0001", "4F0", "ZZZ", "", false
			)
		);

		assertTrue( fameDB.isOpen() );
		assertEquals( dbName, fameDB.toString());

		writer.handle(
			new MessageBegin(
				"MREF00001",
				"DE2",
				"4F0",
				"ECB",
				MessageBegin.STATISTICAL_DATA,
				"test",
				new Contact[]{},
				"BOP",
				MessageBegin.DELETE_ACTION,
				"", "", "",
				"ECB_BOP1"
			 )
		);

		writer.handle(
			new MessageEnd(
				"MREF0001"
			)
		);

		assertTrue( fameDB.isOpen() );

		writer.handle(
			new InterchangeEnd(
				"IREF0001"
			)
		);

		assertTrue( ! fameDB.isOpen() );
	}

}
