// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.fame;

import java.util.Date;

import com.fame.timeiq.*;
import com.fame.timeiq.data.*;
import com.fame.timeiq.dates.*;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import Int.ecb.gesmescb.*;
import Int.ecb.gesmescb.time.*;

/* ***************************** Class ****************************** */

/**
 * <P>Tests the implementation of SimpleFAMEReader class in the
 * Int.ecb.gesmescb.fame package.</P>
 *
 * @author
 * <A HREF="mailto:Sami.Airo@ECB.int">Sami Airo</A>
 *
 * @version @(#)$Id: SimpleFAMEReaderTest.java,v 1.1 2017/01/19 16:14:03 trainot Exp $
 */

public class SimpleFAMEReaderTest extends TestCase {

	/* ========================= Fields ========================= */

	/* ====================== Constructors ====================== */

	/**
	 * Constructs a SimpleFAMEReaderTest with the specified name.
	 *
	 * @param name Test case name
	 */
	public SimpleFAMEReaderTest( final String name ) {
		super( name );
	}

	/* ================== Protected Methods ===================== */

	/**
	 * Sets up the test fixture.
	 */
	protected void setUp() {
	}

	/**
	 * Tears down the test fixture.
	 */
	protected void tearDown() {
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite
	 */
	public static Test suite() {
		TestSuite suite = new TestSuite( SimpleFAMEReaderTest.class );
		return suite;
	}

	/**
	 * Runs the test case
	 */
	public static void main(String args[]) {
		String[] testCaseName = {
			SimpleFAMEReaderTest.class.getName()
		};
		junit.textui.TestRunner.main(testCaseName);
	}

	/* --------------------- Test Methods --------------------- */

	public void testConvertObservationListToTimeSeries () throws Exception {
		SimpleFAMEReader reader = new SimpleFAMEReader();
		RegularInterval interval = new GregorianRegularInterval(RegularInterval.MONTHLY, 2003, 1, 3);
		TiqObject tiqSeries = 
			new RegularSeries(
				IntervalConverter.regularIntervalToIndexValueRange(interval),			
				new double[] { 123.4, 234.5, -345.6 },
				new byte[] { TiqConstants.STATUS_OK, TiqConstants.STATUS_OK, TiqConstants.STATUS_OK }
			);
		TimeSeries series = 
			new TimeSeries(
				"ABC", 
				interval, 
				new Int.ecb.gesmescb.Observation[] { 
					new Int.ecb.gesmescb.Observation("123.4", "A", "", ""),
					new Int.ecb.gesmescb.Observation("234.5", "A", "", ""),
					new Int.ecb.gesmescb.Observation("-345.6", "A", "", "")
				}
			);
		assertEquals("Series 1", series, reader.convertObservationListToTimeSeries("ABC", tiqSeries.getObservations()));
	}


	public void testConvertEmptyObservationListToTimeSeries () throws Exception {
		SimpleFAMEReader reader = new SimpleFAMEReader();
		TimeSeries series = 
			new TimeSeries(
				"ABC", 
				new GregorianRegularInterval(
					RegularInterval.MONTHLY, 
					new Date(0),
					0
				), 
				new Int.ecb.gesmescb.Observation[] {}
			);
		assertEquals(
			"Series 1", 
			series, reader.convertObservationListToTimeSeries(
				"ABC", 
				new ObservationList(TiqType.DOUBLE, TiqFrequency.MONTHLY)
			)
		);
	}

	
	public void testConvertTiqValueToString () throws Exception {
		SimpleFAMEReader reader = new SimpleFAMEReader();
		
		assertEquals( "long 123", "123", reader.convertTiqValueToString( new TiqValue( new Long(123), TiqConstants.STATUS_OK	) ) );
		assertEquals( "int 123", "123", reader.convertTiqValueToString( new TiqValue( new Integer(123), TiqConstants.STATUS_OK ) ) );
		assertEquals( "double 123.4", "123.4", reader.convertTiqValueToString( new TiqValue( new Double(123.4), TiqConstants.STATUS_OK ) ) );
		assertEquals( "float 123.4", "123.4", reader.convertTiqValueToString( new TiqValue(	new Float(123.4),	TiqConstants.STATUS_OK ) ) );
		assertEquals( "double -123.4", "-123.4", reader.convertTiqValueToString( new TiqValue(	new Double(-123.4), TiqConstants.STATUS_OK ) ) );
		assertEquals( "double NC", "-", reader.convertTiqValueToString( new TiqValue(	new Double(123.4), TiqConstants.STATUS_NC ) ) );
		assertEquals( "double ND", "", reader.convertTiqValueToString( new TiqValue(	new Double(123.4), TiqConstants.STATUS_ND ) ) );
		assertEquals( "double NA", "", reader.convertTiqValueToString( new TiqValue(	new Double(123.4), TiqConstants.STATUS_NA ) ) );
		assertEquals( "double NIC", "", reader.convertTiqValueToString( new TiqValue(	new Double(123.4), TiqConstants.STATUS_NIC ) ) );
		assertEquals( "double 123456789.123456789", "1.2345678912345679E8", reader.convertTiqValueToString( new TiqValue( new Double(123456789.123456789), TiqConstants.STATUS_OK ) ) );
		assertEquals( "double 123456789123456789.1234", "1.23456789123456784E17", reader.convertTiqValueToString( new TiqValue( new Double(123456789123456789.1234), TiqConstants.STATUS_OK ) ) );
		assertEquals( "double 1.23456789123456789", "1.234567891234568", reader.convertTiqValueToString( new TiqValue( new Double(1.23456789123456789), TiqConstants.STATUS_OK ) ) );
		assertEquals( "long 123456789123456789", "123456789123456789", reader.convertTiqValueToString( new TiqValue( new Long(123456789123456789L), TiqConstants.STATUS_OK ) ) );
	}


	public void testConvertObsValueToObsStatus () throws Exception {
		SimpleFAMEReader reader = new SimpleFAMEReader();

		assertEquals( "Normal value", "A", reader.convertObsValueToObsStatus("123.4") );
		assertEquals( "Missing value", "M", reader.convertObsValueToObsStatus("-") );
		assertEquals( "Empty value", "", reader.convertObsValueToObsStatus("") );
	}

}
