// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.fame;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;

import java.util.Vector;

/* ***************************** Class ****************************** */

/**
 * <P>Class to hold a list of time series keys. It has a method to
 * read the keys from file. The file must be in ASCII format,
 * containing one time series key per line.</P>
 *
 * @author
 * <A HREF="mailto:klo@fame.com">Kam Lo</A>
 *
 * @version @(#)$Id: TimeSeriesKeyList.java,v 1.1 2017/01/19 16:14:03 trainot Exp $
 */

public class TimeSeriesKeyList extends Vector {

	/**
	 *   serialVersionUID generated by eclipse.
	 */
	private static final long serialVersionUID = 9089455091194029620L;

	/**
	 * Construct a new list of time series keys.
	 */
	public TimeSeriesKeyList () {
	}

	/**
	 * Reads list of time series keys from file.
	 * Format of file is a time series key on each line.
	 *
	 * @param filename name of file containing list of time series keys.
	 *
	 * @throws IOException if an I/O error occurs.
	 */
	public void readFromFile ( String filename )
	    throws IOException
	{

        // Clear list
        removeAllElements();
        
		// Open file
		FileInputStream fileStream = new FileInputStream( filename );
		BufferedReader  fileReader = new BufferedReader(
		                                    new InputStreamReader( fileStream ) );

        // Read in time series keys if file exists
		if ( fileReader != null ) {

			String line;

			// Read each line
			do {

				line = fileReader.readLine();

				// Add to list if not end of file nor blank line
				if ( line != null && line.trim().length() > 0 )
					add( line );

			} while ( line != null );

		}

		// Close file
		fileReader.close();
		fileStream.close();

	}

}
