// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.metabase;

import java.util.Date;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import Int.ecb.gesmescb.*;

import Int.ecb.gesmescb.metabase.Metabase;
import Int.ecb.gesmescb.metabase.MaintenanceAgency;

/* ***************************** Class ****************************** */

/**
 * <P>Tests the implementation of the Int.ecb.gesmesdb.Metabase class.</P>
 *
 * @author
 * <A HREF="mailto:Sami.Airo@ECB.int">Sami Airo</A>
 * @author
 * <A HREF="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</A>
 *
 * @version @(#)$Id: MetabaseTest.java,v 1.1 2017/01/19 16:14:05 trainot Exp $
 */

public final class MetabaseTest extends TestCase {

	/* ========================= Fields ========================= */

	private Metabase metabase;
	private InterchangeBegin interchangeBegin;
	private InterchangeEnd interchangeEnd;
	private MessageBegin messageBeginECB;
	private MessageBegin messageBeginBIS;
	private MessageEnd messageEndECB;
	private MessageEnd messageEndBIS;
	private CodeList codeList;
	private KeyFamily keyFamily;
	private StatisticalConcept statisticalConcept;

	/* ====================== Constructors ====================== */

	/**
	 * Constructs a MetabaseTest with the specified name.
	 *
	 * @param name Test case name.
	 */
	public MetabaseTest ( final String name ) {
		super( name );
	}

	/* ================== Protected Methods ===================== */

	/**
	 * Sets up the test fixture.
	 */
	protected void setUp () {
		metabase = Metabase.getMetabase();
		interchangeBegin = new InterchangeBegin(
			"IREF0001", "DE2", "4F0", "000315124533", false
		);
		interchangeEnd = new InterchangeEnd(
			"IREF0001"
		);
		messageBeginECB = new MessageBegin(
			"MREF0001", "4F0", "4F0", "ECB",
			MessageBegin.STATISTICAL_DEFINITIONS,
			"subject: definitions", new Contact[] {}
		);
		messageEndECB = new MessageEnd(
			"MREF0001"
		);
		messageBeginBIS = new MessageBegin(
			"MREF0002", "4F0", "4F0", "BIS",
			MessageBegin.STATISTICAL_DEFINITIONS,
			"subject: definitions", new Contact[] {}
		);
		messageEndBIS = new MessageEnd(
			"MREF0002"
		);
		codeList = new CodeList(
			"CL_TEST", "name", "desc", new Date(),
			new String[]{"A"}, new String[]{"A"}
		);
		statisticalConcept = new StatisticalConcept(
			"SC_TEST", "Concept 1", "1abc", new Date()
		);
		keyFamily = new KeyFamily(
			"KF_TEST", "name", "desc", new Date(), new KeyFamilyComponent[0]
		);
	}

	/**
	 * Tears down the test fixture.
	 */
	protected void tearDown () {
		metabase.clear();
		interchangeBegin = null;
		interchangeEnd = null;
		messageBeginECB = null;
		messageEndECB = null;
		messageBeginBIS = null;
		messageEndBIS = null;
		codeList = null;
		statisticalConcept = null;
		keyFamily = null;
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite.
	 */
	public static Test suite () {
		return new TestSuite( MetabaseTest.class );
	}

	/**
	 * Runs the test case.
	 */
	public static void main ( final String args[] ) {
		final String[] testCaseName = { MetabaseTest.class.getName() };
		junit.textui.TestRunner.main( testCaseName );
	}

	/* --------------------- Test Methods --------------------- */

	/**
	 * Tests the handling of a normal interchange with two messages.
	 */
	public void testNormalInterchange () {
		MaintenanceAgency agency;

		metabase.clear();
		assertNull( metabase.getMaintenanceAgency( "ECB" ) );

		metabase.handle( interchangeBegin );
		metabase.handle( messageBeginECB );
		metabase.handle( codeList );
		metabase.handle( statisticalConcept );
		metabase.handle( keyFamily );
		metabase.handle( messageEndECB );
		metabase.handle( messageBeginBIS );
		metabase.handle( codeList );
		metabase.handle( statisticalConcept );
		metabase.handle( keyFamily );
		metabase.handle( messageEndBIS );
		metabase.handle( interchangeEnd );

		assertEquals( 2, metabase.getMaintenanceAgencyCount() );

		agency = metabase.getMaintenanceAgency( "ECB" );
		assertNotNull( agency );
		assertEquals( "Name o.k.", "ECB", agency.getId() );

		assertNotNull( agency.getCodeList( "CL_TEST" ) );
		assertNull( agency.getCodeList( "CL_XXX" ) );
		assertNotNull( agency.getStatisticalConcept( "SC_TEST" ) );
		assertNull( agency.getStatisticalConcept( "SC_XXX" ) );
		assertNotNull( agency.getKeyFamily( "KF_TEST" ) );
		assertNull( agency.getKeyFamily( "KF_XXX" ) );

		agency = metabase.getMaintenanceAgency( "BIS" );
		assertNotNull( agency );

		assertNotNull( agency.getCodeList( "CL_TEST" ) );
		assertNull( agency.getCodeList( "CL_XXX" ) );
		assertNotNull( agency.getStatisticalConcept( "SC_TEST" ) );
		assertNull( agency.getStatisticalConcept( "SC_XXX" ) );
		assertNotNull( agency.getKeyFamily( "KF_TEST" ) );
		assertNull( agency.getKeyFamily( "KF_XXX" ) );
	}
}
