// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.semanticchecker;

import java.lang.Object;
import java.lang.String;

import junit.framework.Test;
import junit.framework.TestSuite;

import Int.ecb.gesmescb.*;

import Int.ecb.gesmescb.semanticchecker.ECBRestrictionsChecker;

/* ***************************** Class ****************************** */

/**
 * <P>Tests the implementation of the
 * class {@link ECBRestrictionsChecker}.</P>
 *
 * @author
 * <A HREF="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</A>
 *
 * @version @(#)$Id: ECBRestrictionsCheckerTest.java,v 1.1 2017/01/19 16:14:05 trainot Exp $
 */

public class ECBRestrictionsCheckerTest extends CheckerBaseTest {

	/* ====================== Constructors ====================== */

	/**
	 * Constructs a SemanticCheckerTest with the specified name.
	 *
	 * @param name Test case name
	 */
	public ECBRestrictionsCheckerTest ( final String name ) {
		super( name );
	}

	/* ================== Protected Methods ===================== */

	/**
	 * Factory for ECBRestrictionsCheckers.
	 * Should be overridden by derived classes.
	 *
	 * @see doTest(boolean,Object[])
	 */
	protected CheckerBase checkerFactory () {
		return new ECBRestrictionsChecker();
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite
	 */
	public static Test suite () {
		return new TestSuite( ECBRestrictionsCheckerTest.class );
	}

	/**
	 * Runs the test case
	 */
	public static void main ( String args[] ) {
		final String[] testCaseName
		= { ECBRestrictionsCheckerTest.class.getName() };
		junit.textui.TestRunner.main( testCaseName );
	}

	/* ---------------------- Test Methods ---------------------- */

	/* ------------------- Interchange Checks ------------------- */

	public void testUnknownInterchangeSenderReceiver () {
		doTest( false, new Object[] {
			new InterchangeBegin(
				INTERCHANGE_BEGIN.getIref(),
				"7G7",
				INTERCHANGE_BEGIN.getReceiver(),
				INTERCHANGE_BEGIN.getPreparationDateTime(),
				false
			), ERROR,
			INTERCHANGE_END,
			null
		} );
		doTest( false, new Object[] {
			new InterchangeBegin(
				INTERCHANGE_BEGIN.getIref(),
				INTERCHANGE_BEGIN.getSender(),
				"7G7",
				INTERCHANGE_BEGIN.getPreparationDateTime(),
				false
			), ERROR,
			INTERCHANGE_END,
			null
		} );
	}

	/* --------------------- Message Checks --------------------- */

	public void testUnknownMessageSenderReceiver () {
		doTest( false, new Object[] {
			INTERCHANGE_BEGIN,
			new MessageBegin(
				METADATA_MESSAGE_BEGIN.getMref(),
				"Aardvark",
				METADATA_MESSAGE_BEGIN.getReceiver(),
				METADATA_MESSAGE_BEGIN.getMaintenanceAgencyId(),
				MessageBegin.STATISTICAL_DEFINITIONS,
				METADATA_MESSAGE_BEGIN.getIdentity(),
				contacts
			), ERROR,
			new MessageEnd( "MREF000001" ),
			INTERCHANGE_END,
			null
		} );
		doTest( true, new Object[] {
			INTERCHANGE_BEGIN,
			new MessageBegin(
				METADATA_MESSAGE_BEGIN.getMref(),
				METADATA_MESSAGE_BEGIN.getSender(),
				"Aardvark",
				METADATA_MESSAGE_BEGIN.getMaintenanceAgencyId(),
				MessageBegin.STATISTICAL_DEFINITIONS,
				METADATA_MESSAGE_BEGIN.getIdentity(),
				contacts
			), ERROR,
			new MessageEnd( "MREF000001" ),
			INTERCHANGE_END,
			null
		} );
	}

	public void testAgencyWithoutCL_ORG () {
		doTest( false, new Object[] {
			INTERCHANGE_BEGIN,
			// make BIS known to Metabase.
			new MessageBegin(
				METADATA_MESSAGE_BEGIN.getMref(),
				METADATA_MESSAGE_BEGIN.getSender(),
				METADATA_MESSAGE_BEGIN.getReceiver(),
				"BIS",
				MessageBegin.STATISTICAL_DEFINITIONS,
				METADATA_MESSAGE_BEGIN.getIdentity(),
				contacts
			),
			METADATA_MESSAGE_END,
			new MessageBegin(
				"MREF000002",
				METADATA_MESSAGE_BEGIN.getSender(),
				METADATA_MESSAGE_BEGIN.getReceiver(),
				"BIS",
				MessageBegin.STATISTICAL_DEFINITIONS,
				METADATA_MESSAGE_BEGIN.getIdentity(),
				contacts
			), ERROR,
			getWithRef( METADATA_MESSAGE_END, "MREF000002" ),
			INTERCHANGE_END,
			null
		} );
	}

	public void testAgencyOutsideCL_AGENCY () {
		doTest( false, new Object[] {
			INTERCHANGE_BEGIN,
			new MessageBegin(
				METADATA_MESSAGE_BEGIN.getMref(),
				METADATA_MESSAGE_BEGIN.getSender(),
				METADATA_MESSAGE_BEGIN.getReceiver(),
				"AGOPLEX",
				MessageBegin.STATISTICAL_DEFINITIONS,
				METADATA_MESSAGE_BEGIN.getIdentity(),
				contacts
			),
			CL_ORGANISATION,
			CL_AGENCY,
			METADATA_MESSAGE_END,
			new MessageBegin(
				"MREF000002",
				DATA_MESSAGE_BEGIN.getSender(),
				DATA_MESSAGE_BEGIN.getReceiver(),
				"AGOPLEX",
				DATA_MESSAGE_BEGIN.getFunction(),
				DATA_MESSAGE_BEGIN.getIdentity(),
				contacts,
				DATA_MESSAGE_BEGIN.getDataSetId(),
				DATA_MESSAGE_BEGIN.getAction(),
				DATA_MESSAGE_BEGIN.getExtractionDateTime(),
				DATA_MESSAGE_BEGIN.getReportingPeriod(),
				DATA_MESSAGE_BEGIN.getReportingPeriodFormat(),
				DATA_MESSAGE_BEGIN.getKeyFamilyId()
			), ERROR,
			getWithRef( DATA_MESSAGE_END, "MREF000001" ),
			INTERCHANGE_END,
			null
		} );
	}

	public void testGoodDataSetIds () {
		doTest( false, new Object[] {
			INTERCHANGE_BEGIN,
			new MessageBegin(
				METADATA_MESSAGE_BEGIN.getMref(),
				DATA_MESSAGE_BEGIN.getSender(),
				DATA_MESSAGE_BEGIN.getReceiver(),
				DATA_MESSAGE_BEGIN.getMaintenanceAgencyId(),
				DATA_MESSAGE_BEGIN.getFunction(),
				DATA_MESSAGE_BEGIN.getIdentity(),
				contacts,
				"ECB_BOP1",
				DATA_MESSAGE_BEGIN.getAction(),
				DATA_MESSAGE_BEGIN.getExtractionDateTime(),
				DATA_MESSAGE_BEGIN.getReportingPeriod(),
				DATA_MESSAGE_BEGIN.getReportingPeriodFormat(),
				"ECB_BOP1"
			),
			DATA_MESSAGE_END,

			new MessageBegin(
				"MREF000002",
				DATA_MESSAGE_BEGIN.getSender(),
				DATA_MESSAGE_BEGIN.getReceiver(),
				DATA_MESSAGE_BEGIN.getMaintenanceAgencyId(),
				DATA_MESSAGE_BEGIN.getFunction(),
				DATA_MESSAGE_BEGIN.getIdentity(),
				contacts,
				"ECB_SSS",
				DATA_MESSAGE_BEGIN.getAction(),
				DATA_MESSAGE_BEGIN.getExtractionDateTime(),
				DATA_MESSAGE_BEGIN.getReportingPeriod(),
				DATA_MESSAGE_BEGIN.getReportingPeriodFormat(),
				"ECB_BOP1"
			),
			getWithRef( DATA_MESSAGE_END, "MREF000002" ),

			new MessageBegin(
				"MREF000003",
				DATA_MESSAGE_BEGIN.getSender(),
				DATA_MESSAGE_BEGIN.getReceiver(),
				DATA_MESSAGE_BEGIN.getMaintenanceAgencyId(),
				DATA_MESSAGE_BEGIN.getFunction(),
				DATA_MESSAGE_BEGIN.getIdentity(),
				contacts,
				"ECB_BODI",
				DATA_MESSAGE_BEGIN.getAction(),
				DATA_MESSAGE_BEGIN.getExtractionDateTime(),
				DATA_MESSAGE_BEGIN.getReportingPeriod(),
				DATA_MESSAGE_BEGIN.getReportingPeriodFormat(),
				"ECB_BBODI"
			),
			getWithRef( DATA_MESSAGE_END, "MREF000003" ),
			INTERCHANGE_END,
			null
		} );
	}

	/* -------------------- CodeList Checks --------------------- */

	public void testWhitespaceInCodeDescription () {
		doTest( false, new Object[] {
			INTERCHANGE_BEGIN,
			METADATA_MESSAGE_BEGIN,
			new CodeList(
				"CL_TRIMMED", "Nice descriptions", "",
				new String[]{ "SAU" }, new String[]{ "Female pig" }
			),
			new CodeList(
				"CL_END_SPC", "Whitespace at end descriptions", "",
				new String[]{ "SAU" }, new String[]{ "Female pig " }
			), WARNING,
			new CodeList(
				"CL_BEG_SPC", "Whitepsace at beginning descriptions", "",
				new String[]{ "SAU" }, new String[]{ " " }
			), WARNING,
			new CodeList(
				"CL_BEG_SPC", "Whitepsace at beginning descriptions", "",
				new String[]{ "SAU" }, new String[]{ " female pig" }
			), WARNING,
			new CodeList(
				"CL_MULTI_SPC", "Multiple whitespace descriptions", "",
				new String[]{ "SAU" }, new String[]{ "Female  pig" }
			), WARNING,
			METADATA_MESSAGE_END,
			INTERCHANGE_END,
			null
		} );
	}

	public void testEmptyCodeDescription () {
		doTest( false, new Object[] {
			INTERCHANGE_BEGIN,
			METADATA_MESSAGE_BEGIN,
			new CodeList(
				"CL_END_SPC", "Empty code descriptions", "",
				new String[]{ "SAU" }, new String[]{ "" }
			), WARNING,
			METADATA_MESSAGE_END,
			INTERCHANGE_END,
			null
		} );
	}

	/* --------------- StatisticalConcept Checks ---------------- */

	public void testConceptInsideOutsideCL_CONCEPT () {
		doTest( false, new Object[] {
			INTERCHANGE_BEGIN,
			new MessageBegin(
				METADATA_MESSAGE_BEGIN.getMref(),
				METADATA_MESSAGE_BEGIN.getSender(),
				METADATA_MESSAGE_BEGIN.getReceiver(),
				"EUROSTAT",
				MessageBegin.STATISTICAL_DEFINITIONS,
				METADATA_MESSAGE_BEGIN.getIdentity(),
				contacts
			),
			new CodeList(
				"CL_CONCEPT", "Concepts list", "",
				new String[]{ "SAU" }, new String[]{ "Female pig" }
			),
			new StatisticalConcept(
				"SAU", "Name of female pig", "", ""
			),
			new StatisticalConcept(
				"EBER", "Name of male pig", "", ""
			), ERROR,
			METADATA_MESSAGE_END,
			INTERCHANGE_END,
			null
		} );
	}

	/* ------------------- Key Family Checks -------------------- */

	public void testAssignDimensionWithInvalidCodeListCodes () {
		doTest( false, new Object[] {
			INTERCHANGE_BEGIN,
			METADATA_MESSAGE_BEGIN,
			new CodeList(
				"CL_ATT1", "test undim", "",
				new String[] { "A", "B-C" },
				new String[] { "Alpha", "Beta-Gamma" }
			),
			KF1,
			new CodeList(
				"CL_DIM1", "test undim", "",
				new String[] { "A", "B-C" },
				new String[] { "Alpha", "Beta-Gamma" }
			),
			KF1, ERROR,
			METADATA_MESSAGE_END,
			INTERCHANGE_END,
			null
		} );
	}

	public void testAssignDimensionWithEmptyCodeListCode () {
		doTest( false, new Object[] {
			INTERCHANGE_BEGIN,
			METADATA_MESSAGE_BEGIN,
			new CodeList(
				"CL_ATT1", "test undim", "",
				new String[] { "A", "" },
				new String[] { "Alpha", "Zeta" }
			),
			KF1,
			new CodeList(
				"CL_DIM1", "test undim", "",
				new String[] { "A", "" },
				new String[] { "Alpha", "Zeta" }
			),
			KF1, ERROR,
			METADATA_MESSAGE_END,
			INTERCHANGE_END,
			null
		} );
	}

	/* ------------------- Time Series Checks ------------------- */

	public void testFreqTimeFormatMismatch () {
		doTest( false, new Object[] {
			INTERCHANGE_BEGIN,
			DATA_MESSAGE_BEGIN,
			new TimeSeries(
				"Q.A.DO", "20021", "608", new Observation[]{
					OBS_ARR[1]
				}
			),
			new TimeSeries(
				"H.A.DO", "20021", "608", new Observation[]{
					OBS_ARR[1]
				}
			),
			ERROR,
			DATA_MESSAGE_END,
			INTERCHANGE_END,
			null
		} );
	}

	public void testMissingValue () {
		doTest( false, new Object[] {
			INTERCHANGE_BEGIN,
			DATA_MESSAGE_BEGIN,
			new TimeSeries(
				"Q.A.DO", "20021", "608", new Observation[]{
					new Observation( "13.6E6", "A", "", "" )
				}
			),
			new TimeSeries(
				"Q.A.DO", "20021", "608", new Observation[]{
					new Observation( "-", "A", "", "" )
				}
			),
			ERROR,
			DATA_MESSAGE_END,
			INTERCHANGE_END,
			null
		} );
	}

	public void testMissingStatus () {
		doTest( false, new Object[] {
			INTERCHANGE_BEGIN,
			DATA_MESSAGE_BEGIN,
			new TimeSeries(
				"Q.A.DO", "20021", "608", new Observation[]{
					new Observation( "-", "H", "", "" )
				}
			),
			new TimeSeries(
				"Q.A.DO", "20021", "608", new Observation[]{
					new Observation( "13.6", "H", "", "" )
				}
			),
			ERROR,
			new TimeSeries(
				"Q.A.DO", "20021", "608", new Observation[]{
					new Observation( "-", "L", "", "" )
				}
			),
			new TimeSeries(
				"Q.A.DO", "20021", "608", new Observation[]{
					new Observation( "13.6", "L", "", "" )
				}
			),
			ERROR,
			new TimeSeries(
				"Q.A.DO", "20021", "608", new Observation[]{
					new Observation( "-", "M", "", "" )
				}
			),
			new TimeSeries(
				"Q.A.DO", "20021", "608", new Observation[]{
					new Observation( "13.6", "M", "", "" )
				}
			),
			ERROR,
			new TimeSeries(
				"Q.A.DO", "20021", "608", new Observation[]{
					new Observation( "-", "S", "", "" )
				}
			),
			new TimeSeries(
				"Q.A.DO", "20021", "608", new Observation[]{
					new Observation( "-", "Q", "", "" )
				}
			),
			new TimeSeries(
				"Q.A.DO", "20021", "608", new Observation[]{
					new Observation( "13.6", "Q", "", "" )
				}
			),
			ERROR,
			DATA_MESSAGE_END,
			INTERCHANGE_END,
			null
		} );
	}

	public void testBreakStatus () {
		doTest( false, new Object[] {
			INTERCHANGE_BEGIN,
			DATA_MESSAGE_BEGIN,
			new TimeSeries(
				"Q.A.DO", "20021", "608", new Observation[]{
					new Observation( "12", "A", "C", "14" )
				}
			),
			ERROR,
			new TimeSeries(
				"Q.A.DO", "20021", "608", new Observation[]{
					new Observation( "12", "B", "C", "" )
				}
			),
			WARNING,
			new TimeSeries(
				"Q.A.DO", "20021", "608", new Observation[]{
					new Observation( "12", "B", "C", "-" )
				}
			),
			DATA_MESSAGE_END,
			INTERCHANGE_END,
			null
		} );
	}

	public void testEmptyObservation () {
		doTest( false, new Object[] {
			INTERCHANGE_BEGIN,
			DATA_MESSAGE_BEGIN,
			new TimeSeries(
				"W.A.DI", "200507200511", "716", new Observation[]{
					new Observation( "14", "B", "F", "0.14" ),
					new Observation( "", "", "", "" ),
					Observation.EMPTY_OBSERVATION,
					new Observation( "13.7", "E", "N", "" ),
				}
			),
			DATA_MESSAGE_END,
			INTERCHANGE_END,
			null
		} );
	}
	
	public void testEmptyObsWithBreak () {
		doTest( false, new Object[] {
				INTERCHANGE_BEGIN,
				DATA_MESSAGE_BEGIN,
				new TimeSeries(
						"Q.A.DO1", "20021", "608", new Observation[]{
							new Observation( "-", "B", "", "-" )
						}
				),
				new TimeSeries(
						"Q.A.DO2", "20021", "608", new Observation[]{
							new Observation( "-", "B", "", "45" )
						}
				),
				new TimeSeries(
						"Q.A.DO3", "20021", "608", new Observation[]{
							new Observation( "-", "B", "", "" )
						}
				),
				WARNING,					
				DATA_MESSAGE_END,
				INTERCHANGE_END,
				null
			} );
	}
}
