// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.semanticchecker;

import java.lang.Object;
import java.lang.String;

import junit.framework.Test;
import junit.framework.TestSuite;

import Int.ecb.gesmescb.*;

import Int.ecb.gesmescb.semanticchecker.ECBRestrictionsChecker;

/* ***************************** Class ****************************** */

/**
 * <P>Tests the implementation of the
 * class {@link ECBRestrictionsChecker}.</P>
 *
 * @author
 * <A HREF="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</A>
 *
 * @version @(#)$Id: PreBreakRestrictionsCheckerTest.java,v 1.1 2017/01/19 16:14:05 trainot Exp $
 */

public class PreBreakRestrictionsCheckerTest extends CheckerBaseTest {

	/* ====================== Constructors ====================== */

	/**
	 * Constructs a SemanticCheckerTest with the specified name.
	 *
	 * @param name Test case name
	 */
	public PreBreakRestrictionsCheckerTest ( final String name ) {
		super( name );
	}

	/* ================== Protected Methods ===================== */

	/**
	 * Factory for ECBRestrictionsCheckers.
	 * Should be overridden by derived classes.
	 *
	 * @see doTest(boolean,Object[])
	 */
	protected CheckerBase checkerFactory () {
		return new ECBRestrictionsChecker();
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite
	 */
	public static Test suite () {
		return new TestSuite( PreBreakRestrictionsChecker.class );
	}

	/**
	 * Runs the test case
	 */
	public static void main ( String args[] ) {
		final String[] testCaseName
		= { PreBreakRestrictionsChecker.class.getName() };
		junit.textui.TestRunner.main( testCaseName );
	}

	/* ---------------------- Test Methods ---------------------- */

	/* -------------------- CodeList Checks --------------------- */

	public void testBreakStatus () {
		doTest( false, new Object[] {
			INTERCHANGE_BEGIN,
			DATA_MESSAGE_BEGIN,
			new TimeSeries(
				"Q.A.DO", "20021", "608", new Observation[]{
					new Observation( "12", "A", "C", "14" )
				}
			),
			ERROR,
			new TimeSeries(
				"Q.A.DO", "20021", "608", new Observation[]{
					new Observation( "12", "B", "C", "" )
				}
			),
			WARNING,
			new TimeSeries(
				"Q.A.DO", "20021", "608", new Observation[]{
					new Observation( "12", "B", "C", "-" )
				}
			),
			DATA_MESSAGE_END,
			INTERCHANGE_END,
			null
		} );
	}

	public void testEmptyObservation () {
		doTest( false, new Object[] {
			INTERCHANGE_BEGIN,
			DATA_MESSAGE_BEGIN,
			new TimeSeries(
				"W.A.DI", "200507200511", "716", new Observation[]{
					new Observation( "14", "B", "F", "0.14" ),
					new Observation( "", "", "", "" ),
					Observation.EMPTY_OBSERVATION,
					new Observation( "13.7", "E", "N", "" ),
				}
			),
			DATA_MESSAGE_END,
			INTERCHANGE_END,
			null
		} );
	}
	
	public void testEmptyObsWithBreak () {
		doTest( false, new Object[] {
				INTERCHANGE_BEGIN,
				DATA_MESSAGE_BEGIN,
				new TimeSeries(
						"Q.A.DO1", "20021", "608", new Observation[]{
							new Observation( "-", "B", "", "-" )
						}
				),
				new TimeSeries(
						"Q.A.DO2", "20021", "608", new Observation[]{
							new Observation( "-", "B", "", "45" )
						}
				),
				new TimeSeries(
						"Q.A.DO3", "20021", "608", new Observation[]{
							new Observation( "-", "B", "", "" )
						}
				),
				WARNING,					
				DATA_MESSAGE_END,
				INTERCHANGE_END,
				null
			} );
	}
}
