// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.sieve;

import java.lang.String;

import Int.ecb.gesmescb.util.ComponentHelper;
import Int.ecb.logging.Level;

import Int.ecb.gesmescb.Observation;
import Int.ecb.gesmescb.MessageBegin;

import Int.ecb.gesmescb.time.RegularInterval;

/* **************************** Class ***************************** */

/**
 * <p>Filters CSI (Confidential Statistical Information) observations.</p>
 *
 * @author
 * <A HREF="mailto:Sami.Airo@ECB.int">Sami Airo</A>
 *
 * @version @(#)$Id: CSIObservationFilter.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */

public final class CSIObservationFilter implements ObservationFilter {

	/* ======================= Constants ======================== */

	public final String CSI_CODE = "R";

	/* ========================= Fields ========================= */

	/**
	 * Takes care of logging and prefs
	 */
	private ComponentHelper componentHelper;

	/**
	 * Logging level for rejections
	 */
	private Level rejectionLevel;

	/* ====================== Constructors ====================== */

	public CSIObservationFilter () {
		this( "" );
	}

	/**
	 * Constructs a CSIObservationFilter.
	 */
	public CSIObservationFilter (final String name) {
		componentHelper = new ComponentHelper( name, this );
		rejectionLevel = Level.parse( componentHelper.getPreference("rejectionLevel", "FINE") );
	}

	/* ===================== Public Methods ===================== */

	public boolean accept (
		final MessageBegin messageBegin,
		final String seriesKey,
		final Observation observation,
		final RegularInterval interval
	) {
		componentHelper.logger.log(
			Level.FINE, componentHelper.logName, "accept",
			"checking observation for CSI", interval
		);

		if ( observation.getConfidentiality().equals(CSI_CODE) ) {
			componentHelper.logger.log(
				rejectionLevel,
				componentHelper.logName,
				"accept",
				"Observation rejected: observation confidentiality indicates CSI for interval " + interval
			);
			return false;
		}

		return true;
	}
}
