// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (C) 2004, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.sieve;

import java.util.Properties;

import junit.framework.Test;
import junit.framework.TestSuite;

import Int.ecb.gesmescb.Attribute;

/* ***************************** Class ****************************** */

/**
 * Tests the implementation of the class {@link EmptyObjectFilter}.
 *
 * @author
 * <a href="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</A>
 *
 * @version @(#)$Id: EmptyObjectFilterTest.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */
public class EmptyObjectFilterTest extends GESMESCBObjectFilterAdapterTest {

	/* ====================== Constructors ====================== */

	/**
	 * Constructs a EmptyObjectFilterTest with the specified name.
	 *
	 * @param name Test case name
	 */
	public EmptyObjectFilterTest ( final String name ) {
		super( name );
	}

	/* ================== Protected Methods ===================== */

	/** Sets up the test fixture. */
	protected void setUp () {
		final String className = EmptyObjectFilter.class.getName();
		System.setProperty(
			className + ".AttributeRejector.acceptEmptyAttributeUpdates",
			"false"
		);
	}

	/** Tears down the test fixture. */
	protected void tearDown () {
		final String className = EmptyObjectFilter.class.getName();
		final Properties properties = System.getProperties();
		properties.remove(
			className + ".AttributeRejector.acceptEmptyAttributeUpdates"
		);
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite.
	 */
	public static Test suite () {
		return new TestSuite( EmptyObjectFilterTest.class );
	}

	/**
	 * Runs the test case.
	 */
	public static void main ( final String args[] ) {
		final String[] testCaseName = {
			EmptyObjectFilterTest.class.getName()
		};
		junit.textui.TestRunner.main( testCaseName );
	}

	/* --------------------- Test Methods --------------------- */

	public void testDefault () {
		final GESMESCBObjectFilter filter = new EmptyObjectFilter();
		acceptCodeList( filter );
		acceptStatisticalConcept( filter );
		acceptKeyFamily( filter );
		acceptTimeSeries( filter );
		acceptAttribute( filter );
	}

	public void testOtherObjects () {
		final GESMESCBObjectFilter filter = new EmptyObjectFilter(
			"AttributeRejector"
		);
		acceptCodeList( filter );
		acceptStatisticalConcept( filter );
		acceptKeyFamily( filter );
		acceptTimeSeries( filter );
	}

	public void testAcceptEmptyAttribute () {
		final GESMESCBObjectFilter filter = new EmptyObjectFilter();
		assertTrue( "MessageBegin OK1", filter.accept( DATA_MESSAGE_BEGIN ) );
		assertTrue(
			"Empty attribute by default is OK",
			filter.accept( new Attribute(
				"ABC", "", "", "", ""
			) )
		);
	}

	public void testRejectEmptyAttribute () {
		final GESMESCBObjectFilter filter = new EmptyObjectFilter(
			"AttributeRejector"
		);
		assertTrue( "MessageBegin OK2", filter.accept( DATA_MESSAGE_BEGIN ) );
		assertTrue(
			"Empty attribute not OK",
			! filter.accept( new Attribute(
				"ABC", "", "", "", ""
			) )
		);
		assertTrue(
			"Nonempty attribute is OK",
			filter.accept( new Attribute(
				"ABC", "", "", "", "Schwompf!"
			) )
		);
	}

	public void testAttributeDeletion () {
		final GESMESCBObjectFilter filter = new EmptyObjectFilter(
			"AttributeRejector"
		);
		assertTrue( "MessageBegin OK3", filter.accept( DELETE_MESSAGE_BEGIN ) );
		assertTrue(
			"Empty attribute in deletion is OK",
			filter.accept( new Attribute(
				"ABC", "", "", "", ""
			) )
		);
	}
}
