// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2002, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.sieve;

import Int.ecb.gesmescb.*;

/* **************************** Interface ***************************** */

/**
 * <P>Defines the contract for filter classes
 * used by {@link GESMESCBEventSieve}.</P>
 *
 * <P>A class that implements this interface inspects
 * each {@link GESMESCBObject} received
 * and accepts or rejects it.</P>
 *
 * @author
 * <A HREF="mailto:Sami.Airo@ECB.int">Sami Airo</A>
 * @author
 * <A HREF="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</A>
 *
 * @version @(#)$Id: GESMESCBObjectFilter.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */
public interface GESMESCBObjectFilter {

	/* ======================== Methods ========================= */

	/**
	 * Accepts or rejects a <CODE>InterchangeBegin</CODE>.
	 *
	 * @param interchangeBegin to inspect.
	 *
	 * @return true if interchangeBegin is accepted; false if it is rejected.
	 */
	boolean accept ( InterchangeBegin interchangeBegin );

	/**
	 * Accepts or rejects a <CODE>MessageBegin</CODE>.
	 *
	 * @param messageBegin to inspect.
	 *
	 * @return true if messageBegin is accepted; false if it is rejected.
	 */
	boolean accept ( MessageBegin messageBegin );

	/**
	 * Accepts or rejects a <CODE>CodeList</CODE>.
	 *
	 * @param codeList to inspect.
	 *
	 * @return true if codeList is accepted; false if it is rejected.
	 */
	boolean accept ( CodeList codeList );

	/**
	 * Accepts or rejects a <CODE>StatisticalConcept</CODE>.
	 *
	 * @param statisticalConcept to inspect.
	 *
	 * @return true if statisticalConcept is accepted; false if it is rejected.
	 */
	boolean accept ( StatisticalConcept statisticalConcept );

	/**
	 * Accepts or rejects a <CODE>KeyFamily</CODE>.
	 *
	 * @param keyFamily to inspect.
	 *
	 * @return true if keyFamily is accepted; false if it is rejected.
	 */
	boolean accept ( KeyFamily keyFamily );

	/**
	 * Accepts or rejects a <CODE>TimeSeries</CODE>.
	 *
	 * @param timeSeries to inspect.
	 *
	 * @return true if timeSeries is accepted; false if it is rejected.
	 */
	boolean accept ( TimeSeries timeSeries );

	/**
	 * Accepts or rejects an <CODE>Attribute</CODE>.
	 *
	 * @param attribute to inspect.
	 *
	 * @return true if attribute is accepted; false if it is rejected.
	 */
	boolean accept ( Attribute attribute );
}
