// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (C) 2004, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.sieve;

import Int.ecb.logging.Level;

import Int.ecb.gesmescb.InterchangeBegin;
import Int.ecb.gesmescb.MessageBegin;
import Int.ecb.gesmescb.CodeList;
import Int.ecb.gesmescb.StatisticalConcept;
import Int.ecb.gesmescb.KeyFamily;
import Int.ecb.gesmescb.TimeSeries;
import Int.ecb.gesmescb.Attribute;

import Int.ecb.gesmescb.util.ComponentHelper;

/* **************************** Class ***************************** */

/**
 * <p>Provides default "accept all" to subclasses.
 * This helps implementing {@link GESMESCBObjectFilter}s.</p>
 *
 * @see GESMESCBEventSieve
 *
 * @author
 * <a href="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</A>
 *
 * @version @(#)$Id: GESMESCBObjectFilterAdapter.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */
public abstract class GESMESCBObjectFilterAdapter
	implements GESMESCBObjectFilter
{

	/* ========================= Fields ========================= */

	/** Takes care of logging and preferences. */
	protected final ComponentHelper componentHelper;

	/* ====================== Constructors ====================== */

	/** Constructs a GESMESCBObjectFilterAdapter with the specified name. */
	public GESMESCBObjectFilterAdapter ( final String name ) {
		componentHelper = new ComponentHelper( name, this );
	}

	/* ===================== Public Methods ===================== */

	public boolean accept ( final InterchangeBegin interchangeBegin ) {
		componentHelper.logger.log(
			Level.FINE,
			componentHelper.logName,
			"accept",
			"InterchangeBegin accepted.",
			interchangeBegin
		);
		return true;
	}

	public boolean accept ( final MessageBegin messageBegin ) {
		componentHelper.logger.log(
			Level.FINE,
			componentHelper.logName,
			"accept",
			"MessageBegin accepted.",
			messageBegin
		);
		return true;
	}

	public boolean accept ( final CodeList codeList ) {
		componentHelper.logger.log(
			Level.FINE,
			componentHelper.logName,
			"accept",
			"CodeList accepted.",
			codeList
		);
		return true;
	}

	public boolean accept ( final StatisticalConcept statisticalConcept ) {
		componentHelper.logger.log(
			Level.FINE,
			componentHelper.logName,
			"accept",
			"StatisticalConcept accepted.",
			statisticalConcept
		);
		return true;
	}

	public boolean accept ( final KeyFamily keyFamily ) {
		componentHelper.logger.log(
			Level.FINE,
			componentHelper.logName,
			"accept",
			"KeyFamily accepted.",
			keyFamily
		);
		return true;
	}

	public boolean accept ( final TimeSeries timeSeries ) {
		componentHelper.logger.log(
			Level.FINEST,
			componentHelper.logName,
			"accept",
			"TimeSeries accepted.",
			timeSeries
		);
		return true;
	}

	public boolean accept ( final Attribute attribute ) {
		componentHelper.logger.log(
			Level.FINEST,
			componentHelper.logName,
			"accept",
			"Attribute accepted.",
			attribute
		);
		return true;
	}
}
