// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.sieve;

import java.util.Collections;
import java.text.SimpleDateFormat;
import java.text.ParseException;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import Int.ecb.gesmescb.*;
import Int.ecb.gesmescb.time.*;

/* ***************************** Class ****************************** */

/**
 * Tests the implementation of the class {@link IntervalObservationFilter}.
 *
 * @author
 * <A HREF="mailto:Sami.Airo@ECB.int">Sami Airo</A>
 *
 * @version @(#)$Id: IntervalObservationFilterTest.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */

public class IntervalObservationFilterTest extends TestCase {

	/* ======================= Constants ======================== */

	private final String CLASS_NAME = "Int.ecb.gesmescb.sieve.IntervalObservationFilter";

	/* ========================= Fields ========================= */

	private MessageBegin messageBegin;
	private SimpleDateFormat format = new SimpleDateFormat("yyyyMMdd'T'HHmmss");

	/* ====================== Constructors ====================== */

	/**
	 * Constructs a IntervalObservationFilterTest with the specified name.
	 *
	 * @param name Test case name
	 */
	public IntervalObservationFilterTest ( final String name ) {
		super( name );
	}

	/* ================== Protected Methods ===================== */

	/**
	 * Sets up the test fixture.
	 */
	protected void setUp () {
		messageBegin = new MessageBegin(
			"MREF0001", "SE2", "4F0", "ECB",
			MessageBegin.STATISTICAL_DATA,
			"Test message", Collections.EMPTY_SET,
			"ECB_EXR1",
			MessageBegin.UPDATE_OR_REPLACE_ACTION,
			"20010329:1423", "", "",
			"ECB_EXR1"
		);

		System.setProperty( ".level", "ALL" );
		System.setProperty( "handlers", "Int.ecb.logging.ConsoleHandler" );
		System.setProperty(
			"Int.ecb.logging.ConsoleHandler.formatter",
			"Int.ecb.gesmescb.util.NormalLogFormatter"
		);
	}

	/**
	 * Tears down the test fixture.
	 */
	protected void tearDown () {
		messageBegin = null;
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite.
	 */
	public static Test suite () {
		return new TestSuite( IntervalObservationFilterTest.class );
	}

	/**
	 * Runs the test case.
	 */
	public static void main ( final String args[] ) {
		final String[] testCaseName = {
			IntervalObservationFilterTest.class.getName()
		};
		junit.textui.TestRunner.main( testCaseName );
	}

	/* --------------------- Test Methods --------------------- */

	public void testAcceptObservation () 
		throws ParseException
	{
		System.setProperty(CLASS_NAME + ".beginDate", "20030315T000000");
		System.setProperty(CLASS_NAME + ".endDate", "20030320T000000");
	
		IntervalObservationFilter filter = new IntervalObservationFilter();
				
		assertTrue("Interval OK", 
			filter.accept(
				messageBegin, 
				"SERIES.KEY", 
				Observation.EMPTY_OBSERVATION, 
				new GregorianRegularInterval(
					RegularInterval.DAILY,
					format.parse("20030319T000000"),
					format.parse("20030320T000000")
				)
			)
		);

		assertTrue("Interval NOK", 
			filter.accept(
				messageBegin, 
				"SERIES.KEY", 
				Observation.EMPTY_OBSERVATION, 
				new GregorianRegularInterval(
					RegularInterval.DAILY,
					format.parse("20030315T000000"),
					format.parse("20030320T000000")
				)
			)
		);

		assertTrue("Interval NOK", 
			! filter.accept(
				messageBegin, 
				"SERIES.KEY", 
				Observation.EMPTY_OBSERVATION, 
				new GregorianRegularInterval(
					RegularInterval.DAILY,
					format.parse("20030301T000000"),
					format.parse("20030302T000000")
				)
			)
		);

		assertTrue("Interval NOK", 
			! filter.accept(
				messageBegin, 
				"SERIES.KEY", 
				Observation.EMPTY_OBSERVATION, 
				new GregorianRegularInterval(
					RegularInterval.DAILY,
					format.parse("20030301T000000"),
					format.parse("20030325T000000")
				)
			)
		);

		assertTrue("Interval NOK", 
			! filter.accept(
				messageBegin, 
				"SERIES.KEY", 
				Observation.EMPTY_OBSERVATION, 
				new GregorianRegularInterval(
					RegularInterval.DAILY,
					format.parse("20030324T000000"),
					format.parse("20030325T000000")
				)
			)
		);

		assertTrue("Interval NOK", 
			! filter.accept(
				messageBegin, 
				"SERIES.KEY", 
				Observation.EMPTY_OBSERVATION, 
				new GregorianRegularInterval(
					RegularInterval.DAILY,
					format.parse("20030314T000000"),
					format.parse("20030318T000000")
				)
			)
		);

		assertTrue("Interval NOK", 
			! filter.accept(
				messageBegin, 
				"SERIES.KEY", 
				Observation.EMPTY_OBSERVATION, 
				new GregorianRegularInterval(
					RegularInterval.DAILY,
					format.parse("20030318T000000"),
					format.parse("20030321T000000")
				)
			)
		);

	}
}
