// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.sieve;

import java.io.*;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import Int.ecb.gesmescb.*;

import Int.ecb.gesmescb.event.TestEventListener;
import Int.ecb.gesmescb.event.TestEventSource;

/* ***************************** Class ****************************** */

/**
 * <P>Tests the implementation of {@link ObservationSieve}.</P>
 *
 * @author
 * <A HREF="mailto:Sami.Airo@ECB.int">Sami Airo</A>
 *
 * @version @(#)$Id: ObservationSieveTest.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */

public class ObservationSieveTest extends TestCase {

	/* ======================= Constants ======================== */

	/* ========================= Fields ========================= */

	private ObservationSieve sieve;
	private TestEventSource source;
	private TestEventListener listener;

	private final MessageBegin messageBegin = new MessageBegin(
		"MREF0001",
		"4F0",
		"4F0",
		"ECB",
		MessageBegin.STATISTICAL_DATA,
		"De subject",
		new Contact[]{}
	);

	/* ====================== Constructors ====================== */

	/**
	 * Constructs a ObservationSieveOldTest with the specified name.
	 *
	 * @param name Test case name
	 */
	public ObservationSieveTest ( final String name ) {
		super( name );
	}

	/* ================== Protected Methods ===================== */

	/**
	 * Sets up the test fixture.
	 */
	protected void setUp () throws IOException {
		System.setProperty(
			"Int.ecb.gesmescb.sieve.ObservationSieve.observationFilterClass.1", 
			"Int.ecb.gesmescb.sieve.CSIObservationFilter"
		);
		
		System.setProperty( "Int.ecb.logging.config.class", "Int.ecb.gesmescb.util.LoggingPropertiesReader");
		System.setProperty( ".level", "ALL" );
		System.setProperty( "handlers", "Int.ecb.logging.ConsoleHandler" );
		System.setProperty( "Int.ecb.logging.ConsoleHandler.formatter", "Int.ecb.gesmescb.util.NormalLogFormatter" );
//		System.setProperty( "Int.ecb.logging.ConsoleHandler.level", "ALL" );

		sieve = new ObservationSieve();		
		source = new TestEventSource();
		listener = new TestEventListener( 10 );
		source.addListener( sieve, true );
		sieve.addListener( listener, true );
		source.dispatch( messageBegin );
	}
	
	protected void tearDown () {
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite.
	 */
	public static Test suite () {
		return new TestSuite( ObservationSieveTest.class );
	}

	/**
	 * Runs the test case.
	 */
	public static void main ( final String args[] ) {
		String[] testCaseName = {
			ObservationSieveTest.class.getName()
		};
		junit.textui.TestRunner.main( testCaseName );
	}

	/* --------------------- Test Methods --------------------- */


	public void testMessageBegin () {
		assertSame("messageBegin", messageBegin, listener.getLastObject() );
	}

	public void testNormalTimeSeries () {
		TimeSeries sourceSeries = 
			new TimeSeries(
				"A.B.C",
				"19992000",
				"702",
				new Observation[]{
					new Observation("10","A","",""),
					new Observation("10","A","","")
				}
			);

		TimeSeries filteredSeries = sourceSeries;
		
		source.dispatch( sourceSeries );
		assertEquals("unfiltered series", filteredSeries, listener.getLastObject() );
	}

	public void testTimeSeriesWithR () {
		TimeSeries sourceSeries = 
			new TimeSeries(
				"A.B.C",
				"19992000",
				"702",
				new Observation[]{
					new Observation("10","A","R",""),
					new Observation("10","A","","")
				}
			);

		TimeSeries filteredSeries = 
			new TimeSeries(
				"A.B.C",
				"2000",
				"602",
				new Observation[]{
					new Observation("10","A","","")
				}
			);

		source.dispatch( sourceSeries );
		assertEquals("filtered series", filteredSeries, listener.getLastObject() );
	}

	public void testTimeSeriesHoles () {
		TimeSeries sourceSeries = 
			new TimeSeries(
				"A.B.C",
				"19992003",
				"702",
				new Observation[]{
					Observation.EMPTY_OBSERVATION,
					new Observation("2000","A","",""),
					new Observation("2001","A","",""),
					Observation.EMPTY_OBSERVATION,
					new Observation("2003","A","","")
				}
			);

		TimeSeries filteredSeries = 
			new TimeSeries(
				"A.B.C",
				"20002003",
				"702",
				new Observation[]{
					new Observation("2000","A","",""),
					new Observation("2001","A","",""),
					Observation.EMPTY_OBSERVATION,
					new Observation("2003","A","","")
				}
			);

		source.dispatch( sourceSeries );
		assertEquals("filtered series", filteredSeries, listener.getLastObject() );
	}
}
