// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.sieve;

import Int.ecb.logging.Level;

import Int.ecb.gesmescb.*;

import Int.ecb.gesmescb.util.ComponentHelper;

/* **************************** Class ***************************** */

/**
 * <P>Filter interchanges based on the test indicator.</P>
 *
 * <P>Accepts normal interchanges and rejects test interchanges.
 * </P>
 *
 * @see GESMESCBEventSieve
 *
 * @author
 * <A HREF="mailto:Sami.Airo@ECB.int">Sami Airo</A>
 *
 * @version @(#)$Id: TestInterchangeFilter.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */
public class TestInterchangeFilter implements GESMESCBObjectFilter {

	/* ========================= Fields ========================= */

	/**
	 * Takes care of logging and preferences.
	 */
	protected final ComponentHelper componentHelper;

	/* ====================== Constructors ====================== */

	/**
	 * Constructs a TestInterchangeFilter with the default name.
	 */
	public TestInterchangeFilter () {
		this( "" );
	}

	/**
	 * Constructs a TestInterchangeFilter with the specified name.
	 *
	 * @param name		name for this TestInterchangeFilter
	 */
	public TestInterchangeFilter ( final String name ) {
		componentHelper = new ComponentHelper( name, this );
	}

	/* ===================== Public Methods ===================== */

	public boolean accept ( final InterchangeBegin interchangeBegin ) {
		if (interchangeBegin.isTest()) {
			componentHelper.logger.log(
				Level.FINE,
				componentHelper.logName,
				"accept",
				"Test interchange rejected.",
				interchangeBegin
			);
		} else {
			componentHelper.logger.log(
				Level.FINE,
				componentHelper.logName,
				"accept",
				"Normal interchange accepted.",
				interchangeBegin
			);
		}
		
		return !interchangeBegin.isTest();
	}
	
	public boolean accept ( final MessageBegin messageBegin ) {
		return true;
	}

	public boolean accept ( final CodeList codeList ) {
		return true;
	}

	public boolean accept ( final StatisticalConcept statisticalConcept ) {
		return true;
	}

	public boolean accept ( final KeyFamily keyFamily ) {
		return true;
	}

	public boolean accept ( final TimeSeries timeSeries ) {
		return true;
	}

	public boolean accept ( final Attribute attribute ) {
		return true;
	}
}
