// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.sieve;

import Int.ecb.gesmescb.*;

import Int.ecb.gesmescb.sieve.GESMESCBObjectFilter;

/* **************************** Class ***************************** */

/**
 * Assists for testing the class {@link GESMESCBEventSieve}.
 *
 * @author
 * <A HREF="mailto:Sami.Airo@ECB.int">Sami Airo</A>
 * @author
 * <A HREF="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</A>
 *
 * @version @(#)$Id: TestObjectFilter.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */
class TestObjectFilter implements GESMESCBObjectFilter {

	/* ========================= Fields ========================= */

	/**
	 * Last event received
	 */
	private GESMESCBObject lastObject;

	/**
	 * Default response
	 */
	private boolean defaultResponse;

	/* ====================== Constructors ====================== */

	public TestObjectFilter () {}

	/* ===================== Public Methods ===================== */

	/**
	 * Returns the last received object.
	 */
	public GESMESCBObject getLastObject () {
		return lastObject;
	}

	/**
	 * Sets the default answer.
	 *
	 * @param response true to make all events accepted; false otherwise
	 */
	public void setDefaultResponse ( final boolean response ) {
		defaultResponse = response;
	}

	public boolean accept ( final InterchangeBegin interchangeBegin ) {
		lastObject = interchangeBegin;
		return defaultResponse;
	}

	public boolean accept ( final MessageBegin messageBegin ) {
		lastObject = messageBegin;
		return defaultResponse;
	}

	public boolean accept ( final CodeList codeList ) {
		lastObject = codeList;
		return defaultResponse;
	}

	public boolean accept ( final StatisticalConcept statisticalConcept ) {
		lastObject = statisticalConcept;
		return defaultResponse;
	}

	public boolean accept ( final KeyFamily keyFamily ) {
		lastObject = keyFamily;
		return defaultResponse;
	}

	public boolean accept ( final TimeSeries timeSeries ) {
		lastObject = timeSeries;
		return defaultResponse;
	}

	public boolean accept ( final Attribute attribute ) {
		lastObject = attribute;
		return defaultResponse;
	}
}
