// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.sql;

import java.text.ParseException;
import java.util.Iterator;
import java.io.IOException;

import Int.ecb.logging.Level;

import Int.ecb.gesmescb.*;

import Int.ecb.gesmescb.util.ComponentHelper;

/* **************************** Class ***************************** */

/**
 * SQL writer for metadata.
 *
 * @author
 * <A HREF="mailto:Sami.Airo@ECB.int">Sami Airo</A>
 *
 * @version @(#)$Id: SQLDataDefinitionWriter.java,v 1.1 2017/01/19 16:14:02 trainot Exp $
 */

public class SQLDataDefinitionWriter 
	extends SQLWriter
{

	/* ======================= Constants ======================== */

	/* ========================= Fields ========================= */

	/* ====================== Constructors ====================== */

	public SQLDataDefinitionWriter ()
		throws IOException
	{
		this( "" );
	}

	public SQLDataDefinitionWriter (final String name)
		throws IOException
	{
		componentHelper = new ComponentHelper( name, this );
		setSQLSyntax(componentHelper.getPreference("SQLSyntax", "SQL92"));
		open();
	}

	/* ===================== Public Methods ===================== */

	public void handle ( final MessageBegin messageBegin ) {
		componentHelper.logger.log(
			Level.FINE,
			componentHelper.logName,
			"handle",
			"Handling MessageBegin"
		);
		
		if (!messageBegin.getFunction().equals(MessageBegin.STATISTICAL_DEFINITIONS)) {
			return;
		}
		
		writer.println("\nCREATE SCHEMA GESMESCB_MA_" + messageBegin.getMaintenanceAgencyId() + getStatementTerminator());
		
		writer.println("\nCREATE TABLE STATISTICAL_CONCEPTS");
		writer.println("\t( ID VARCHAR(35),");
		writer.println("\tNAME VARCHAR(70),");
		writer.println("\tPRIMARY KEY (ID) )" + getStatementTerminator());
	}

	public void handle ( final CodeList codeList ) {
		componentHelper.logger.log(
			Level.FINE,
			componentHelper.logName,
			"handle",
			"Handling CodeList"
		);
		
		writer.println("\nCREATE TABLE " + codeList.getId());
		writer.println("\t( VALUE VARCHAR(12),");
		writer.println("\tDESCRIPTION VARCHAR(350),");
		writer.println("\tPRIMARY KEY (VALUE) )" + getStatementTerminator());
	}

	public void handle ( final KeyFamily keyFamily ) {
		componentHelper.logger.log(
			Level.FINE,
			componentHelper.logName,
			"handle",
			"Handling KeyFamily"
		);

		writer.println( getDataSetTableDefinition( keyFamily ) );
		writer.println( getSiblingTableDefinition( keyFamily ) );
		writer.println( getSeriesTableDefinition( keyFamily ) );
		writer.println( getDataTableDefinition( keyFamily ) );
	}
	
			
	public String getSeriesTableDefinition( final KeyFamily keyFamily ) {
		StringBuffer keyColumns = new StringBuffer();
		StringBuffer attributeColumns = new StringBuffer();
		StringBuffer constraints = new StringBuffer();
		StringBuffer primaryKeys = new StringBuffer("SIBLING_KEY");
		
		Iterator it = keyFamily.componentIterator();		
		while (it.hasNext()) {
			KeyFamilyComponent component = (KeyFamilyComponent) it.next();			
			String definition = getColumnDefinition(component);
			String constraint = getColumnConstraint(component);
			
			if (0 < constraint.length()) {
				constraint = ",\n\t" + constraint;
			}						

			if (component.isKeyStructureDimension()) {
				if (component.isTimeDimension()) {
					continue;
				}

				if (!component.getConceptId().equals("FREQ")) {
					continue;
				}

				keyColumns.append(",\n\t" + definition);
				primaryKeys.append("," + component.getConceptId());
				constraints.append(constraint);				
				continue;
			}
			
			if (!component.isAttachedToTimeSeries()) {
				continue;
			}

			attributeColumns.append(",\n\t" + definition);
			constraints.append(constraint);
		}		

		return	"\nCREATE TABLE " + keyFamily.getId() + "_TIME_SERIES" 
					+ "\n\t( " + getKeyDefinition(keyFamily)
					+ ",\n\t" + getSiblingKeyDefinition(keyFamily)
					+ keyColumns.toString()
					+ attributeColumns.toString()
					+ ",\n\tPRIMARY KEY (" + primaryKeys + ")" 
					+ ",\n\tUNIQUE (SERIES_KEY)"
					+ ",\n\tFOREIGN KEY (SIBLING_KEY) REFERENCES " + keyFamily.getId() + "_SIBLING_GROUPS (SIBLING_KEY)" 
					+ constraints + " )" + getStatementTerminator();
	}


	public String getSiblingTableDefinition( final KeyFamily keyFamily ) {
		StringBuffer keyColumns = new StringBuffer();
		StringBuffer attributeColumns = new StringBuffer();
		StringBuffer primaryKey = new StringBuffer("DATA_SET");
		StringBuffer constraints = new StringBuffer();
		
		Iterator it = keyFamily.componentIterator();		
		while (it.hasNext()) {
			KeyFamilyComponent component = (KeyFamilyComponent) it.next();			
			String definition = getColumnDefinition(component);
			String constraint = getColumnConstraint(component);
			
			if (0 < constraint.length()) {
				constraint = ",\n\t" + constraint;
			}						

			if (component.isKeyStructureDimension()) {
				if (component.isTimeDimension()) {
					continue;
				}

				if (component.getConceptId().equals("FREQ")) {
					continue;
				}

				keyColumns.append(",\n\t" + definition);
				primaryKey.append("," + component.getConceptId());
				constraints.append(constraint);
				continue;
			}
			
			if (!component.isAttachedToSiblingGroup()) {
				continue;
			}

			attributeColumns.append(",\n\t" + definition);
			constraints.append(constraint);
		}		

		return	"\nCREATE TABLE " + keyFamily.getId() + "_SIBLING_GROUPS" 
					+ "\n\t( " + getSiblingKeyDefinition(keyFamily)
					+ ",\n\tDATA_SET VARCHAR(35)"
					+ keyColumns
					+ attributeColumns
					+ ",\n\tPRIMARY KEY (" + primaryKey + ")" 
					+ ",\n\tUNIQUE (SIBLING_KEY)" 
					+ ",\n\tFOREIGN KEY (DATA_SET) REFERENCES " + keyFamily.getId() + "_DATA_SETS" 
					+ constraints + " )" + getStatementTerminator();
	}


	public String getDataSetTableDefinition( final KeyFamily keyFamily ) {
		StringBuffer attributeColumns = new StringBuffer();
		StringBuffer constraints = new StringBuffer();
		
		Iterator it = keyFamily.componentIterator();		
		while (it.hasNext()) {
			KeyFamilyComponent component = (KeyFamilyComponent) it.next();
			
			String definition = getColumnDefinition(component);
			String constraint = getColumnConstraint(component);
			if (0 < constraint.length()) {
				constraint = ",\n\t" + constraint;
			}						
			
			if (component.isAttachedToDataSet()) {
				attributeColumns.append(",\n\t" + definition);
				constraints.append(constraint);
				continue;
			}
		}		

		return	"\nCREATE TABLE " + keyFamily.getId() + "_DATA_SETS" 
					+ "\n\t( DATA_SET VARCHAR(35)"
					+ attributeColumns.toString()
					+ ",\n\tPRIMARY KEY ( DATA_SET )" 
					+ constraints.toString() + " )" + getStatementTerminator();
	}


	public String getDataTableDefinition( final KeyFamily keyFamily ) {
		StringBuffer arrayCellColumns = new StringBuffer();
		StringBuffer attributeColumns = new StringBuffer();
		StringBuffer constraints = new StringBuffer();
		
		Iterator it = keyFamily.componentIterator();		
		while (it.hasNext()) {
			KeyFamilyComponent component = (KeyFamilyComponent) it.next();
			
			String definition = getColumnDefinition(component);
			String constraint = getColumnConstraint(component);
			if (0 < constraint.length()) {
				constraint = ",\n\t" + constraint;
			}						

			if (component.isArrayCell()) {
				arrayCellColumns.append(",\n\t" + definition);
				constraints.append(constraint);
				continue;
			}
			
			if (component.isAttachedToObservation()) {
				attributeColumns.append(",\n\t" + definition);
				constraints.append(constraint);
				continue;
			}
		}		

		if (attributeColumns.toString().equals("")) {
			return "";
		}

		return	"\nCREATE TABLE " + keyFamily.getId() + "_OBSERVATIONS" 
					+ "\n\t( " + getKeyDefinition(keyFamily)
					+ ",\n\t PERIOD " + getDateTimeDataType()
					+ arrayCellColumns
					+ attributeColumns
					+ ",\n\tPRIMARY KEY ( SERIES_KEY, PERIOD )" 
					+ ",\n\tFOREIGN KEY (SERIES_KEY) REFERENCES " + keyFamily.getId() + "_TIME_SERIES (SERIES_KEY)" 
					+ constraints + " )" + getStatementTerminator();
	}
	
		
	public int getKeyLength( final KeyFamily keyFamily ) {
		int length = 0;
		Iterator it = keyFamily.componentIterator();		
		while (it.hasNext()) {
			KeyFamilyComponent component = (KeyFamilyComponent) it.next();
			if (!component.isKeyStructureDimension()) continue;
			if (component.isTimeDimension()) continue;	
	
			TextFormat format = null;
			
			try {
				format = TextFormat.parse(component.getFormat());
			} catch (ParseException e) {
				componentHelper.logger.log(
					Level.SEVERE,
					componentHelper.logName,
					"getKeyLength",
					"Error in parsing format: " + e
				);			
				return -1;
			}			
				
			length += format.getReferenceLength() + 1;
		}
		return length - 1;
	}
	
	
	public String getKeyDefinition ( final KeyFamily keyFamily ) {
		return "SERIES_KEY VARCHAR(" + (getKeyLength( keyFamily ) + 35) + ")";
	}


	public String getSiblingKeyDefinition ( final KeyFamily keyFamily ) {
		return "SIBLING_KEY VARCHAR(" + (getKeyLength( keyFamily ) + 35 - 1) + ")";
	}


	public String getColumnDefinition(final KeyFamilyComponent component) {
		if (component.getConceptId().equals("OBS_VALUE")) return "OBS_VALUE DOUBLE PRECISION";
		if (component.getConceptId().equals("OBS_PRE_BREAK")) return "OBS_PRE_BREAK DOUBLE PRECISION";

		StringBuffer buffer = new StringBuffer(component.getConceptId());
		TextFormat format = null;
				
		try {
			format = TextFormat.parse(component.getFormat());
		} catch (ParseException e) {
			componentHelper.logger.log(
				Level.SEVERE,
				componentHelper.logName,
				"getColumnDefinition",
				"Error in parsing format: " + e
			);			
			return "";
		}			

		if (component.isCoded()) {
			buffer.append(" VARCHAR");
		} else {		
			if (format.isVariableLength()) {
				buffer.append(" VARCHAR");
			} else {
				buffer.append(" CHAR");
			}
		}

		buffer.append("(" + format.getReferenceLength() + ")");						
		
		return buffer.toString();
	}


	public String getColumnConstraint(final KeyFamilyComponent component) {
		StringBuffer buffer = new StringBuffer();
		if (component.isCoded()) {
			buffer.append("FOREIGN KEY (" + component.getConceptId() + ")" 
								+ " REFERENCES " + component.getCodeListId());
								
			if (currentSQLSyntax == SQL_SYNTAX_SQL92) {
				if (component.isKeyStructureDimension()) {
					buffer.append(" MATCH FULL");
				} else {
					buffer.append(" MATCH PARTIAL");
				}			
			}
		}
		return buffer.toString();
	}
	
	public String getDateTimeDataType() {
		if (currentSQLSyntax == SQL_SYNTAX_ORACLE8) return "DATE";
		return "TIMESTAMP";
	}
		
}
