/** Test Hyperion UDFs. */

package Int.ecb.gesmescb.test;

import java.lang.reflect.Method;
import java.io.*;
import java.util.*;
import java.text.*;

import com.hyperion.essbase.calculator.Context;
import com.hyperion.essbase.calculator.InvalidContextException;

/** Offers test methods to be used for EAACS testing.
 *
 * @author
 * <a href="mailto:sis.external@ecb.int">Bernhard Bodenstorfer</a>
 *
 * @version @(#)$Id: TestHyperion.java,v 1.1 2017/01/19 16:14:05 trainot Exp $
 */
public class TestHyperion {

	private static final DateFormat FORMAT
		= new SimpleDateFormat( "yyyy-MM-dd'T'HH:mm:ss" );

	private static int counter = 0;

	private static double accumulator = 0.0;

	public static String version () {
		return "0.20";
	}

	public static double add ( final double[] x, final double[] y ) {
		double value = 0;
		for ( int i = 0; i < x.length; ++i ) {
			value += x[i];
		}
		for ( int i = 0; i < y.length; ++i ) {
			value += y[i];
		}
		return value;
	}

	public static String exception ( final String message ) throws Exception {
		throw new Exception( message );
	}

	public static String runtimeException ( final String message ) throws RuntimeException {
		throw new RuntimeException( message );
	}

	public static String read ( final String filename ) {
		final StringBuffer buffer = new StringBuffer();
		try {
			final BufferedReader reader = new BufferedReader( new FileReader( filename ) );
			String line;
			while ( null != ( line = reader.readLine() ) ) {
				buffer.append( line );
				buffer.append( '\n' );
			}
			reader.close();
		} catch ( final Exception e ) {
			e.printStackTrace();
			return e.toString();
		}
		return buffer.toString();
	}

	public static String write ( final String filename, final String text ) {
		if ( ! filename.startsWith( "/tmp/" ) ) {
			throw new SecurityException( "Refused to write non-tmp file." );
		}
		try {
			final Writer writer = new FileWriter( filename );
			writer.write( text );
			writer.close();
		} catch ( final Exception e ) {
			e.printStackTrace();
			return e.toString();
		}
		return "Ok.";
	}

	public static String stdout ( final String text ) {
		try {
			System.out.println( text );
		} catch ( final Exception e ) {
			return e.toString();
		}
		return "Ok.";
	}

	public static double stdout ( final String[] items ) {
		try {
			for ( int i = 0; i < items.length; ++i ) {
				System.out.println( "item[" + i + "]=" + items[i] );
			}
		} catch ( final Exception e ) {
			return e.toString().length();
		}
		return 0;
	}

	public static String stderr ( final String text ) {
		try {
			System.err.println( text );
		} catch ( final Exception e ) {
			return e.toString();
		}
		return "Ok.";
	}

	public static String stderr (
		final String[] dimensions,
		final String[] coordinates,
		final double[] values
	) {
		for ( int i = 0; i < dimensions.length; ++i ) {
			System.out.println(
				dimensions[i]
				+ " = "
				+ coordinates[i]
			);
		}
		for ( int i = 0; i < values.length; ++i ) {
			System.out.println( "value[" + i + "] = " + values[i] );
		}
		return "Ok.";
	}

	public static String stderr (
		final String[] dimensions,
		final String[] coordinates,
		final double value
	) {
		for ( int i = 0; i < dimensions.length; ++i ) {
			System.out.println(
				dimensions[i]
				+ " = "
				+ coordinates[i]
			);
		}
		System.out.println( "value = " + value );
		return "Ok.";
	}

	public static String cp ( final String source, final String destination ) {
		try {
			if (
				! destination.startsWith( "/tmp/" )
				&&
				0 > destination.indexOf( "/udf/" )
			) {
				throw new SecurityException( "Refused to cp non-tmp file." );
			}
			final InputStream in = new BufferedInputStream( new FileInputStream( source ) );
			final OutputStream out = new BufferedOutputStream( new FileOutputStream( destination ) );
			int c;
			while ( 0 <= ( c = in.read() ) ) {
				out.write( c );
			}
			out.close();
			in.close();
			return "Ok.";
		} catch ( final Exception e ) {
			return e.toString();
		}
	}

	public static String rm ( final String filename )
		throws IOException
	{
		if (
			! filename.startsWith( "/tmp/" )
			&&
			0 > filename.indexOf( "/udf/" )
		) {
			throw new SecurityException( "Refused to remove non-tmp file." );
		}
		return new File( filename ).delete() ? "Ok." : "Ko.";
	}

	public static double zero () {
		return 0;
	}

	public static double addOne ( final double x ) {
		return x + 1;
	}

	public static double divide ( final double x, final double y ) {
		return x / y;
	}

	public static double length ( final String text ) {
		return text.length();
	}

	public static String ok () {
		return "Ok.";
	}

	public static String toEuro ( final double amount ) {
		return Double.toString( amount ) + " EUR";
	}

	public static String toAmount ( final double amount, final String currency ) {
		return Double.toString( amount ) + " " + currency;
	}

	public static double count ( final double[] values ) {
		return values.length;
	}

	public static double sum ( final double[] data ) {
		int i, n = data.length;
		double sum = 0.0d;
		for ( i = 0; i < n; ++i ) {
			double d = data [i];
			sum = sum + d;
		}
		return sum;
	}

	public static String concat ( final String[] strings ) {
		final StringBuffer buffer = new StringBuffer();
		for ( int i = 0; i < strings.length; ++i ) {
			buffer.append( strings[ i ] );
		}
		return buffer.toString();
	}

	public static void main ( final String[] args ) {
		System.out.println( "Hello EAACS! " + version() );
	}

	public static double accumulate ( double x ) {
		return accumulator += x;
	}

	public static void touch (
		final String filename, final String timeStamp
	) throws ParseException {
		final File file = new File( filename );

		final Date date;
		synchronized ( FORMAT ) {
			date = FORMAT.parse( timeStamp );
		}
		file.setLastModified( date.getTime() );
	}

	public static void appendLogLine(
		final String filename, final String text
	) {
		final String message =
		Integer.toString( counter++ )
		+ FORMAT.format( new Date() )
		+ ": "
		+ text
		+ "\n";
		try {
			final Writer writer = new FileWriter( filename, true );
			writer.write( message );
			writer.write( '\n' );
			writer.close();
		} catch ( final IOException ioe ) {
			System.err.println(
				"Could not append \"" + message + "\": " + ioe
			);
		}
	}

	public static void run ( final String command ) {
		try {
			final Runtime rt = Runtime.getRuntime();
			System.out.println( "Execute: " + command );
			final Process proc = rt.exec( command );

			final StreamGobbler outputGobbler
				= new StreamGobbler( proc.getInputStream(), "OUTPUT" );

			final StreamGobbler errorGobbler
				= new StreamGobbler( proc.getErrorStream(), "ERROR" );

			outputGobbler.start();
			errorGobbler.start();

			final int exitVal = proc.waitFor();
			System.out.println( "Exit value: " + exitVal );
		} catch ( final Throwable t ) {
			t.printStackTrace();
		}
	}

	/** Provides the entry point for the <code>RUNJAVA</code> statement. */
	public static void main ( final Context context, final String[] args ) {
		if ( 0 == args.length ) {
			try {
				context.sendMessage(
					"Usage: RUNJAVA "
					+ TestHyperion.class.getName()
					+ " method_name [arguments]"
				);
			} catch ( final InvalidContextException e ) {
				System.err.println( "SendMessage failed: " + e.toString() );
			}
		} else {
			try {
				final int argN = args.length - 1;
				final Method method = TestHyperion.class.getMethod(
					args[0],
					(Class[]) Collections.nCopies( argN, String.class )
					.toArray( new Class[0] )
				);
				final String[] argV = new String[ argN ];
				System.arraycopy( args, 1, argV, 0, argN );
				method.invoke( null, argV );
			} catch ( final Exception e ) {
				System.err.println(
					"Call \"" + args[0] + "\" failed with parameters:"
				);
				for ( int i = 1; i < args.length; ++i ) {
					System.err.println( "arg[" + i + "]=\"" + args[i] + "\"" );
				}
				System.err.println( "Exception was: " + e );
			}
		}
	}
}

class StreamGobbler extends Thread {
	final InputStream is;
	final String type;

	public StreamGobbler ( final InputStream is, final String type ) {
	this.is = is;
		this.type = type;
	}

	public void run () {
		try {
			final InputStreamReader isr = new InputStreamReader( is );
			final BufferedReader br = new BufferedReader( isr );
			try {
				String line = null;
				while ( ( line = br.readLine() ) != null) {
					System.out.println( type + ">" + line );
				}
			} finally {
				br.close();
				isr.close();
			}
		} catch ( final IOException ioe ) {
			ioe.printStackTrace();
		}
	}
}
