FUNCTION $compare_scalar
ARGUMENT a_scalar1, a_scalar2, a_log
	IF CLASS(a_scalar1) NE "SCALAR"
		WRITE NAME(a_scalar1) + " CLASS: " + CLASS(a_scalar1) + " != SCALAR" TO a_log
		RETURN FALSE
	END IF

	IF CLASS(a_scalar2) NE "SCALAR"
		WRITE NAME(a_scalar2) + " CLASS: " + CLASS(a_scalar2) + " != SCALAR" TO a_log
		RETURN FALSE
	END IF
	
	IF a_scalar1 NE a_scalar2 
		WRITE NAME(a_scalar1) + ": " + STRING(a_scalar1) + " != " + STRING(a_scalar2) TO a_log
		RETURN FALSE
	END IF
	
	RETURN TRUE
END FUNCTION


FUNCTION $compare_object
ARGUMENT a_object1, a_object2, a_log
	LOCAL SCALAR l_ok:BOOLEAN = TRUE

	IF CLASS(a_object1) NE CLASS(a_object2)
		WRITE NAME(a_object1) + " CLASS: " + CLASS(a_object1) + " != " +  CLASS(a_object2) TO a_log
		SET l_ok = FALSE
	END IF

	IF TYPE(a_object1) NE TYPE(a_object2)
		WRITE NAME(a_object1) + " TYPE: " + TYPE(a_object1) + " != " +  TYPE(a_object2) TO a_log
		SET l_ok = FALSE
	END IF

	IF DESCRIPTION(a_object1) NE DESCRIPTION(a_object2)
		WRITE NAME(a_object1) + " DESC: " + STRING(DESCRIPTION(a_object1)) + " != " +  STRING(DESCRIPTION(a_object2)) TO a_log
		SET l_ok = FALSE
	END IF

	IF DOCUMENTATION(a_object1) NE DOCUMENTATION(a_object2)
		WRITE NAME(a_object1) + " DOC: " + STRING(DOCUMENTATION(a_object1)) + " != " + STRING(DOCUMENTATION(a_object2)) TO a_log
		SET l_ok = FALSE
	END IF

	IF ASSOCIATE(a_object1) NE ASSOCIATE(a_object2)
		WRITE <CASE *> NAME(a_object1) + " ASSOCIATE: " + STRING(SUM(SL(ASSOCIATE(a_object1)))) + " != " + STRING(SUM(SL(ASSOCIATE(a_object2)))) TO a_log
		SET l_ok = FALSE
	END IF

	IF ALIAS(a_object1) NE ALIAS(a_object2)
		WRITE <CASE *> NAME(a_object1) + " ALIAS: " + STRING(SUM(SL(ALIAS(a_object1)))) + " != " + STRING(SUM(SL(ALIAS(a_object2)))) TO a_log
		SET l_ok = FALSE
	END IF

	RETURN l_ok
END FUNCTION


FUNCTION $compare_series
ARGUMENT a_series1, a_series2, a_log
	LOCAL SCALAR l_ok:BOOLEAN = TRUE

	IF CLASS(a_series1) NE "SERIES"
		WRITE NAME(a_series1) + " CLASS: " + CLASS(a_series1) + " != SERIES" TO a_log
		RETURN FALSE
	END IF

	IF CLASS(a_series2) NE "SERIES"
		WRITE NAME(a_series2) + " CLASS: " + CLASS(a_series2) + " != SERIES" TO a_log
		RETURN FALSE
	END IF
	
	IF INDEX(a_series1) NE INDEX(a_series2)
		WRITE NAME(a_series1) + " INDEX: " + INDEX(a_series1) + " != " + INDEX(a_series2) TO a_log
		RETURN FALSE
	END IF

	IF INDEX(a_series1) EQ "DATE" AND FREQUENCY(a_series1) NE FREQUENCY(a_series2)
		WRITE NAME(a_series1) + " FREQUENCY: " + FREQUENCY(a_series1) + " != " + FREQUENCY(a_series2) TO a_log
		RETURN FALSE
	END IF
	
	IF INDEX(a_series1) EQ "DATE" AND OBSERVED(a_series1) NE OBSERVED(a_series2)
		WRITE NAME(a_series1) + " OBSERVED: " + OBSERVED(a_series1) + " != " + OBSERVED(a_series2) TO a_log
		SET l_ok = FALSE
	END IF

	IF INDEX(a_series1) EQ "DATE" AND BASIS(a_series1) NE BASIS(a_series2)
		WRITE NAME(a_series1) + " BASIS: " + BASIS(a_series1) + " != " + BASIS(a_series2) TO a_log
		SET l_ok = FALSE
	END IF

	IF FIRSTVALUE(a_series1) NE FIRSTVALUE(a_series2)
		WRITE NAME(a_series1) + " FIRSTVALUE: " + STRING(FIRSTVALUE(a_series1)) + " != " + STRING(FIRSTVALUE(a_series2)) TO a_log
		SET l_ok = FALSE
	END IF

	IF LASTVALUE(a_series1) NE LASTVALUE(a_series2)
		WRITE NAME(a_series1) + " LASTVALUE: " + STRING(LASTVALUE(a_series1)) + " != " + STRING(LASTVALUE(a_series2)) TO a_log
		SET l_ok = FALSE
	END IF

	IF NOT MISSING(FIRSTVALUE(a_series1))
		IF INDEX(a_series1) EQ "DATE"
			DATE FIRSTVALUE(a_series1) TO LASTVALUE(a_series1)
		ELSE
			CASE FIRSTVALUE(a_series1) TO LASTVALUE(a_series1)
		END IF
		WHICH a_series1 NE a_series2
		IF @WHICH.PASSED GT 0
			IF INDEX(a_series1) EQ "DATE"
				WRITE NAME(a_series1) + " VALUES NOT EQUAL: " + @DATE TO a_log
			ELSE
				WRITE NAME(a_series1) + " VALUES NOT EQUAL: " + @CASE TO a_log
			END IF
			SET l_ok = FALSE
		END IF
	END IF

	RETURN l_ok
END FUNCTION


FUNCTION $compare_formula
ARGUMENT a_formula1, a_formula2, a_log
	IF CLASS(a_formula1) NE "FORMULA"
		WRITE NAME(a_formula1) + " CLASS: " + CLASS(a_series1) + " != FORMULA" TO a_log
		RETURN FALSE
	END IF

	IF CLASS(a_formula2) NE "FORMULA"
		WRITE NAME(a_formula2) + " CLASS: " + CLASS(a_series2) + " != FORMULA" TO a_log
		RETURN FALSE
	END IF

	IF SOURCE(a_formula1) NE SOURCE(a_formula2)
		WRITE NAME(a_formula1) + " SOURCE: " + STRING(SOURCE(a_formula1)) + " != " + STRING(SOURCE(a_formula2)) TO a_log
		RETURN FALSE
	END IF

	RETURN TRUE
END FUNCTION


FUNCTION $compare_database
ARGUMENT a_db1, a_db2, a_log
	LOCAL SCALAR l_ok:BOOLEAN = TRUE
	LOCAL SCALAR l_db1_objects:NAMELIST = WILDLIST(a_db1, "?")
	LOCAL SCALAR l_db2_objects:NAMELIST = WILDLIST(a_db2, "?")
	
	WRITE "Starting to compare " + FILESPEC(a_db1) + " to " + FILESPEC(a_db2)
	
	IF DOCUMENTATION(a_db1) NE DOCUMENTATION(a_db2)
		WRITE "DOCUMENTATION: " + STRING(DOCUMENTATION(a_db1)) + " != " +  STRING(DOCUMENTATION(a_db2)) TO a_log
		SET l_ok = FALSE
	END IF

	IF DESCRIPTION(a_db1) NE DESCRIPTION(a_db2)
		WRITE "DESCRIPTION: " + STRING(DESCRIPTION(a_db1)) + " != " +  STRING(DESCRIPTION(a_db2)) TO a_log
		SET l_ok = FALSE
	END IF

	LOOP FOR l_name IN l_db1_objects EXCEPT l_db2_objects
		WRITE NAME(l_name) + " not in db2" TO a_log
		SET l_ok = FALSE
	END LOOP

	LOOP FOR l_name IN l_db2_objects EXCEPT l_db1_objects
		WRITE NAME(l_name) + " not in db1" TO a_log
		SET l_ok = FALSE
	END LOOP	

	LOOP FOR l_name IN l_db2_objects INTERSECT l_db1_objects
		SET l_ok = ($compare_object(a_db1'l_name, a_db2'l_name, a_log) AND l_ok)

		IF CLASS(a_db1'l_name) EQ "SCALAR"
			SET l_ok = ($compare_scalar(a_db1'l_name, a_db2'l_name, a_log) AND l_ok)
		ELSE IF CLASS(a_db1'l_name) EQ "SERIES"
			SET l_ok = ($compare_series(a_db1'l_name, a_db2'l_name, a_log) AND l_ok)
		ELSE IF CLASS(a_db1'l_name) EQ "FORMULA"
			SET l_ok = ($compare_formula(a_db1'l_name, a_db2'l_name, a_log) AND l_ok)
		ELSE
			TYPE NAME(l_name) + " not checked"
		END IF
	END LOOP

	WRITE "Finished comparing"
	
	RETURN l_ok
END FUNCTION
