// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.time;

import java.lang.ClassCastException;
import java.util.Date;

import Int.ecb.gesmescb.time.Interval;

/* ***************************** Class ****************************** */

/**
 * <P>Provides a sceletal implementation of {@link Interval}.</P>
 *
 * @author
 * <A HREF="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</A>
 *
 * @version @(#)$Id: AbstractInterval.java,v 1.1 2017/01/19 16:14:03 trainot Exp $
 */
public abstract class AbstractInterval implements Interval {

	/* ====================== Constructors ====================== */

	/** Construct one. */
	protected AbstractInterval () {}

	/* ===================== Public Methods ===================== */

	public boolean contains ( final Date date ) {
		return
			getBegin().compareTo( date ) <= 0
			&&
			getEnd().compareTo( date ) > 0;
	}

	public int compareTo ( final Object o ) throws ClassCastException {
		return compareTo( (Interval) o );
	}

	/**
	 * Acts like {@link #compareTo(Object)} for <CODE>Interval</CODE>s.
	 */
	public int compareTo ( final Interval interval ) {
		final int rv = getBegin().compareTo( interval.getBegin() );
		if ( rv != 0 ) return rv;
		return getEnd().compareTo( interval.getEnd() );
	}

	/** Tests equality of two objects concerning their
	 * <code>Interval</code> properties.
	 */
	public static boolean equals (
		final Interval interval1, final Interval interval2
	) {
		return
			interval1.getBegin().equals( interval2.getBegin() )
			&&
			interval1.getEnd().equals( interval2.getEnd() );
	}

	/**
	 * Computes a hash code compatible with {@link #equals(java.lang.Object)}.
	 */
	public int hashCode () {
		return
			getBegin().hashCode()
			^
			getEnd().hashCode();
	}

	/** Gives a human readable representation. */
	public String toString () {
		return
			getClass().getName() + "(from "
			+ getBegin() + " to " + getEnd() + ")";
	}
}
