// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.time;

import java.util.Date;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import Int.ecb.gesmescb.time.AbstractInterval;

/* ***************************** Class ****************************** */

/**
 * Tests the class {@link AbstractInterval}.
 *
 * @author
 * <A HREF="mailto:Sami.Airo@ECB.int">Sami Airo</A>
 * @author
 * <A HREF="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</A>
 *
 * @version @(#)$Id: AbstractIntervalTest.java,v 1.1 2017/01/19 16:14:03 trainot Exp $
 */

public class AbstractIntervalTest extends TestCase {

	/* ======================= Constants ======================== */

	protected static final DateFormat FORMAT = new SimpleDateFormat( "yyyyMMddHHmmss" );

	/* ====================== Constructors ====================== */

	/**
	 * Constructs an AbstractIntervalTest with the specified name.
	 *
	 * @param name Test case name
	 */
	public AbstractIntervalTest ( final String name ) {
		super( name );
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite
	 */
	public static Test suite () {
		TestSuite suite = new TestSuite( AbstractIntervalTest.class );
		return suite;
	}

	/**
	 * Runs the test case.
	 */
	public static void main ( final String args[] ) {
		String[] testCaseName = { AbstractIntervalTest.class.getName() };
		junit.textui.TestRunner.main( testCaseName );
	}

	/* ---------------------- Test Methods ---------------------- */

	public void testCompare () {
		final TestInterval[][] interval
		= new TestInterval[4][4];
		for ( int i = 0; i < 4; ++i )
			for ( int j = 0; j < 4; ++j )
				interval[i][j] = new TestInterval(
					new Date( i ), new Date( j )
				);

		assertEquals(
			"diagonal 1", 0, interval[0][0].compareTo( interval[0][0] )
		);
		assertEquals(
			"diagonal 2", 0, interval[2][3].compareTo( interval[2][3] )
		);

		assertEquals(
			"weak sub 1", -1, interval[0][0].compareTo( interval[0][1] )
		);
		assertEquals(
			"weak sub 2", -1, interval[1][2].compareTo( interval[1][3] )
		);
		assertEquals(
			"strong sub 1", -1, interval[1][0].compareTo( interval[2][3] )
		);
		assertEquals(
			"strong sub 2", -1, interval[1][3].compareTo( interval[2][0] )
		);
		assertEquals(
			"strong sub 3", -1, interval[0][3].compareTo( interval[1][3] )
		);

		assertEquals(
			"weak super 1", 1, interval[0][3].compareTo( interval[0][1] )
		);
		assertEquals(
			"weak super 2", 1, interval[1][2].compareTo( interval[1][1] )
		);
		assertEquals(
			"strong super 1", 1, interval[1][0].compareTo( interval[0][1] )
		);
		assertEquals(
			"strong super 2", 1, interval[1][2].compareTo( interval[0][1] )
		);
		assertEquals(
			"strong super 3", 1, interval[1][2].compareTo( interval[0][3] )
		);
	}

	public void testContains () {
		final Date
			d1 = new Date( 1 ),
			d2 = new Date( 2 ),
			d3 = new Date( 3 ),
			d4 = new Date( 4 ),
			d5 = new Date( 5 );
		final TestInterval
			ti1 = new TestInterval( new Date( 2 ), new Date( 4 ) ),
			ti2 = new TestInterval( new Date( 2 ), new Date( 2 ) ),
			ti3 = new TestInterval( new Date( 4 ), new Date( 2 ) );

		assertTrue( "ti1 nc d1", !ti1.contains( d1 ) );
		assertTrue( "ti1 c d2", ti1.contains( d2 ) );
		assertTrue( "ti1 c d3", ti1.contains( d3 ) );
		assertTrue( "ti1 nc d4", !ti1.contains( d4 ) );

		assertTrue( "ti2 nc d1", !ti2.contains( d1 ) );
		assertTrue( "ti2 nc d2", !ti2.contains( d2 ) );
		assertTrue( "ti2 nc d3", !ti2.contains( d3 ) );

		assertTrue( "ti3 nc d1", !ti3.contains( d1 ) );
		assertTrue( "ti3 nc d2", !ti3.contains( d2 ) );
		assertTrue( "ti3 nc d3", !ti3.contains( d3 ) );
		assertTrue( "ti3 nc d4", !ti3.contains( d4 ) );
		assertTrue( "ti3 nc d5", !ti3.contains( d5 ) );
	}
}

/** A very straightforward {@link Date}-based implementation for test. */
class TestInterval extends AbstractInterval {
	private final Date begin, end;
	public TestInterval ( final Date begin, final Date end ) {
		this.begin = begin;
		this.end = end;
	}
	public Date getBegin () { return begin; }
	public Date getEnd () { return end; }
}
