// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2009, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.time;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/* ***************************** Class ****************************** */

/**
 * Tests the class {@link TimeZoneUtilTest}.
 *
 * @author
 * <A HREF="mailto:Francoid.Hoeltzli@ecb.europa.eu">Francois Hoeltzli</A>
 * @author
 * <A HREF="mailto:Xavier.Sosnovsky@ecb.europa.eu">Xavier Sosnovsky</A>
 *
 * @version @(#)$Id: TimeZoneUtilTest.java,v 1.2 2018/03/08 10:16:27 kortest Exp $
 */

public class TimeZoneUtilTest extends TestCase {

	/* ======================== Fields ========================== */
	private TimeZoneUtil util;
	
	/* ====================== Constructors ====================== */

	/**
	 * Constructs a TimeZoneUtilTest with the specified name.
	 *
	 * @param name Test case name
	 */
	public TimeZoneUtilTest ( final String name ) {
		super( name );
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite
	 */
	public static Test suite () {
		TestSuite suite = new TestSuite( TimeZoneUtilTest.class );
		return suite;
	}

	/**
	 * Runs the test case.
	 */
	public static void main ( final String args[] ) {
		String[] testCaseName = { TimeZoneUtilTest.class.getName() };
		junit.textui.TestRunner.main( testCaseName );
	}

	/* ---------------------- Test Methods ---------------------- */
	
	public void setUp() {
		util = TimeZoneUtil.getInstance();
	}
	
	public void testNothingToDo() {
		String input, output, expected;
		input = "2009-04-08T15:43:59+0100";
		output = util.normalize( input );
		expected = "2009-04-08T15:43:59+0100";
        assertEquals(
        	"tried to change a string already conforming rfc822!",
        	expected, output
        );
        // try invalid one
        input    = "2009-04-08T15:43:59+43:21";
        expected = "2009-04-08T15:43:59+4321";
		output = util.normalize( input );
		assertEquals(
        	"tried to change a string conforming rfc822 format but invalid (+4321)",
        	expected, output
        );        
		// try the horror that RFC822 allows
		input = "2009-04-08T15:43:59+01:00";
		output = util.normalize( input, util.ISO8601 );
		expected = "2009-04-08T15:43:59+01:00";
        assertEquals(
        	"failed to change a string already conforming rfc822 with colon in the zone!",
        	expected, output
        );
	}
	
	public void testMilitaryZone() {
		String input, expected, output;
		input = "2009-04-08T15:43:59A";
		expected = "2009-04-08T15:43:59+0100";
		output = util.normalize( input );
        assertEquals(
        	"wrong A zone",
        	expected, output
        );
        input = "2009-04-08T15:43:59M";
		expected = "2009-04-08T15:43:59+1200";
		output = util.normalize( input );
        assertEquals(
        	"wrong M zone",
        	expected, output
        );
        input = "2009-04-08T15:43:59N";
		expected = "2009-04-08T15:43:59-1200";
		output = util.normalize( input );
        assertEquals(
        	"wrong N zone",
        	expected, output
        );
        input = "2009-04-08T15:43:59Y";
		expected = "2009-04-08T15:43:59-0100";
		output = util.normalize( input );
        assertEquals(
        	"wrong Y zone",
        	expected, output
        );
        input = "2009-04-08T15:43:59Z";
		expected = "2009-04-08T15:43:59+0000";
		output = util.normalize( input );
        assertEquals(
        	"wrong Z zone",
        	expected, output
        );
	}
	
	public void testNoZone() {
		String input, expected, output;
		input = "2009-04-08T15:43:59";
		expected = "2009-04-08T15:43:59+0000";
		output = util.normalize( input );
        assertEquals(
        	"wrong no zone",
        	expected, output
        );
	}


	public void testOtherZones() {
		String input, expected, output;
		input = "2009-04-08T15:43:59MSK";
		expected = "2009-04-08T15:43:59+0300";
		output = util.normalize( input );
        assertEquals(
        	"failed from moscow timezone",
        	expected, output
        );
        input = "2009-04-08T15:43:59WWWW";
		expected = null;
		output = util.normalize( input );
        assertEquals(
        	"failed to return null with WWWW",
        	expected, output
        );
        
	}
	
	public void testTimeStampCrappy() {
		String input, expected, output;
		input = "2009-04-08 15:43:59Z";
		expected = null;
		output = util.normalize( input );
        assertEquals(
        	"failed to return null with badly formatted timestamp",
        	expected, output
        );
	}
	
	public void testWithMS (){
		String input = "2018-02-27T11:23:25.895-0700";
		String expected = "2018-02-27T11:23:25-0700";
		String output = util.normalize( input );
		assertEquals("Normalizing with MS failed: ", expected, output);
	}
	
}
