// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (C) 2004, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.util;

import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;

import Int.ecb.gesmescb.KeyFamily;
import Int.ecb.gesmescb.KeyFamilyComponent;
import Int.ecb.gesmescb.Attribute;
import Int.ecb.gesmescb.util.FilterIterator.Condition;

/* ***************************** Class ****************************** */

/**
 * Filters {@link Int.ecb.gesmescb.AbstractCube} objects
 * with a given <code>key</code>.
 *
 * @author
 * <A HREF="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</A>
 *
 * @version @(#)$Id: AttachmentLevelEqualsCondition.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */

public class AttachmentLevelEqualsCondition implements Condition {

	/* ========================= Fields ========================= */

	/** The names of all concepts with the given attachment level. */
	private final Collection conceptIds;

	/* ====================== Constructors ====================== */

	/**
	 * Construct a new <code>AttachmentLevelEqualsCondition</code>
	 * to filter all {@link Attribute}s with matching attachment level.
	 *
	 * @param keyFamily corresponding to the data set.
	 * @param attachmentLevel to require from accepted attributes.
	 *
	 * @see KeyFamilyComponent for <code>attachmentLevel</code> codes.
	 */
	public AttachmentLevelEqualsCondition (
		final KeyFamily keyFamily,
		final String attachmentLevel
	) {
		conceptIds = new HashSet();
		for (
			final Iterator iterator = keyFamily.componentIterator();
			iterator.hasNext();
		) {
			final KeyFamilyComponent component
			= (KeyFamilyComponent) iterator.next();
			if (
				attachmentLevel.equals( component.getAttachmentLevel() )
			) {
				conceptIds.add( component.getConceptId() );
			}
		}
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Returns whether the <code>object</code> is an
	 * {@link Int.ecb.gesmescb.AbstractCube} with matching <code>key</code>.
	 */
	public boolean accept ( final Object object ) {
		return object instanceof Attribute
			? conceptIds.contains( ( (Attribute) object ).getName() )
			: false;
	}

	public String toString () {
		return super.toString() + "( \"" + conceptIds + "\" )";
	}
}
