// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2004, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.util;

import java.util.Date;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import Int.ecb.gesmescb.*;

/* ***************************** Class ****************************** */

/**
 * Tests the {@link AttachmentLevelEqualsCondition}.
 *
 * @author
 * <a href="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</a>
 *
 * @version @(#)$Id: AttachmentLevelEqualsConditionTest.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */
public class AttachmentLevelEqualsConditionTest extends TestCase {

	/* ======================= Constants ======================== */

	static final KeyFamily KEY_FAMILY = new KeyFamily(
		"KEY_FAMILY", "A key family", "", new Date(),
		new KeyFamilyComponent[]{
			new KeyFamilyComponent(
				"FREQ", KeyFamilyComponent.FREQUENCY_DIMENSION,
				"1", "A1", "", "", "CL_FREQ", true
			),
			new KeyFamilyComponent(
				"DIM", KeyFamilyComponent.KEY_STRUCTURE_DIMENSION,
				"2", "AN..6", "", "", "CL_DIM", true
			),
			new KeyFamilyComponent(
				"ATT1", KeyFamilyComponent.ATTRIBUTE,
				"", "AN..4",
				KeyFamilyComponent.MANDATORY,
				KeyFamilyComponent.ATTACHED_TO_DATA_SET, ""
			),
			new KeyFamilyComponent(
				"ATT2", KeyFamilyComponent.ATTRIBUTE,
				"", "AN..4",
				KeyFamilyComponent.MANDATORY,
				KeyFamilyComponent.ATTACHED_TO_SIBLING_GROUP, ""
			),
			new KeyFamilyComponent(
				"ATT3", KeyFamilyComponent.ATTRIBUTE,
				"", "AN..4",
				KeyFamilyComponent.MANDATORY,
				KeyFamilyComponent.ATTACHED_TO_TIME_SERIES, ""
			),
			new KeyFamilyComponent(
				"ATT4", KeyFamilyComponent.ATTRIBUTE,
				"", "AN..4",
				KeyFamilyComponent.MANDATORY,
				KeyFamilyComponent.ATTACHED_TO_OBSERVATION, ""
			)
		}
	);

	private static final Attribute
		AT1 = new Attribute( "", "", "", "ATT1", "VAL" ),
		AT2 = new Attribute( "PT", "", "", "ATT2", "VAL" ),
		AT3 = new Attribute( "Q.PT", "", "", "ATT3", "VAL" ),
		AT4 = new Attribute( "Q.PT", "20041", "608", "ATT4", "VAL" );

	/* ====================== Constructors ====================== */

	public AttachmentLevelEqualsConditionTest( final String name ) {
		super( name );
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite
	 */
	public static Test suite () {
		final TestSuite suite = new TestSuite(
			AttachmentLevelEqualsConditionTest.class
		);
		return suite;
	}

	/**
	 * @param arg the array of argument {@link String}s
	 */
	public static void main ( final String[] arg ) {
		final String[] testCaseName = {
			AttachmentLevelEqualsConditionTest.class.getName()
		};
		junit.textui.TestRunner.main( testCaseName );
	}

	/* ---------------------- Test Methods ---------------------- */

	public void test () {
		final FilterIterator.Condition dataSetCondition
		= new AttachmentLevelEqualsCondition(
			KEY_FAMILY, KeyFamilyComponent.ATTACHED_TO_DATA_SET
		);
		assertTrue( "ds accepts at1", dataSetCondition.accept( AT1 ) );
		assertTrue( "ds rejects at2", ! dataSetCondition.accept( AT2 ) );
		assertTrue( "ds rejects at3", ! dataSetCondition.accept( AT3 ) );
		assertTrue( "ds rejects at4", ! dataSetCondition.accept( AT4 ) );

		final FilterIterator.Condition groupCondition
		= new AttachmentLevelEqualsCondition(
			KEY_FAMILY, KeyFamilyComponent.ATTACHED_TO_SIBLING_GROUP
		);
		assertTrue( "sg rejects at1", ! groupCondition.accept( AT1 ) );
		assertTrue( "sg accepts at2", groupCondition.accept( AT2 ) );
		assertTrue( "sg rejects at3", ! groupCondition.accept( AT3 ) );
		assertTrue( "sg rejects at4", ! groupCondition.accept( AT4 ) );

		final FilterIterator.Condition seriesCondition
		= new AttachmentLevelEqualsCondition(
			KEY_FAMILY, KeyFamilyComponent.ATTACHED_TO_TIME_SERIES
		);
		assertTrue( "ts rejects at1", ! seriesCondition.accept( AT1 ) );
		assertTrue( "ts rejects at2", ! seriesCondition.accept( AT2 ) );
		assertTrue( "ts accepts at3", seriesCondition.accept( AT3 ) );
		assertTrue( "ts rejects at4", ! seriesCondition.accept( AT4 ) );

		final FilterIterator.Condition obsCondition
		= new AttachmentLevelEqualsCondition(
			KEY_FAMILY, KeyFamilyComponent.ATTACHED_TO_OBSERVATION
		);
		assertTrue( "obs rejects at1", ! obsCondition.accept( AT1 ) );
		assertTrue( "obs rejects at2", ! obsCondition.accept( AT2 ) );
		assertTrue( "obs rejects at3", ! obsCondition.accept( AT3 ) );
		assertTrue( "obs accepts at4", obsCondition.accept( AT4 ) );
	}
}
