// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (C) 2004, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.util;

/* ***************************** Class ****************************** */

/**
 * <p>Filters given class or interface instances.</p>
 *
 * @author
 * <a href="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</a>
 *
 * @version @(#)$Id: CombinedCondition.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */
public class CombinedCondition implements FilterIterator.Condition {

	/* ======================= Constants ======================== */

	public static final int AND = 1;
	public static final int OR = 2;

	/* ========================= Fields ========================= */

	private final boolean isAnd;
	private final FilterIterator.Condition[] conditions;

	/* ====================== Constructors ====================== */

	/** Create a new instance.
	 * @param operation to combine the conditions.
	 * Must be one of {@link #AND} or {@link #OR}.
	 * @param conditions to be combined
	 */
	public CombinedCondition (
		final int operation,
		final FilterIterator.Condition[] conditions
	) {
		if ( operation != AND && operation != OR ) {
			throw new IllegalArgumentException(
				"Invalid operation " + operation
				+ " to construct a CombinedCondition."
			);
		}
		isAnd = operation == AND;
		this.conditions = (FilterIterator.Condition[]) conditions.clone();
	}

	/* ===================== Public Methods ===================== */

	/** Whether the object belongs to the class given at construction. */
	public boolean accept ( final Object object ) {
		for ( int i = 0; i < conditions.length; ++i ) {
			if ( isAnd ^ conditions[i].accept( object ) ) {
				return !isAnd;
			}
		}
		return isAnd;
	}

	public String toString () {
		return getClass().getName()
		+ "(" + conditions + ")";
	}
}
