// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2004, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.util;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/* ***************************** Class ****************************** */

/**
 * Tests the {@link CombinedCondition}.
 *
 * @author
 * <a href="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</a>
 *
 * @version @(#)$Id: CombinedConditionTest.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */
public class CombinedConditionTest extends TestCase {

	/* ====================== Constructors ====================== */

	public CombinedConditionTest( final String name ) {
		super( name );
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite
	 */
	public static Test suite () {
		final TestSuite suite = new TestSuite( CombinedConditionTest.class );
		return suite;
	}

	/**
	 * @param arg the array of argument {@link String}s
	 */
	public static void main ( final String[] arg ) {
		final String[] testCaseName = {
			CombinedConditionTest.class.getName()
		};
		junit.textui.TestRunner.main( testCaseName );
	}

	/* ---------------------- Test Methods ---------------------- */

	public void testNoConditions () {
		final FilterIterator.Condition andEmpty = new CombinedCondition(
			CombinedCondition.AND,
			new FilterIterator.Condition[]{}
		);
		final FilterIterator.Condition orEmpty = new CombinedCondition(
			CombinedCondition.OR,
			new FilterIterator.Condition[]{}
		);
		assertTrue( "No AND", andEmpty.accept( new Object() ) );
		assertTrue( "No OR", !orEmpty.accept( new Object() ) );
	}

	public void testOneCondition () {
		final FilterIterator.Condition andTrue = new CombinedCondition(
			CombinedCondition.AND,
			new FilterIterator.Condition[]{
				new ConstantCondition( true )
			}
		);
		final FilterIterator.Condition andFalse = new CombinedCondition(
			CombinedCondition.AND,
			new FilterIterator.Condition[]{
				new ConstantCondition( false )
			}
		);
		final FilterIterator.Condition orTrue = new CombinedCondition(
			CombinedCondition.OR,
			new FilterIterator.Condition[]{
				new ConstantCondition( true )
			}
		);
		final FilterIterator.Condition orFalse = new CombinedCondition(
			CombinedCondition.OR,
			new FilterIterator.Condition[]{
				new ConstantCondition( false )
			}
		);
		assertTrue( "True AND", andTrue.accept( new Object() ) );
		assertTrue( "False AND", !andFalse.accept( new Object() ) );
		assertTrue( "True OR", orTrue.accept( new Object() ) );
		assertTrue( "False OR", !orFalse.accept( new Object() ) );
	}

	public void testTwoConditions () {
		final FilterIterator.Condition andTrue = new CombinedCondition(
			CombinedCondition.AND,
			new FilterIterator.Condition[]{
				new ConstantCondition( true ),
				new ConstantCondition( true )
			}
		);
		final FilterIterator.Condition andFalse = new CombinedCondition(
			CombinedCondition.AND,
			new FilterIterator.Condition[]{
				new ConstantCondition( true ),
				new ConstantCondition( false )
			}
		);
		final FilterIterator.Condition orTrue = new CombinedCondition(
			CombinedCondition.OR,
			new FilterIterator.Condition[]{
				new ConstantCondition( false ),
				new ConstantCondition( true )
			}
		);
		final FilterIterator.Condition orFalse = new CombinedCondition(
			CombinedCondition.OR,
			new FilterIterator.Condition[]{
				new ConstantCondition( false ),
				new ConstantCondition( false )
			}
		);
		assertTrue( "True AND", andTrue.accept( new Object() ) );
		assertTrue( "False AND", !andFalse.accept( new Object() ) );
		assertTrue( "True OR", orTrue.accept( new Object() ) );
		assertTrue( "False OR", !orFalse.accept( new Object() ) );
	}
}

class ConstantCondition implements FilterIterator.Condition {

	private final boolean acceptance;

	public ConstantCondition ( final boolean acceptance ) {
		this.acceptance = acceptance;
	}

	public boolean accept ( final Object object ) {
		return acceptance;
	}
}
