// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.util;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import Int.ecb.logging.Logger;

import Int.ecb.gesmescb.util.Named;
import Int.ecb.gesmescb.util.ComponentHelper;

/* ***************************** Class ****************************** */

/**
 * <P>Tests the implementation of the class {@link ComponentHelper}.</P>
 *
 * @author
 * <A HREF="mailto:Sami.Airo@ECB.int">Sami Airo</A>
 * @author
 * <A HREF="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</A>
 *
 * @version @(#)$Id: ComponentHelperTest.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */

public class ComponentHelperTest extends TestCase {

	/* ========================= Fields ========================= */

	private final String[] names;
	private final TestComponent[] components;

	/* ====================== Constructors ====================== */

	/**
	 * Constructs a ComponentHelperTest with the specified name.
	 *
	 * @param name Test case name
	 */
	public ComponentHelperTest ( final String name ) {
		super( name );
		names = new String[] { "Sami", "", null };
		components = new TestComponent[names.length];
	}

	/* ================== Protected Methods ===================== */

	/**
	 * Sets up the test fixture.
	 */
	protected void setUp() {
		System.setProperty(
			TestComponent.class.getName() + ".Sami.someprop",
			"some"
		);
		System.setProperty(
			TestComponent.class.getName() + ".anyprop",
			"any"
		);
		// Cannot be initialized together with components, because
		// this would result in a premature attempt to
		// load the properties file.
		for ( int i = 0; i < names.length; ++i ) {
			components[i] = new TestComponent( names[i] );
		}
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite
	 */
	public static Test suite () {
		TestSuite suite = new TestSuite( ComponentHelperTest.class );
		return suite;
	}

	/**
	 * Run the test case.
	 */
	public static void main ( final String args[] ) {
		String[] testCaseName = { ComponentHelperTest.class.getName() };
		junit.textui.TestRunner.main( testCaseName );
	}

	/* ---------------------- Test Methods ---------------------- */

	public void testName () {
		for ( int i = 0; i < names.length; ++i ) {
			assertSame( names[i], components[i].getName() );
		}
	}

	public void testLogName () {
		assertEquals(
			"Int.ecb.gesmescb.util.TestComponent.Sami",
			components[0].ch.logName
		);
		assertEquals(
			"Int.ecb.gesmescb.util.TestComponent",
			components[1].ch.logName
		);
		assertEquals(
			"Int.ecb.gesmescb.util.TestComponent",
			components[2].ch.logName
		);
	}

	public void testLogger () {
		assertSame(
			Logger.getLogger(
				"Int.ecb.gesmescb.util.TestComponent.Sami"
			),
			components[0].ch.logger
		);
		assertSame(
			Logger.getLogger(
				"Int.ecb.gesmescb.util.TestComponent"
			),
			components[1].ch.logger
		);
		assertSame(
			Logger.getLogger(
				"Int.ecb.gesmescb.util.TestComponent"
			),
			components[2].ch.logger
		);
	}

	public void testPreference () {
		assertEquals(
			"some",
			components[0].ch.getPreference( "someprop", "nix" )
		);
		for ( int i = 1; i < names.length; ++i ) {
			assertEquals(
				"components[" + i + "].anyprop",
				"any",
				components[i].ch.getPreference( "anyprop", "nix" )
			);
		}
	}
}

class TestComponent implements Named {
	public final ComponentHelper ch;
	public TestComponent ( final String name ) {
		ch = new ComponentHelper( name, this );
	}
	public String getName () {
		return ch.name;
	}
}
