// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2007, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.util;

import java.util.List;
import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;

/* ***************************** Class ****************************** */

/**
 * Decomposes keys and gives access to the used coordinate code values.
 *
 * @author
 * <a href="mailto:sis.external@ecb.int">Bernhard Bodenstorfer</a>
 *
 * @version @(#)$Id: CoordinateCollector.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */
public class CoordinateCollector {

	/* ========================= Fields ========================= */

	private Map[] dimensionMaps;

	/* ====================== Constructors ====================== */

	/** Create a new instance.
	 * @param iterator to get {@link AbstractCube}s from.
	 */
	public CoordinateCollector () {
		dimensionMaps = null;
	}

	/* ===================== Public Methods ===================== */

	public void addKey ( final String key ) throws DimensionCountException {
		final List coordinates = new ArrayList();
		for (
			final KeyTokenizer tokenizer = new KeyTokenizer( key );
			tokenizer.hasNext();
		) {
			coordinates.add( tokenizer.nextToken() );
		}
		if ( null == dimensionMaps ) {
			dimensionMaps = new Map[ coordinates.size() ];
			for ( int i = 0; i < dimensionMaps.length; ++i ) {
				dimensionMaps[ i ] = new HashMap();
			}
		} else if ( dimensionMaps.length != coordinates.size() ) {
			throw new DimensionCountException(
				"Series key \"" + key
				+ "\" should have " + dimensionMaps.length
				+ " dimensions, but has " + coordinates.size() + "."
			);
		}

		final Iterator iterator = coordinates.iterator();
		for (
			int i = 0;
			i < dimensionMaps.length;
			++i
		) {
			final String coordinate = (String) iterator.next();
			if ( ! dimensionMaps[ i ].containsKey( coordinate ) ) {
				dimensionMaps[ i ].put(
					coordinate,
					new Integer( dimensionMaps[i].size() )
				);
			}
		}
	}

	public int getIndex(
		final int dimIndex, final String coordinate
	) throws UnknownCodeException {
		if ( null == dimensionMaps ) {
			throw new IllegalStateException( "No keys registered yet." );
		}
		final Integer index
			= (Integer) dimensionMaps[ dimIndex ].get( coordinate );
		if ( null == index ) {
			throw new UnknownCodeException(
				"Dimension[" + dimIndex + "] does not know code \""
				+ coordinate + "\"."
			);
		} else {
			return index.intValue();
		}
	}

	public void clear () {
		dimensionMaps = null;
	}
}

/* ***************************** Class ****************************** */

class DimensionCountException extends Exception {
	/**
	 *   serialVersionUID generated by eclipse.
	 */
	private static final long serialVersionUID = 6594210566792364124L;

	public DimensionCountException ( final String message ) {
		super( message );
	}
}

/* ***************************** Class ****************************** */

class UnknownCodeException extends Exception {
	/**
	 *   serialVersionUID generated by eclipse.
	 */
	private static final long serialVersionUID = -2404017209773215580L;

	public UnknownCodeException ( final String message ) {
		super( message );
	}
}
