// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (C) 2007, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.util;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/* ***************************** Class ****************************** */

/**
 * Tests the {@link CoordinateCollector}.
 *
 * @author
 * <a href="mailto:sis.external@ecb.int">Bernhard Bodenstorfer</a>
 *
 * @version @(#)$Id: CoordinateCollectorTest.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */
public class CoordinateCollectorTest extends TestCase {

	/* ====================== Constructors ====================== */

	public CoordinateCollectorTest( final String name ) {
		super( name );
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite
	 */
	public static Test suite () {
		final TestSuite suite = new TestSuite( CoordinateCollectorTest.class );
		return suite;
	}

	/**
	 * @param arg the array of argument {@link String}s
	 */
	public static void main ( final String[] arg ) {
		final String[] testCaseName = {
			CoordinateCollectorTest.class.getName()
		};
		junit.textui.TestRunner.main( testCaseName );
	}

	/* ---------------------- Test Methods ---------------------- */

	public void testEmpty () throws Exception {
		final CoordinateCollector collector = new CoordinateCollector();
		try {
			collector.getIndex( 0, "E" );
			fail( "Must recognise absence of keys." );
		} catch ( final IllegalStateException e ) {}

		collector.addKey( "H" );
		try {
			collector.getIndex( 0, "Q" );
			fail( "Q not registered." );
		} catch ( final UnknownCodeException e ) {}
		assertEquals( "H is in.", 0, collector.getIndex( 0, "H" ) );

		collector.addKey( "Q" );
		assertEquals( "Q is in.", 1, collector.getIndex( 0, "Q" ) );

		collector.addKey( "A" );
		assertEquals( "A is in.", 2, collector.getIndex( 0, "A" ) );

		try {
			collector.addKey( "A.P" );
			fail( "Must recognise dimension count mismatch." );
		} catch ( final DimensionCountException e ) {}

		try {
			collector.getIndex( 1, "E" );
			fail( "Must recognise too high index." );
		} catch ( final ArrayIndexOutOfBoundsException e ) {}

		collector.clear();
		try {
			collector.getIndex( 0, "E" );
			fail( "Must respect clear." );
		} catch ( final IllegalStateException e ) {}

		collector.addKey( "A.P" );
		collector.addKey( "Q.BE" );

		assertEquals( "First A", 0, collector.getIndex( 0, "A" ) );
		assertEquals( "Second Q", 1, collector.getIndex( 0, "Q" ) );
		assertEquals( "First P", 0, collector.getIndex( 1, "P" ) );
		assertEquals( "Second BE", 1, collector.getIndex( 1, "BE" ) );
	}
}
