// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2004, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.util;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import Int.ecb.gesmescb.*;

/* ***************************** Class ****************************** */

/**
 * Tests the {@link CubeClassCondition}.
 *
 * @author
 * <a href="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</a>
 *
 * @version @(#)$Id: CubeClassConditionTest.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */
public class CubeClassConditionTest extends TestCase {

	/* ======================= Constants ======================== */

	private static final KeyFamily KEY_FAMILY
	= AttachmentLevelEqualsConditionTest.KEY_FAMILY;

	private static final TimeSeries
		TS1 = new TimeSeries( "Q.PT", "", "", new Observation[]{} );

	private static final Attribute
		AT1 = new Attribute( "", "", "", "ATT1", "VAL" ),
		AT2 = new Attribute( "PT", "", "", "ATT2", "VAL" ),
		AT3 = new Attribute( "Q.PT", "", "", "ATT3", "VAL" ),
		AT4 = new Attribute( "Q.PT", "20041", "608", "ATT4", "VAL" );

	/* ====================== Constructors ====================== */

	public CubeClassConditionTest( final String name ) {
		super( name );
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite
	 */
	public static Test suite () {
		final TestSuite suite = new TestSuite( CubeClassConditionTest.class );
		return suite;
	}

	/**
	 * @param arg the array of argument {@link String}s
	 */
	public static void main ( final String[] arg ) {
		final String[] testCaseName = {
			CubeClassConditionTest.class.getName()
		};
		junit.textui.TestRunner.main( testCaseName );
	}

	/* ---------------------- Test Methods ---------------------- */

	public void test () {
		final FilterIterator.Condition throwAwayCondition
		= new CubeClassCondition(
			KEY_FAMILY, 0
		);
		assertTrue( "ta rejects ts1", ! throwAwayCondition.accept( TS1 ) );
		assertTrue( "ta rejects at1", ! throwAwayCondition.accept( AT1 ) );
		assertTrue( "ta rejects at2", ! throwAwayCondition.accept( AT2 ) );
		assertTrue( "ta rejects at3", ! throwAwayCondition.accept( AT3 ) );
		assertTrue( "ta rejects at4", ! throwAwayCondition.accept( AT4 ) );

		final FilterIterator.Condition dataSetCondition
		= new CubeClassCondition(
			KEY_FAMILY, CubeClassCondition.DATA_SET
		);
		assertTrue( "ds rejects ts1", ! dataSetCondition.accept( TS1 ) );
		assertTrue( "ds accepts at1", dataSetCondition.accept( AT1 ) );
		assertTrue( "ds rejects at2", ! dataSetCondition.accept( AT2 ) );
		assertTrue( "ds rejects at3", ! dataSetCondition.accept( AT3 ) );
		assertTrue( "ds rejects at4", ! dataSetCondition.accept( AT4 ) );

		final FilterIterator.Condition groupCondition = new CubeClassCondition(
			KEY_FAMILY, CubeClassCondition.SIBLING_GROUP
		);
		assertTrue( "sg rejects ts1", ! groupCondition.accept( TS1 ) );
		assertTrue( "sg rejects at1", ! groupCondition.accept( AT1 ) );
		assertTrue( "sg accepts at2", groupCondition.accept( AT2 ) );
		assertTrue( "sg rejects at3", ! groupCondition.accept( AT3 ) );
		assertTrue( "sg rejects at4", ! groupCondition.accept( AT4 ) );

		final FilterIterator.Condition seriesCondition = new CubeClassCondition(
			KEY_FAMILY, CubeClassCondition.TIME_SERIES
		);
		assertTrue( "ts accepts ts1", seriesCondition.accept( TS1 ) );
		assertTrue( "ts rejects at1", ! seriesCondition.accept( AT1 ) );
		assertTrue( "ts rejects at2", ! seriesCondition.accept( AT2 ) );
		assertTrue( "ts accepts at3", seriesCondition.accept( AT3 ) );
		assertTrue( "ts rejects at4", ! seriesCondition.accept( AT4 ) );

		final FilterIterator.Condition obsCondition = new CubeClassCondition(
			KEY_FAMILY, CubeClassCondition.OBSERVATION
		);
		assertTrue( "obs rejects ts1", ! obsCondition.accept( TS1 ) );
		assertTrue( "obs rejects at1", ! obsCondition.accept( AT1 ) );
		assertTrue( "obs rejects at2", ! obsCondition.accept( AT2 ) );
		assertTrue( "obs rejects at3", ! obsCondition.accept( AT3 ) );
		assertTrue( "obs accepts at4", obsCondition.accept( AT4 ) );

		final FilterIterator.Condition mixCondition = new CubeClassCondition(
			KEY_FAMILY,
			CubeClassCondition.DATA_SET
			|
			CubeClassCondition.TIME_SERIES
			|
			CubeClassCondition.OBSERVATION
		);
		assertTrue( "mix accepts ts1", mixCondition.accept( TS1 ) );
		assertTrue( "mix accepts at1", mixCondition.accept( AT1 ) );
		assertTrue( "mix rejects at2", ! mixCondition.accept( AT2 ) );
		assertTrue( "mix accepts at3", mixCondition.accept( AT3 ) );
		assertTrue( "mix accepts at4", mixCondition.accept( AT4 ) );
	}
}
