// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2007, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.util;


import java.util.Iterator;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import Int.ecb.gesmescb.*;

import Int.ecb.gesmescb.event.TestEventListener;


/* ***************************** Class ****************************** */

/**
 * <P>Tests the {@link DecimalsTrimmer}.</P>
 *
 * @author
 * <A HREF="mailto:francois.hoeltzli@ECB.int">Fran&ccedil;ois Hoeltzli</A>
 *
 * @version @(#)$Id: DecimalsTrimmerTest.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */

public class DecimalsTrimmerTest extends TestCase {

	/* ======================= Constants ======================== */


	/* ========================= Fields ========================= */


	private final MessageBegin messageBegin = new MessageBegin(
		"MREF0001",
		"4F0",
		"4F0",
		"ECB",
		MessageBegin.STATISTICAL_DATA,
		"De subject",
		new Contact[]{},
		"DSI",
		MessageBegin.UPDATE_OR_REPLACE_ACTION,
		"",
		"",
		"",
		"KEY_FAMILY"
	);

	protected final Attribute attribute = new Attribute(
		"A.B.C", "", "", "TEST_ATT", "0"
	);

	private final MessageEnd messageEnd = new MessageEnd(
		"MREF0001"
	);

	private final TimeSeries series_ABC_19972003 = new TimeSeries(
		"A.B.C",
		"19972003",
		"702",
		new Observation[]{
			new Observation("1997","A","",""),
			new Observation("1998.1","A","",""),
			new Observation("1999.12","A","",""),
			new Observation("2000.123","A","",""),
			new Observation("2001.1234","A","",""),
			Observation.EMPTY_OBSERVATION,
			new Observation(Observation.MISSING_VALUE,"A","","")
		}
	);

	private final TimeSeries series_ABC_19972003_T3 = new TimeSeries(
		"A.B.C",
		"19972003",
		"702",
		new Observation[]{
			new Observation("1997","A","",""),
			new Observation("1998.1","A","",""),
			new Observation("1999.12","A","",""),
			new Observation("2000.123","A","",""),
			new Observation("2001.123","A","",""),
			Observation.EMPTY_OBSERVATION,
			new Observation(Observation.MISSING_VALUE,"A","","")
		}
	);

	private final TimeSeries series_ABC_19972003_T13 = new TimeSeries(
		"A.B.C",
		"19972003",
		"702",
		new Observation[]{
			new Observation("1997","A","",""),
			new Observation("1998.1","A","",""),
			new Observation("1999.12","A","",""),
			new Observation("2000.123","A","",""),
			new Observation("2001.1234","A","",""),
			Observation.EMPTY_OBSERVATION,
			new Observation(Observation.MISSING_VALUE,"A","","")
		}
	);

	private final TimeSeries series_ABC_19972003_T0 = new TimeSeries(
		"A.B.C",
		"19972003",
		"702",
		new Observation[]{
			new Observation("1997","A","",""),
			new Observation("1998","A","",""),
			new Observation("1999","A","",""),
			new Observation("2000","A","",""),
			new Observation("2001","A","",""),
			Observation.EMPTY_OBSERVATION,
			new Observation(Observation.MISSING_VALUE,"A","","")
		}
	);
	
	private final TimeSeries series_ABC_19912002_FIVE = new TimeSeries(
		"A.B.C",
		"19912002",
		"702",
		new Observation[]{
			new Observation("1.0499999999995","A","",""),
			new Observation("-1.0499999999999","A","",""),
			new Observation("-1.0499999999995","A","",""),
			new Observation("-0.05","A","",""),
			new Observation("-0.15","A","",""),
			new Observation("-1234.85","A","",""),
			new Observation("-1234.95","A","",""),
			new Observation("1.0499999999999","A","",""),
			new Observation("0.05","A","",""),
			new Observation("0.15","A","",""),
			new Observation("1234.85","A","",""),
			new Observation("1234.95","A","",""),
			new Observation("3.85","A","","")
		}
	);
	private final TimeSeries series_ABC_19912002_FIVE_HALF_EVEN = new TimeSeries(
			"A.B.C",
			"19912002",
			"702",
			new Observation[]{
					new Observation("1.05","A","",""),
					new Observation("-1.05","A","",""),
					new Observation("-1.05","A","",""),
					new Observation("-0.05","A","",""),
					new Observation("-0.15","A","",""),
					new Observation("-1234.85","A","",""),
					new Observation("-1234.95","A","",""),
					new Observation("1.05","A","",""),
					new Observation("0.05","A","",""),
					new Observation("0.15","A","",""),
					new Observation("1234.85","A","",""),
					new Observation("1234.95","A","",""),
					new Observation("3.85","A","","")
			}
		);
	
	/* if digit just after the dispatched decimals is a 5:
	   always round up for positive value,
	   always round down for negative value.
	*/
	private final TimeSeries series_ABC_19912002_FIVE_T1 = new TimeSeries(
		"A.B.C",
		"19912002",
		"702",
		new Observation[]{
			new Observation("1","A","",""),
			new Observation("-1","A","",""),
			new Observation("-1","A","",""),
			new Observation("-0.1","A","",""),
			new Observation("-0.2","A","",""),
			new Observation("-1234.9","A","",""),
			new Observation("-1235","A","",""),
			new Observation("1","A","",""),
			new Observation("0.1","A","",""),
			new Observation("0.2","A","",""),
			new Observation("1234.9","A","",""),
			new Observation("1235","A","",""),
			new Observation("3.9","A","","")
		}
	);

	
	/* ===================== Protected Methods ===================== */

	protected void setUp() {
	}
	protected void tearDown() {
		System.getProperties().remove(
			"Int.ecb.gesmescb.util.DecimalsTrimmer.dispatchedDecimals"
		);
	}

	/* ====================== Constructors ====================== */

	/**
	 * Constructs a DecimalsTrimmerTest with the specified name.
	 *
	 * @param name Test case name
	 */
	public DecimalsTrimmerTest ( final String name ) {
		super( name );
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite.
	 */
	public static Test suite () {
		return new TestSuite( DecimalsTrimmerTest.class );
	}

	/**
	 * Runs the test case.
	 */
	public static void main ( final String args[] ) {
		String[] testCaseName = {
			DecimalsTrimmerTest.class.getName()
		};
		junit.textui.TestRunner.main( testCaseName );
//		junit.swingui.TestRunner.main( testCaseName );
	}

	/* --------------------- Test Methods --------------------- */

	/**
	 * Test the trimmer 
	 */
	public void testTrimNothing () {
		final DecimalsTrimmer trimmer = new DecimalsTrimmer();
		final TestEventListener listener = new TestEventListener( 10 );
		trimmer.addListener( listener, true );
		
		trimmer.handle(messageBegin);
		assertSame("messageBegin", messageBegin, listener.getLastObject() );
		trimmer.handle(series_ABC_19972003);
		assertSame(
			"series_ABC_19972003",
			series_ABC_19972003,
			listener.getLastObject()
		);
		trimmer.handle(messageEnd);
		assertSame("messageEnd", messageEnd, listener.getLastObject() );		
		assertEquals("size", 3, listener.size() );		
	}
	
	public void testTrimNegative () {
		// should behave like trim nothing
		System.setProperty(
			"Int.ecb.gesmescb.util.DecimalsTrimmer.dispatchedDecimals", "-1"
		);
		final DecimalsTrimmer trimmer = new DecimalsTrimmer();
		final TestEventListener listener = new TestEventListener( 10 );
		trimmer.addListener( listener, true );
		
		trimmer.handle(messageBegin);
		assertSame("messageBegin", messageBegin, listener.getLastObject() );
		trimmer.handle(series_ABC_19972003);
		assertSame(
			"series_ABC_19972003",
			series_ABC_19972003,
			listener.getLastObject()
		);
		trimmer.handle(messageEnd);
		assertSame("messageEnd", messageEnd, listener.getLastObject() );		
		assertEquals("size", 3, listener.size() );		
	}


	public void testTrimTo3Decimals () {
		System.setProperty(
			"Int.ecb.gesmescb.util.DecimalsTrimmer.dispatchedDecimals", "3"
		);
		final DecimalsTrimmer trimmer = new DecimalsTrimmer();
		final TestEventListener listener = new TestEventListener( 10 );

		trimmer.addListener( listener, true );
		trimmer.handle(messageBegin);
		assertSame("messageBegin", messageBegin, listener.getLastObject() );
		trimmer.handle(series_ABC_19972003);
		TimeSeries gottenTS = (TimeSeries) listener.getLastObject();
		assertEquals(
			"series_ABC_19972003_T3",
			series_ABC_19972003_T3,
			gottenTS
		);
		trimmer.handle(messageEnd);
		assertSame("messageEnd", messageEnd, listener.getLastObject() );		
		assertEquals("size", 3, listener.size() );		
	}

	public void testTrimToMaxDecimals () {
		System.setProperty(
			"Int.ecb.gesmescb.util.DecimalsTrimmer.dispatchedDecimals", "213"
		);
		final DecimalsTrimmer trimmer = new DecimalsTrimmer();
		final TestEventListener listener = new TestEventListener( 10 );

		trimmer.addListener( listener, true );
		trimmer.handle(messageBegin);
		assertSame("messageBegin", messageBegin, listener.getLastObject() );
		trimmer.handle(series_ABC_19972003);
		TimeSeries gottenTS = (TimeSeries) listener.getLastObject();
		assertEquals(
			"series_ABC_19972003_T13",
			series_ABC_19972003_T13,
			gottenTS
		);
		trimmer.handle(messageEnd);
		assertSame("messageEnd", messageEnd, listener.getLastObject() );		
		assertEquals("size", 3, listener.size() );		
	}

	public void testTrimTo0Decimals () {
		System.setProperty(
			"Int.ecb.gesmescb.util.DecimalsTrimmer.dispatchedDecimals", "0"
		);
		final DecimalsTrimmer trimmer = new DecimalsTrimmer();
		final TestEventListener listener = new TestEventListener( 10 );

		trimmer.addListener( listener, true );
		trimmer.handle(messageBegin);
		assertSame("messageBegin", messageBegin, listener.getLastObject() );
		trimmer.handle(series_ABC_19972003);
		TimeSeries gottenTS = (TimeSeries) listener.getLastObject();
		assertEquals(
			"series_ABC_19972003_T0",
			series_ABC_19972003_T0,
			gottenTS
		);
		trimmer.handle(messageEnd);
		assertSame("messageEnd", messageEnd, listener.getLastObject() );		
		assertEquals("size", 3, listener.size() );		
	}

	public void testRuleForFive () {
		System.setProperty(
			"Int.ecb.gesmescb.util.DecimalsTrimmer.dispatchedDecimals", "1"
		);
		final DecimalsTrimmer trimmer = new DecimalsTrimmer();
		final TestEventListener listener = new TestEventListener( 10 );

		trimmer.addListener( listener, true );
		trimmer.handle(messageBegin);
        assertSame("messageBegin", messageBegin, listener.getLastObject() );
		trimmer.handle(series_ABC_19912002_FIVE);
		TimeSeries gottenTS = (TimeSeries) listener.getLastObject();
		assertEquals(
			"series_ABC_19912002_FIVE",
			series_ABC_19912002_FIVE_T1,
			gottenTS
		);
		trimmer.handle(messageEnd);
		assertSame("messageEnd", messageEnd, listener.getLastObject() );		
		assertEquals("size", 3, listener.size() );		
	}
	
	public void testStaticFormatHalfUp() {
		System.setProperty(
				"Int.ecb.gesmescb.util.DecimalsTrimmer.dispatchedDecimals", "6"
		);
		int decimals = 1;
		int mode = DecimalsTrimmer.HALF_UP_MODE;
		
		Iterator originalIterator = series_ABC_19912002_FIVE.observationIterator();
		Iterator roundedIterator = series_ABC_19912002_FIVE_T1.observationIterator();
		
		while( originalIterator.hasNext() && roundedIterator.hasNext() ) {
			
			String testValue = ((Observation) roundedIterator.next()).getValue();
			String roundedValue = "";
			try {
				roundedValue = DecimalsTrimmer.format(
						Double.parseDouble(((Observation) originalIterator.next()).getValue()),
						decimals,
						mode
				);
			} catch (NumberFormatException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			assertEquals("observation value " + roundedValue + "should be equal to " + testValue,
					testValue, roundedValue);
		}
		assertTrue("None of the iterators should have remaining values",
				! originalIterator.hasNext() && ! roundedIterator.hasNext() );
	}

	public void testStaticFormatHalfEven() {
		System.setProperty(
				"Int.ecb.gesmescb.util.DecimalsTrimmer.dispatchedDecimals", "6"
		);
		int decimals = 3;
		int mode = DecimalsTrimmer.HALF_EVEN_MODE;
		
		Iterator originalIterator = series_ABC_19912002_FIVE.observationIterator();
		Iterator roundedIterator = series_ABC_19912002_FIVE_HALF_EVEN.observationIterator();
		
		while( originalIterator.hasNext() && roundedIterator.hasNext() ) {
			
			String testValue = ((Observation) roundedIterator.next()).getValue();
			String roundedValue = "";
			try {
				roundedValue = DecimalsTrimmer.format(
						Double.parseDouble(((Observation) originalIterator.next()).getValue()),
						decimals,
						mode
				);
			} catch (NumberFormatException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			assertEquals("observation value " + roundedValue + "should be equal to " + testValue,
					testValue, roundedValue);
		}
		assertTrue("None of the iterators should have remaining values",
				! originalIterator.hasNext() && ! roundedIterator.hasNext() );
	}
	
}
