// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.util;

import java.util.ArrayList;
import java.util.HashMap;

import Int.ecb.gesmescb.Attribute;
import Int.ecb.gesmescb.CodeList;
import Int.ecb.gesmescb.InterchangeBegin;
import Int.ecb.gesmescb.InterchangeEnd;
import Int.ecb.gesmescb.KeyFamily;
import Int.ecb.gesmescb.MessageBegin;
import Int.ecb.gesmescb.MessageEnd;
import Int.ecb.gesmescb.StatisticalConcept;
import Int.ecb.gesmescb.TimeSeries;
import Int.ecb.gesmescb.event.DataListener;
import Int.ecb.gesmescb.event.GESMESCBEventListenerAdapter;
import Int.ecb.gesmescb.event.GESMESCBWriter;
import Int.ecb.gesmescb.event.StructureListener;

/* **************************** Class ***************************** */

/**
 * <p>Filters parameters needed for the EXDI/STC interface</p>
 * <br/>
 * <p>
 * 	<ul>
 * 		<li>IREF</li>
 * 		<li>DSIs</li>
 * 		<li>Function</li>
 *	</ul>
 * </p>
 * @author
 * <A HREF="mailto:Thomas.Michelbach@ECB.int">Sami Airo</A>
 *
 * 
 */

public final class EXDIParameterFilter extends GESMESCBEventListenerAdapter
	implements DataListener, StructureListener, GESMESCBWriter 
{

	/* ======================= Constants ======================== */
	
	/* ========================= Fields ========================= */

	/** 
	 * function and DSIs for the current interchange
	 * 
	 * function is always equal for all messages in one interchange
	 * DSIs can be different, so treat it as a list
	 */
	private String function ;
	private ArrayList DSIList;
	private HashMap DSIListUniqueValues;
	private String IRef ;
	private String sender;
	private String receiver;

	
	private Object event;

	

	/* ====================== Constructors ====================== */

	public EXDIParameterFilter () {
		
		DSIList = new ArrayList();
		DSIListUniqueValues = new HashMap();
		
	}



	/* ===================== Public Methods ===================== */

	/**
	 * Inspect and process a <CODE>MessageBegin</CODE>.
	 *
	 * @param messageBegin to be handled.
	 */
	public void handle ( final MessageBegin messageBegin ) {
		
		String dsi = messageBegin.getDataSetId();
		this.DSIList.add( dsi );
		this.DSIListUniqueValues.put(dsi, dsi);
		
		this.function = messageBegin.getFunction();
		
		event = ( (Object)messageBegin );
	
	}
	
	/**
	 * Inspect and process a <CODE>InterchangeBegin</CODE>.
	 *
	 * @param interchangeBegin to be handled.
	 */
	public void handle(InterchangeBegin interchangeBegin) {
		this.IRef = interchangeBegin.getIref();
		this.sender = interchangeBegin.getSender();
		this.receiver = interchangeBegin.getReceiver();
		event = (Object) interchangeBegin;	
	}
	
	/**
	 * 
	 * @return
	 */
	public String getFunction(){
		return this.function;
	}
	
	/**
	 * 
	 * @return
	 */
	public ArrayList getDSIList(){
		return this.DSIList;
	}

	/**
	 * 
	 * @return
	 */
	public ArrayList getDSIListUniqueValues(){
		return new ArrayList(this.DSIListUniqueValues.values());
	}


	public String getIRef(){
		return this.IRef;
	}
	
	public String getSender(){
		return this.sender;
	}
	
	public String getReceiver(){
		return this.receiver;
	}
	
	public Object getLastObject () {
		return event;
	}
	

	public void handle(TimeSeries timeSeries) {
		// TODO Auto-generated method stub
		
	}

	public void handle(Attribute attribute) {
		// TODO Auto-generated method stub
		
	}

	
	public void handle(InterchangeEnd interchangeEnd) {
		// TODO Auto-generated method stub
		
	}



	public void handle(MessageEnd messageEnd) {
		// TODO Auto-generated method stub
		
	}



	public void handle(CodeList codeList) {
		// TODO Auto-generated method stub
		
	}



	public void handle(StatisticalConcept statisticalConcept) {
		// TODO Auto-generated method stub
		
	}



	public void handle(KeyFamily keyFamily) {
		// TODO Auto-generated method stub
		
	}
	



	public void close() {
		// TODO Auto-generated method stub
		
	}


	
	

}
