// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2007, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.util;

import java.io.IOException;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import Int.ecb.gesmescb.Contact;
import Int.ecb.gesmescb.InterchangeBegin;
import Int.ecb.gesmescb.MessageBegin;
import Int.ecb.gesmescb.event.TestEventSource;

/* ***************************** Class ****************************** */

/**
 * <P>Tests the implementation of {@link MessageParameterSieve}.</P>
 *
 * @author
 * <A HREF="mailto:Thomas.Michelbach@ECB.int">Thomas Michelbach</A>
 *
 * @version 
 */

public class EXDIParameterFilterTest extends TestCase {

	/* ======================= Constants ======================== */

	/* ========================= Fields ========================= */

	private TestEventSource source;
	private EXDIParameterFilter listener;
	
	

	
	private final MessageBegin messageBegin = new MessageBegin(
		"MREF0001",
		"4F0",
		"4F0",
		"ECB",
		MessageBegin.STATISTICAL_DATA,
		"De subject",
		new Contact[]{}
	);
	
	private final InterchangeBegin interchangeBegin = new InterchangeBegin(
			"IREF0001", "DE2", "4F0", "000315:1299", false
	);


	/* ====================== Constructors ====================== */

	/**
	 * Constructs a ObservationSieveTest with the specified name.
	 *
	 * @param name Test case name
	 */
	public EXDIParameterFilterTest ( final String name ) {
		super( name );
	}

	/* ================== Protected Methods ===================== */

	/**
	 * Sets up the test fixture.
	 */
	protected void setUp () throws IOException {

		System.setProperty( "Int.ecb.logging.config.class", "Int.ecb.gesmescb.util.LoggingPropertiesReader");
		System.setProperty( ".level", "ALL" );
		System.setProperty( "handlers", "Int.ecb.logging.ConsoleHandler" );
		System.setProperty( "Int.ecb.logging.ConsoleHandler.formatter", "Int.ecb.gesmescb.util.NormalLogFormatter" );

		listener = new EXDIParameterFilter();		

		source = new TestEventSource();
	
		source.addListener( listener, true );

		
		
	}
	
	protected void tearDown () {
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite.
	 */
	public static Test suite () {
		return new TestSuite( EXDIParameterFilterTest.class );
	}

	/**
	 * Runs the test case.
	 */
	public static void main ( final String args[] ) {
		String[] testCaseName = {
				EXDIParameterFilterTest.class.getName()
		};
		junit.textui.TestRunner.main( testCaseName );
	}

	/* --------------------- Test Methods --------------------- */

	/**
	 * 
	 */
	public void testMessageBegin () {
		
		source.dispatch( messageBegin );
		assertSame("messageBegin", messageBegin, listener.getLastObject() );
	}
	
	
	
	/**
	 * 
	 *
	 */
	
	public void testFunction () {
		
		source.dispatch( messageBegin );
		assertEquals("messageBegin", messageBegin.getFunction(), listener.getFunction());
	}
	
	/**
	 * 
	 *
	 */
	
	public void testDSIList () {
		source.dispatch( messageBegin );
		assertSame("messageBegin", messageBegin.getDataSetId() , (String)listener.getDSIList().get(0));
	}
	
	/**
	 * 
	 */
	public void testInterchangeBegin () {
		source.dispatch(interchangeBegin);
		assertSame("interchangeBegin", interchangeBegin, listener.getLastObject() );
	}
	
	/**
	 * 
	 */
	public void testIRef () {
		source.dispatch(interchangeBegin);
		assertEquals("interchangeBegin", interchangeBegin.getIref(), listener.getIRef() );
	}

	
}
