// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (C) 2004, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.util;

import Int.ecb.gesmescb.AbstractCube;
import Int.ecb.gesmescb.time.RegularInterval;
import Int.ecb.gesmescb.util.FilterIterator.Condition;

/* ***************************** Class ****************************** */

/**
 * Filters {@link AbstractCube} objects with a given <code>interval</code>.
 *
 * @author
 * <A HREF="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</A>
 *
 * @version @(#)$Id: IntervalEqualsCondition.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */

public class IntervalEqualsCondition implements Condition {

	/* ========================= Fields ========================= */

	/** The interval to look for. */
	private final RegularInterval interval;

	/* ====================== Constructors ====================== */

	/**
	 * Construct a new <code>IntervalEqualsCondition</code>
	 * to filter all {@link AbstractCube}s with matching <code>interval</code>.
	 *
	 * @param interval to match
	 */
	public IntervalEqualsCondition ( final RegularInterval interval ) {
		if ( ( this.interval = interval ) == null ) {
			throw new NullPointerException(
				"Trying to create an IntervalEqualsCondition with null interval."
			);
		}
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Returns whether the <code>object</code> is an
	 * {@link AbstractCube} with matching <code>interval</code>.
	 */
	public boolean accept ( final Object object ) {
		if ( object instanceof AbstractCube ) {
			final AbstractCube cube = (AbstractCube) object;
			final RegularInterval range = cube.getInterval();
			return
				null != range
				&&
				interval.equals( range );
		}
		return false;
	}

	public String toString () {
		return super.toString() + "( \"" + interval + "\" )";
	}
}
