// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2002, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.util;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.util.*;

/* ***************************** Class ****************************** */

/**
 * Tests the {@link JointIterator}.
 *
 * @author
 * <A HREF="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</A>
 *
 * @version @(#)$Id: JointIteratorTest.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */
public class JointIteratorTest extends TestCase {

	/* ====================== Constructors ====================== */

	public JointIteratorTest( final String name ) {
		super( name );
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite
	 */
	public static Test suite () {
		final TestSuite suite = new TestSuite( JointIteratorTest.class );
		return suite;
	}

	/**
	 * @param arg the array of argument {@link String}s
	 */
	public static void main ( final String[] arg ) {
		final String[] testCaseName = {
			JointIteratorTest.class.getName()
		};
		junit.textui.TestRunner.main( testCaseName );
	}

	/* ---------------------- Test Methods ---------------------- */

	public void testConstructFromIterator () {
		assertTrue(
			"No members",
			! new JointIterator( Collections.EMPTY_LIST.iterator() ).hasNext()
		);
		assertTrue(
			"No members",
			! new JointIterator(
				Collections.singleton(
					Collections.EMPTY_LIST.iterator()
				).iterator()
			).hasNext()
		);
		final Iterator joint = new JointIterator(
			Arrays.asList(
				new Iterator[]{
					Arrays.asList( new String[]{} ).iterator(),
					Arrays.asList( new String[]{ "A", "B" } ).iterator(),
					Arrays.asList( new String[]{} ).iterator(),
					Arrays.asList( new String[]{} ).iterator(),
					Arrays.asList( new String[]{ "C" } ).iterator(),
					Arrays.asList( new String[]{} ).iterator(),
				}
			).iterator()
		);
		assertEquals( "A", joint.next() );
		assertEquals( "B", joint.next() );
		assertEquals( "C", joint.next() );
		assertTrue( "No more", ! joint.hasNext() );
	}

	public void testHasNext () {
		assertTrue(
			"No members",
			!new JointIterator( new Iterator[0] ).hasNext()
		);
		assertTrue(
			"Empty members",
			!new JointIterator(
				new Iterator[]{
					Collections.EMPTY_SET.iterator(),
					Collections.EMPTY_SET.iterator(),
					Collections.EMPTY_SET.iterator()
				}
			).hasNext()
		);
		assertTrue(
			"Nonempty members",
			new JointIterator(
				new Iterator[]{
					Collections.EMPTY_SET.iterator(),
					Collections.EMPTY_SET.iterator(),
					Arrays.asList( new Thread[1] ).iterator()
				}
			).hasNext()
		);
	}

	public void testNext () {
		try {
			new JointIterator( new Iterator[0] ).next();
			fail( "No members" );
		} catch ( final NoSuchElementException nse ) {}
		try {
			new JointIterator(
				new Iterator[]{
					Collections.EMPTY_SET.iterator(),
					Collections.EMPTY_SET.iterator(),
					Collections.EMPTY_SET.iterator()
				}
			).next();
			fail( "Empty members" );
		} catch ( final NoSuchElementException nse ) {}
		assertSame(
			"Nonempty members",
			Thread.currentThread(),
			new JointIterator(
				new Iterator[]{
					Collections.EMPTY_SET.iterator(),
					Collections.EMPTY_SET.iterator(),
					Arrays.asList(
						new Thread[]{ Thread.currentThread() }
					).iterator()
				}
			).next()
		);
	}

	public void testRemove () {
		final Collection c1 = new ArrayList();
		c1.add( "A" );
		c1.add( "B" );
		final Collection c2 = Collections.unmodifiableCollection(
			Arrays.asList( new String[]{ "C", "D" } )
		);
		final Iterator joint = new JointIterator(
			new Iterator[]{ c1.iterator(), c2.iterator() }
		);

		try {
			joint.remove();
			fail( "No preceding next" );
		} catch ( final IllegalStateException ise ) {}

		assertEquals( "A", joint.next() );
		joint.remove();
		assertEquals( 1, c1.size() );
		assertEquals( "B", joint.next() );
		joint.remove();
		assertEquals( 0, c1.size() );

		assertEquals( "C", joint.next() );
		try {
			joint.remove();
			fail( "Should forward UnsupportedOperationException" );
		} catch ( final UnsupportedOperationException uoe ) {}
		assertEquals( "D", joint.next() );
		try {
			joint.remove();
			fail( "Should forward UnsupportedOperationException" );
		} catch ( final UnsupportedOperationException uoe ) {}
		assertEquals( 2, c2.size() );

		try {
			joint.next();
			fail( "Nothing left" );
		} catch ( final NoSuchElementException nse ) {}
		try {
			joint.remove();
			fail( "Must still point to last c2 entry" );
		} catch ( final UnsupportedOperationException uoe ) {}
	}
}
