// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (C) 2004, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.util;

import Int.ecb.gesmescb.AbstractCube;
import Int.ecb.gesmescb.util.FilterIterator.Condition;

/* ***************************** Class ****************************** */

/**
 * Filters {@link AbstractCube} objects with a given <code>key</code>.
 *
 * @author
 * <A HREF="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</A>
 *
 * @version @(#)$Id: KeyEqualsCondition.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */

public class KeyEqualsCondition implements Condition {

	/* ========================= Fields ========================= */

	/** The key to look for. */
	private final String key;

	/* ====================== Constructors ====================== */

	/**
	 * Construct a new <code>KeyEqualsCondition</code>
	 * to filter all {@link AbstractCube}s with matching <code>key</code>.
	 *
	 * @param key to match
	 */
	public KeyEqualsCondition ( final String key ) {
		if ( ( this.key = key ) == null ) {
			throw new NullPointerException(
				"Trying to create a KeyEqualsCondition with null key."
			);
		}
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Returns whether the <code>object</code> is an
	 * {@link AbstractCube} with matching <code>key</code>.
	 */
	public boolean accept ( final Object object ) {
		return object instanceof AbstractCube
			? key.equals( ( (AbstractCube) object ).getKey() )
			: false;
	}

	public String toString () {
		return super.toString() + "( \"" + key + "\" )";
	}
}
