// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.util;

import java.lang.Object;
import java.lang.String;
import java.lang.UnsupportedOperationException;
import java.util.NoSuchElementException;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import Int.ecb.gesmescb.util.KeyTokenizer;

/* ***************************** Class ****************************** */

/**
 * <P>Tests the implementation of the class {@link KeyTokenizer}.</P>
 *
 * @author
 * <A HREF="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</A>
 *
 * @version @(#)$Id: KeyTokenizerTest.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */
public class KeyTokenizerTest extends TestCase {

	/* ====================== Constructors ====================== */

	/**
	 * Constructs a KeyTokenizerTest with the specified name.
	 *
	 * @param name Test case name
	 */
	public KeyTokenizerTest ( String name ) {
		super( name );
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite
	 */
	public static Test suite () {
		TestSuite suite = new TestSuite( KeyTokenizerTest.class );
		return suite;
	}

	/**
	 * Runs the test case.
	 */
	public static void main ( String args[] ) {
		String[] testCaseName = { KeyTokenizerTest.class.getName() };
		junit.textui.TestRunner.main( testCaseName );
	}

	/* ---------------------- Test Methods ---------------------- */

	/**
	 * Test with regular keys.
	 */
	public void testNormal () {
		doTest( "A", new String[] { "A" } );
		doTest( "A.BC.GEF", new String[] { "A", "BC", "GEF" } );
		doTest( "AGURKF", new String[] { "AGURKF" } );
	}

	/**
	 * Test with empty key.
	 */
	public void testEmpty () {
		doTest( "", new String[] { "" } );
	}

	/**
	 * Test with irregular keys.
	 */
	public void testIrr () {
		doTest( ".B.DE", new String[] { "", "B", "DE" } );
		doTest( ".", new String[] { "", "" } );
		doTest( "..BOD..", new String[] { "", "", "BOD", "", "" } );
	}

	/**
	 * Test exception on remove.
	 */
	public void testRemove () {
		final KeyTokenizer tokenizer = new KeyTokenizer( "A.B" );
		tokenizer.next();
		try {
			tokenizer.remove();
			fail( "No UnsupportedOperationException." );
		} catch ( UnsupportedOperationException uso ) {}
	}

	/* ==================== Private Methods ===================== */

	private void doTest( final String key, final String[] expect ) {
		for ( int methodChoice = 0; methodChoice < 2; ++ methodChoice ) {
			final KeyTokenizer tokenizer = new KeyTokenizer( key );
			for ( int i = 0; i < expect.length; ++i ) {
				Object token = null;
				assertTrue( "Token " + i, tokenizer.hasNext() );
				switch ( methodChoice ) {
					case 0: token = tokenizer.nextToken(); break;
					case 1: token = tokenizer.next(); break;
					default: fail( "Internal test problem." );
				}
				assertEquals( "Token " + i + " equals", expect[i], token );
			}
			assertTrue( "no Token " + expect.length , !tokenizer.hasNext() );
			try {
				tokenizer.next();
				fail( "No NoSuchElementException." );
			} catch ( NoSuchElementException nse ) {}
		}
	}
}
