// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.util;

import Int.ecb.logging.LogManager;
import Int.ecb.logging.Level;

import java.io.InputStream;
import java.io.FileInputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.SequenceInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

/* ***************************** Class ****************************** */

/**
 * Helper class for reading in properties for the LogManager.
 *
 * @author
 * <A HREF="mailto:Sami.Airo@ECB.int">Sami Airo</A>
 *
 * @version @(#)$Id: LoggingPropertiesReader.java,v 1.2 2020/07/20 07:02:08 delical Exp $
 */
public class LoggingPropertiesReader {

	/* ======================= Constants ======================== */

	/* ========================= Fields ========================= */

	/* Class name for logging. */
	private final String className = getClass().getName();

	/* ====================== Constructors ====================== */

	/**
	 * Construct a new instance and read in the properties.
	 */
	public LoggingPropertiesReader () {
		InputStream infile = null;
		InputStream inbyte = null;

		LogManager manager = LogManager.getLogManager();
		String packageName = manager.getClass().getPackage().getName();

		ByteArrayOutputStream outbyte = new ByteArrayOutputStream();

		try {
			System.getProperties().store(outbyte, "system properties");
		} catch (IOException e) {
			manager.log(
				Level.SEVERE, className, "Constructor",
				"cannot read system properties", e
			);
		}

		inbyte = new ByteArrayInputStream(outbyte.toByteArray());

		// Try to open the standard logging properties file
		String ps = System.getProperty("file.separator");
		String fileName = System.getProperty(
			packageName + ".config.file",
			System.getProperty("java.home")
			+ ps + "lib" + ps + "logging.properties"
		);

		try {
			infile = new FileInputStream(fileName);
		} catch (FileNotFoundException e) {
			manager.log(
				Level.SEVERE, className, "Constructor",
				"configuration file not found", e
			);
		} finally {
			if (infile == null) {
				infile = new ByteArrayInputStream("".getBytes());
			}
		}

		manager.readConfiguration(new SequenceInputStream(infile, inbyte));
	}
}
