// ECB/SIS Public License, version 1.0, document reference SIS/2001/116

package Int.ecb.gesmescb.util;

import Int.ecb.gesmescb.MessageBegin;
import Int.ecb.gesmescb.event.GESMESCBEventBroker;
import Int.ecb.logging.Level;

/* **************************** Class ***************************** */

/**
 * <P>Overwrite the {@link MessageBegin} according 
 * to preferences.</P>
 *
 * @author
 * <A HREF="mailto:francois.hoeltzli@ECB.int">Fran&ccedil;ois Hoeltzli</A>
 *
 * @version @(#)$Id: MessageBeginOverwriter.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */
public class MessageBeginOverwriter extends GESMESCBEventBroker {

	/* ======================= Constants ======================== */

	/* ========================= Fields ========================= */

	final String originalDataSetId;
	
	/* ====================== Constructors ====================== */
	
	public MessageBeginOverwriter () {
		this( "" );
	}

	/**
	 * Constructs a MessageBeginOverwriter.
	 */
	public MessageBeginOverwriter ( final String name ) {

		super( name, true );
		originalDataSetId = componentHelper.getPreference(
				"originalDataSetId", null);
		componentHelper.logger.log(
			Level.CONFIG, componentHelper.logName,
			"MessageBeginOverwriter",
			originalDataSetId == null
				? "originalDataSetId not reconfigured."
				: "originalDataSetId reconfigured to " + originalDataSetId
		);
	}
	/* ===================== Public Methods ===================== */

	/**
	 * Dispatches the MessageBegin as reconfigured.
	 *
	 * @param MessageBegin coming in.
	 */
	public void handle ( final MessageBegin messageBegin ) {
		
		componentHelper.logger.log(
			Level.FINE, componentHelper.logName, "handle",
			"handling MessageBegin", messageBegin
		);
		
		dispatch ( ( null == originalDataSetId )
			? messageBegin
			: new MessageBegin(
				messageBegin.getMref(),
				messageBegin.getSender(),
				messageBegin.getReceiver(),
				messageBegin.getMaintenanceAgencyId(),
				messageBegin.getFunction(),
				messageBegin.getIdentity(),
				messageBegin.contactAsCollection(),
				messageBegin.getDataSetId(),
				messageBegin.getAction(),
				messageBegin.getExtractionDateTime(),
				messageBegin.getReportingPeriod(),
				messageBegin.getReportingPeriodFormat(),
				messageBegin.getKeyFamilyId(),
				originalDataSetId
			)
		);
	}
	
	/* ==================== Private Methods ===================== */
}