// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2007, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.util;


import java.util.Arrays;
import java.util.Collection;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import Int.ecb.gesmescb.*;

import Int.ecb.gesmescb.event.GESMESCBEventListener;
import Int.ecb.gesmescb.event.TestEventListener;


/* ***************************** Class ****************************** */

/**
 * <P>Tests the {@link MessageBeginOverwriter}.</P>
 *
 * @author
 * <A HREF="mailto:francois.hoeltzli@ECB.int">Fran&ccedil;ois Hoeltzli</A>
 *
 * @version @(#)$Id: MessageBeginOverwriterTest.java,v 1.2 2019/01/04 09:19:33 kortest Exp $
 */

public class MessageBeginOverwriterTest extends TestCase {

	/* ======================= Constants ======================== */


	/* ========================= Fields ========================= */
	
	private static final MessageBegin MBOriginal = new MessageBegin(
		"MREF0001",
		"DE2",
		"4F0",
		"ECB",
		MessageBegin.STATISTICAL_DATA,
		"The identity",
		(Collection) Arrays.asList ( new Contact[] {
			new Contact(
				Contact.DATA_PROCESSING, "ECB-SIS", "Quasimodo",
				new ContactChannel[] {
					new ContactChannel(
						ContactChannel.EMAIL,
						"s-xtermal@ecb.int"
					)
				}
			)
		}),
		"DATASET1",
		MessageBegin.DELETE_ACTION,
		"200103201423",
		"1999",
		"602",
		"ECB_BOP1",
		"BSI_BOPT"
	);

	private static final String OVER_DSI = "OVERWRITTEN WITH THIS";

	private static final MessageBegin MBOverwritten = new MessageBegin(
			"MREF0001",
			"DE2",
			"4F0",
			"ECB",
			MessageBegin.STATISTICAL_DATA,
			"The identity",
			(Collection) Arrays.asList ( new Contact[] {
				new Contact(
					Contact.DATA_PROCESSING, "ECB-SIS", "Quasimodo",
					new ContactChannel[] {
						new ContactChannel(
							ContactChannel.EMAIL,
							"s-xtermal@ecb.int"
						)
					}
				)
			}),
			"DATASET1",
			MessageBegin.DELETE_ACTION,
			"200103201423",
			"1999",
			"602",
			"ECB_BOP1",
			OVER_DSI
	);
			
	/* ===================== Protected Methods ===================== */

	protected void setUp() {}

	protected void tearDown() {
		System.getProperties().remove(
			"Int.ecb.gesmescb.util.MessageBeginOverwriter.isTest"
		);	
	}

	/* ====================== Constructors ====================== */

	/**
	 * Constructs a MessageBeginOverwriterTest with the specified name.
	 *
	 * @param name Test case name
	 */
	public MessageBeginOverwriterTest ( final String name ) {
		super( name );
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite.
	 */
	public static Test suite () {
		return new TestSuite( MessageBeginOverwriterTest.class );
	}

	/**
	 * Runs the test case.
	 */
	public static void main ( final String args[] ) {
		String[] testCaseName = {
			MessageBeginOverwriterTest.class.getName()
		};
		junit.textui.TestRunner.main( testCaseName );
//		junit.swingui.TestRunner.main( testCaseName );
	}

	/* --------------------- Test Methods --------------------- */

	/**
	 * Test the MessageBeginOverwriter 
	 */
	public void testOverwriteNothing () {

		final MessageBeginOverwriter writer = new MessageBeginOverwriter();
		final TestEventListener listener = new TestEventListener( 10 );
		writer.addListener( listener, true );
		
		writer.handle( MBOriginal );
		assertEquals("size", 1, listener.size() );		
		assertSame("MessageBegin", MBOriginal, listener.getLastObject() );		
	}

	public void testOverwriteAsInProperties () {
		System.setProperty(
				"Int.ecb.gesmescb.util.MessageBeginOverwriter.originalDataSetId",
				OVER_DSI
			);
		final MessageBeginOverwriter writer = new MessageBeginOverwriter();
		final TestEventListener listener = new TestEventListener( 10 );
		writer.addListener( listener, true );
	
		writer.handle( MBOriginal );
		assertEquals("MessageBegin", MBOverwritten, listener.getLastObject() );
		System.getProperties().remove(
				"Int.ecb.gesmescb.util.MessageBeginOverwriter.originalDataSetId"
		);	
	}	

}
