// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.util;

import java.lang.String;

import java.text.SimpleDateFormat;
import java.util.Date;

import Int.ecb.logging.LogRecord;
import Int.ecb.logging.Formatter;
import Int.ecb.logging.Handler;

/**
 * Print the and message of the LogRecord.
 *
 * @author
 * <A HREF="mailto:Sami.Airo@ECB.int">Sami Airo</A>
 * @author
 * <A HREF="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</A>
 *
 * @version @(#)$Id: NormalLogFormatter.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */
public class NormalLogFormatter extends Formatter {

	/* ========================= Fields ========================= */

	/** The line separator to use in the output. */
	private final String lineSeparator;

	/* ====================== Constructors ====================== */

	/** Construct an instance. */
	public NormalLogFormatter () {
		lineSeparator = System.getProperty( "line.separator", "\n" );
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Format the given LogRecord.
	 *
	 * @param record the log record to be formatted.
	 * @return			a formatted log record
	 */
	public String format ( final LogRecord record ) {
		return record.getLevel() + "::"
			+ record.getMessage()
			+ (record.getThrown() == null
				? ""
				: " [" + record.getThrown().getMessage() + "]"
			) + lineSeparator;
	}

	/**
	 * Return the header string
	 *
	 * @return	a header string
	 */
	public String getHead ( final Handler h ) {
		final SimpleDateFormat formatter = new SimpleDateFormat();
		return "Log opened: " + formatter.format( new Date() ) + lineSeparator;
	}


	/**
	 * Return the tail string
	 *
	 * @return	a tail string
	 */
	public String getTail ( final Handler h ) {
		final SimpleDateFormat formatter = new SimpleDateFormat();
		return "Log closed: " + formatter.format( new Date() ) + lineSeparator;
	}
}
