// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (C) 2004, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.util;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.util.*;

import Int.ecb.gesmescb.*;
import Int.ecb.gesmescb.time.*;

/* ***************************** Class ****************************** */

/**
 * Tests the {@link ObsIntervalExtractor}.
 *
 * @author
 * <a href="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</a>
 *
 * @version @(#)$Id: ObsIntervalExtractorTest.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */
public class ObsIntervalExtractorTest extends TestCase {

	/* ====================== Constructors ====================== */

	public ObsIntervalExtractorTest( final String name ) {
		super( name );
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite
	 */
	public static Test suite () {
		final TestSuite suite = new TestSuite( ObsIntervalExtractorTest.class );
		return suite;
	}

	/**
	 * @param arg the array of argument {@link String}s
	 */
	public static void main ( final String[] arg ) {
		final String[] testCaseName = {
			ObsIntervalExtractorTest.class.getName()
		};
		junit.textui.TestRunner.main( testCaseName );
	}

	/* ---------------------- Test Methods ---------------------- */

	public void testEmpty () {
		final Collection objects = new ArrayList();
		final Iterator intervals = new ObsIntervalExtractor( objects.iterator() );
		assertTrue( "Nothing", ! intervals.hasNext() );
		try {
			intervals.next();
			fail( "Really had them" );
		} catch ( final NoSuchElementException nse ) {}
	}

	public void testMix () {
		final Collection objects = new ArrayList();
		final Observation obs = new Observation( "1.4", "A", "C", "" );
		objects.add(
			new TimeSeries(
				"M.BO.LO", "200601200604", "710",
				new Observation[]{ obs, obs, obs, obs }
			)
		);
		objects.add( new Attribute( "", "", "", "DS_ATT", "nobody" ) );
		objects.add( new Attribute( "BO.K", "", "", "SG_ATT", "anybody" ) );
		objects.add( new Attribute( "Q.UA.K", "", "", "TS_ATT", "somebody" ) );
		objects.add( new Attribute( "Q.UI.E", "2006", "602", "O_ATT", "all" ) );

		final Iterator intervals = new ObsIntervalExtractor( objects.iterator() );

		assertTrue( "Has Obs1", intervals.hasNext() );
		assertEquals(
			"Obs1",
			new GregorianRegularInterval( RegularInterval.MONTHLY, 2006, 0, 1 ),
			intervals.next()
		);
		assertTrue( "Has Obs2", intervals.hasNext() );
		assertEquals(
			"Obs2",
			new GregorianRegularInterval( RegularInterval.MONTHLY, 2006, 1, 1 ),
			intervals.next()
		);
		assertTrue( "Has Obs3", intervals.hasNext() );
		assertEquals(
			"Obs3",
			new GregorianRegularInterval( RegularInterval.MONTHLY, 2006, 2, 1 ),
			intervals.next()
		);
		assertTrue( "Has Obs1", intervals.hasNext() );
		assertEquals(
			"Obs4",
			new GregorianRegularInterval( RegularInterval.MONTHLY, 2006, 3, 1 ),
			intervals.next()
		);
		assertTrue( "Has Att1", intervals.hasNext() );
		assertEquals(
			"Att1",
			new GregorianRegularInterval( RegularInterval.ANNUAL, 2006, 0, 1 ),
			intervals.next()
		);
		assertTrue( "Had all", ! intervals.hasNext() );
		try {
			intervals.next();
			fail( "Really had them" );
		} catch ( final NoSuchElementException nse ) {}
	}
}
