package Int.ecb.gesmescb.util;

import java.util.ArrayList;
import java.util.Iterator;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import Int.ecb.gesmescb.*;

import Int.ecb.gesmescb.event.TestEventListener;

/* ***************************** Class ****************************** */

/**
 * <P>Tests the {@link RuleBasedMapper}.</P>
 *
 * @author
 * <A HREF="mailto:francois.hoeltzli@ECB.int">Fran&ccedil;ois Hoeltzli</A>
 *
 * @version @(#)$Id: RuleBasedMapperTest.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */

/* TODO: refactor so as to program to a mapper interface
 * TODO: program the ArrayLists to lists ??? (status!)
 */

public class RuleBasedMapperTest extends TestCase {

	/* ======================= Constants ======================== */
	
	private final MessageBegin messageBegin = new MessageBegin(
		"MREF0001",
		"4F0",
		"4F0",
		"ECB",
		MessageBegin.STATISTICAL_DATA,
		"De subject",
		new Contact[]{},
		"DSI",
		MessageBegin.UPDATE_OR_REPLACE_ACTION,
		"",
		"",
		"",
		"KEY_FAMILY"
	);
	private final MessageEnd messageEnd = new MessageEnd(
		"MREF0001"
	);	
	private final TimeSeries series_ABC = new TimeSeries(
		"A.B.C",
		"19971998",
		"702",
		new Observation[]{
			new Observation("1997.0","A","",""),
			new Observation("1998.1","A","",""),
			Observation.EMPTY_OBSERVATION,
			new Observation(Observation.MISSING_VALUE,"A","","")
		}
	);
	private final TimeSeries series_AZC = new TimeSeries(
		"A.Z.C",
		"19971998",
		"702",
		new Observation[]{
			new Observation("1997.0","A","",""),
			new Observation("1998.1","A","",""),
			Observation.EMPTY_OBSERVATION,
			new Observation(Observation.MISSING_VALUE,"A","","")
		}
	);
	private final Attribute dom_ser_id
	= new Attribute(
		"A.Z.C",       // key is the new key
		"",
		"",
		"DOM_SER_IDS", // name is set by properties normally.
		"PREFIX.A.B.C"        // value is the old key
	);
	private final Attribute attribute_BC
	= new Attribute(
		".B.C",       // key is the new key
		"",
		"",
		"attname", // name is set by properties normally.
		"value"        // value is the old key
	);
	private final Attribute attribute_ZC
	= new Attribute(
		".Z.C",       // key is the new key
		"",
		"",
		"attname", // name is set by properties normally.
		"value"        // value is the old key
	);

	/* ========================= Fields ========================= */
	/* ==================== Protected Methods =================== */

	protected void setUp() {	
		System.setProperty( // always use fast exit.
			"Int.ecb.gesmescb.util.SimpleRuleBasedMapper.fastExit",	"true"
		);
		System.setProperty( 
			"Int.ecb.gesmescb.util.SimpleRuleBasedMapper.allowPartialMap",
			"true"
		);
		System.setProperty( 
			"Int.ecb.gesmescb.util.RuleBasedMapper.dispatchWhenIncompleteMapping",
			"true"
		);
	}
	/* ====================== Constructors ====================== */

	/**
	 * Constructs a RuleBasedMapper with the specified name.
	 *
	 * @param name Test case name
	 */
	public RuleBasedMapperTest ( final String name ) {
		super( name );
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite.
	 */
	public static Test suite () {
		return new TestSuite( RuleBasedMapperTest.class );
	}

	/**
	 * Runs the test case.
	 */
	public static void main ( final String args[] ) {
		String[] testCaseName = {
			RuleBasedMapperTest.class.getName()
		};
		junit.textui.TestRunner.main( testCaseName );
//		junit.swingui.TestRunner.main( testCaseName );
	}

	/* ---------------------- Test Methods ---------------------- */
	
	/**
	 * Test the mapper 
	 */
	public void testMessageBegin() {
		System.setProperty( 
			"Int.ecb.gesmescb.util.RuleBasedMapper.dispatchWhenIncompleteMapping",
			"false"
		);
		final RuleBasedMapper mapper = new RuleBasedMapper();
		SimpleRuleBasedMapper temp = null;
		temp = mapper.addLine("{", temp, null);
		temp = mapper.addLine("(*).*.*->()", temp, null);
		temp = mapper.addLine("}", temp, null);
		temp = mapper.addLine("{", temp, null);
		temp = mapper.addLine("*.(B).C->Z", temp, null);
		temp = mapper.addLine("}", temp, null);
		temp = mapper.addLine("{", temp, null);
		temp = mapper.addLine("*.*.(*)->()", temp, null);
		temp = mapper.addLine("}", temp, null);
		final TestEventListener listener = new TestEventListener( 10 );
		mapper.addListener( listener, true );
		mapper.handle(messageBegin);
		assertSame("messageBegin", messageBegin, listener.getLastObject() );
		mapper.handle(series_ABC);
		assertEquals(
			"series_AZC",
			series_AZC,
			listener.getLastObject()
		);
		mapper.handle(attribute_BC);
		assertEquals(
			"attribute_ZC",
			attribute_ZC,
			listener.getLastObject()
		);
		mapper.handle(messageEnd);
		assertSame("messageEnd", messageEnd, listener.getLastObject() );		
		assertEquals("size", 4, listener.size() );
	}

	public void testRemap() {
		final RuleBasedMapper mapper = new RuleBasedMapper();
		SimpleRuleBasedMapper temp = null;
		temp = mapper.addLine("{", temp, null);
		temp = mapper.addLine("A.(B).C->Z", temp, null);
		temp = mapper.addLine("}", temp, null);
		String key = "A.B.C";
		ArrayList status = new ArrayList();
		status.add(new Integer(0));
		status.add(new Integer(0));
		status.add(new Integer(0));
		ArrayList expectedStatus = new ArrayList();
		expectedStatus.add(new Integer(0));
		expectedStatus.add(new Integer(1));
		expectedStatus.add(new Integer(0));
		String remappedKey = mapper.remap(key, status);
		assertEquals("TestRemap", "A.Z.C", remappedKey);
		assertEquals("TestRemapStatus", expectedStatus, status);
	}
	public void testMultiRemap() {
		final RuleBasedMapper mapper = new RuleBasedMapper();
		SimpleRuleBasedMapper temp = null;
		temp = mapper.addLine("{", temp, null);
		temp = mapper.addLine("A.(B).C->Z", temp, null);
		temp = mapper.addLine("}", temp, null);
		temp = mapper.addLine("{", temp, null);
		temp = mapper.addLine("A.(Z).C->B", temp, null);
		temp = mapper.addLine("}", temp, null);
		String key = "A.B.C";
		ArrayList status = new ArrayList();
		status.add(new Integer(0));
		status.add(new Integer(0));
		status.add(new Integer(0));
		ArrayList expectedStatus = new ArrayList();
		expectedStatus.add(new Integer(0));
		expectedStatus.add(new Integer(1));
		expectedStatus.add(new Integer(0));
		String remappedKey = mapper.remap(key, status);
		assertEquals("TestRemap", "A.B.C", remappedKey);
		assertEquals("TestRemapStatus", expectedStatus, status);
	}
	public void testEmptyRemap() {
		final RuleBasedMapper mapper = new RuleBasedMapper();
		String key = "A.B.C";
		ArrayList status = new ArrayList();
		status.add(new Integer(0));
		status.add(new Integer(0));
		status.add(new Integer(0));
		ArrayList expectedStatus = new ArrayList();
		expectedStatus.add(new Integer(0));
		expectedStatus.add(new Integer(0));
		expectedStatus.add(new Integer(0));
		String remappedKey = mapper.remap(key, status);
		assertEquals("TestRemap", "A.B.C", remappedKey);
		assertEquals("TestRemapStatus", expectedStatus, status);
	}
	public void testMultiRemapWithSelfRemap() {
		final RuleBasedMapper mapper = new RuleBasedMapper();
		SimpleRuleBasedMapper temp = null;
		temp = mapper.addLine("{", temp, null);
		temp = mapper.addLine("A.(B).C->Z", temp, null);
		temp = mapper.addLine("}", temp, null);
		temp = mapper.addLine("{", temp, null);
		temp = mapper.addLine("(A.Z.C)->()", temp, null);
		temp = mapper.addLine("}", temp, null);
		String key = "A.B.C";
		ArrayList status = new ArrayList();
		status.add(new Integer(0));
		status.add(new Integer(0));
		status.add(new Integer(0));
		ArrayList expectedStatus = new ArrayList();
		expectedStatus.add(new Integer(1));
		expectedStatus.add(new Integer(1));
		expectedStatus.add(new Integer(1));
		String remappedKey = mapper.remap(key, status);
		assertEquals("TestRemap", "A.Z.C", remappedKey);
		assertEquals("TestRemapStatus", expectedStatus, status);
	}	
	public void testSendingOldName() {
		System.setProperty( 
			"Int.ecb.gesmescb.util.RuleBasedMapper.originalNamePrefix",
			"PREFIX"
		);
		System.setProperty( 
			"Int.ecb.gesmescb.util.RuleBasedMapper.originalNameAttribute",
			"DOM_SER_IDS"
		);
		final RuleBasedMapper mapper = new RuleBasedMapper();
		SimpleRuleBasedMapper temp = null;
		temp = mapper.addLine("{", temp, null);
		temp = mapper.addLine("A.(B).C->Z", temp, null);
		temp = mapper.addLine("}", temp, null);

		final TestEventListener listener = new TestEventListener( 10 );
		mapper.addListener( listener, true );
		
		mapper.handle(messageBegin);
		mapper.handle(series_ABC);
		mapper.handle(messageEnd);
		
		Iterator iterator = listener.iterator();
		assertTrue("We should have something...", iterator.hasNext());
		assertSame("messageBegin", messageBegin, iterator.next() );
		assertEquals(
			"series_AZC",
			series_AZC,
			iterator.next()
		);
		assertEquals(
			"attribute dom_ser_id",
			dom_ser_id,
			iterator.next()
		);
		assertSame("messageEnd", messageEnd, iterator.next() );		
		assertEquals("size", 4, listener.size() );

		System.clearProperty( 
				"Int.ecb.gesmescb.util.RuleBasedMapper.originalNameAttribute"
		);
	}
	
	public void testDeprecatedReordering() {
		System.setProperty( 
			"Int.ecb.gesmescb.util.RuleBasedMapper.reordering",
			"1.2.3.7.5.6.8.4.9.10"
		);
		final RuleBasedMapper mapper = new RuleBasedMapper();
		String key          = "Q.IE.N.L.B1QG00.0000.1000.TTTT.N.A";
		String expectedKey  = "Q.IE.N.1000.B1QG00.0000.TTTT.L.N.A";
		String reorderedKey = mapper.reorder(key);
		assertEquals("TestReorder", expectedKey, reorderedKey);
		System.clearProperty( 
			"Int.ecb.gesmescb.util.RuleBasedMapper.reordering"
		);
	}	

	public void if_you_have_one_you_can_testFromFile() {
		System.setProperty( 
			"Int.ecb.gesmescb.util.RuleBasedMapper.reordering",
		//	"0.1.2.7.4.3.5.6.8.9" no! we made it user friendly 1-based,
			"1.2.3.6.5.7.8.4.9.10"
		);
		System.setProperty( 
			"Int.ecb.gesmescb.util.RuleBasedMapper.filename",
			"D:\\Data\\java\\ges\\maprulesEstat.txt"
		//	"/global/proddatadg/sdpe/production/EFS/GES/inp/mapping_rules_ESTAT_ESAAK_to_ECB_ESA1"
		);
		final RuleBasedMapper mapper = new RuleBasedMapper();
		ArrayList status = new ArrayList();
		for(int i=0; i<12; ++i) {
			status.add(new Integer(0));
		}
		String key1          = "A.SI.N.Z.EPEA.Z.Z.Z.Z.Z.M.0110";
		String expectedKey1  = "A.SI.N.0000.ECACPO.1000.TTTT.N.P.A";
		String remappedKey = mapper.remap(key1, status);
		String reorderedKey = mapper.reorder(remappedKey);
		assertEquals( "key1 not correctly remapped", expectedKey1, reorderedKey);
		status = new ArrayList();
		for(int i=0; i<12; ++i) {
			status.add(new Integer(0));
		}
		String key2          = "Q.LT.W.L.B1G.Z.S1.Y.Z.3.N.0101";
		String expectedKey2  = "Q.LT.W.0000.B1G000.1000.TTTT.L.N.A";
		remappedKey = mapper.remap(key2, status);
		reorderedKey = mapper.reorder(remappedKey);
		assertEquals( "key1 not correctly remapped", expectedKey2, reorderedKey);
		System.clearProperty( 
			"Int.ecb.gesmescb.util.RuleBasedMapper.reordering"
		);
		System.clearProperty( 
			"Int.ecb.gesmescb.util.RuleBasedMapper.filename"
		);
	}
	public void if_you_have_another_file() {
		System.setProperty( 
			"Int.ecb.gesmescb.util.RuleBasedMapper.filename",
			//"D:\\Data\\java\\ges\\map.txt"
			//"/global/proddatadg/sdpe/production/EFS/GES/inp/mapping_and_swapping_rules_ICPT_to_ECB_ICP1"
			"D:\\Data\\java\\ges\\maprules.txt"
			//"/global/proddatadg/sdpe/production/EFS/GES/inp/mapping_and_swapping_rules_ICPT_to_ECB_ICP1"
		);
		final RuleBasedMapper mapper = new RuleBasedMapper();
		ArrayList status = new ArrayList();
		for(int i=0; i<2; ++i) {
			status.add(new Integer(0));
		}
		//String key1          = "A.VAL.COWE15.AUT";
		//String expectedKey1  = "A.VAL.COWE15.AUT.00"; //expectedKey1  = "A.AT.N.000000.4.D2W";
		String key1          = "A.VAL";
		String expectedKey1  = "preums.deuze";
		String remappedKey = mapper.remap(key1, status);
		assertEquals( "key1 not correctly remapped", expectedKey1, remappedKey);
		boolean fullyRemapped = SimpleRuleBasedMapper.fullyRemapped( status );
		assertTrue( fullyRemapped );
		System.clearProperty( 
			"Int.ecb.gesmescb.util.RuleBasedMapper.reordering"
		);
		System.clearProperty( 
			"Int.ecb.gesmescb.util.RuleBasedMapper.filename"
		);
	}

}
