// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2007, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.util;


import java.util.ArrayList;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import Int.ecb.gesmescb.*;

import Int.ecb.gesmescb.event.TestEventListener;


/* ***************************** Class ****************************** */

/**
 * <P>Tests the {@link SimpleRuleBasedMapper}.</P>
 *
 * @author
 * <A HREF="mailto:francois.hoeltzli@ECB.int">Fran&ccedil;ois Hoeltzli</A>
 *
 * @version @(#)$Id: SimpleRuleBasedMapperTest.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */

public class SimpleRuleBasedMapperTest extends TestCase {

	/* ======================= Constants ======================== */


	/* ========================= Fields ========================= */


	private final MessageBegin messageBegin = new MessageBegin(
		"MREF0001",
		"4F0",
		"4F0",
		"ECB",
		MessageBegin.STATISTICAL_DATA,
		"De subject",
		new Contact[]{},
		"DSI",
		MessageBegin.UPDATE_OR_REPLACE_ACTION,
		"",
		"",
		"",
		"KEY_FAMILY"
	);

	protected final Attribute attributeABC = new Attribute(
		"A.B.C", "200109200110", "710", "TEST_ATT", "0"
	);
	protected final Attribute attribute_BC = new Attribute(
		".B.C", "200109200110", "710", "TEST_ATT", "0"
	);
	protected final Attribute attributeAZC = new Attribute(
		"A.Z.C", "200109200110", "710", "TEST_ATT", "0"
	);
	protected final Attribute attributeW = new Attribute(
		"W", "200109200110", "710", "TEST_ATT", "0"
	);
	
	

	private final MessageEnd messageEnd = new MessageEnd(
		"MREF0001"
	);

	private final TimeSeries series_ABC = new TimeSeries(
		"A.B.C",
		"19971998",
		"702",
		new Observation[]{
			new Observation("1997.0","A","",""),
			new Observation("1998.1","A","",""),
			Observation.EMPTY_OBSERVATION,
			new Observation(Observation.MISSING_VALUE,"A","","")
		}
	);

	private final TimeSeries series_AB00 = new TimeSeries(
			"A.B.00",
			"19971998",
			"702",
			new Observation[]{
				new Observation("1997.0","A","",""),
				new Observation("1998.1","A","",""),
				Observation.EMPTY_OBSERVATION,
				new Observation(Observation.MISSING_VALUE,"A","","")
			}
		);
	
	private final TimeSeries series_BCA = new TimeSeries(
		"B.C.A",
		"19971998",
		"702",
		new Observation[]{
			new Observation("1997.0","A","",""),
			new Observation("1998.1","A","",""),
			Observation.EMPTY_OBSERVATION,
			new Observation(Observation.MISSING_VALUE,"A","","")
		}
	);

	private final TimeSeries series_AZC = new TimeSeries(
			"A.Z.C",
			"19971998",
			"702",
			new Observation[]{
				new Observation("1997.0","A","",""),
				new Observation("1998.1","A","",""),
				Observation.EMPTY_OBSERVATION,
				new Observation(Observation.MISSING_VALUE,"A","","")
			}
		);

	private final TimeSeries series_AWC = new TimeSeries(
			"A.W.C",
			"19971998",
			"702",
			new Observation[]{
				new Observation("1997.0","A","",""),
				new Observation("1998.1","A","",""),
				Observation.EMPTY_OBSERVATION,
				new Observation(Observation.MISSING_VALUE,"A","","")
			}
		);
	
	/* ===================== Protected Methods ===================== */

	protected void setUp() {
		// for the sake of the test we accept any partial mapping
		System.setProperty( 
			"Int.ecb.gesmescb.util.SimpleRuleBasedMapper.allowPartialMap",
			"true"
		);
	}
	protected void tearDown() {
		System.getProperties().remove(
			"Int.ecb.gesmescb.util.SimpleRuleBasedMapper.rule"
		);
		System.getProperties().remove(
			"Int.ecb.gesmescb.util.SimpleRuleBasedMapper.rule.0"
		);
		System.getProperties().remove(
			"Int.ecb.gesmescb.util.SimpleRuleBasedMapper.rule.1"
		);
		System.getProperties().remove(
			"Int.ecb.gesmescb.util.SimpleRuleBasedMapper.rule.2"
		);
		System.getProperties().remove(
			"Int.ecb.gesmescb.util.SimpleRuleBasedMapper.rule.3"
		);
		System.getProperties().remove(
			"Int.ecb.gesmescb.util.SimpleRuleBasedMapper.fastExit"
		);
		System.getProperties().remove(
			"Int.ecb.gesmescb.util.SimpleRuleBasedMapper.swapRule"
		);
	}

	/* ====================== Constructors ====================== */

	/**
	 * Constructs a SimpleRuleBasedMapper with the specified name.
	 *
	 * @param name Test case name
	 */
	public SimpleRuleBasedMapperTest ( final String name ) {
		super( name );
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite.
	 */
	public static Test suite () {
		return new TestSuite( SimpleRuleBasedMapperTest.class );
	}

	/**
	 * Runs the test case.
	 */
	public static void main ( final String args[] ) {
		String[] testCaseName = {
			SimpleRuleBasedMapperTest.class.getName()
		};
		junit.textui.TestRunner.main( testCaseName );
//		junit.swingui.TestRunner.main( testCaseName );
	}

	/* --------------------- Test Methods --------------------- */

	/**
	 * Test the mapper 
	 */
	public void testMapABCToAZC() {
		System.setProperty(
			"Int.ecb.gesmescb.util.SimpleRuleBasedMapper.rule.0",
			"A.(B).C->Z"
		);
		final SimpleRuleBasedMapper mapper = new SimpleRuleBasedMapper();
		final TestEventListener listener = new TestEventListener( 10 );
		mapper.addListener( listener, true );
		
		mapper.handle(messageBegin);
		assertSame("messageBegin", messageBegin, listener.getLastObject() );
		mapper.handle(series_ABC);
		assertEquals(
			"series_AZC",
			series_AZC,
			listener.getLastObject()
		);
		mapper.handle(messageEnd);
		assertSame("messageEnd", messageEnd, listener.getLastObject() );		
		assertEquals("size", 3, listener.size() );
	}

	public void testWildMapABCToAZC() {
		// same test with wildcarded left and right contexts
		System.setProperty(
			"Int.ecb.gesmescb.util.SimpleRuleBasedMapper.rule.0",
			"*.(B).*->Z"
		);
		final SimpleRuleBasedMapper mapper = new SimpleRuleBasedMapper();
		final TestEventListener listener = new TestEventListener( 10 );
		mapper.addListener( listener, true );
		
		mapper.handle(messageBegin);
		assertSame("messageBegin", messageBegin, listener.getLastObject() );
		mapper.handle(series_ABC);
		assertEquals(
			"series_AZC",
			series_AZC,
			listener.getLastObject()
		);
		mapper.handle(messageEnd);
		assertSame("messageEnd", messageEnd, listener.getLastObject() );		
		assertEquals("size", 3, listener.size() );
	}

	public void testEmptyLeftContextC() {
		System.setProperty(
			"Int.ecb.gesmescb.util.SimpleRuleBasedMapper.rule.0",
			"(A.B).*->A.Z"
		);
		final SimpleRuleBasedMapper mapper = new SimpleRuleBasedMapper();
		final TestEventListener listener = new TestEventListener( 10 );
		mapper.addListener( listener, true );
		
		mapper.handle(messageBegin);
		assertSame("messageBegin", messageBegin, listener.getLastObject() );
		mapper.handle(series_ABC);
		assertEquals(
			"series_AZC",
			series_AZC,
			listener.getLastObject()
		);
		mapper.handle(series_AZC);
		assertEquals(
			"series_AZC",
			series_AZC,
			listener.getLastObject()
		);
		mapper.handle(messageEnd);
		assertSame("messageEnd", messageEnd, listener.getLastObject() );		
		assertEquals("size", 4, listener.size() );
	}

	public void testEmptyRightContextC() {
		System.setProperty(
			"Int.ecb.gesmescb.util.SimpleRuleBasedMapper.rule.0",
			"*.(B.C)->Z.C"
		);
		final SimpleRuleBasedMapper mapper = new SimpleRuleBasedMapper();
		final TestEventListener listener = new TestEventListener( 10 );
		mapper.addListener( listener, true );
		
		mapper.handle(messageBegin);
		assertSame("messageBegin", messageBegin, listener.getLastObject() );
		mapper.handle(series_ABC);
		assertEquals(
			"series_AZC",
			series_AZC,
			listener.getLastObject()
		);
		mapper.handle(messageEnd);
		assertSame("messageEnd", messageEnd, listener.getLastObject() );		
		assertEquals("size", 3, listener.size() );
	}

	public void testRuleDoesNotApply() {
		System.setProperty(
			"Int.ecb.gesmescb.util.SimpleRuleBasedMapper.rule.0",
			"*.(B).C->W"
		);
		final SimpleRuleBasedMapper mapper = new SimpleRuleBasedMapper();
		final TestEventListener listener = new TestEventListener( 10 );
		mapper.addListener( listener, true );
		
		mapper.handle(messageBegin);
		assertSame("messageBegin", messageBegin, listener.getLastObject() );
		mapper.handle(series_AZC);
		assertEquals(
			"series_AZC",
			series_AZC,
			listener.getLastObject()
		);
		mapper.handle(messageEnd);
		assertSame("messageEnd", messageEnd, listener.getLastObject() );		
		assertEquals("size", 3, listener.size() );
	}
	
	// Testing with multiple rules
	public void test2RulesApply() {
		System.setProperty(
			"Int.ecb.gesmescb.util.SimpleRuleBasedMapper.rule.0",
			"*.(B).C->W"
		);
		SimpleRuleBasedMapper mapper = new SimpleRuleBasedMapper();
		final TestEventListener listener = new TestEventListener( 10 );
		mapper.addListener( listener, true );
		mapper.handle(messageBegin);
		assertSame("messageBegin", messageBegin, listener.getLastObject() );
		mapper.handle(series_ABC);
		TimeSeries ts = (TimeSeries) listener.getLastObject(); // should get AWC
		// and we feed it a second time in another mapper to get back to ABC
		System.setProperty(
				"Int.ecb.gesmescb.util.SimpleRuleBasedMapper.rule.0",
				"*.(W).C->B"
		);
		mapper = new SimpleRuleBasedMapper();
		mapper.addListener( listener, true );
		mapper.handle(ts);
		assertEquals(
			"series_ABC",
			series_ABC,
			listener.getLastObject()
		);
		mapper.handle(messageEnd);
		assertSame("messageEnd", messageEnd, listener.getLastObject() );		
		assertEquals("size", 4, listener.size() );
	}

	// Testing with multiple rules, but now we let the mapper loop for us over the
	// list of rules
	public void testRulesApply() {
		System.setProperty(
			"Int.ecb.gesmescb.util.SimpleRuleBasedMapper.rule.0",
			"*.(B).C->W"
		);
		System.setProperty(
			"Int.ecb.gesmescb.util.SimpleRuleBasedMapper.rule.1",
			"*.(W).C->B"
		);
		SimpleRuleBasedMapper mapper = new SimpleRuleBasedMapper();
		final TestEventListener listener = new TestEventListener( 10 );
		mapper.addListener( listener, true );
		mapper.handle(series_ABC);
		assertEquals(
			"series_ABC",
			series_ABC,
			listener.getLastObject()
		);
		mapper.handle(messageEnd);
		assertSame("messageEnd", messageEnd, listener.getLastObject() );		
		assertEquals("size", 2, listener.size() );
	}

	public void testRulesApply4() {
		System.setProperty( // A.B.C -> A.W.C
			"Int.ecb.gesmescb.util.SimpleRuleBasedMapper.rule.0",
			"*.(B).C->W"
		);
		System.setProperty( // A.X.C should not apply
			"Int.ecb.gesmescb.util.SimpleRuleBasedMapper.rule.1",
			"*.(X).C->A.B.C.D.E.F"
		);
		System.setProperty( // A.W.C -> A.Z.Z.C
			"Int.ecb.gesmescb.util.SimpleRuleBasedMapper.rule.2",
			"*.(W).C->Z.Z"
		);
		System.setProperty( // A.Z.Z.C -> A.Z.C
			"Int.ecb.gesmescb.util.SimpleRuleBasedMapper.rule.3",
			"(A.Z).*.*->A"
		);
		SimpleRuleBasedMapper mapper = new SimpleRuleBasedMapper();
		final TestEventListener listener = new TestEventListener( 10 );
		mapper.addListener( listener, true );
		mapper.handle(series_ABC);
		assertEquals(
			"series_AZC",
			series_AZC,
			listener.getLastObject()
		);
		assertEquals("size", 1, listener.size() );
	}	

	public void testBlockingOfPartialMaps() {
		System.setProperty( // A.B.C -> A.W.C
			"Int.ecb.gesmescb.util.SimpleRuleBasedMapper.rule.0",
			"*.(B).C->W"
		);
		System.setProperty( // A.Z.Z.C -> A.Z.C
			"Int.ecb.gesmescb.util.SimpleRuleBasedMapper.rule.1",
			"(A.Z).*.*->A"
		);
		System.setProperty( 
			"Int.ecb.gesmescb.util.SimpleRuleBasedMapper.allowPartialMap",
			"false"
		);
		SimpleRuleBasedMapper mapper = new SimpleRuleBasedMapper();
		final TestEventListener listener = new TestEventListener( 10 );
		mapper.addListener( listener, true );
		mapper.handle(series_ABC);
		assertEquals(
			"series_AZC",
			null,
			listener.getLastObject()
		);
		assertEquals("size", 0, listener.size() );
	}	

	public void testFastExit() {
		System.setProperty( // A.B.C -> A.W.C
			"Int.ecb.gesmescb.util.SimpleRuleBasedMapper.rule.0",
			"*.(B).C->W"
		);
		System.setProperty( 
			"Int.ecb.gesmescb.util.SimpleRuleBasedMapper.allowPartialMap",
			"true"
		);
		System.setProperty( 
			"Int.ecb.gesmescb.util.SimpleRuleBasedMapper.fastExit",
			"true"
		);
		System.setProperty( // A.W.C -> A.Z.Z.C # shouldn't apply due to fast exit
			"Int.ecb.gesmescb.util.SimpleRuleBasedMapper.rule.1",
			"*.(W).C->Z.Z"
		);
		SimpleRuleBasedMapper mapper = new SimpleRuleBasedMapper();
		final TestEventListener listener = new TestEventListener( 10 );
		mapper.addListener( listener, true );
		mapper.handle(series_ABC);
		assertEquals(
			"series_AWC",
			series_AWC,
			listener.getLastObject()
		);
		assertEquals("size", 1, listener.size() );
	}	

	public void testRemapToSelf() {
		System.setProperty( // A.B.C -> A.B.C
			"Int.ecb.gesmescb.util.SimpleRuleBasedMapper.rule.0",
			"*.(B).C->()"
		);
		System.setProperty( 
			"Int.ecb.gesmescb.util.SimpleRuleBasedMapper.allowPartialMap",
			"true"
		);
		System.setProperty( 
			"Int.ecb.gesmescb.util.SimpleRuleBasedMapper.fastExit",
			"false"
		);
		System.setProperty( // A.W.C -> A.Z.Z.C 
			"Int.ecb.gesmescb.util.SimpleRuleBasedMapper.rule.1",
			"*.(W).C->Z.Z"
		);
		SimpleRuleBasedMapper mapper = new SimpleRuleBasedMapper();
		final TestEventListener listener = new TestEventListener( 10 );
		mapper.addListener( listener, true );
		mapper.handle(series_ABC);
		assertEquals(
			"series_ABC",
			series_ABC,
			listener.getLastObject()
		);
		assertEquals("size", 1, listener.size() );
	}	

	public void testStatusWhenRemapToSelf() {
		System.setProperty( 
			"Int.ecb.gesmescb.util.SimpleRuleBasedMapper.rule.0",
			"(*).*.*->()"
		);
		System.setProperty(
			"Int.ecb.gesmescb.util.SimpleRuleBasedMapper.rule.1",
			"*.(*).*->()"
		);
		System.setProperty(
			"Int.ecb.gesmescb.util.SimpleRuleBasedMapper.rule.2",
			"*.*.(*)->()"
		);
		System.setProperty( 
			"Int.ecb.gesmescb.util.SimpleRuleBasedMapper.allowPartialMap",
			"false"
		);
		System.setProperty( 
			"Int.ecb.gesmescb.util.SimpleRuleBasedMapper.fastExit",
			"false"
		);
		SimpleRuleBasedMapper mapper = new SimpleRuleBasedMapper();
		final TestEventListener listener = new TestEventListener( 10 );
		mapper.addListener( listener, true );
		mapper.handle(attribute_BC);
		assertEquals(
			"attribute_BC",
			attribute_BC,
			listener.getLastObject()
		);
		assertEquals("size", 1, listener.size() );
	}	

	public void testStatusAloneWhenRemapToSelf() {
		ArrayList status = new ArrayList();
		status.add(SimpleRuleBasedMapper.ORIGINAL);
		status.add(SimpleRuleBasedMapper.ORIGINAL);
		status.add(SimpleRuleBasedMapper.ORIGINAL);
		SimpleRuleBasedMapper.computeStatus(status, ".", ".C", "B");
		assertEquals(
			"dimensionality",
			3,
			status.size()
		);
		assertEquals("first elem", SimpleRuleBasedMapper.ORIGINAL, status.get(0));
		assertEquals("2ndt elem", SimpleRuleBasedMapper.REPLACED, status.get(1));
		assertEquals("3rd elem", SimpleRuleBasedMapper.ORIGINAL, status.get(2));
	}	

	public void testConcatStatusAloneWhenRemapToSelf() {
		// .B.C is the input string, .(B).C->() is the rule
		ArrayList leftStatus = new ArrayList();
		leftStatus.add(SimpleRuleBasedMapper.ORIGINAL);
		leftStatus.add(SimpleRuleBasedMapper.ORIGINAL);
		
		ArrayList replacementStatus = new ArrayList();
		replacementStatus.add(SimpleRuleBasedMapper.REPLACED);
		
		ArrayList rightStatus = new ArrayList();
		rightStatus.add(SimpleRuleBasedMapper.ORIGINAL);
		rightStatus.add(SimpleRuleBasedMapper.ORIGINAL);
		
		SimpleRuleBasedMapper.concatStatus(
				".", leftStatus,"B", replacementStatus);

		assertEquals( "result status is 2", 2, leftStatus.size() );
		SimpleRuleBasedMapper.concatStatus(
				".B", leftStatus,".C", rightStatus);
		
		assertEquals(
			"dimensionality",
			3,
			leftStatus.size()
		);
		assertEquals("first elem", SimpleRuleBasedMapper.ORIGINAL, leftStatus.get(0));
		assertEquals("2ndt elem", SimpleRuleBasedMapper.REPLACED, leftStatus.get(1));
		assertEquals("3rd elem", SimpleRuleBasedMapper.ORIGINAL, leftStatus.get(2));
	}	
	
	public void testSwap() {
		System.setProperty( 
			"Int.ecb.gesmescb.util.SimpleRuleBasedMapper.swapRule",
			"2,3,1"
		);
		SimpleRuleBasedMapper mapper = new SimpleRuleBasedMapper();
		final TestEventListener listener = new TestEventListener( 10 );
		mapper.addListener( listener, true );
		mapper.handle(series_ABC);
		assertEquals(
			"swapping series_ABC",
			series_BCA,
			listener.getLastObject()
		);
		assertEquals("size", 1, listener.size() );
	}	

	public void testAddSwaprule() {
		System.setProperty( 
			"Int.ecb.gesmescb.util.SimpleRuleBasedMapper.fastExit",
			"false"
		);
		SimpleRuleBasedMapper mapper = new SimpleRuleBasedMapper();
		mapper.addRule("swap 2,3,1");
		mapper.addRule("(B.C.A)->A.W.C");
		final TestEventListener listener = new TestEventListener( 10 );
		mapper.addListener( listener, true );
		mapper.handle(series_ABC);
		assertEquals(
			"swapping and add rules",
			series_AWC,
			listener.getLastObject()
		);
		assertEquals("size", 1, listener.size() );
	}	
	
	public void testComputeStatus() {
		// I simulate A(.B).C -> 
		// where A and B are already replaced, and C still untouched
		ArrayList status;
		status = SimpleRuleBasedMapper.createStatus( "A.B", SimpleRuleBasedMapper.REPLACED );
		status.add(SimpleRuleBasedMapper.ORIGINAL); // A.B.C w C still untouched.
		// now assume we delete .B
		SimpleRuleBasedMapper.computeStatus(status, "A", //".B",
				".C", "");
		assertEquals( "status should have 2 elements",
				2, status.size());
		assertEquals( "last elem should be original...",
				SimpleRuleBasedMapper.ORIGINAL, status.get(1));
		//
		status = SimpleRuleBasedMapper.createStatus(
				"M.AT.S.todelete.ESI.000.TT", SimpleRuleBasedMapper.REPLACED );
		status.add(SimpleRuleBasedMapper.ORIGINAL); // M still untouched.
		assertEquals( "status should have 8 elements",
				8, status.size());
		// now assume remove the dot between todelete and ESI 
		SimpleRuleBasedMapper.computeStatus(
				status, "M.AT.S.todelete", //".", 
				"ESI.000.TT.M", "");
		assertEquals( "status should have 7 elements",
				7, status.size());
		assertEquals( "last elem should be original...",
				SimpleRuleBasedMapper.ORIGINAL, status.get(6));
	}

	public void testConcatStatus() {
		ArrayList LStatus = new ArrayList();
		LStatus.add( SimpleRuleBasedMapper.REPLACED );
		LStatus.add( SimpleRuleBasedMapper.REPLACED );
		LStatus.add( SimpleRuleBasedMapper.REPLACED );
		LStatus.add( SimpleRuleBasedMapper.ORIGINAL );
		String leftContext = "q.se.bpm.fcc12";
		ArrayList RStatus = new ArrayList();
		String rightContext = "";
		SimpleRuleBasedMapper.concatStatus(
				leftContext, LStatus, rightContext, RStatus
		);
		assertEquals( "new Status",	LStatus.size(), 4);
		assertEquals( "1.dim1", SimpleRuleBasedMapper.REPLACED, (Integer) LStatus.get(0));
		assertEquals( "1.dim2", SimpleRuleBasedMapper.REPLACED, (Integer) LStatus.get(1));
		assertEquals( "1.dim3", SimpleRuleBasedMapper.REPLACED, (Integer) LStatus.get(2));
		assertEquals( "1.dim4", SimpleRuleBasedMapper.ORIGINAL, (Integer) LStatus.get(3));
		assertEquals( "1. R Status", 0, RStatus.size() );
		
		RStatus.add( SimpleRuleBasedMapper.ORIGINAL );
		RStatus.add( SimpleRuleBasedMapper.REPLACED );
		RStatus.add( SimpleRuleBasedMapper.REPLACED );
		RStatus.add( SimpleRuleBasedMapper.REPLACED );
		rightContext = ".n.4.000";
		SimpleRuleBasedMapper.concatStatus(
				leftContext, LStatus, rightContext, RStatus
		);
		assertEquals( "new Status",	LStatus.size(), 7);
		assertEquals( "2.dim1", SimpleRuleBasedMapper.REPLACED, (Integer) LStatus.get(0));
		assertEquals( "2.dim2", SimpleRuleBasedMapper.REPLACED, (Integer) LStatus.get(1));
		assertEquals( "2.dim3", SimpleRuleBasedMapper.REPLACED, (Integer) LStatus.get(2));
		assertEquals( "2.dim4", SimpleRuleBasedMapper.ORIGINAL, (Integer) LStatus.get(3));
		assertEquals( "2.dim5", SimpleRuleBasedMapper.REPLACED, (Integer) LStatus.get(4));
		assertEquals( "2.dim6", SimpleRuleBasedMapper.REPLACED, (Integer) LStatus.get(5));
		assertEquals( "2.dim7", SimpleRuleBasedMapper.REPLACED, (Integer) LStatus.get(6));
	}

	public void testSimulateReplaceWithinLastDim() {
		// I simulate AA.BB.CC() -> .00
		ArrayList status;
		status = SimpleRuleBasedMapper.createStatus( "A.B.C", SimpleRuleBasedMapper.ORIGINAL );
		// now assume we augment CC
		SimpleRuleBasedMapper.computeStatus(status, "AA.BB.CC", //"",
				"", ".00");
		assertEquals( "status should have 4 elements",
				4, status.size());
		assertEquals( "last elem should be new...",
				SimpleRuleBasedMapper.REPLACED, status.get(3));
	}

	public void testdoReplaceWithinLastDim() {
		// AOWO.BOWO.COWO.DOWO
		SimpleRuleBasedMapper mapper = new SimpleRuleBasedMapper();
		mapper.addRule("*.*.COW*.*()->.00");
		final TestEventListener listener = new TestEventListener( 10 );
		mapper.addListener( listener, true );

		TimeSeries series_COW = new TimeSeries(
				"AOWO.BOWO.COWO.DOWO", "19971998", "702", new Observation[]{
						new Observation(Observation.MISSING_VALUE,"A","",""),
						new Observation(Observation.MISSING_VALUE,"A","","")
				});
		TimeSeries expected = new TimeSeries(
				"AOWO.BOWO.COWO.DOWO.00", "19971998", "702", new Observation[]{
						new Observation(Observation.MISSING_VALUE,"A","",""),
						new Observation(Observation.MISSING_VALUE,"A","","")
				});
		mapper.handle( series_COW );
		assertEquals("size", 1, listener.size() );
		assertEquals(
			"series_COW",
			expected,
			listener.getLastObject()
		);
	}
}
	
