// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2004, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.util;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import Int.ecb.gesmescb.*;

/* ***************************** Class ****************************** */

/**
 * Tests the {@link SubcubeCondition}.
 *
 * @author
 * <a href="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</a>
 *
 * @version @(#)$Id: SubcubeConditionTest.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */
public class SubcubeConditionTest extends TestCase {

	/* ======================= Constants ======================== */

	private static final KeyFamily KEY_FAMILY
	= AttachmentLevelEqualsConditionTest.KEY_FAMILY;

	private static final TimeSeries
		TS1 = new TimeSeries( "Q.PT", "", "", new Observation[]{} ),
		TS2 = new TimeSeries( "M.PT", "", "", new Observation[]{} ),
		TS3 = new TimeSeries( "Q.BE", "", "", new Observation[]{} );

	private static final Attribute
		AT1 = new Attribute( "Q.PT", "20041", "608", "NAM", "VAL" ),
		AT2 = new Attribute( "M.PT", "200401", "610", "NAM", "VAL" ),
		AT3 = new Attribute( "Q.PT", "", "", "NAM", "VAL" ),
		AT4 = new Attribute( "PT", "", "", "NAM", "VAL" ),
		AT5 = new Attribute( "", "", "", "NAM", "VAL" ),
		AT6 = new Attribute( "Q.BE", "", "", "NAM", "VAL" ),
		AT7 = new Attribute( "BE", "", "", "NAM", "VAL" );

	/* ====================== Constructors ====================== */

	public SubcubeConditionTest( final String name ) {
		super( name );
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite
	 */
	public static Test suite () {
		final TestSuite suite = new TestSuite( SubcubeConditionTest.class );
		return suite;
	}

	/**
	 * @param arg the array of argument {@link String}s
	 */
	public static void main ( final String[] arg ) {
		final String[] testCaseName = {
			SubcubeConditionTest.class.getName()
		};
		junit.textui.TestRunner.main( testCaseName );
	}

	/* ---------------------- Test Methods ---------------------- */

	public void test () {
		final FilterIterator.Condition dataSetCondition = new SubcubeCondition(
			KEY_FAMILY, ""
		);
		assertTrue( "ds accepts ts1", dataSetCondition.accept( TS1 ) );
		assertTrue( "ds accepts ts2", dataSetCondition.accept( TS2 ) );
		assertTrue( "ds accepts ts3", dataSetCondition.accept( TS3 ) );
		assertTrue( "ds accepts at1", dataSetCondition.accept( AT1 ) );
		assertTrue( "ds accepts at2", dataSetCondition.accept( AT2 ) );
		assertTrue( "ds accepts at3", dataSetCondition.accept( AT3 ) );
		assertTrue( "ds accepts at4", dataSetCondition.accept( AT4 ) );
		assertTrue( "ds accepts at5", dataSetCondition.accept( AT5 ) );
		assertTrue( "ds accepts at6", dataSetCondition.accept( AT6 ) );
		assertTrue( "ds accepts at7", dataSetCondition.accept( AT7 ) );

		final FilterIterator.Condition groupCondition = new SubcubeCondition(
			KEY_FAMILY, "PT"
		);
		assertTrue( "sg accepts ts1", groupCondition.accept( TS1 ) );
		assertTrue( "sg accepts ts2", groupCondition.accept( TS2 ) );
		assertTrue( "sg rejects ts3", ! groupCondition.accept( TS3 ) );
		assertTrue( "sg accepts at1", groupCondition.accept( AT1 ) );
		assertTrue( "sg accepts at2", groupCondition.accept( AT2 ) );
		assertTrue( "sg accepts at3", groupCondition.accept( AT3 ) );
		assertTrue( "sg accepts at4", groupCondition.accept( AT4 ) );
		assertTrue( "sg rejects at5", ! groupCondition.accept( AT5 ) );
		assertTrue( "sg rejects at6", ! groupCondition.accept( AT6 ) );
		assertTrue( "sg rejects at7", ! groupCondition.accept( AT7 ) );

		final FilterIterator.Condition seriesCondition = new SubcubeCondition(
			KEY_FAMILY, "Q.PT"
		);
		assertTrue( "ts accepts ts1", seriesCondition.accept( TS1 ) );
		assertTrue( "ts rejects ts2", ! seriesCondition.accept( TS2 ) );
		assertTrue( "ts rejects ts3", ! seriesCondition.accept( TS3 ) );
		assertTrue( "ts accepts at1", seriesCondition.accept( AT1 ) );
		assertTrue( "ts rejects at2", ! seriesCondition.accept( AT2 ) );
		assertTrue( "ts accepts at3", seriesCondition.accept( AT3 ) );
		assertTrue( "ts rejects at4", ! seriesCondition.accept( AT4 ) );
		assertTrue( "ts rejects at5", ! seriesCondition.accept( AT5 ) );
		assertTrue( "ts rejects at6", ! seriesCondition.accept( AT6 ) );
		assertTrue( "ts rejects at7", ! seriesCondition.accept( AT7 ) );
	}
}
