// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2004, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.util;

import java.util.Iterator;

import Int.ecb.gesmescb.AbstractCube;

/* ***************************** Class ****************************** */

/**
 * <p>Translates the objects that another {@link Iterator} retrieves.</p>
 *
 * <p>An instance wraps around the given iterator and delegates to this.
 * Override the method {@link #next()} to implement the desired translation.
 * The class is abstract to signal to user that without overriding
 * of <code>next</code>,
 * it provides just a useless delegation to another iterator.</p>
 *
 * @author
 * <a href="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</a>
 *
 * @version @(#)$Id: TranslatingIterator.java,v 1.1 2017/01/19 16:14:04 trainot Exp $
 */
public abstract class TranslatingIterator implements Iterator {

	private final Iterator iterator;

	/* ====================== Constructors ====================== */

	/** Create a new instance.
	 * @param iterator to get {@link AbstractCube}s from.
	 */
	public TranslatingIterator (
		final Iterator iterator
	) {
		if ( ( this.iterator = iterator ) == null ) {
			throw new NullPointerException(
				"Trying to construct a TranslatingIterator from null iterator."
			);
		}
	}

	/* ===================== Public Methods ===================== */

	public boolean hasNext () {
		return iterator.hasNext();
	}

	/** Override this method to implement the desired translation. */
	public Object next () {
		return iterator.next();
	}

	public void remove () {
		iterator.remove();
	}
}
