// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (C) 2005-2006, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.xml;

import java.io.IOException;

import junit.framework.Test;
import junit.framework.TestSuite;

/* ***************************** Class ****************************** */

/**
 * Implements a few tests for the {@link SDMXMLWriter}.
 *
 * @author
 * <a href="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</a>
 *
 * @version @(#)$Id: SDMXMLWriterTest.java,v 1.11 2015/07/20 06:54:09 trainot Exp $
 */
public class SDMXMLWriterTest extends XMLWriterTest {

	/* ======================= Constants ======================== */

	private static final String EPSILON_BEGIN_OUT
	= "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" + NEW_LINE + NEW_LINE
		+ "<!-- SDMX-ML from Gesmes/TS Suite -->" + NEW_LINE
		+ "<!-- " + WILDCARD + " -->" + NEW_LINE;

	private static final String STRUCTURE_BEGIN_OUT
	= "<!DOCTYPE Structure SYSTEM \"structure.dtd\">" + NEW_LINE
		+ "<?xml-stylesheet href=\"structure.xsl\" type=\"text/xsl\"?>" + NEW_LINE + NEW_LINE
		+ "<Structure"
		+ " xmlns=\"http://www.SDMX.org/resources/SDMXML/schemas/v1_0/message\""
		+ " xmlns:message=\"http://www.SDMX.org/resources/SDMXML/schemas/v1_0/message\""
		+ " xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\""
		+ " xsi:schemaLocation=\"http://www.SDMX.org/resources/SDMXML/schemas/v1_0/message message.xsd http://www.SDMX.org/resources/SDMXML/schemas/v1_0/structure structure.xsd\">" + NEW_LINE
		+ "\t<Header>" + NEW_LINE
		+ "\t\t<ID>IREF000012</ID>" + NEW_LINE
		+ "\t\t<Test>true</Test>" + NEW_LINE
		+ "\t\t<Name>The subject</Name>" + NEW_LINE
		+ "\t\t<Prepared>2002-08-17T17:48:00+02:00</Prepared>" + NEW_LINE
		+ "\t\t<Sender id=\"BBB\"/>" + NEW_LINE
		+ "\t\t<Receiver id=\"ZZZ\"/>" + NEW_LINE
		+ "\t</Header>" + NEW_LINE;

	private static final String STRUCTURE_END_OUT = "</Structure>" + NEW_LINE;

	private static final String DATA_BEGIN_OUT
	= "<!DOCTYPE MessageGroup SYSTEM \"compact.dtd\">" + NEW_LINE
		+ "<?xml-stylesheet href=\"compact.xsl\" type=\"text/xsl\"?>" + NEW_LINE + NEW_LINE
		+ "<MessageGroup"
		+ " xmlns=\"http://www.SDMX.org/resources/SDMXML/schemas/v1_0/message\""
		+ " xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\""
		+ " xsi:schemaLocation=\"http://www.SDMX.org/resources/SDMXML/schemas/v1_0/message message.xsd\">" + NEW_LINE
		+ "\t<Header>" + NEW_LINE
		+ "\t\t<ID>IREF000012</ID>" + NEW_LINE
		+ "\t\t<Test>true</Test>" + NEW_LINE
		+ "\t\t<Name>The subject</Name>" + NEW_LINE
		+ "\t\t<Prepared>2002-08-17T17:48:00+02:00</Prepared>" + NEW_LINE
		+ "\t\t<Sender id=\"BBB\">" + NEW_LINE
		+ "\t\t\t<Contact>" + NEW_LINE
		+ "\t\t\t\t<Name>BOP Hotline</Name>" + NEW_LINE
		+ "\t\t\t\t<Department>BOP</Department>" + NEW_LINE
		+ "\t\t\t\t<Role>Responsible for Information Production</Role>" + NEW_LINE
		+ "\t\t\t\t<Telephone>++34987230763</Telephone>" + NEW_LINE
		+ "\t\t\t</Contact>" + NEW_LINE
		+ "\t\t\t<Contact>" + NEW_LINE
		+ "\t\t\t\t<Name>BOP Head</Name>" + NEW_LINE
		+ "\t\t\t\t<Department>BOP</Department>" + NEW_LINE
		+ "\t\t\t\t<Role>Head of Information Production</Role>" + NEW_LINE
		+ "\t\t\t\t<Fax>++34987234762</Fax>" + NEW_LINE
		+ "\t\t\t</Contact>" + NEW_LINE
		+ "\t\t\t<Contact>" + NEW_LINE
		+ "\t\t\t\t<Name>External Systems Hotline</Name>" + NEW_LINE
		+ "\t\t\t\t<Department>SIS</Department>" + NEW_LINE
		+ "\t\t\t\t<Role>Responsible for Data Processing</Role>" + NEW_LINE
		+ "\t\t\t\t<URI>mailto:sis.external@ecb.int</URI>" + NEW_LINE
		+ "\t\t\t</Contact>" + NEW_LINE
		+ "\t\t\t<Contact>" + NEW_LINE
		+ "\t\t\t\t<Name>SIS Head</Name>" + NEW_LINE
		+ "\t\t\t\t<Department>SIS</Department>" + NEW_LINE
		+ "\t\t\t\t<Role>Head of Data Processing</Role>" + NEW_LINE
		+ "\t\t\t\t<X400>CO=DE; REST=unknown</X400>" + NEW_LINE
		+ "\t\t\t</Contact>" + NEW_LINE
		+ "\t\t</Sender>" + NEW_LINE
		+ "\t\t<Receiver id=\"ZZZ\"/>" + NEW_LINE
		+ "\t\t<KeyFamilyRef>KFSUMPF</KeyFamilyRef>" + NEW_LINE
		+ "\t\t<KeyFamilyAgency>BODI</KeyFamilyAgency>" + NEW_LINE
		+ "\t\t<DataSetAgency>ECB</DataSetAgency>" + NEW_LINE
		+ "\t\t<DataSetID>DS1</DataSetID>" + NEW_LINE
		+ "\t\t<DataSetAction>Update</DataSetAction>" + NEW_LINE
		+ "\t\t<Extracted>2000-11-01T13:10:00+01:00</Extracted>" + NEW_LINE
		+ "\t\t<ReportingBegin>2000-01-01T00:00:00+01:00</ReportingBegin>" + NEW_LINE
		+ "\t\t<ReportingEnd>2001-01-01T00:00:00+01:00</ReportingEnd>" + NEW_LINE
		+ "\t</Header>" + NEW_LINE;

	private static final String DELETE_MESSAGE_BEGIN_OUT
	= "<!DOCTYPE MessageGroup SYSTEM \"compact.dtd\">" + NEW_LINE
		+ "<?xml-stylesheet href=\"compact.xsl\" type=\"text/xsl\"?>" + NEW_LINE + NEW_LINE
		+ "<MessageGroup"
		+ " xmlns=\"http://www.SDMX.org/resources/SDMXML/schemas/v1_0/message\""
		+ " xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\""
		+ " xsi:schemaLocation=\"http://www.SDMX.org/resources/SDMXML/schemas/v1_0/message message.xsd\">" + NEW_LINE
		+ "\t<Header>" + NEW_LINE
		+ "\t\t<ID>IREF000012</ID>" + NEW_LINE
		+ "\t\t<Test>true</Test>" + NEW_LINE
		+ "\t\t<Name>Subject is \"Deletor\"</Name>" + NEW_LINE
		+ "\t\t<Prepared>2002-08-17T17:48:00+02:00</Prepared>" + NEW_LINE
		+ "\t\t<Sender id=\"BBB\"/>" + NEW_LINE
		+ "\t\t<Receiver id=\"ZZZ\"/>" + NEW_LINE
		+ "\t\t<KeyFamilyRef>KFSUMPF</KeyFamilyRef>" + NEW_LINE
        + "\t\t<KeyFamilyAgency>BODI</KeyFamilyAgency>" + NEW_LINE
		+ "\t\t<DataSetAgency>ECB</DataSetAgency>" + NEW_LINE
		+ "\t\t<DataSetID>DS1</DataSetID>" + NEW_LINE
		+ "\t\t<DataSetAction>Delete</DataSetAction>" + NEW_LINE
		+ "\t\t<Extracted>2000-11-01T13:10:00+01:00</Extracted>" + NEW_LINE
		+ "\t\t<ReportingBegin>2000-01-01T00:00:00+01:00</ReportingBegin>" + NEW_LINE
		+ "\t\t<ReportingEnd>2000-07-01T00:00:00+02:00</ReportingEnd>" + NEW_LINE
		+ "\t</Header>" + NEW_LINE;

	private static final String CODE_LIST_OUT
	= "<message:CodeLists xmlns=\"http://www.SDMX.org/resources/SDMXML/schemas/v1_0/structure\">" + NEW_LINE
		+ "\t<CodeList agency=\"BODI\" id=\"CL_SUMPF\">" + NEW_LINE
		+ "\t\t<Name>Sumpfige CodeList</Name>" + NEW_LINE
		+ "\t\t<Code value=\"S\">" + NEW_LINE
		+ "\t\t\t<Description>Sumpf</Description>" + NEW_LINE
		+ "\t\t</Code>" + NEW_LINE
		+ "\t\t<Code value=\"U\">" + NEW_LINE
		+ "\t\t\t<Description>Und</Description>" + NEW_LINE
		+ "\t\t</Code>" + NEW_LINE
		+ "\t\t<Code value=\"M\">" + NEW_LINE
		+ "\t\t\t<Description>Manche</Description>" + NEW_LINE
		+ "\t\t</Code>" + NEW_LINE
		+ "\t\t<Code value=\"P\">" + NEW_LINE
		+ "\t\t\t<Description>Parteien</Description>" + NEW_LINE
		+ "\t\t</Code>" + NEW_LINE
		+ "\t\t<Code value=\"F\">" + NEW_LINE
		+ "\t\t\t<Description>Finanzierung</Description>" + NEW_LINE
		+ "\t\t</Code>" + NEW_LINE
		+ "\t</CodeList>" + NEW_LINE
		+ "</message:CodeLists>" + NEW_LINE;

	private static final String DATA_END_OUT = "</MessageGroup>" + NEW_LINE;

	private static final String CONCEPT_OUT
	= "<message:Concepts xmlns=\"http://www.SDMX.org/resources/SDMXML/schemas/v1_0/structure\">" + NEW_LINE
		+ "\t<Concept agency=\"BODI\" id=\"SUMPF\">" + NEW_LINE
		+ "\t\t<Name>Sumpfiges Concept</Name>" + NEW_LINE
		+ "\t</Concept>" + NEW_LINE
		+ "</message:Concepts>" + NEW_LINE;

	private static final String KEY_FAMILY_OUT
	= "<message:KeyFamilies xmlns=\"http://www.SDMX.org/resources/SDMXML/schemas/v1_0/structure\">" + NEW_LINE
		+ "\t<KeyFamily agency=\"BODI\" id=\"KFSUMPF\" uri=\"http://www.sumpfland.org/vocabulary/2002-09-12/kfsumpf\">" + NEW_LINE
		+ "\t\t<Name>Sample Key Family</Name>" + NEW_LINE
		+ "\t\t<Components>" + NEW_LINE
		+ "\t\t\t<Dimension concept=\"FREQ\" codelist=\"CL_FREQ\" isFrequencyDimension=\"true\"/>" + NEW_LINE
		+ "\t\t\t<Dimension concept=\"SUMPF\" codelist=\"CL_SUMPF\" isFrequencyDimension=\"false\"/>" + NEW_LINE
		+ "\t\t\t<TimeDimension concept=\"TIME_PERIOD\"/>" + NEW_LINE
		+ "\t\t\t<Group id=\"Group\">" + NEW_LINE
		+ "\t\t\t\t<DimensionRef>SUMPF</DimensionRef>" + NEW_LINE
		+ "\t\t\t</Group>" + NEW_LINE
		+ "\t\t\t<PrimaryMeasure concept=\"OBS_VALUE\"/>" + NEW_LINE
		+ "\t\t\t<Attribute concept=\"TIME_FORMAT\" attachmentLevel=\"Series\" codelist=\"CL_TIME_FORMAT\" assignmentStatus=\"Mandatory\" isTimeFormat=\"true\">" + NEW_LINE
		+ "\t\t\t\t<TextFormat TextType=\"AlphaNumFixed\" length=\"3\"/>" + NEW_LINE
		+ "\t\t\t</Attribute>" + NEW_LINE
		+ "\t\t\t<Attribute concept=\"OBS_STATUS\" attachmentLevel=\"Observation\" codelist=\"CL_OBS_STATUS\" assignmentStatus=\"Mandatory\" isTimeFormat=\"false\">" + NEW_LINE
		+ "\t\t\t\t<TextFormat TextType=\"AlphaFixed\" length=\"1\"/>" + NEW_LINE
		+ "\t\t\t</Attribute>" + NEW_LINE
		+ "\t\t\t<Attribute concept=\"OBS_CONF\" attachmentLevel=\"Observation\" codelist=\"CL_OBS_CONF\" assignmentStatus=\"Conditional\" isTimeFormat=\"false\">" + NEW_LINE
		+ "\t\t\t\t<TextFormat TextType=\"AlphaFixed\" length=\"1\"/>" + NEW_LINE
		+ "\t\t\t</Attribute>" + NEW_LINE
		+ "\t\t\t<Attribute concept=\"OBS_PRE_BREAK\" attachmentLevel=\"Observation\" assignmentStatus=\"Conditional\" isTimeFormat=\"false\">" + NEW_LINE
		+ "\t\t\t\t<TextFormat TextType=\"AlphaNum\" length=\"15\"/>" + NEW_LINE
		+ "\t\t\t</Attribute>" + NEW_LINE
		+ "\t\t\t<Attribute concept=\"OBS_COM\" attachmentLevel=\"Observation\" assignmentStatus=\"Conditional\" isTimeFormat=\"false\">" + NEW_LINE
		+ "\t\t\t\t<TextFormat TextType=\"AlphaNum\" length=\"1050\"/>" + NEW_LINE
		+ "\t\t\t</Attribute>" + NEW_LINE
		+ "\t\t\t<Attribute concept=\"UNIT\" attachmentLevel=\"Series\" codelist=\"CL_UNIT\" assignmentStatus=\"Mandatory\" isTimeFormat=\"false\">" + NEW_LINE
		+ "\t\t\t\t<TextFormat TextType=\"AlphaNum\" length=\"4\"/>" + NEW_LINE
		+ "\t\t\t</Attribute>" + NEW_LINE
		+ "\t\t\t<Attribute concept=\"TITLE\" attachmentLevel=\"Group\" assignmentStatus=\"Conditional\" isTimeFormat=\"false\">" + NEW_LINE
		+ "\t\t\t\t<TextFormat TextType=\"AlphaNum\" length=\"64\"/>" + NEW_LINE
		+ "\t\t\t\t<AttachmentGroup>Group</AttachmentGroup>" + NEW_LINE
		+ "\t\t\t</Attribute>" + NEW_LINE
		+ "\t\t\t<Attribute concept=\"DS_COMMENT\" attachmentLevel=\"DataSet\" assignmentStatus=\"Conditional\" isTimeFormat=\"false\">" + NEW_LINE
		+ "\t\t\t\t<TextFormat TextType=\"AlphaNum\" length=\"64\"/>" + NEW_LINE
		+ "\t\t\t</Attribute>" + NEW_LINE
		+ "\t\t</Components>" + NEW_LINE
		+ "\t</KeyFamily>" + NEW_LINE
		+ "</message:KeyFamilies>" + NEW_LINE;

	private static final String KEY_FAMILY_OUT_V2
	= "<message:KeyFamilies xmlns=\"http://www.SDMX.org/resources/SDMXML/schemas/v1_0/structure\">" + NEW_LINE
		+ "\t<KeyFamily agency=\"BODI\" id=\"KFSUMPF\" uri=\"http://www.sumpfland.org/vocabulary/2002-09-12/kfsumpf\">" + NEW_LINE
		+ "\t\t<Name>Sample Key Family</Name>" + NEW_LINE
		+ "\t\t<Components>" + NEW_LINE
		+ "\t\t\t<Dimension conceptRef=\"FREQ\" codelist=\"CL_FREQ\" isFrequencyDimension=\"true\"/>" + NEW_LINE
		+ "\t\t\t<Dimension conceptRef=\"SUMPF\" codelist=\"CL_SUMPF\" isFrequencyDimension=\"false\"/>" + NEW_LINE
		+ "\t\t\t<TimeDimension conceptRef=\"TIME_PERIOD\"/>" + NEW_LINE
		+ "\t\t\t<Group id=\"Group\">" + NEW_LINE
		+ "\t\t\t\t<DimensionRef>SUMPF</DimensionRef>" + NEW_LINE
		+ "\t\t\t</Group>" + NEW_LINE
		+ "\t\t\t<PrimaryMeasure conceptRef=\"OBS_VALUE\"/>" + NEW_LINE
		+ "\t\t\t<Attribute conceptRef=\"TIME_FORMAT\" attachmentLevel=\"Series\" codelist=\"CL_TIME_FORMAT\" assignmentStatus=\"Mandatory\" isTimeFormat=\"true\">" + NEW_LINE
		+ "\t\t\t\t<TextFormat textType=\"String\" minLength=\"3\" maxLength=\"3\"/>" + NEW_LINE
		+ "\t\t\t</Attribute>" + NEW_LINE
		+ "\t\t\t<Attribute conceptRef=\"OBS_STATUS\" attachmentLevel=\"Observation\" codelist=\"CL_OBS_STATUS\" assignmentStatus=\"Mandatory\" isTimeFormat=\"false\">" + NEW_LINE
		+ "\t\t\t\t<TextFormat textType=\"String\" minLength=\"1\" maxLength=\"1\"/>" + NEW_LINE
		+ "\t\t\t</Attribute>" + NEW_LINE
		+ "\t\t\t<Attribute conceptRef=\"OBS_CONF\" attachmentLevel=\"Observation\" codelist=\"CL_OBS_CONF\" assignmentStatus=\"Conditional\" isTimeFormat=\"false\">" + NEW_LINE
		+ "\t\t\t\t<TextFormat textType=\"String\" minLength=\"1\" maxLength=\"1\"/>" + NEW_LINE
		+ "\t\t\t</Attribute>" + NEW_LINE
		+ "\t\t\t<Attribute conceptRef=\"OBS_PRE_BREAK\" attachmentLevel=\"Observation\" assignmentStatus=\"Conditional\" isTimeFormat=\"false\">" + NEW_LINE
		+ "\t\t\t\t<TextFormat textType=\"String\" minLength=\"0\" maxLength=\"15\"/>" + NEW_LINE
		+ "\t\t\t</Attribute>" + NEW_LINE
		+ "\t\t\t<Attribute conceptRef=\"OBS_COM\" attachmentLevel=\"Observation\" assignmentStatus=\"Conditional\" isTimeFormat=\"false\">" + NEW_LINE
		+ "\t\t\t\t<TextFormat textType=\"String\" minLength=\"0\" maxLength=\"1050\"/>" + NEW_LINE
		+ "\t\t\t</Attribute>" + NEW_LINE
		+ "\t\t\t<Attribute conceptRef=\"UNIT\" attachmentLevel=\"Series\" codelist=\"CL_UNIT\" assignmentStatus=\"Mandatory\" isTimeFormat=\"false\">" + NEW_LINE
		+ "\t\t\t\t<TextFormat textType=\"String\" minLength=\"0\" maxLength=\"4\"/>" + NEW_LINE
		+ "\t\t\t</Attribute>" + NEW_LINE
		+ "\t\t\t<Attribute conceptRef=\"TITLE\" attachmentLevel=\"Group\" assignmentStatus=\"Conditional\" isTimeFormat=\"false\">" + NEW_LINE
		+ "\t\t\t\t<TextFormat textType=\"String\" minLength=\"0\" maxLength=\"64\"/>" + NEW_LINE
		+ "\t\t\t\t<AttachmentGroup>Group</AttachmentGroup>" + NEW_LINE
		+ "\t\t\t</Attribute>" + NEW_LINE
		+ "\t\t\t<Attribute conceptRef=\"DS_COMMENT\" attachmentLevel=\"DataSet\" assignmentStatus=\"Conditional\" isTimeFormat=\"false\">" + NEW_LINE
		+ "\t\t\t\t<TextFormat textType=\"String\" minLength=\"0\" maxLength=\"64\"/>" + NEW_LINE
		+ "\t\t\t</Attribute>" + NEW_LINE
		+ "\t\t</Components>" + NEW_LINE
		+ "\t</KeyFamily>" + NEW_LINE
		+ "</message:KeyFamilies>" + NEW_LINE;

	private static final String DATA1_SORTED_OUT
	= "<DataSet xmlns=\"http://www.sumpfland.org/vocabulary/2002-09-12/kfsumpf\" xsi:schemaLocation=\"http://www.sumpfland.org/vocabulary/2002-09-12/kfsumpf compact.xsd\" DS_COMMENT=\"dscom\">" + NEW_LINE
		+ "\t<Group SUMPF=\"P\" TITLE=\"sgcom\"/>" + NEW_LINE
		+ "\t<Series FREQ=\"M\" SUMPF=\"P\" TIME_FORMAT=\"P1M\" UNIT=\"tscom\">" + NEW_LINE
		+ "\t\t<Obs TIME_PERIOD=\"2003-01\" OBS_VALUE=\"18.5\" OBS_STATUS=\"A\" OBS_CONF=\"F\" OBS_PRE_BREAK=\"NaN\"/>" + NEW_LINE
		+ "\t\t<Obs TIME_PERIOD=\"2003-02\" OBS_VALUE=\"NaN\" OBS_STATUS=\"M\" OBS_CONF=\"C\" OBS_PRE_BREAK=\"NaN\"/>" + NEW_LINE
		+ "\t\t<Obs TIME_PERIOD=\"2003-03\" OBS_COM=\"obscom\"/>" + NEW_LINE
		+ "\t</Series>" + NEW_LINE
		+ "</DataSet>" + NEW_LINE;

	private static final String DATA1_UNSORTED_OUT
	= "<DataSet xmlns=\"http://www.sumpfland.org/vocabulary/2002-09-12/kfsumpf\" xsi:schemaLocation=\"http://www.sumpfland.org/vocabulary/2002-09-12/kfsumpf compact.xsd\">" + NEW_LINE
		+ "\t<Series FREQ=\"M\" SUMPF=\"P\" TIME_FORMAT=\"P1M\" UNIT=\"tscom\">" + NEW_LINE
		+ "\t\t<Obs TIME_PERIOD=\"2003-01\" OBS_VALUE=\"18.5\" OBS_STATUS=\"A\" OBS_CONF=\"F\" OBS_PRE_BREAK=\"NaN\"/>" + NEW_LINE
		+ "\t\t<Obs TIME_PERIOD=\"2003-02\" OBS_VALUE=\"NaN\" OBS_STATUS=\"M\" OBS_CONF=\"C\" OBS_PRE_BREAK=\"NaN\"/>" + NEW_LINE
		+ "\t</Series>" + NEW_LINE
		+ "</DataSet>" + NEW_LINE
		+ "<DataSet xmlns=\"http://www.sumpfland.org/vocabulary/2002-09-12/kfsumpf\" xsi:schemaLocation=\"http://www.sumpfland.org/vocabulary/2002-09-12/kfsumpf compact.xsd\" DS_COMMENT=\"dscom\">" + NEW_LINE
		+ "\t<Group SUMPF=\"P\" TITLE=\"sgcom\"/>" + NEW_LINE
		+ "\t<Series FREQ=\"M\" SUMPF=\"P\" TIME_FORMAT=\"P1M\">" + NEW_LINE
		+ "\t\t<Obs TIME_PERIOD=\"2003-03\" OBS_COM=\"obscom\"/>" + NEW_LINE
		+ "\t</Series>" + NEW_LINE
		+ "</DataSet>" + NEW_LINE;

	private static final String DELETE_SORTED_OUT
	= "<DataSet xmlns=\"http://www.sumpfland.org/vocabulary/2002-09-12/kfsumpf\" xsi:schemaLocation=\"http://www.sumpfland.org/vocabulary/2002-09-12/kfsumpf compact.xsd\" DS_COMMENT=\"dscom\">" + NEW_LINE
		+ "\t<Group SUMPF=\"P\" TITLE=\"sgcom\"/>" + NEW_LINE
		+ "\t<Series FREQ=\"M\" SUMPF=\"P\" UNIT=\"tscom\">" + NEW_LINE
		+ "\t\t<Obs TIME_PERIOD=\"2003-01\" OBS_VALUE=\"18.5\" OBS_STATUS=\"A\" OBS_CONF=\"F\" OBS_PRE_BREAK=\"NaN\"/>" + NEW_LINE
		+ "\t\t<Obs TIME_PERIOD=\"2003-02\" OBS_VALUE=\"NaN\" OBS_STATUS=\"M\" OBS_CONF=\"C\" OBS_PRE_BREAK=\"NaN\"/>" + NEW_LINE
		+ "\t\t<Obs TIME_PERIOD=\"2003-03\" OBS_COM=\"obscom\"/>" + NEW_LINE
		+ "\t</Series>" + NEW_LINE
		+ "\t<Series FREQ=\"Q\" SUMPF=\"P\">" + NEW_LINE
		+ "\t\t<Obs TIME_PERIOD=\"2000-Q1\"/>" + NEW_LINE
		+ "\t\t<Obs TIME_PERIOD=\"2000-Q2\"/>" + NEW_LINE
		+ "\t</Series>" + NEW_LINE
		+ "\t<Series FREQ=\"M\" SUMPF=\"U\"/>" + NEW_LINE
		+ "</DataSet>" + NEW_LINE;

	private static final String DELETE_UNSORTED_OUT
	= "<DataSet xmlns=\"http://www.sumpfland.org/vocabulary/2002-09-12/kfsumpf\" xsi:schemaLocation=\"http://www.sumpfland.org/vocabulary/2002-09-12/kfsumpf compact.xsd\">" + NEW_LINE
		+ "\t<Series FREQ=\"M\" SUMPF=\"P\" UNIT=\"tscom\">" + NEW_LINE
		+ "\t\t<Obs TIME_PERIOD=\"2003-01\" OBS_VALUE=\"18.5\" OBS_STATUS=\"A\" OBS_CONF=\"F\" OBS_PRE_BREAK=\"NaN\"/>" + NEW_LINE
		+ "\t\t<Obs TIME_PERIOD=\"2003-02\" OBS_VALUE=\"NaN\" OBS_STATUS=\"M\" OBS_CONF=\"C\" OBS_PRE_BREAK=\"NaN\"/>" + NEW_LINE
		+ "\t</Series>" + NEW_LINE
		+ "\t<Series FREQ=\"M\" SUMPF=\"U\"/>" + NEW_LINE
		+ "\t<Series FREQ=\"Q\" SUMPF=\"P\">" + NEW_LINE
		+ "\t\t<Obs TIME_PERIOD=\"2000-Q1\"/>" + NEW_LINE
		+ "\t\t<Obs TIME_PERIOD=\"2000-Q2\"/>" + NEW_LINE
		+ "\t</Series>" + NEW_LINE
		+ "</DataSet>" + NEW_LINE
		+ "<DataSet xmlns=\"http://www.sumpfland.org/vocabulary/2002-09-12/kfsumpf\" xsi:schemaLocation=\"http://www.sumpfland.org/vocabulary/2002-09-12/kfsumpf compact.xsd\" DS_COMMENT=\"dscom\">" + NEW_LINE
		+ "\t<Group SUMPF=\"P\" TITLE=\"sgcom\"/>" + NEW_LINE
		+ "\t<Series FREQ=\"M\" SUMPF=\"P\">" + NEW_LINE
		+ "\t\t<Obs TIME_PERIOD=\"2003-03\" OBS_COM=\"obscom\"/>" + NEW_LINE
		+ "\t</Series>" + NEW_LINE
		+ "</DataSet>" + NEW_LINE;

	private static final String DATA1_SORTED_OUT_NODEFAULTS
	= "<DataSet xmlns=\"http://www.sumpfland.org/vocabulary/2002-09-12/kfsumpf\" xsi:schemaLocation=\"http://www.sumpfland.org/vocabulary/2002-09-12/kfsumpf compact.xsd\" DS_COMMENT=\"dscom\">" + NEW_LINE
		+ "\t<Group SUMPF=\"P\" TITLE=\"sgcom\"/>" + NEW_LINE
		+ "\t<Series FREQ=\"M\" SUMPF=\"P\" TIME_FORMAT=\"P1M\" UNIT=\"tscom\">" + NEW_LINE
		+ "\t\t<Obs TIME_PERIOD=\"2003-01\" OBS_VALUE=\"18.5\" OBS_STATUS=\"A\"/>" + NEW_LINE
		+ "\t\t<Obs TIME_PERIOD=\"2003-02\" OBS_VALUE=\"NaN\" OBS_STATUS=\"M\" OBS_CONF=\"C\"/>" + NEW_LINE
		+ "\t\t<Obs TIME_PERIOD=\"2003-03\" OBS_COM=\"obscom\"/>" + NEW_LINE
		+ "\t</Series>" + NEW_LINE
		+ "</DataSet>" + NEW_LINE;
	
	

	/* ====================== Constructors ====================== */

	public SDMXMLWriterTest( final String name ) {
		super( name );
		
	}

	/* ================== Protected Methods ===================== */

	protected void setUp () {
		super.setUp();
		System.setProperty(
			"Int.ecb.gesmescb.xml.SDMXMLWriter.message.schema",
			"message.xsd"
		);
		System.setProperty(
			"Int.ecb.gesmescb.xml.SDMXMLWriter.structure.dtd",
			"structure.dtd"
		);
		System.setProperty(
			"Int.ecb.gesmescb.xml.SDMXMLWriter.structure.schema",
			"structure.xsd"
		);
		System.setProperty(
			"Int.ecb.gesmescb.xml.SDMXMLWriter.structure.stylesheet",
			"structure.xsl"
		);
		System.setProperty(
			"Int.ecb.gesmescb.xml.KeyFamilyURIMapping.BODI.KFSUMPF.dtd",
			"compact.dtd"
		);
		System.setProperty(
			"Int.ecb.gesmescb.xml.KeyFamilyURIMapping.BODI.KFSUMPF.schema",
			"compact.xsd"
		);
		System.setProperty(
			"Int.ecb.gesmescb.xml.KeyFamilyURIMapping.BODI.KFSUMPF.stylesheet",
			"compact.xsl"
		);
		System.setProperty(
			"Int.ecb.gesmescb.xml.SDMXMLWriter.defaultObsConf",
			"F"
		);
		System.setProperty(
			"Int.ecb.gesmescb.xml.SDMXMLWriter.defaultObsPreBreak",
			"NaN"
		);
		System.setProperty(
			"Int.ecb.gesmescb.xml.SDMXMLWriter.dataSetAgency",
			"ECB"
		);
	}

	protected void tearDown () {
		System.getProperties().remove(
			"Int.ecb.gesmescb.xml.SDMXMLWriter.sdmxVersion"
		);
		System.getProperties().remove(
			"Int.ecb.gesmescb.xml.SDMXMLWriter.message.schema"
		);
		System.getProperties().remove(
			"Int.ecb.gesmescb.xml.SDMXMLWriter.structure.dtd"
		);
		System.getProperties().remove(
			"Int.ecb.gesmescb.xml.SDMXMLWriter.structure.schema"
		);
		System.getProperties().remove(
			"Int.ecb.gesmescb.xml.SDMXMLWriter.structure.stylesheet"
		);
		System.getProperties().remove(
			"Int.ecb.gesmescb.xml.KeyFamilyURIMapping.BODI.KFSUMPF.dtd"
		);
		System.getProperties().remove(
			"Int.ecb.gesmescb.xml.KeyFamilyURIMapping.BODI.KFSUMPF.schema"
		);
		System.getProperties().remove(
			"Int.ecb.gesmescb.xml.KeyFamilyURIMapping.BODI.KFSUMPF.stylesheet"
		);
		System.getProperties().remove(
			"Int.ecb.gesmescb.xml.SDMXMLWriter.defaultObsConf"
		);
		System.getProperties().remove(
			"Int.ecb.gesmescb.xml.SDMXMLWriter.defaultObsPreBreak"
		);
		System.getProperties().remove(
			"Int.ecb.gesmescb.xml.SDMXMLWriter.dataSetAgency"
		);
		super.tearDown();
	}

	protected String getClassName () {
		return "Int.ecb.gesmescb.xml.SDMXMLWriter";
	}

	protected XMLWriter factory () throws IOException {
		return new SDMXMLWriter( "" );
	}
	
	protected XMLWriter streamFactory () throws IOException {
		return new SDMXMLWriter( "" ,  this.outputStream );
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite
	 */
	public static Test suite () {
		final TestSuite suite = new TestSuite( SDMXMLWriterTest.class );
		return suite;
	}

	/**
	 * @param arg the array of argument {@link String}s
	 */
	public static void main ( final String[] arg ) {
		final String[] testCaseName = {
			SDMXMLWriterTest.class.getName()
		};
		junit.textui.TestRunner.main( testCaseName );
	}

	/* ---------------------- Test Methods ---------------------- */
	
	public void testStructureMessageBeginStream () throws Exception {
		writeStream ( new Object[]{
			INTERCHANGE_BEGIN,
			STRUCTURE_MESSAGE_BEGIN,
			MESSAGE_END,
			INTERCHANGE_END
		} );
		assertEqualsLinewise(
			EPSILON_BEGIN_OUT
			+ STRUCTURE_BEGIN_OUT
			+ STRUCTURE_END_OUT,
			readTempFile()
		);
	}
	
	public void testStructureMessageBegin () throws Exception {
		writeTempFile ( new Object[]{
			INTERCHANGE_BEGIN,
			STRUCTURE_MESSAGE_BEGIN,
			MESSAGE_END,
			INTERCHANGE_END
		} );
		assertEqualsLinewise(
			EPSILON_BEGIN_OUT
			+ STRUCTURE_BEGIN_OUT
			+ STRUCTURE_END_OUT,
			readTempFile()
		);
	}

	public void testStructureMessageBeginV2 () throws Exception {
		System.setProperty(
			"Int.ecb.gesmescb.xml.SDMXMLWriter.sdmxVersion",
			"2.0"
		);
		writeTempFile ( new Object[]{
			INTERCHANGE_BEGIN,
			STRUCTURE_MESSAGE_BEGIN,
			MESSAGE_END,
			INTERCHANGE_END
		} );
		assertEqualsLinewise(
			version1to2(
				EPSILON_BEGIN_OUT
				+ STRUCTURE_BEGIN_OUT
				+ STRUCTURE_END_OUT
			),
			readTempFile()
		);
	}

	public void testDataMessageBegin () throws Exception {
		writeTempFile ( new Object[]{
			INTERCHANGE_BEGIN,
			MESSAGE_BEGIN,
			MESSAGE_END,
			INTERCHANGE_END
		} );
		assertEqualsLinewise(
			EPSILON_BEGIN_OUT
			+ DATA_BEGIN_OUT
			+ DATA_END_OUT,
			readTempFile()
		);
	}

	public void testDataMessageBeginV2 () throws Exception {
		System.setProperty(
			"Int.ecb.gesmescb.xml.SDMXMLWriter.sdmxVersion",
			"2.0"
		);
		writeTempFile ( new Object[]{
			INTERCHANGE_BEGIN,
			MESSAGE_BEGIN,
			MESSAGE_END,
			INTERCHANGE_END
		} );
		assertEqualsLinewise(
			version1to2(
				EPSILON_BEGIN_OUT
				+ DATA_BEGIN_OUT
				+ DATA_END_OUT
			),
			readTempFile()
		);
	}

	public void testCodeList () throws Exception {
		writeTempFile ( new Object[]{
			INTERCHANGE_BEGIN,
			STRUCTURE_MESSAGE_BEGIN,
			CODE_LIST,
			MESSAGE_END,
			INTERCHANGE_END
		} );
		assertEqualsLinewise(
			EPSILON_BEGIN_OUT
			+ STRUCTURE_BEGIN_OUT
			+ moreIndent( CODE_LIST_OUT, 1 )
			+ STRUCTURE_END_OUT,
			readTempFile()
		);
	}

	public void testCodeListV2 () throws Exception {
		System.setProperty(
			"Int.ecb.gesmescb.xml.SDMXMLWriter.sdmxVersion",
			"2.0"
		);
		writeTempFile ( new Object[]{
			INTERCHANGE_BEGIN,
			STRUCTURE_MESSAGE_BEGIN,
			CODE_LIST,
			MESSAGE_END,
			INTERCHANGE_END
		} );
		assertEqualsLinewise(
			version1to2(
				EPSILON_BEGIN_OUT
				+ STRUCTURE_BEGIN_OUT
				+ moreIndent( CODE_LIST_OUT, 1 )
				+ STRUCTURE_END_OUT
			),
			readTempFile()
		);
	}

	public void testConcept () throws Exception {
		writeTempFile ( new Object[]{
			INTERCHANGE_BEGIN,
			STRUCTURE_MESSAGE_BEGIN,
			CONCEPT,
			MESSAGE_END,
			INTERCHANGE_END
		} );
		assertEqualsLinewise(
			EPSILON_BEGIN_OUT
			+ STRUCTURE_BEGIN_OUT
			+ moreIndent( CONCEPT_OUT, 1 )
			+ STRUCTURE_END_OUT,
			readTempFile()
		);
	}

	public void testConceptV2 () throws Exception {
		System.setProperty(
			"Int.ecb.gesmescb.xml.SDMXMLWriter.sdmxVersion",
			"2.0"
		);
		writeTempFile ( new Object[]{
			INTERCHANGE_BEGIN,
			STRUCTURE_MESSAGE_BEGIN,
			CONCEPT,
			MESSAGE_END,
			INTERCHANGE_END
		} );
		assertEqualsLinewise(
			version1to2(
				EPSILON_BEGIN_OUT
				+ STRUCTURE_BEGIN_OUT
				+ moreIndent( CONCEPT_OUT, 1 )
				+ STRUCTURE_END_OUT
			),
			readTempFile()
		);
	}

	public void testKeyFamily () throws Exception {
		writeTempFile ( new Object[]{
			INTERCHANGE_BEGIN,
			STRUCTURE_MESSAGE_BEGIN,
			KEY_FAMILY,
			MESSAGE_END,
			INTERCHANGE_END
		} );
		assertEqualsLinewise(
			EPSILON_BEGIN_OUT
			+ STRUCTURE_BEGIN_OUT
			+ moreIndent( KEY_FAMILY_OUT, 1 )
			+ STRUCTURE_END_OUT,
			readTempFile()
		);
	}

	public void testKeyFamilyV2 () throws Exception {
		System.setProperty(
			"Int.ecb.gesmescb.xml.SDMXMLWriter.sdmxVersion",
			"2.0"
		);
		writeTempFile ( new Object[]{
			INTERCHANGE_BEGIN,
			STRUCTURE_MESSAGE_BEGIN,
			KEY_FAMILY,
			MESSAGE_END,
			INTERCHANGE_END
		} );
		assertEqualsLinewise(
			version1to2(
				EPSILON_BEGIN_OUT
				+ STRUCTURE_BEGIN_OUT
				+ moreIndent( KEY_FAMILY_OUT_V2, 1 )
				+ STRUCTURE_END_OUT
			),
			readTempFile()
		);
	}

	public void testSortedUpdate () throws Exception {
		writeTempFile ( new Object[]{
			INTERCHANGE_BEGIN,
			MESSAGE_BEGIN,
			DS_ATT,
			SG_ATT,
			TS_ATT,
			SERIES1,
			OBS_ATT,
			SERIES2,
			MESSAGE_END,
			INTERCHANGE_END
		} );
		assertEqualsLinewise(
			EPSILON_BEGIN_OUT
			+ DATA_BEGIN_OUT
			+ moreIndent( DATA1_SORTED_OUT, 1 )
			+ DATA_END_OUT,
			readTempFile()
		);
	}

	public void testUnsortedUpdate () throws Exception {
		writeTempFile ( new Object[]{
			INTERCHANGE_BEGIN,
			MESSAGE_BEGIN,
			SERIES1,
			SERIES2,
			DS_ATT,
		    TS_ATT,
			SG_ATT,
			OBS_ATT,
			MESSAGE_END,
			INTERCHANGE_END
		} );
		assertEqualsLinewise(
			EPSILON_BEGIN_OUT
			+ DATA_BEGIN_OUT
			+ moreIndent( DATA1_UNSORTED_OUT, 1 )
			+ DATA_END_OUT,
			readTempFile()
		);
	}

	public void testSortedDelete () throws Exception {
		writeTempFile ( new Object[]{
			INTERCHANGE_BEGIN,
			DELETE_MESSAGE_BEGIN,
			DS_ATT,
			SG_ATT,
			TS_ATT,
			SERIES1,
			OBS_ATT,
			SERIES3,
			SERIES2,
			MESSAGE_END,
			INTERCHANGE_END
		} );
		assertEqualsLinewise(
			EPSILON_BEGIN_OUT
			+ DELETE_MESSAGE_BEGIN_OUT
			+ moreIndent( DELETE_SORTED_OUT, 1 )
			+ DATA_END_OUT,
			readTempFile()
		);
	}

	public void testUnsortedDelete () throws Exception {
		writeTempFile ( new Object[]{
			INTERCHANGE_BEGIN,
			DELETE_MESSAGE_BEGIN,
			SERIES1,
			SERIES2,
			SERIES3,
			DS_ATT,
			SG_ATT,
			OBS_ATT,
			TS_ATT,
			MESSAGE_END,
			INTERCHANGE_END
		} );
		assertEqualsLinewise(
			EPSILON_BEGIN_OUT
			+ DELETE_MESSAGE_BEGIN_OUT
			+ moreIndent( DELETE_UNSORTED_OUT, 1 )
			+ DATA_END_OUT,
			readTempFile()
		);
	}

	public void testEmptyDefaultConfAndBreak () throws Exception {
		System.getProperties().remove(
			"Int.ecb.gesmescb.xml.SDMXMLWriter.defaultObsConf"
		);
		System.getProperties().remove(
			"Int.ecb.gesmescb.xml.SDMXMLWriter.defaultObsPreBreak"
		);

		writeTempFile ( new Object[]{
			INTERCHANGE_BEGIN,
			MESSAGE_BEGIN,
			DS_ATT,
			SG_ATT,
			TS_ATT,
			SERIES1,
			OBS_ATT,
			SERIES2,
			MESSAGE_END,
			INTERCHANGE_END
		} );
		assertEqualsLinewise(
			EPSILON_BEGIN_OUT
			+ DATA_BEGIN_OUT
			+ moreIndent( DATA1_SORTED_OUT_NODEFAULTS, 1 )
			+ DATA_END_OUT,
			readTempFile()
		);
	}

	public void testWith2Messages () throws Exception {
		//System.setProperty(
	//		"Int.ecb.gesmescb.xml.SDMXMLWriter.sdmxVersion",
//			"2.0"
//		);
		writeTempFile ( new Object[]{
			INTERCHANGE_BEGIN,
			DELETE_MESSAGE_BEGIN,
			DS_ATT,
			SG_ATT,
			TS_ATT,
			SERIES1,
			OBS_ATT,
			SERIES3,
			SERIES2,
			MESSAGE_END, 
			MESSAGE_BEGIN,
			DS_ATT,
			SG_ATT,
			TS_ATT,
			SERIES1,
			OBS_ATT,
			SERIES2,
			MESSAGE_END,
			INTERCHANGE_END
		} );
		String expectedContents =
				EPSILON_BEGIN_OUT
				+ DELETE_MESSAGE_BEGIN_OUT
				+ moreIndent(DELETE_SORTED_OUT,1)
				+ moreIndent(DATA1_SORTED_OUT,1)
				+ DATA_END_OUT;
		assertEqualsLinewise(
			expectedContents
			,
			readTempFile()
		);
	}
	
	/* ==================== Private Methods ===================== */

	/** Converts expected test output from version 1 to version 2. */
	private static String version1to2 ( final String version1text ) {
		return version1text.replaceAll(
				"http://www.SDMX.org/resources/SDMXML/schemas/v1_0/message",
				"http://www.SDMX.org/resources/SDMXML/schemas/v2_0/message"
			).replaceAll(
				"http://www.SDMX.org/resources/SDMXML/schemas/v1_0/structure",
				"http://www.SDMX.org/resources/SDMXML/schemas/v2_0/structure"
			).replaceAll(
				" agency=",
				" agencyID="
			).replaceAll(
				"Update",
				"Replace"
			).replaceAll(
				"\t\t<DataSetAction>Update</DataSetAction>" + NEW_LINE,
				""
			).replaceAll(
				"\t\t<DataSetAction>Replace</DataSetAction>" + NEW_LINE,
				""
			);
	}
}
