// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2004, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.xml;

import java.io.IOException;

import junit.framework.Test;
import junit.framework.TestSuite;

/* ***************************** Class ****************************** */

/**
 * Implements a few tests for the {@link CaseStudyXMLWriter}.
 *
 * @author
 * <a href="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</a>
 *
 * @version @(#)$Id: CaseStudyXMLWriterTest.java,v 1.1 2017/01/19 16:14:02 trainot Exp $
 */
public class CaseStudyXMLWriterTest extends XMLWriterTest {

	/* ======================= Constants ======================== */

	private static final String EPSILON_BEGIN_OUT
	= "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" + NEW_LINE + NEW_LINE

		+ "<!-- SDMX Case Study's draft XML from Gesmes/TS Suite -->" + NEW_LINE
		+ "<!-- " + WILDCARD + " -->" + NEW_LINE
		+ "<?xml-stylesheet href=\"csxml.xsl\" type=\"text/xsl\" media=\"screen,html\"?>" + NEW_LINE + NEW_LINE;

	private static final String INTERCHANGE_BEGIN_OUT
	= "<SDMXTimeSeries"
		+ " xmlns=\"http://www.SDMX.org/resources/SDMXML/schemas/draft/SDMXML\""
		+ " xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\""
		+ " xsi:schemaLocation=\"http://www.SDMX.org/resources/SDMXML/schemas/draft/SDMXML csxml.xsd\">"
		+ NEW_LINE
		+ "\t<Header>" + NEW_LINE
		+ "\t\t<Name>IREF000012</Name>" + NEW_LINE
		+ "\t\t<Sender>BBB</Sender>" + NEW_LINE
		+ "\t\t<Receiver>ZZZ</Receiver>" + NEW_LINE
		+ "\t\t<TestIndicator>true</TestIndicator>" + NEW_LINE
		+ "\t\t<Prepared>2002-08-17T17:48:00</Prepared>" + NEW_LINE
		+ "\t</Header>" + NEW_LINE;

	private static final String INTERCHANGE_END_OUT
	= "</SDMXTimeSeries>" + NEW_LINE;

	private static final String STRUCTURE_MESSAGE_BEGIN_OUT = "";

	//private static final String STRUCTURE_MESSAGE_END_OUT = "";

	private static final String MESSAGE_BEGIN_OUT
	= "<StructureDefinition>" + NEW_LINE
		+ "\t<KeyFamilies>" + NEW_LINE
		+ "\t\t<KeyFamily uniqueID=\"KFSUMPF\" name=\"Sample Key Family\">" + NEW_LINE
		+ "\t\t\t<ExtKeyRef extKeyURI=\"http://www.sumpfland.org/vocabulary/2002-09-12/kfsumpf\">Sample Key Family</ExtKeyRef>" + NEW_LINE
		+ "\t\t</KeyFamily>" + NEW_LINE
		+ "\t</KeyFamilies>" + NEW_LINE
		+ "</StructureDefinition>" + NEW_LINE;

	private static final String MESSAGE_END_OUT = "";

	private static final String CODE_LIST_OUT
	= "<CodeList uniqueID=\"CL_SUMPF\" name=\"Sumpfige CodeList\">" + NEW_LINE
		+ "\t<Codes>" + NEW_LINE
		+ "\t\t<Code>" + NEW_LINE
		+ "\t\t\t<Value>S</Value>" + NEW_LINE
		+ "\t\t\t<Description>Sumpf</Description>" + NEW_LINE
		+ "\t\t</Code>" + NEW_LINE
		+ "\t\t<Code>" + NEW_LINE
		+ "\t\t\t<Value>U</Value>" + NEW_LINE
		+ "\t\t\t<Description>Und</Description>" + NEW_LINE
		+ "\t\t</Code>" + NEW_LINE
		+ "\t\t<Code>" + NEW_LINE
		+ "\t\t\t<Value>M</Value>" + NEW_LINE
		+ "\t\t\t<Description>Manche</Description>" + NEW_LINE
		+ "\t\t</Code>" + NEW_LINE
		+ "\t\t<Code>" + NEW_LINE
		+ "\t\t\t<Value>P</Value>" + NEW_LINE
		+ "\t\t\t<Description>Parteien</Description>" + NEW_LINE
		+ "\t\t</Code>" + NEW_LINE
		+ "\t\t<Code>" + NEW_LINE
		+ "\t\t\t<Value>F</Value>" + NEW_LINE
		+ "\t\t\t<Description>Finanzierung</Description>" + NEW_LINE
		+ "\t\t</Code>" + NEW_LINE
		+ "\t</Codes>" + NEW_LINE
		+ "</CodeList>" + NEW_LINE;

	private static final String CONCEPT_OUT
	= "<Concept>" + NEW_LINE
		+ "\t<Identifier>SUMPF</Identifier>" + NEW_LINE
		+ "\t<Label>Sumpfiges Concept</Label>" + NEW_LINE
		+ "</Concept>" + NEW_LINE;

	private static final String KEY_FAMILY_OUT
	= "<KeyFamily uniqueID=\"KFSUMPF\" name=\"Sample Key Family\">" + NEW_LINE
		+ "\t<Components>" + NEW_LINE
		+ "\t\t<Dimension conceptName=\"FREQ\" representation=\"CL_FREQ\"/>" + NEW_LINE
		+ "\t\t<Dimension conceptName=\"SUMPF\" description=\"Sumpfiges Concept\" representation=\"CL_SUMPF\"/>" + NEW_LINE
		+ "\t\t<Attribute conceptName=\"OBS_STATUS\" assignmentLevel=\"Observation\" assignmentStatus=\"Mandatory\">" + NEW_LINE
		+ "\t\t\t<Coded>CL_OBS_STATUS</Coded>" + NEW_LINE
		+ "\t\t</Attribute>" + NEW_LINE
		+ "\t\t<Attribute conceptName=\"OBS_CONF\" assignmentLevel=\"Observation\" assignmentStatus=\"Optional\">" + NEW_LINE
		+ "\t\t\t<Coded>CL_OBS_CONF</Coded>" + NEW_LINE
		+ "\t\t</Attribute>" + NEW_LINE
		+ "\t\t<Attribute conceptName=\"OBS_PRE_BREAK\" assignmentLevel=\"Observation\" assignmentStatus=\"Optional\">" + NEW_LINE
		+ "\t\t\t<String maxlength=\"15\"/>" + NEW_LINE
		+ "\t\t</Attribute>" + NEW_LINE
		+ "\t\t<Attribute conceptName=\"OBS_COM\" assignmentLevel=\"Observation\" assignmentStatus=\"Optional\">" + NEW_LINE
		+ "\t\t\t<String maxlength=\"1050\"/>" + NEW_LINE
		+ "\t\t</Attribute>" + NEW_LINE
		+ "\t\t<Attribute conceptName=\"UNIT\" assignmentLevel=\"TimeSeries\" assignmentStatus=\"Mandatory\">" + NEW_LINE
		+ "\t\t\t<Coded>CL_UNIT</Coded>" + NEW_LINE
		+ "\t\t</Attribute>" + NEW_LINE
		+ "\t\t<Attribute conceptName=\"TITLE\" assignmentLevel=\"SiblingGroup\" assignmentStatus=\"Optional\">" + NEW_LINE
		+ "\t\t\t<String maxlength=\"64\"/>" + NEW_LINE
		+ "\t\t</Attribute>" + NEW_LINE
		+ "\t\t<Attribute conceptName=\"DS_COMMENT\" assignmentLevel=\"DataSet\" assignmentStatus=\"Optional\">" + NEW_LINE
		+ "\t\t\t<String maxlength=\"64\"/>" + NEW_LINE
		+ "\t\t</Attribute>" + NEW_LINE
		+ "\t</Components>" + NEW_LINE
		+ "</KeyFamily>" + NEW_LINE;

	private static final String DATA1_OUT
	= "<Data>" + NEW_LINE
		+ "\t<DataSet name=\"DS1\">" + NEW_LINE
		+ "\t\t<AttributeList>" + NEW_LINE
		+ "\t\t\t<AttVal name=\"DS_COMMENT\">" + NEW_LINE
		+ "\t\t\t\t<Value>dscom</Value>" + NEW_LINE
		+ "\t\t\t</AttVal>" + NEW_LINE
		+ "\t\t</AttributeList>" + NEW_LINE
		+ "\t\t<KeyRef>KFSUMPF</KeyRef>" + NEW_LINE
		+ "\t\t<SiblingGroup variableDimension=\"FREQ\">" + NEW_LINE
		+ "\t\t\t<AttributeList>" + NEW_LINE
		+ "\t\t\t\t<AttVal name=\"TITLE\">" + NEW_LINE
		+ "\t\t\t\t\t<Value>sgcom</Value>" + NEW_LINE
		+ "\t\t\t\t</AttVal>" + NEW_LINE
		+ "\t\t\t</AttributeList>" + NEW_LINE
		+ "\t\t\t<TimeSeries>" + NEW_LINE
		+ "\t\t\t\t<AttributeList>" + NEW_LINE
		+ "\t\t\t\t\t<AttVal name=\"UNIT\">" + NEW_LINE
		+ "\t\t\t\t\t\t<Value>tscom</Value>" + NEW_LINE
		+ "\t\t\t\t\t</AttVal>" + NEW_LINE
		+ "\t\t\t\t</AttributeList>" + NEW_LINE
		+ "\t\t\t\t<Key>" + NEW_LINE
		+ "\t\t\t\t\t<KeyValue name=\"FREQ\">" + NEW_LINE
		+ "\t\t\t\t\t\t<Value>M</Value>" + NEW_LINE
		+ "\t\t\t\t\t</KeyValue>" + NEW_LINE
		+ "\t\t\t\t\t<KeyValue name=\"SUMPF\">" + NEW_LINE
		+ "\t\t\t\t\t\t<Value>P</Value>" + NEW_LINE
		+ "\t\t\t\t\t</KeyValue>" + NEW_LINE
		+ "\t\t\t\t</Key>" + NEW_LINE
		+ "\t\t\t\t<Observations>" + NEW_LINE
		+ "\t\t\t\t\t<Observation>" + NEW_LINE
		+ "\t\t\t\t\t\t<AttributeList>" + NEW_LINE
		+ "\t\t\t\t\t\t\t<AttVal name=\"OBS_STATUS\">" + NEW_LINE
		+ "\t\t\t\t\t\t\t\t<Value>A</Value>" + NEW_LINE
		+ "\t\t\t\t\t\t\t</AttVal>" + NEW_LINE
		+ "\t\t\t\t\t\t\t<AttVal name=\"OBS_CONF\">" + NEW_LINE
		+ "\t\t\t\t\t\t\t\t<Value/>" + NEW_LINE
		+ "\t\t\t\t\t\t\t</AttVal>" + NEW_LINE
		+ "\t\t\t\t\t\t\t<AttVal name=\"OBS_PRE_BREAK\">" + NEW_LINE
		+ "\t\t\t\t\t\t\t\t<Value/>" + NEW_LINE
		+ "\t\t\t\t\t\t\t</AttVal>" + NEW_LINE
		+ "\t\t\t\t\t\t</AttributeList>" + NEW_LINE
		+ "\t\t\t\t\t\t<Time>2003-01-01</Time>" + NEW_LINE
		+ "\t\t\t\t\t\t<Value>18.5</Value>" + NEW_LINE
		+ "\t\t\t\t\t</Observation>" + NEW_LINE
		+ "\t\t\t\t\t<Observation>" + NEW_LINE
		+ "\t\t\t\t\t\t<AttributeList>" + NEW_LINE
		+ "\t\t\t\t\t\t\t<AttVal name=\"OBS_STATUS\">" + NEW_LINE
		+ "\t\t\t\t\t\t\t\t<Value>M</Value>" + NEW_LINE
		+ "\t\t\t\t\t\t\t</AttVal>" + NEW_LINE
		+ "\t\t\t\t\t\t\t<AttVal name=\"OBS_CONF\">" + NEW_LINE
		+ "\t\t\t\t\t\t\t\t<Value>C</Value>" + NEW_LINE
		+ "\t\t\t\t\t\t\t</AttVal>" + NEW_LINE
		+ "\t\t\t\t\t\t\t<AttVal name=\"OBS_PRE_BREAK\">" + NEW_LINE
		+ "\t\t\t\t\t\t\t\t<Value/>" + NEW_LINE
		+ "\t\t\t\t\t\t\t</AttVal>" + NEW_LINE
		+ "\t\t\t\t\t\t</AttributeList>" + NEW_LINE
		+ "\t\t\t\t\t\t<Time>2003-02-01</Time>" + NEW_LINE
		+ "\t\t\t\t\t\t<Value>NaN</Value>" + NEW_LINE
		+ "\t\t\t\t\t</Observation>" + NEW_LINE
		+ "\t\t\t\t\t<Observation>" + NEW_LINE
		+ "\t\t\t\t\t\t<AttributeList>" + NEW_LINE
		+ "\t\t\t\t\t\t\t<AttVal name=\"OBS_COM\">" + NEW_LINE
		+ "\t\t\t\t\t\t\t\t<Value>obscom</Value>" + NEW_LINE
		+ "\t\t\t\t\t\t\t</AttVal>" + NEW_LINE
		+ "\t\t\t\t\t\t</AttributeList>" + NEW_LINE
		+ "\t\t\t\t\t\t<Time>2003-03-01</Time>" + NEW_LINE
		+ "\t\t\t\t\t</Observation>" + NEW_LINE
		+ "\t\t\t\t</Observations>" + NEW_LINE
		+ "\t\t\t</TimeSeries>" + NEW_LINE
		+ "\t\t</SiblingGroup>" + NEW_LINE
		+ "\t</DataSet>" + NEW_LINE
		+ "</Data>" + NEW_LINE;

	private static final String DATA2_OUT
	= "<Data/>" + NEW_LINE;

	/* ====================== Constructors ====================== */

	public CaseStudyXMLWriterTest( final String name ) {
		super( name );
	}

	/* ================== Protected Methods ===================== */

	protected void setUp () {
		super.setUp();
		System.setProperty(
			"Int.ecb.gesmescb.xml.CaseStudyXMLWriter.stylesheet",
			"csxml.xsl"
		);
		System.setProperty(
			"Int.ecb.gesmescb.xml.CaseStudyXMLWriter.schemaLocation",
			"csxml.xsd"
		);
	}

	protected void tearDown () {
		System.getProperties().remove(
			"Int.ecb.gesmescb.xml.CaseStudyXMLWriter.schemaLocation"
		);
		System.getProperties().remove(
			"Int.ecb.gesmescb.xml.CaseStudyXMLWriter.stylesheet"
		);
		super.tearDown();
	}

	protected String getClassName () {
		return "Int.ecb.gesmescb.xml.CaseStudyXMLWriter";
	}

	protected XMLWriter factory () throws IOException {
		return new CaseStudyXMLWriter( "" );
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite
	 */
	public static Test suite () {
		final TestSuite suite = new TestSuite( CaseStudyXMLWriterTest.class );
		return suite;
	}

	/**
	 * @param arg the array of argument {@link String}s
	 */
	public static void main ( final String[] arg ) {
		final String[] testCaseName = {
			CaseStudyXMLWriterTest.class.getName()
		};
		junit.textui.TestRunner.main( testCaseName );
	}
	
	/**
	 * to make eclipse happy
	 * @throws Exception
	 */
	public XMLWriter streamFactory(){ return null; }

	/* ---------------------- Test Methods ---------------------- */

	public void testInterchangeBegin () throws Exception {
		writeTempFile ( new Object[]{
			INTERCHANGE_BEGIN,
			INTERCHANGE_END
		} );
		assertEqualsLinewise(
			EPSILON_BEGIN_OUT
			+ INTERCHANGE_BEGIN_OUT
			+ INTERCHANGE_END_OUT,
			readTempFile()
		);
	}

	public void testMessageBegin () throws Exception {
		writeTempFile ( new Object[]{
			MESSAGE_BEGIN,
			MESSAGE_END
		} );
		assertEqualsLinewise(
			EPSILON_BEGIN_OUT
			+ MESSAGE_BEGIN_OUT
			+ "<Data/>" + NEW_LINE
			+ MESSAGE_END_OUT,
			readTempFile()
		);
	}

	public void testStructure () throws Exception {
		writeTempFile ( new Object[]{
			STRUCTURE_MESSAGE_BEGIN,
			CODE_LIST,
			CONCEPT,
			KEY_FAMILY,
			MESSAGE_END
		} );
		assertEqualsLinewise(
			EPSILON_BEGIN_OUT
			+ STRUCTURE_MESSAGE_BEGIN_OUT
			+ "<StructureDefinition>" + NEW_LINE
			+ moreIndent(
				"<KeyFamilies>" + NEW_LINE
				+ moreIndent( KEY_FAMILY_OUT, 1 )
				+ "</KeyFamilies>" + NEW_LINE
				+ "<CodeLists>" + NEW_LINE
				+ moreIndent( CODE_LIST_OUT, 1 )
				+ "</CodeLists>" + NEW_LINE
				+ "<ConceptList>" + NEW_LINE
				+ moreIndent( CONCEPT_OUT, 1 )
				+ "</ConceptList>" + NEW_LINE,
				1
			)
			+ "</StructureDefinition>" + NEW_LINE
			+ MESSAGE_END_OUT,
			readTempFile()
		);
	}

	public void testData () throws Exception {
		writeTempFile ( new Object[]{
			MESSAGE_BEGIN,
			SERIES1,
			DS_ATT,
			SG_ATT,
			TS_ATT,
			OBS_ATT,
			MESSAGE_END
		} );
		assertEqualsLinewise(
			EPSILON_BEGIN_OUT
			+ MESSAGE_BEGIN_OUT
			+ DATA1_OUT
			+ MESSAGE_END_OUT,
			readTempFile()
		);
	}

	public void testEmptySeries () throws Exception {
		writeTempFile ( new Object[]{
			MESSAGE_BEGIN,
			SERIES2,
			MESSAGE_END
		} );
		assertEqualsLinewise(
			EPSILON_BEGIN_OUT
			+ MESSAGE_BEGIN_OUT
			+ DATA2_OUT
			+ MESSAGE_END_OUT,
			readTempFile()
		);
	}
}
