// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2003, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.xml;

import java.io.*;

import junit.framework.Test;
import junit.framework.TestSuite;

/* ***************************** Class ****************************** */

/**
 * Implements a few tests for the {@link CheckingXMLWriteBuffer}.
 *
 * @author
 * <A HREF="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</A>
 *
 * @version @(#)$Id: CheckingXMLWriteBufferTest.java,v 1.1 2017/01/19 16:14:02 trainot Exp $
 */
public class CheckingXMLWriteBufferTest extends XMLWriteBufferTest {

	/* ======================= Constants ======================== */

	private static final String NEW_LINE
	= System.getProperty( "line.separator" );

	/* ====================== Constructors ====================== */

	public CheckingXMLWriteBufferTest( final String name ) {
		super( name );
	}

	/* =================== Protected Methods ==================== */

	protected XMLWriteBuffer factory (
		final Writer writer,
		final char indentChar,
		final int indentStep,
		final String newLine
	) {
		return new CheckingXMLWriteBuffer(
			writer, indentChar, indentStep, newLine
		);
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite
	 */
	public static Test suite () {
		final TestSuite suite = new TestSuite( CheckingXMLWriteBufferTest.class );
		return suite;
	}

	/**
	 * @param arg the array of argument {@link String}s
	 */
	public static void main ( final String[] arg ) {
		final String[] testCaseName = {
			CheckingXMLWriteBufferTest.class.getName()
		};
		junit.textui.TestRunner.main( testCaseName );
	}

	/* ---------------------- Test Methods ---------------------- */

	public void testElementMismatch () throws IOException {
		final CharArrayWriter writer = new CharArrayWriter();
		final XMLWriteBuffer buffer = factory(
			writer, '\t', 1, NEW_LINE
		);

		buffer.openElement( "test" );
		try {
			buffer.closeElement( "fest" );
			fail( "No exception thrown." );
		}
		catch ( final RuntimeException e ) {}
	}

	public void testElementCloseTooMany () throws IOException {
		final CharArrayWriter writer = new CharArrayWriter();
		final XMLWriteBuffer buffer = factory(
			writer, '\t', 1, NEW_LINE
		);

		try {
			buffer.closeElement( "fest" );
			fail( "No exception thrown." );
		}
		catch ( final RuntimeException e ) {}
	}

	public void testCloseTooFew () throws IOException {
		final CharArrayWriter writer = new CharArrayWriter();
		final XMLWriteBuffer buffer = factory(
			writer, '\t', 1, NEW_LINE
		);

		buffer.openElement( "test" );
		try {
			buffer.close();
			fail( "No exception thrown." );
		}
		catch ( final RuntimeException e ) {}
	}

	public void testBadElement () throws IOException {
		final CharArrayWriter writer = new CharArrayWriter();
		final XMLWriteBuffer buffer = factory(
			writer, '\t', 1, NEW_LINE
		);

		try {
			buffer.openElement( "9est" );
			fail( "No exception thrown for first." );
		}
		catch ( final RuntimeException e ) {}
		try {
			buffer.openElement( "te*st" );
			fail( "No exception thrown for later." );
		}
		catch ( final RuntimeException e ) {}
	}

	public void testBadAttribute () throws IOException {
		final CharArrayWriter writer = new CharArrayWriter();
		final XMLWriteBuffer buffer = factory(
			writer, '\t', 1, NEW_LINE
		);

		buffer.openElement( "test" );
		try {
			buffer.writeAttribute( "4est", "3" );
			fail( "No exception thrown for first." );
		}
		catch ( final RuntimeException e ) {}
		try {
			buffer.writeAttribute( "te*st", "3" );
			fail( "No exception thrown for later." );
		}
		catch ( final RuntimeException e ) {}
	}

	public void testMisplacedAttribute () throws IOException {
		final CharArrayWriter writer = new CharArrayWriter();
		final XMLWriteBuffer buffer = factory(
			writer, '\t', 1, NEW_LINE
		);

		try {
			buffer.writeAttribute( "test", "3" );
			fail( "No exception thrown for prepositioned." );
		}
		catch ( final RuntimeException e ) {}
		buffer.openElement( "test" );
		buffer.writeCharacters( "jest" );
		try {
			buffer.writeAttribute( "gest", "3" );
			fail( "No exception thrown for after characters." );
		}
		catch ( final RuntimeException e ) {}
		buffer.openElement( "test" );
		buffer.writeAttribute( "geht", "3" );
		buffer.closeElement( "test" );
		try {
			buffer.writeAttribute( "tey", "3" );
			fail( "No exception thrown for after element close." );
		}
		catch ( final RuntimeException e ) {}
	}

	public void testDuplicateAttribute () throws IOException {
		final CharArrayWriter writer = new CharArrayWriter();
		final XMLWriteBuffer buffer = factory(
			writer, '\t', 1, NEW_LINE
		);

		buffer.openElement( "test" );
		buffer.writeAttribute( "est", "3" );
		try {
			buffer.writeAttribute( "est", "3" );
			fail( "No exception thrown." );
		}
		catch ( final RuntimeException e ) {}
	}

	public void testAttributeSort () throws IOException {
		final CharArrayWriter writer = new CharArrayWriter();
		final CheckingXMLWriteBuffer buffer = (CheckingXMLWriteBuffer) factory(
			writer, '\t', 1, NEW_LINE
		);

		buffer.openSortedElement( "test", null );
		buffer.writeAttribute( "est", "1" );
		buffer.writeAttribute( "frit", "2" );
		buffer.writeAttribute( "hust", "3" );
		buffer.writeAttribute( "pust", "4" );
		buffer.writeAttribute( "mut", "5" );
		buffer.writeAttribute( "lunix", "6" );
		buffer.closeElement( "test" );
		buffer.close();

		assertEqualsLinewise(
			"<test est=\"1\" frit=\"2\" hust=\"3\""
			+ " lunix=\"6\" mut=\"5\" pust=\"4\"/>" + NEW_LINE,
			writer.toString()
		);
	}
}
