// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2002, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.xml;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.util.*;

import Int.ecb.gesmescb.KeyFamily;
import Int.ecb.gesmescb.KeyFamilyComponent;

/* ***************************** Class ****************************** */

/**
 * Tests the class {@link ConceptIdComparator}.
 *
 * @author
 * <A HREF="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</A>
 *
 * @version @(#)$Id: ConceptIdComparatorTest.java,v 1.1 2017/01/19 16:14:02 trainot Exp $
 */
public class ConceptIdComparatorTest extends TestCase {

	/* ======================= Constants ======================== */

	/**
	 * Version date of the Key Family.
	 */
	private static final Date VERSION_DATE;
	static {
		final Calendar calendar = Calendar.getInstance();
		calendar.clear();
		calendar.set( 2002, Calendar.AUGUST, 16, 14, 10 );
		VERSION_DATE = calendar.getTime();
	}

	/* ====================== Constructors ====================== */

	public ConceptIdComparatorTest( final String name ) {
		super( name );
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite
	 */
	public static Test suite () {
		return new TestSuite( ConceptIdComparatorTest.class );
	}

	/**
	 * @param arg the array of argument {@link String}s
	 */
	public static void main ( final String[] arg ) {
		final String[] testCaseName = {
			ConceptIdComparatorTest.class.getName()
		};
		junit.textui.TestRunner.main( testCaseName );
	}

	/* ---------------------- Test Methods ---------------------- */

	public void test () {
		final KeyFamily keyFamily = new KeyFamily(
			"ECB_TEST", "test KF", "", VERSION_DATE,
			new KeyFamilyComponent[]{
				new KeyFamilyComponent(
					"SG_COM", KeyFamilyComponent.ATTRIBUTE,
					"", "", KeyFamilyComponent.CONDITIONAL,
					KeyFamilyComponent.ATTACHED_TO_SIBLING_GROUP,
					""
				),
				new KeyFamilyComponent(
					"TIME", KeyFamilyComponent.TIME_DIMENSION,
					"2", "", "", "", ""
				),
				new KeyFamilyComponent(
					"FREQ", KeyFamilyComponent.KEY_STRUCTURE_DIMENSION,
					"1", "", "", "", "CL_FREQ"
				),
				new KeyFamilyComponent(
					"OBS_COM", KeyFamilyComponent.ATTRIBUTE,
					"", "", KeyFamilyComponent.CONDITIONAL,
					KeyFamilyComponent.ATTACHED_TO_OBSERVATION,
					""
				),
				new KeyFamilyComponent(
					"OBS_VALUE", KeyFamilyComponent.ARRAY_CELL,
					"3", "AN..16", "", "", ""
				),
				new KeyFamilyComponent(
					"DS_COM", KeyFamilyComponent.ATTRIBUTE,
					"", "", KeyFamilyComponent.CONDITIONAL,
					KeyFamilyComponent.ATTACHED_TO_DATA_SET,
					""
				),
				new KeyFamilyComponent(
					"TS_COM", KeyFamilyComponent.ATTRIBUTE,
					"", "AN..4", KeyFamilyComponent.MANDATORY,
					KeyFamilyComponent.ATTACHED_TO_TIME_SERIES,
					""
				)
			}
		);
		final Comparator comparator = new ConceptIdComparator( keyFamily );
		assertEquals( "FREQ==FREQ", 0, comparator.compare( "FREQ", "FREQ" ) );
		assertTrue( "FREQ<TIME", comparator.compare( "FREQ", "TIME" ) < 0 );
		assertTrue( "TIME>FREQ", comparator.compare( "TIME", "FREQ" ) > 0 );
		assertTrue( "FREQ<TS_COM", comparator.compare( "FREQ", "TS_COM" ) < 0 );
		assertTrue( "TIME<TS_COM", comparator.compare( "TIME", "TS_COM" ) < 0 );
		assertEquals(
			"TS_COM==TS_COM", 0, comparator.compare( "TS_COM", "TS_COM" )
		);
		assertTrue(
			"TS_COM>OBS_VALUE", comparator.compare( "TS_COM", "OBS_VALUE" ) > 0
		);
		assertTrue(
			"TS_COM>OBS_COM", comparator.compare( "TS_COM", "OBS_COM" ) > 0
		);
		assertTrue(
			"DS_COM>OBS_COM", comparator.compare( "DS_COM", "OBS_COM" ) > 0
		);
		assertTrue(
			"DS_COM>SG_COM", comparator.compare( "DS_COM", "SG_COM" ) > 0
		);
		assertTrue(
			"TS_COM<SG_COM", comparator.compare( "TS_COM", "SG_COM" ) < 0
		);
	}
}
