// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2002-2006, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.xml;

import java.io.IOException;

import junit.framework.Test;
import junit.framework.TestSuite;

/* ***************************** Class ****************************** */

/**
 * Implements a few tests for the {@link GesmesTSXMLWriter}.
 *
 * @author
 * <a href="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</a>
 * @author
 * <a href="mailto:Paula.Rocha@ECB.int">Paula Rocha</a>
 *
 * @version @(#)$Id: GesmesTSXMLWriterTest.java,v 1.1 2017/01/19 16:14:02 trainot Exp $
 */
public class GesmesTSXMLWriterTest extends XMLWriterTest {

	/* ======================= Constants ======================== */

	private static final String EPSILON_BEGIN_OUT
	= "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" + NEW_LINE + NEW_LINE

		+ "<!-- Gesmes/TS-XML from Gesmes/TS Suite -->" + NEW_LINE
		+ "<!-- " + WILDCARD + " -->" + NEW_LINE
		+ "<?xml-stylesheet href=\"gesmes.xsl\" type=\"text/xsl\" media=\"screen,html\"?>" + NEW_LINE + NEW_LINE;

	private static final String INTERCHANGE_BEGIN_OUT
	= "<Envelope id=\"IREF000012\""
		+ " xmlns=\"http://www.gesmes.org/xml/2002-08-01\""
		+ " xmlns:gesmes=\"http://www.gesmes.org/xml/2002-08-01\""
		+ " xmlns:edifact=\"http://www.gesmes.org/xml/edifact/2002-08-01\""
		+ " xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">"
		+ NEW_LINE
		+ "\t<test/>" + NEW_LINE
		+ "\t<prepared>2002-08-17T17:48:00</prepared>" + NEW_LINE
		+ "\t<Sender id=\"BBB\"/>" + NEW_LINE
		+ "\t<Receiver id=\"ZZZ\"/>" + NEW_LINE;

	private static final String INTERCHANGE_END_OUT
	= "</Envelope>" + NEW_LINE;

	private static final String STRUCTURE_MESSAGE_BEGIN_OUT
	= "<Envelope id=\"MREF000001\">" + NEW_LINE
		+ "\t<name>The subject</name>" + NEW_LINE
		+ "\t<Sender id=\"BBB\"/>" + NEW_LINE
		+ "\t<Receiver id=\"ZZZ\"/>" + NEW_LINE
		+ "\t<Extension>" + NEW_LINE
		+ "\t\t<edifact:maintenanceAgencyId>BODI</edifact:maintenanceAgencyId>" + NEW_LINE
		+ "\t</Extension>" + NEW_LINE;

	private static final String MESSAGE_BEGIN_OUT
	= "<Envelope id=\"MREF000001\">" + NEW_LINE
		+ "\t<name>The subject</name>" + NEW_LINE
		+ "\t<prepared>2000-11-01T13:10:00</prepared>" + NEW_LINE
		+ "\t<Sender id=\"BBB\">" + NEW_LINE
		+ "\t\t<Contact>" + NEW_LINE
		+ "\t\t\t<name>BOP Hotline</name>" + NEW_LINE
		+ "\t\t\t<department>BOP</department>" + NEW_LINE
		+ "\t\t\t<role>Responsible for Information Production</role>" + NEW_LINE
		+ "\t\t\t<telephone>++34987230763</telephone>" + NEW_LINE
		+ "\t\t</Contact>" + NEW_LINE
		+ "\t\t<Contact>" + NEW_LINE
		+ "\t\t\t<name>BOP Head</name>" + NEW_LINE
		+ "\t\t\t<department>BOP</department>" + NEW_LINE
		+ "\t\t\t<role>Head of Information Production</role>" + NEW_LINE
		+ "\t\t\t<fax>++34987234762</fax>" + NEW_LINE
		+ "\t\t</Contact>" + NEW_LINE
		+ "\t\t<Contact>" + NEW_LINE
		+ "\t\t\t<name>External Systems Hotline</name>" + NEW_LINE
		+ "\t\t\t<department>SIS</department>" + NEW_LINE
		+ "\t\t\t<role>Responsible for Data Processing</role>" + NEW_LINE
		+ "\t\t\t<uri>mailto:sis.external@ecb.int</uri>" + NEW_LINE
		+ "\t\t</Contact>" + NEW_LINE
		+ "\t\t<Contact>" + NEW_LINE
		+ "\t\t\t<name>SIS Head</name>" + NEW_LINE
		+ "\t\t\t<department>SIS</department>" + NEW_LINE
		+ "\t\t\t<role>Head of Data Processing</role>" + NEW_LINE
		+ "\t\t\t<x400>CO=DE; REST=unknown</x400>" + NEW_LINE
		+ "\t\t</Contact>" + NEW_LINE
		+ "\t</Sender>" + NEW_LINE
		+ "\t<Receiver id=\"ZZZ\"/>" + NEW_LINE
		+ "\t<reportingBegin>2000-01-01T00:00:00</reportingBegin>" + NEW_LINE
		+ "\t<reportingEnd>2001-01-01T00:00:00</reportingEnd>" + NEW_LINE
		+ "\t<dataSetId>DS1</dataSetId>" + NEW_LINE
		+ "\t<action>update</action>" + NEW_LINE
		+ "\t<Extension>" + NEW_LINE
		+ "\t\t<edifact:maintenanceAgencyId>BODI</edifact:maintenanceAgencyId>" + NEW_LINE
		+ "\t\t<edifact:keyFamilyId>KFSUMPF</edifact:keyFamilyId>" + NEW_LINE
		+ "\t</Extension>" + NEW_LINE;

	private static final String DELETE_MESSAGE_BEGIN_OUT
	= "<Envelope id=\"MREF000001\">" + NEW_LINE
		+ "\t<name>Subject is \"Deletor\"</name>" + NEW_LINE
		+ "\t<prepared>2000-11-01T13:10:00</prepared>" + NEW_LINE
		+ "\t<Sender id=\"BBB\"/>" + NEW_LINE
		+ "\t<Receiver id=\"ZZZ\"/>" + NEW_LINE
		+ "\t<reportingBegin>2000-01-01T00:00:00</reportingBegin>" + NEW_LINE
		+ "\t<reportingEnd>2000-07-01T00:00:00</reportingEnd>" + NEW_LINE
		+ "\t<dataSetId>DS1</dataSetId>" + NEW_LINE
		+ "\t<action>delete</action>" + NEW_LINE
		+ "\t<Extension>" + NEW_LINE
		+ "\t\t<edifact:maintenanceAgencyId>BODI</edifact:maintenanceAgencyId>" + NEW_LINE
		+ "\t\t<edifact:keyFamilyId>KFSUMPF</edifact:keyFamilyId>" + NEW_LINE
		+ "\t</Extension>" + NEW_LINE;

	private static final String MESSAGE_END_OUT
	= "</Envelope>" + NEW_LINE;

	private static final String CODE_LIST_OUT
	= "<CodeList id=\"CL_SUMPF\">" + NEW_LINE
		+ "\t<name>Sumpfige CodeList</name>" + NEW_LINE
		+ "\t<Code value=\"S\">" + NEW_LINE
		+ "\t\t<description>Sumpf</description>" + NEW_LINE
		+ "\t</Code>" + NEW_LINE
		+ "\t<Code value=\"U\">" + NEW_LINE
		+ "\t\t<description>Und</description>" + NEW_LINE
		+ "\t</Code>" + NEW_LINE
		+ "\t<Code value=\"M\">" + NEW_LINE
		+ "\t\t<description>Manche</description>" + NEW_LINE
		+ "\t</Code>" + NEW_LINE
		+ "\t<Code value=\"P\">" + NEW_LINE
		+ "\t\t<description>Parteien</description>" + NEW_LINE
		+ "\t</Code>" + NEW_LINE
		+ "\t<Code value=\"F\">" + NEW_LINE
		+ "\t\t<description>Finanzierung</description>" + NEW_LINE
		+ "\t</Code>" + NEW_LINE
		+ "</CodeList>" + NEW_LINE;

	private static final String CONCEPT_OUT
	= "<Concept id=\"SUMPF\">" + NEW_LINE
		+ "\t<name>Sumpfiges Concept</name>" + NEW_LINE
		+ "</Concept>" + NEW_LINE;

	private static final String KEY_FAMILY_OUT
	= "<KeyFamily id=\"KFSUMPF\">" + NEW_LINE
		+ "\t<name>Sample Key Family</name>" + NEW_LINE
		+ "\t<uri>urn:sdmx:org.sdmx.infomodel.datastructure.DataStructure=BODI:KFSUMPF(1.0):compact</uri>" + NEW_LINE
		+ "\t<Concept ref=\"FREQ\">" + NEW_LINE
		+ "\t\t<TextFormat>" + NEW_LINE
		+ "\t\t\t<cType>alphabetic</cType>" + NEW_LINE
		+ "\t\t\t<length exact=\"true\">1</length>" + NEW_LINE
		+ "\t\t</TextFormat>" + NEW_LINE
		+ "\t\t<CodeList ref=\"CL_FREQ\"/>" + NEW_LINE
		+ "\t\t<Extension>" + NEW_LINE
		+ "\t\t\t<edifact:conceptType>frequency dimension</edifact:conceptType>" + NEW_LINE
		+ "\t\t</Extension>" + NEW_LINE
		+ "\t</Concept>" + NEW_LINE
		+ "\t<Concept ref=\"SUMPF\">" + NEW_LINE
		+ "\t\t<TextFormat>" + NEW_LINE
		+ "\t\t\t<cType>alphabetic</cType>" + NEW_LINE
		+ "\t\t\t<length exact=\"true\">1</length>" + NEW_LINE
		+ "\t\t</TextFormat>" + NEW_LINE
		+ "\t\t<CodeList ref=\"CL_SUMPF\"/>" + NEW_LINE
		+ "\t\t<Extension>" + NEW_LINE
		+ "\t\t\t<edifact:conceptType>dimension</edifact:conceptType>" + NEW_LINE
		+ "\t\t</Extension>" + NEW_LINE
		+ "\t</Concept>" + NEW_LINE
		+ "\t<Concept ref=\"TIME_PERIOD\">" + NEW_LINE
		+ "\t\t<TextFormat>" + NEW_LINE
		+ "\t\t\t<cType>numeric</cType>" + NEW_LINE
		+ "\t\t\t<length exact=\"false\">16</length>" + NEW_LINE
		+ "\t\t</TextFormat>" + NEW_LINE
		+ "\t\t<TimePointSet>" + NEW_LINE
		+ "\t\t\t<periodicity>" + NEW_LINE
		+ "\t\t\t\t<value-of concept=\"TIME_FORMAT\"/>" + NEW_LINE
		+ "\t\t\t</periodicity>" + NEW_LINE
		+ "\t\t</TimePointSet>" + NEW_LINE
		+ "\t\t<Extension>" + NEW_LINE
		+ "\t\t\t<edifact:conceptType>time dimension</edifact:conceptType>" + NEW_LINE
		+ "\t\t</Extension>" + NEW_LINE
		+ "\t</Concept>" + NEW_LINE
		+ "\t<Concept ref=\"TIME_FORMAT\">" + NEW_LINE
		+ "\t\t<usage>mandatory</usage>" + NEW_LINE
		+ "\t\t<TextFormat>" + NEW_LINE
		+ "\t\t\t<cType>numeric</cType>" + NEW_LINE
		+ "\t\t\t<length exact=\"true\">3</length>" + NEW_LINE
		+ "\t\t</TextFormat>" + NEW_LINE
		+ "\t\t<TimeDurationSet/>" + NEW_LINE
		+ "\t\t<CodeList ref=\"CL_TIME_FORMAT\"/>" + NEW_LINE
		+ "\t\t<Extension>" + NEW_LINE
		+ "\t\t\t<edifact:conceptType>duration attribute</edifact:conceptType>" + NEW_LINE
		+ "\t\t\t<edifact:attachmentLevel>time series</edifact:attachmentLevel>" + NEW_LINE
		+ "\t\t</Extension>" + NEW_LINE
		+ "\t</Concept>" + NEW_LINE
		+ "\t<Concept ref=\"OBS_VALUE\">" + NEW_LINE
		+ "\t\t<usage>mandatory</usage>" + NEW_LINE
		+ "\t\t<TextFormat>" + NEW_LINE
		+ "\t\t\t<length exact=\"false\">16</length>" + NEW_LINE
		+ "\t\t</TextFormat>" + NEW_LINE
		+ "\t\t<RealNumberSet/>" + NEW_LINE
		+ "\t\t<Extension>" + NEW_LINE
		+ "\t\t\t<edifact:conceptType>array cell attribute</edifact:conceptType>" + NEW_LINE
		+ "\t\t\t<edifact:attachmentLevel>observation</edifact:attachmentLevel>" + NEW_LINE
		+ "\t\t</Extension>" + NEW_LINE
		+ "\t</Concept>" + NEW_LINE
		+ "\t<Concept ref=\"OBS_STATUS\">" + NEW_LINE
		+ "\t\t<usage>mandatory</usage>" + NEW_LINE
		+ "\t\t<TextFormat>" + NEW_LINE
		+ "\t\t\t<cType>alphabetic</cType>" + NEW_LINE
		+ "\t\t\t<length exact=\"true\">1</length>" + NEW_LINE
		+ "\t\t</TextFormat>" + NEW_LINE
		+ "\t\t<CodeList ref=\"CL_OBS_STATUS\"/>" + NEW_LINE
		+ "\t\t<Extension>" + NEW_LINE
		+ "\t\t\t<edifact:conceptType>array cell attribute</edifact:conceptType>" + NEW_LINE
		+ "\t\t\t<edifact:attachmentLevel>observation</edifact:attachmentLevel>" + NEW_LINE
		+ "\t\t</Extension>" + NEW_LINE
		+ "\t</Concept>" + NEW_LINE
		+ "\t<Concept ref=\"OBS_CONF\">" + NEW_LINE
		+ "\t\t<usage>conditional</usage>" + NEW_LINE
		+ "\t\t<TextFormat>" + NEW_LINE
		+ "\t\t\t<cType>alphabetic</cType>" + NEW_LINE
		+ "\t\t\t<length exact=\"true\">1</length>" + NEW_LINE
		+ "\t\t</TextFormat>" + NEW_LINE
		+ "\t\t<CodeList ref=\"CL_OBS_CONF\"/>" + NEW_LINE
		+ "\t\t<Extension>" + NEW_LINE
		+ "\t\t\t<edifact:conceptType>array cell attribute</edifact:conceptType>" + NEW_LINE
		+ "\t\t\t<edifact:attachmentLevel>observation</edifact:attachmentLevel>" + NEW_LINE
		+ "\t\t</Extension>" + NEW_LINE
		+ "\t</Concept>" + NEW_LINE
		+ "\t<Concept ref=\"OBS_PRE_BREAK\">" + NEW_LINE
		+ "\t\t<usage>conditional</usage>" + NEW_LINE
		+ "\t\t<TextFormat>" + NEW_LINE
		+ "\t\t\t<length exact=\"false\">15</length>" + NEW_LINE
		+ "\t\t</TextFormat>" + NEW_LINE
		+ "\t\t<RealNumberSet/>" + NEW_LINE
		+ "\t\t<Extension>" + NEW_LINE
		+ "\t\t\t<edifact:conceptType>array cell attribute</edifact:conceptType>" + NEW_LINE
		+ "\t\t\t<edifact:attachmentLevel>observation</edifact:attachmentLevel>" + NEW_LINE
		+ "\t\t</Extension>" + NEW_LINE
		+ "\t</Concept>" + NEW_LINE
		+ "\t<Concept ref=\"OBS_COM\">" + NEW_LINE
		+ "\t\t<usage>conditional</usage>" + NEW_LINE
		+ "\t\t<TextFormat>" + NEW_LINE
		+ "\t\t\t<length exact=\"false\">1050</length>" + NEW_LINE
		+ "\t\t</TextFormat>" + NEW_LINE
		+ "\t\t<Extension>" + NEW_LINE
		+ "\t\t\t<edifact:conceptType>attribute</edifact:conceptType>" + NEW_LINE
		+ "\t\t\t<edifact:attachmentLevel>observation</edifact:attachmentLevel>" + NEW_LINE
		+ "\t\t</Extension>" + NEW_LINE
		+ "\t</Concept>" + NEW_LINE
		+ "\t<Concept ref=\"UNIT\">" + NEW_LINE
		+ "\t\t<usage>mandatory</usage>" + NEW_LINE
		+ "\t\t<TextFormat>" + NEW_LINE
		+ "\t\t\t<length exact=\"false\">4</length>" + NEW_LINE
		+ "\t\t</TextFormat>" + NEW_LINE
		+ "\t\t<CodeList ref=\"CL_UNIT\"/>" + NEW_LINE
		+ "\t\t<Extension>" + NEW_LINE
		+ "\t\t\t<edifact:conceptType>attribute</edifact:conceptType>" + NEW_LINE
		+ "\t\t\t<edifact:attachmentLevel>time series</edifact:attachmentLevel>" + NEW_LINE
		+ "\t\t</Extension>" + NEW_LINE
		+ "\t</Concept>" + NEW_LINE
		+ "\t<Concept ref=\"TITLE\">" + NEW_LINE
		+ "\t\t<usage>conditional</usage>" + NEW_LINE
		+ "\t\t<TextFormat>" + NEW_LINE
		+ "\t\t\t<length exact=\"false\">64</length>" + NEW_LINE
		+ "\t\t</TextFormat>" + NEW_LINE
		+ "\t\t<Extension>" + NEW_LINE
		+ "\t\t\t<edifact:conceptType>attribute</edifact:conceptType>" + NEW_LINE
		+ "\t\t\t<edifact:attachmentLevel>sibling group</edifact:attachmentLevel>" + NEW_LINE
		+ "\t\t</Extension>" + NEW_LINE
		+ "\t</Concept>" + NEW_LINE
		+ "\t<Concept ref=\"DS_COMMENT\">" + NEW_LINE
		+ "\t\t<usage>conditional</usage>" + NEW_LINE
		+ "\t\t<TextFormat>" + NEW_LINE
		+ "\t\t\t<length exact=\"false\">64</length>" + NEW_LINE
		+ "\t\t</TextFormat>" + NEW_LINE
		+ "\t\t<Extension>" + NEW_LINE
		+ "\t\t\t<edifact:conceptType>attribute</edifact:conceptType>" + NEW_LINE
		+ "\t\t\t<edifact:attachmentLevel>data set</edifact:attachmentLevel>" + NEW_LINE
		+ "\t\t</Extension>" + NEW_LINE
		+ "\t</Concept>" + NEW_LINE
		+ "\t<Level dimensions=\"FREQ SUMPF TIME_PERIOD\" attributes=\"OBS_VALUE OBS_STATUS OBS_CONF OBS_PRE_BREAK OBS_COM\"/>" + NEW_LINE
		+ "\t<Level dimensions=\"FREQ SUMPF\" attributes=\"TIME_FORMAT UNIT\"/>" + NEW_LINE
		+ "\t<Level dimensions=\"SUMPF\" attributes=\"TITLE\"/>" + NEW_LINE
		+ "\t<Level dimensions=\"\" attributes=\"DS_COMMENT\"/>" + NEW_LINE
		+ "</KeyFamily>" + NEW_LINE;

	private static final String DATA_SET_OUT
	= "<DataSet xmlns=\"urn:sdmx:org.sdmx.infomodel.datastructure.DataStructure=BODI:KFSUMPF(1.0):compact\">"
		+ NEW_LINE;

	private static final String DATA_SET_CLOSE_OUT
	= "</DataSet>" + NEW_LINE;

	private static final String SERIES1_OUT
	= "<Series FREQ=\"M\" SUMPF=\"P\" TIME_FORMAT=\"P1M\">" + NEW_LINE
		+ "\t<Obs TIME_PERIOD=\"2003-01\" OBS_VALUE=\"18.5\" OBS_STATUS=\"A\" OBS_CONF=\"\" OBS_PRE_BREAK=\"\"/>" + NEW_LINE
		+ "\t<Obs TIME_PERIOD=\"2003-02\" OBS_VALUE=\"NaN\" OBS_STATUS=\"M\" OBS_CONF=\"C\" OBS_PRE_BREAK=\"\"/>" + NEW_LINE
		+ "</Series>" + NEW_LINE;

	private static final String SERIES2_OUT
	= "<Series FREQ=\"M\" SUMPF=\"U\"/>" + NEW_LINE;

	private static final String ATT_OUT
	= "<DataSet xmlns=\"urn:sdmx:org.sdmx.infomodel.datastructure.DataStructure=BODI:KFSUMPF(1.0):compact\" DS_COMMENT=\"dscom\">" + NEW_LINE
		+ "\t<Group SUMPF=\"P\" TITLE=\"sgcom\">" + NEW_LINE
		+ "\t\t<Series FREQ=\"M\" TIME_FORMAT=\"P1M\" UNIT=\"tscom\">" + NEW_LINE
		+ "\t\t\t<Obs TIME_PERIOD=\"2003-03\" OBS_COM=\"obscom\"/>" + NEW_LINE
		+ "\t\t\t<SeriesAtt UNIT=\"tscom\"/>" + NEW_LINE
		+ "\t\t</Series>" + NEW_LINE
		+ "\t\t<GroupAtt TITLE=\"sgcom\"/>" + NEW_LINE
		+ "\t</Group>" + NEW_LINE
		+ "\t<DataSetAtt DS_COMMENT=\"dscom\"/>" + NEW_LINE
		+ "</DataSet>" + NEW_LINE;

	private static final String DELETE_SORTED_OUT
	= "<DataSet xmlns=\"urn:sdmx:org.sdmx.infomodel.datastructure.DataStructure=BODI:KFSUMPF(1.0):compact\" DS_COMMENT=\"dscom\">" + NEW_LINE
		+ "\t<Group SUMPF=\"P\" TITLE=\"sgcom\">" + NEW_LINE
		+ "\t\t<Series FREQ=\"M\" UNIT=\"tscom\">" + NEW_LINE
		+ "\t\t\t<Obs TIME_PERIOD=\"2003-01\" OBS_VALUE=\"18.5\" OBS_STATUS=\"A\" OBS_CONF=\"\" OBS_PRE_BREAK=\"\"/>" + NEW_LINE
		+ "\t\t\t<Obs TIME_PERIOD=\"2003-02\" OBS_VALUE=\"NaN\" OBS_STATUS=\"M\" OBS_CONF=\"C\" OBS_PRE_BREAK=\"\"/>" + NEW_LINE
		+ "\t\t\t<Obs TIME_PERIOD=\"2003-03\" OBS_COM=\"obscom\"/>" + NEW_LINE
		+ "\t\t</Series>" + NEW_LINE
		+ "\t\t<Series FREQ=\"Q\">" + NEW_LINE
		+ "\t\t\t<Obs TIME_PERIOD=\"2000-01\"/>" + NEW_LINE
		+ "\t\t\t<Obs TIME_PERIOD=\"2000-04\"/>" + NEW_LINE
		+ "\t\t</Series>" + NEW_LINE
		+ "\t</Group>" + NEW_LINE
		+ "\t<Series FREQ=\"M\" SUMPF=\"U\"/>" + NEW_LINE
		+ "</DataSet>" + NEW_LINE;

	private static final String DELETE_UNSORTED_OUT
	= "<DataSet xmlns=\"urn:sdmx:org.sdmx.infomodel.datastructure.DataStructure=BODI:KFSUMPF(1.0):compact\">" + NEW_LINE
		+ "\t<Series FREQ=\"M\" SUMPF=\"P\">" + NEW_LINE
		+ "\t\t<Obs TIME_PERIOD=\"2003-01\" OBS_VALUE=\"18.5\" OBS_STATUS=\"A\" OBS_CONF=\"\" OBS_PRE_BREAK=\"\"/>" + NEW_LINE
		+ "\t\t<Obs TIME_PERIOD=\"2003-02\" OBS_VALUE=\"NaN\" OBS_STATUS=\"M\" OBS_CONF=\"C\" OBS_PRE_BREAK=\"\"/>" + NEW_LINE
		+ "\t</Series>" + NEW_LINE
		+ "\t<Series FREQ=\"M\" SUMPF=\"U\"/>" + NEW_LINE
		+ "\t<Series FREQ=\"Q\" SUMPF=\"P\">" + NEW_LINE
		+ "\t\t<Obs TIME_PERIOD=\"2000-01\"/>" + NEW_LINE
		+ "\t\t<Obs TIME_PERIOD=\"2000-04\"/>" + NEW_LINE
		+ "\t</Series>" + NEW_LINE
		+ "\t<Series FREQ=\"M\" SUMPF=\"P\">" + NEW_LINE
		+ "\t\t<Obs TIME_PERIOD=\"2003-03\" OBS_COM=\"obscom\"/>" + NEW_LINE
		+ "\t\t<SeriesAtt UNIT=\"tscom\"/>" + NEW_LINE
		+ "\t</Series>" + NEW_LINE
		+ "\t<Group SUMPF=\"P\" TITLE=\"sgcom\"/>" + NEW_LINE
		+ "\t<DataSetAtt DS_COMMENT=\"dscom\"/>" + NEW_LINE
		+ "</DataSet>" + NEW_LINE;

	/* ====================== Constructors ====================== */

	public GesmesTSXMLWriterTest( final String name ) {
		super( name );
	}

	/* ================== Protected Methods ===================== */

	public XMLWriter streamFactory() { return null; /* make eclipse happy */ }
	
	protected void setUp () {
		super.setUp();
		System.setProperty(
			"Int.ecb.gesmescb.xml.GesmesTSXMLWriter.stylesheet",
			"gesmes.xsl"
		);
	}

	protected void tearDown () {
		System.getProperties().remove(
			"Int.ecb.gesmescb.xml.GesmesTSXMLWriter.stylesheet"
		);
		super.tearDown();
	}

	protected String getClassName () {
		return "Int.ecb.gesmescb.xml.GesmesTSXMLWriter";
	}

	protected XMLWriter factory () throws IOException {
		return new GesmesTSXMLWriter( "" );
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite
	 */
	public static Test suite () {
		final TestSuite suite = new TestSuite( GesmesTSXMLWriterTest.class );
		return suite;
	}

	/**
	 * @param arg the array of argument {@link String}s
	 */
	public static void main ( final String[] arg ) {
		final String[] testCaseName = {
			GesmesTSXMLWriterTest.class.getName()
		};
		junit.textui.TestRunner.main( testCaseName );
	}

	/* ---------------------- Test Methods ---------------------- */

	public void testInterchangeBegin () throws Exception {
		writeTempFile ( new Object[]{
			INTERCHANGE_BEGIN,
			INTERCHANGE_END
		} );
		assertEqualsLinewise(
			EPSILON_BEGIN_OUT
			+ INTERCHANGE_BEGIN_OUT
			+ INTERCHANGE_END_OUT,
			readTempFile()
		);
	}

	public void testMessageBegin () throws Exception {
		writeTempFile ( new Object[]{
			MESSAGE_BEGIN,
			MESSAGE_END
		} );
		assertEqualsLinewise(
			EPSILON_BEGIN_OUT
			+ MESSAGE_BEGIN_OUT
			+ MESSAGE_END_OUT,
			readTempFile()
		);
	}

	public void testCodeList () throws Exception {
		writeTempFile ( CODE_LIST );
		assertEqualsLinewise(
			EPSILON_BEGIN_OUT
			+ CODE_LIST_OUT,
			readTempFile()
		);
	}

	public void testConcept () throws Exception {
		writeTempFile ( CONCEPT );
		assertEqualsLinewise(
			EPSILON_BEGIN_OUT
			+ CONCEPT_OUT,
			readTempFile()
		);
	}

	public void testKeyFamily () throws Exception {
		writeTempFile ( new Object[]{
			STRUCTURE_MESSAGE_BEGIN,
			KEY_FAMILY,
			MESSAGE_END
		} );
		assertEqualsLinewise(
			EPSILON_BEGIN_OUT
			+ STRUCTURE_MESSAGE_BEGIN_OUT
			+ moreIndent( KEY_FAMILY_OUT, 1 )
			+ MESSAGE_END_OUT,
			readTempFile()
		);
	}

	public void testTimeSeries () throws Exception {
		writeTempFile ( new Object[]{
			MESSAGE_BEGIN,
			SERIES1,
			SERIES2,
			MESSAGE_END
		} );
		assertEqualsLinewise(
			EPSILON_BEGIN_OUT
			+ MESSAGE_BEGIN_OUT
			+ moreIndent( DATA_SET_OUT, 1 )
			+ moreIndent( SERIES1_OUT + SERIES2_OUT, 2 )
			+ moreIndent( DATA_SET_CLOSE_OUT, 1 )
			+ MESSAGE_END_OUT,
			readTempFile()
		);
	}

	public void testAttributes () throws Exception {
		writeTempFile ( new Object[]{
			MESSAGE_BEGIN,
			DS_ATT,
			SG_ATT,
			TS_ATT,
			OBS_ATT,
			TS_ATT,
			SG_ATT,
			DS_ATT,
			MESSAGE_END
		} );
		assertEqualsLinewise(
			EPSILON_BEGIN_OUT
			+ MESSAGE_BEGIN_OUT
			+ moreIndent( ATT_OUT, 1 )
			+ MESSAGE_END_OUT,
			readTempFile()
		);
	}

	public void testSortedDelete () throws Exception {
		writeTempFile ( new Object[]{
			DELETE_MESSAGE_BEGIN,
			DS_ATT,
			SG_ATT,
			TS_ATT,
			SERIES1,
			OBS_ATT,
			SERIES3,
			SERIES2,
			MESSAGE_END
		} );
		assertEqualsLinewise(
			EPSILON_BEGIN_OUT
			+ DELETE_MESSAGE_BEGIN_OUT
			+ moreIndent( DELETE_SORTED_OUT, 1 )
			+ MESSAGE_END_OUT,
			readTempFile()
		);
	}

	public void testUnsortedDelete () throws Exception {
		writeTempFile ( new Object[]{
			DELETE_MESSAGE_BEGIN,
			SERIES1,
			SERIES2,
			SERIES3,
			OBS_ATT,
			TS_ATT,
			SG_ATT,
			DS_ATT,
			MESSAGE_END
		} );
		assertEqualsLinewise(
			EPSILON_BEGIN_OUT
			+ DELETE_MESSAGE_BEGIN_OUT
			+ moreIndent( DELETE_UNSORTED_OUT, 1 )
			+ MESSAGE_END_OUT,
			readTempFile()
		);
	}
}
