// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2002, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.xml;

import java.util.List;
import java.util.ArrayList;
import java.util.Map;
import java.util.TreeMap;
import java.util.Iterator;
import java.util.ListIterator;
import java.util.NoSuchElementException;

/* ***************************** Class ****************************** */

/**
 * <p>Maintains a stack of {@link Map}s with downwards lookup.</p>
 *
 * @author
 * <A HREF="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</A>
 *
 * @version @(#)$Id: MapStack.java,v 1.1 2017/01/19 16:14:03 trainot Exp $
 */
public class MapStack {

	/* ========================= Fields ========================= */

	/** The list used as a stack. */
	private final List stack;

	/* ====================== Constructors ====================== */


	public MapStack () {
		stack = new ArrayList();
	}

	/* ===================== Public Methods ===================== */

	public void push ( final Map map ) {
		if ( map == null ) {
			throw new IllegalArgumentException(
				"Trying to push null to MapStack"
			);
		}
		stack.add( map );
	}

	public void push () {
		push( new TreeMap() );
	}

	public Map pop () {
		final Map map;
		final int lastIndex = stack.size() - 1;
		if ( lastIndex >= 0 ) {
			map = (Map) stack.remove( lastIndex );
		} else {
			map = null;
		}
		return map;
	}

	public void put ( final String key, final String value ) {
		top().put( key, value );
	}

	public String get ( final String key ) {
		Object value = null;
		final ListIterator iterator = stack.listIterator( stack.size() );
		while ( value == null && iterator.hasPrevious() ) {
			final Map map = (Map) iterator.previous();
			value = map.get( key );
		}
		return (String) value;
	}

	public Iterator topEntryIterator () {
		return top().entrySet().iterator();
	}

	/* ==================== Private Methods ===================== */

	private Map top () {
		final int lastIndex = stack.size() - 1;
		if ( lastIndex < 0 ) {
			throw new NoSuchElementException(
				"Trying to put a value to an empty MapStack"
			);
		}
		return (Map) stack.get( lastIndex );
	}
	

	public List getStack() {
		return stack;
	}
}
