// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2002, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.xml;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.util.*;

/* ***************************** Class ****************************** */

/**
 * Tests the class {@link MapStack}.
 *
 * @author
 * <A HREF="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</A>
 *
 * @version @(#)$Id: MapStackTest.java,v 1.1 2017/01/19 16:14:03 trainot Exp $
 */
public class MapStackTest extends TestCase {

	/* ====================== Constructors ====================== */

	public MapStackTest( final String name ) {
		super( name );
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite
	 */
	public static Test suite () {
		return new TestSuite( MapStackTest.class );
	}

	/**
	 * @param arg the array of argument {@link String}s
	 */
	public static void main ( final String[] arg ) {
		final String[] testCaseName = { MapStackTest.class.getName() };
		junit.textui.TestRunner.main( testCaseName );
	}

	/* ---------------------- Test Methods ---------------------- */

	public void testPushPop () {
		final Map map1 = new TreeMap();
		final Map map3 = new HashMap();
		final MapStack mapStack = new MapStack();
		assertNull( mapStack.pop() );
		mapStack.push( map1 );
		mapStack.push();
		mapStack.push( map3 );
		assertSame( map3, mapStack.pop() );
		assertNotNull( mapStack.pop() );
		assertSame( map1, mapStack.pop() );
		assertNull( mapStack.pop() );
	}

	public void testPushPutGetPop () {
		final MapStack mapStack = new MapStack();
		mapStack.push();
		mapStack.put( "FREQ", "M" );
		assertEquals( "M", mapStack.get( "FREQ" ) );
		mapStack.push();
		mapStack.put( "TIME", "2002-08-20" );
		assertEquals( "M", mapStack.get( "FREQ" ) );
		assertEquals( "2002-08-20", mapStack.get( "TIME" ) );
		mapStack.put( "FREQ", "MM" );
		assertEquals( "MM", mapStack.get( "FREQ" ) );
		final Map map2 = mapStack.pop();
		assertEquals( "M", mapStack.get( "FREQ" ) );
		assertNull( mapStack.get( "TIME" ) );
		final Map map1 = mapStack.pop();
		assertNull( mapStack.get( "FREQ" ) );

		assertEquals( "M", map1.get( "FREQ" ) );
		assertNull( map1.get( "TIME" ) );
		assertEquals( "MM", map2.get( "FREQ" ) );
		assertEquals( "2002-08-20", map2.get( "TIME" ) );
	}

	public void testTopEntryIterator () {
		final MapStack mapStack = new MapStack();
		mapStack.push( new TreeMap() ); // To provide predictable ordering.
		mapStack.put( "FREQ", "M" );
		mapStack.put( "REF_AREA", "FR" );
		mapStack.put( "TIME", "2002-08-20" );
		final Iterator iterator = mapStack.topEntryIterator();
		final Map.Entry entry1 = (Map.Entry) iterator.next();
		final Map.Entry entry2 = (Map.Entry) iterator.next();
		final Map.Entry entry3 = (Map.Entry) iterator.next();
		assertTrue( "no more", !iterator.hasNext() );

		assertEquals( "FREQ", entry1.getKey() );
		assertEquals( "M", entry1.getValue() );
		assertEquals( "REF_AREA", entry2.getKey() );
		assertEquals( "FR", entry2.getValue() );
		assertEquals( "TIME", entry3.getKey() );
		assertEquals( "2002-08-20", entry3.getValue() );
	}
}
