package Int.ecb.gesmescb.xml;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileDescriptor;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.io.SyncFailedException;

public  class RandomFileOutputStream extends ByteArrayOutputStream {

	// *****************************************************************************
	// INSTANCE PROPERTIES
	// *****************************************************************************

	protected RandomAccessFile randomFile; // the random file to write to
	protected boolean sync; // whether to synchronize every write

	// *****************************************************************************
	// INSTANCE CONSTRUCTION/INITIALIZATON/FINALIZATION, OPEN/CLOSE
	// *****************************************************************************

	public RandomFileOutputStream(String fnm) throws IOException {
		this(fnm, false);
	}

	public RandomFileOutputStream(String fnm, boolean syn)
			throws IOException {
		this(new File(fnm), syn);
	}

	public RandomFileOutputStream(File fil) throws IOException {
		this(fil, false);
	}

	public RandomFileOutputStream(File fil, boolean syn) throws IOException {
		super();

		randomFile = new RandomAccessFile(fil, "rw");
		randomFile.setLength(0); //wipe existing content !!
		sync = syn;
	}
	
	// for compatibility with ByteArrayOutputStream
	public RandomFileOutputStream() throws IOException {
		super();
	}
	
	// *****************************************************************************
	// INSTANCE METHODS - OUTPUT STREAM IMPLEMENTATION
	// *****************************************************************************

	public void write(int val)  {
		try {
			randomFile.write(val);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		if (sync) {
			try {
				randomFile.getFD().sync();
			} catch (SyncFailedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	public void write(byte[] val) throws IOException {
		randomFile.write(val);
		if (sync) {
			randomFile.getFD().sync();
		}
	}

	public void write(byte[] val, int off, int len) {
		try {
			randomFile.write(val, off, len);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		if (sync) {
			try {
				randomFile.getFD().sync();
			} catch (SyncFailedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	public void flush() throws IOException {
		if (sync) {
			randomFile.getFD().sync();
		}
	}

	public void close() throws IOException {
		randomFile.close();
	}

	// *****************************************************************************
	// INSTANCE METHODS - RANDOM ACCESS EXTENSIONS
	// *****************************************************************************

	public long getFilePointer() throws IOException {
		return randomFile.getFilePointer();
	}

	public void setFilePointer(long pos) throws IOException {
		randomFile.seek(pos);
	}

	public long getFileSize() throws IOException {
		return randomFile.length();
	}

	public void setFileSize(long len) throws IOException {
		randomFile.setLength(len);
	}

	public FileDescriptor getFD() throws IOException {
		return randomFile.getFD();
	}

	public RandomAccessFile getRandomFile() {
		return randomFile;
	}
}