// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (C) 2005-2006, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.xml;

import java.io.IOException;

import junit.framework.Test;
import junit.framework.TestSuite;

/* ***************************** Class ****************************** */

/**
 * Implements a few tests for the {@link SDMXMLWriter}.
 *
 * @author
 * <a href="mailto:Maria.Stahl@ECB.int">Maria Stahl</a>
 *
 * @version @(#)$Id: SDMXMLWriter20Test.java,v 1.1 2017/01/19 16:14:02 trainot Exp $
 */
public class SDMXMLWriter20Test extends XMLWriterTest {

	/* ======================= Constants ======================== */

	private static final String EPSILON_BEGIN_OUT
	= "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" + NEW_LINE + NEW_LINE
		+ "<!-- SDMX-ML from Gesmes/TS Suite -->" + NEW_LINE
        + "<!-- " + WILDCARD + " -->" + NEW_LINE;

	private static final String DELETE_MESSAGE_BEGIN_OUT
	= "<!DOCTYPE MessageGroup SYSTEM \"compact.dtd\">" + NEW_LINE
		+ "<?xml-stylesheet href=\"compact.xsl\" type=\"text/xsl\"?>" + NEW_LINE + NEW_LINE
		+ "<MessageGroup"
		+ " xmlns=\"http://www.SDMX.org/resources/SDMXML/schemas/v2_0/message\""
		+ " xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\""
		+ " xsi:schemaLocation=\"http://www.SDMX.org/resources/SDMXML/schemas/v2_0/message message.xsd\">" + NEW_LINE
		+ "\t<Header>" + NEW_LINE
		+ "\t\t<ID>IREF000012</ID>" + NEW_LINE
		+ "\t\t<Test>true</Test>" + NEW_LINE
		+ "\t\t<Name>Subject is \"Deletor\"</Name>" + NEW_LINE
		+ "\t\t<Prepared>2002-08-17T17:48:00+02:00</Prepared>" + NEW_LINE
		+ "\t\t<Sender id=\"BBB\"/>" + NEW_LINE
		+ "\t\t<Receiver id=\"ZZZ\"/>" + NEW_LINE
		+ "\t\t<KeyFamilyRef>KFSUMPF</KeyFamilyRef>" + NEW_LINE
        + "\t\t<KeyFamilyAgency>BODI</KeyFamilyAgency>" + NEW_LINE
		+ "\t\t<DataSetAgency>ECB</DataSetAgency>" + NEW_LINE
		+ "\t\t<DataSetID>DS1</DataSetID>" + NEW_LINE
		+ "\t\t<Extracted>2000-11-01T13:10:00+01:00</Extracted>" + NEW_LINE
		+ "\t\t<ReportingBegin>2000-01-01T00:00:00+01:00</ReportingBegin>" + NEW_LINE
		+ "\t\t<ReportingEnd>2000-07-01T00:00:00+02:00</ReportingEnd>" + NEW_LINE
		+ "\t</Header>" + NEW_LINE;

	private static final String DATA_END_OUT = "</MessageGroup>" + NEW_LINE;

	private static final String DATA1_SORTED_OUT
	= "\t<DataSet action=\"Replace\" xmlns=\"http://www.sumpfland.org/vocabulary/2002-09-12/kfsumpf\" xsi:schemaLocation=\"http://www.sumpfland.org/vocabulary/2002-09-12/kfsumpf compact.xsd\" DS_COMMENT=\"dscom\">" + NEW_LINE 
		+ "\t\t<Group SUMPF=\"P\" TITLE=\"sgcom\"/>" + NEW_LINE
		+ "\t\t<Series FREQ=\"M\" SUMPF=\"P\" TIME_FORMAT=\"P1M\" UNIT=\"tscom\"" + NEW_LINE
		+ "\t\t\t<Obs TIME_PERIOD=\"2003-01\" OBS_VALUE=\"18.5\" OBS_STATUS=\"A\" OBS_CONF=\"F\" OBS_PRE_BREAK=\"NaN\"/>" + NEW_LINE
		+ "\t\t\t<Obs TIME_PERIOD=\"2003-02\" OBS_VALUE=\"NaN\" OBS_STATUS=\"M\" OBS_CONF=\"C\" OBS_PRE_BREAK=\"NaN\"/>" + NEW_LINE
		+ "\t\t\t<Obs TIME_PERIOD=\"2003-03\" OBS_COM=\"obscom\"/>" + NEW_LINE
		+ "\t\t</Series>" + NEW_LINE
		+ "\t</DataSet>" + NEW_LINE;

	private static final String DELETE_SORTED_OUT
	= "\t<DataSet action=\"Delete\" xmlns=\"http://www.sumpfland.org/vocabulary/2002-09-12/kfsumpf\" xsi:schemaLocation=\"http://www.sumpfland.org/vocabulary/2002-09-12/kfsumpf compact.xsd\" DS_COMMENT=\"dscom\">" + NEW_LINE
		+ "\t\t<Group SUMPF=\"P\" TITLE=\"sgcom\"/>" + NEW_LINE
		+ "\t\t<Series FREQ=\"M\" SUMPF=\"P\" UNIT=\"tscom\">" + NEW_LINE
		+ "\t\t\t<Obs TIME_PERIOD=\"2003-01\" OBS_VALUE=\"18.5\" OBS_STATUS=\"A\" OBS_CONF=\"F\" OBS_PRE_BREAK=\"NaN\"/>" + NEW_LINE
		+ "\t\t\t<Obs TIME_PERIOD=\"2003-02\" OBS_VALUE=\"NaN\" OBS_STATUS=\"M\" OBS_CONF=\"C\" OBS_PRE_BREAK=\"NaN\"/>" + NEW_LINE
		+ "\t\t\t<Obs TIME_PERIOD=\"2003-03\" OBS_COM=\"obscom\"/>" + NEW_LINE
		+ "\t\t</Series>" + NEW_LINE
		+ "\t\t<Series FREQ=\"Q\" SUMPF=\"P\">" + NEW_LINE
		+ "\t\t\t<Obs TIME_PERIOD=\"2000-Q1\"/>" + NEW_LINE
		+ "\t\t\t<Obs TIME_PERIOD=\"2000-Q2\"/>" + NEW_LINE
		+ "\t\t</Series>" + NEW_LINE
		+ "\t\t<Series FREQ=\"M\" SUMPF=\"U\"/>" + NEW_LINE
		+ "\t</DataSet>" + NEW_LINE;

	/* ====================== Constructors ====================== */

	public SDMXMLWriter20Test( final String name ) {
		super( name );
		
	}

	/* ================== Protected Methods ===================== */

	protected void setUp () {
		super.setUp();
		System.setProperty(
			"Int.ecb.gesmescb.xml.SDMXMLWriter.message.schema",
			"message.xsd"
		);
		System.setProperty(
			"Int.ecb.gesmescb.xml.SDMXMLWriter.structure.dtd",
			"structure.dtd"
		);
		System.setProperty(
			"Int.ecb.gesmescb.xml.SDMXMLWriter.structure.schema",
			"structure.xsd"
		);
		System.setProperty(
			"Int.ecb.gesmescb.xml.SDMXMLWriter.structure.stylesheet",
			"structure.xsl"
		);
		System.setProperty(
			"Int.ecb.gesmescb.xml.KeyFamilyURIMapping.BODI.KFSUMPF.dtd",
			"compact.dtd"
		);
		System.setProperty(
			"Int.ecb.gesmescb.xml.KeyFamilyURIMapping.BODI.KFSUMPF.schema",
			"compact.xsd"
		);
		System.setProperty(
			"Int.ecb.gesmescb.xml.KeyFamilyURIMapping.BODI.KFSUMPF.stylesheet",
			"compact.xsl"
		);
		System.setProperty(
			"Int.ecb.gesmescb.xml.SDMXMLWriter.defaultObsConf",
			"F"
		);
		System.setProperty(
			"Int.ecb.gesmescb.xml.SDMXMLWriter.defaultObsPreBreak",
			"NaN"
		);
		System.setProperty(
			"Int.ecb.gesmescb.xml.SDMXMLWriter.dataSetAgency",
			"ECB"
		);
	}

	protected void tearDown () {
		System.getProperties().remove(
			"Int.ecb.gesmescb.xml.SDMXMLWriter.sdmxVersion"
		);
		System.getProperties().remove(
			"Int.ecb.gesmescb.xml.SDMXMLWriter.message.schema"
		);
		System.getProperties().remove(
			"Int.ecb.gesmescb.xml.SDMXMLWriter.structure.dtd"
		);
		System.getProperties().remove(
			"Int.ecb.gesmescb.xml.SDMXMLWriter.structure.schema"
		);
		System.getProperties().remove(
			"Int.ecb.gesmescb.xml.SDMXMLWriter.structure.stylesheet"
		);
		System.getProperties().remove(
			"Int.ecb.gesmescb.xml.KeyFamilyURIMapping.BODI.KFSUMPF.dtd"
		);
		System.getProperties().remove(
			"Int.ecb.gesmescb.xml.KeyFamilyURIMapping.BODI.KFSUMPF.schema"
		);
		System.getProperties().remove(
			"Int.ecb.gesmescb.xml.KeyFamilyURIMapping.BODI.KFSUMPF.stylesheet"
		);
		System.getProperties().remove(
			"Int.ecb.gesmescb.xml.SDMXMLWriter.defaultObsConf"
		);
		System.getProperties().remove(
			"Int.ecb.gesmescb.xml.SDMXMLWriter.defaultObsPreBreak"
		);
		System.getProperties().remove(
			"Int.ecb.gesmescb.xml.SDMXMLWriter.dataSetAgency"
		);
		super.tearDown();
	}

	protected String getClassName () {
		return "Int.ecb.gesmescb.xml.SDMXMLWriter";
	}

	protected XMLWriter factory () throws IOException {
		return new SDMXMLWriter( "" );
	}
	
	protected XMLWriter streamFactory () throws IOException {
		return new SDMXMLWriter( "" ,  this.outputStream );
	}

	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite
	 */
	public static Test suite () {
		final TestSuite suite = new TestSuite( SDMXMLWriter20Test.class );
		return suite;
	}

	/**
	 * @param arg the array of argument {@link String}s
	 */
	public static void main ( final String[] arg ) {
		final String[] testCaseName = {
			SDMXMLWriter20Test.class.getName()
		};
		junit.textui.TestRunner.main( testCaseName );
	}

	/* ---------------------- Test Methods ---------------------- */
	
	public void testWith2Messages () throws Exception {
		System.setProperty(
			"Int.ecb.gesmescb.xml.SDMXMLWriter.sdmxVersion",
			"2.0"
		);
		writeTempFile ( new Object[]{
			INTERCHANGE_BEGIN,
			DELETE_MESSAGE_BEGIN,
			DS_ATT,
			SG_ATT,
			TS_ATT,
			SERIES1,
			OBS_ATT,
			SERIES3,
			SERIES2,
			MESSAGE_END, 
			MESSAGE_BEGIN,
			DS_ATT,
			SG_ATT,
			TS_ATT,
			SERIES1,
			OBS_ATT,
			SERIES2,
			MESSAGE_END,
			INTERCHANGE_END
		} );
		String expectedContents =
				EPSILON_BEGIN_OUT
				+ DELETE_MESSAGE_BEGIN_OUT
				+ DELETE_SORTED_OUT
				+ DATA1_SORTED_OUT
				+ DATA_END_OUT;
		assertEqualsLinewise(
			expectedContents
			,
			readTempFile()
		);
	}
	
	/* ==================== Private Methods ===================== */
	

}
