// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2003, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.xml;

import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import org.xml.sax.Attributes;

/* **************************** Class ***************************** */

/**
 * Partially implements the {@link Attributes} interface
 * for testing {@SAX2Reader}s.
 *
 * @author
 * <a href="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</a>
 *
 * @version @(#)$Id: TestAttributes.java,v 1.1 2017/01/19 16:14:02 trainot Exp $
 */
public final class TestAttributes implements Attributes {

	/* ========================= Fields ========================= */

	private final String[] names;
	private final String[] values;

	/* ====================== Constructors ====================== */

	public TestAttributes ( final String[] qNames, final String[] values ) {
		if ( qNames.length != values.length ) {
			throw new IllegalArgumentException(
				"There are " + qNames.length
				+ " for " + values.length + " values."
			);
		}
		this.names = qNames;
		this.values = values;
	}

	/** Construct from space-separated list of assignments.
	 * Something like "<tt>FREQ=M ITEM=602 TIME_FORMAT=P1M</tt>".
	 */
	public TestAttributes( final String string ) {
		final List
			nameList = new ArrayList(),
			valueList = new ArrayList();
		for (
			final StringTokenizer tokenizer = new StringTokenizer( string );
			tokenizer.hasMoreTokens();
		) {
			final String token = tokenizer.nextToken();
			final int sep = token.indexOf( '=' );
			nameList.add( token.substring( 0, sep ) );
			valueList.add( token.substring( sep + 1 ) );
		}
		this.names = (String[]) nameList.toArray( new String[]{} );
		this.values = (String[]) valueList.toArray( new String[]{} );
	}

	/* ===================== Public Methods ===================== */

	public int getIndex ( final String qName ) {
		for ( int i = 0; i < names.length; ++i ) {
			if ( names[i].equals( qName ) ) return i;
		}
		return -1;
	}

	public int getIndex ( final String uri, final String localName ) {
		throw new UnsupportedOperationException(
			"TestAttributes does not implement all Attributes methods."
		);
	}

	public int getLength () {
		return names.length;
	}

	public String getLocalName ( final int index ) {
		final String qName = getQName( index );
		final int colonPosition = qName.indexOf( ':' );
		if ( colonPosition >= 0 ) {
			return qName.substring( colonPosition + 1 );
		} else {
			return qName;
		}
	}

	public String getQName ( final int index ) {
		return names[index];
	}

	public String getType ( final int index ) {
		throw new UnsupportedOperationException(
			"TestAttributes does not implement all Attributes methods."
		);
	}

	public String getType ( final String qName ) {
		throw new UnsupportedOperationException(
			"TestAttributes does not implement all Attributes methods."
		);
	}

	public String getType ( final String uri, final String localName ) {
		throw new UnsupportedOperationException(
			"TestAttributes does not implement all Attributes methods."
		);
	}

	public String getURI ( final int index ) {
		throw new UnsupportedOperationException(
			"TestAttributes does not implement all Attributes methods."
		);
	}

	public String getValue ( final int index ) {
		return values[index];
	}

	public String getValue ( final String qName ) {
		final int index = getIndex( qName );
		if ( index >= 0 ) {
			return getValue( index );
		} else {
			return null;
		}
	}

	public String getValue ( final String uri, final String localName ) {
		/*throw new UnsupportedOperationException(
			"TestAttributes does not implement all Attributes methods."
		);*/
	    return getValue( localName );
	}

	public String toString () {
		final StringBuffer buffer = new StringBuffer();
		for ( int i = 0; i < names.length && i < values.length; ++i ) {
			if ( i > 0 ) {
				buffer.append( ' ' );
			}
			buffer.append( names[i] );
			buffer.append( '=' );
			buffer.append( values[i] );
		}
		return buffer.toString();
	}
}
