// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2003, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.xml;

import java.util.Date;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.text.ParsePosition;

import Int.ecb.gesmescb.*;

/* *************************** Interface **************************** */

/**
 * <p>Holds generally useful constants for testing
 * {@link Int.ecb.gesmescb.xml}.
 *
 * @author
 * <a href="mailto:sis.external@ecb.int">Bernhard Bodenstorfer</A>
 *
 * @version @(#)$Id: TestConstants.java,v 1.1 2017/01/19 16:14:02 trainot Exp $
 */
interface TestConstants extends Constants {

	/* ======================= Constants ======================== */

	DateFormat DATE_FORMAT = new SimpleDateFormat( "yyyy-MM-dd'T'HH:mm:ss" );

	Date VERSION_DATE = DATE_FORMAT.parse( "2003-11-10T15:22:00", new ParsePosition( 0 ) );

	KeyFamilyComponent
		FREQ = new KeyFamilyComponent(
			"FREQ", KeyFamilyComponent.KEY_STRUCTURE_DIMENSION,
			"1", "AN..1", "", "", ""
		),
		ITEM = new KeyFamilyComponent(
			"ITEM", KeyFamilyComponent.KEY_STRUCTURE_DIMENSION,
			"2", "AN1", "", "", ""
		),
		TIME_PERIOD = new KeyFamilyComponent(
			"TIME_PERIOD", KeyFamilyComponent.TIME_DIMENSION,
			"4", "AN..35", "", "", ""
		),
		TIME_FORMAT = new KeyFamilyComponent(
			"TIME_FORMAT", KeyFamilyComponent.TIME_DIMENSION,
			"5", "AN3", "", "", ""
		),
		OBS_VALUE = new KeyFamilyComponent(
			"OBS_VALUE", KeyFamilyComponent.ARRAY_CELL,
			"6", "AN..15", "", "", ""
		),
		OBS_STATUS = new KeyFamilyComponent(
			"OBS_STATUS", KeyFamilyComponent.ARRAY_CELL,
			"7", "AN1", KeyFamilyComponent.MANDATORY,
			KeyFamilyComponent.ATTACHED_TO_OBSERVATION, ""
		),
		OBS_CONF = new KeyFamilyComponent(
			"OBS_CONF", KeyFamilyComponent.ARRAY_CELL,
			"8", "AN1", KeyFamilyComponent.CONDITIONAL,
			KeyFamilyComponent.ATTACHED_TO_OBSERVATION, ""
		),
		OBS_PRE_BREAK = new KeyFamilyComponent(
			"OBS_PRE_BREAK", KeyFamilyComponent.ARRAY_CELL,
			"9", "AN..15", KeyFamilyComponent.CONDITIONAL,
			KeyFamilyComponent.ATTACHED_TO_OBSERVATION, ""
		),
		OBS_ATT = new KeyFamilyComponent(
			"OBS_ATT", KeyFamilyComponent.ATTRIBUTE,
			"", "AN..20", KeyFamilyComponent.CONDITIONAL,
			KeyFamilyComponent.ATTACHED_TO_OBSERVATION, ""
		),
		TS_ATT = new KeyFamilyComponent(
			"TS_ATT", KeyFamilyComponent.ATTRIBUTE,
			"", "A2", KeyFamilyComponent.MANDATORY,
			KeyFamilyComponent.ATTACHED_TO_TIME_SERIES, ""
		),
		SG_ATT = new KeyFamilyComponent(
			"SG_ATT", KeyFamilyComponent.ATTRIBUTE,
			"", "N..10", KeyFamilyComponent.CONDITIONAL,
			KeyFamilyComponent.ATTACHED_TO_SIBLING_GROUP, ""
		),
		DS_ATT = new KeyFamilyComponent(
			"DS_ATT", KeyFamilyComponent.ATTRIBUTE,
			"", "A..10", KeyFamilyComponent.MANDATORY,
			KeyFamilyComponent.ATTACHED_TO_DATA_SET, ""
		);

	KeyFamily KF = new KeyFamily(
		"KF", "test KF", "", VERSION_DATE,
		new KeyFamilyComponent[]{
			FREQ, ITEM, TIME_PERIOD, TIME_FORMAT,
			OBS_VALUE, OBS_STATUS, OBS_CONF, OBS_PRE_BREAK,
			OBS_ATT, TS_ATT, SG_ATT, DS_ATT
		}
	);
	
	KeyFamily ECB_EXR1 = new KeyFamily(
			"ECB_EXR1", "test ECB_EXR1", "", VERSION_DATE,
			new KeyFamilyComponent[]{
				FREQ, ITEM, TIME_PERIOD, TIME_FORMAT,
				OBS_VALUE, OBS_STATUS, OBS_CONF, OBS_PRE_BREAK,
				OBS_ATT, TS_ATT, SG_ATT, DS_ATT
			}
		);
}
