// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2003, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.xml;

// import org.xml.sax.XMLReader;
import org.xml.sax.InputSource;
import org.xml.sax.ContentHandler;
import org.xml.sax.DTDHandler;
import org.xml.sax.EntityResolver;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;

/* **************************** Class ***************************** */

/**
 * Partially implements the {@link org.xml.sax.XMLReader} interface
 * for testing {@SAX2Reader}s.
 *
 * @author
 * <a href="mailto:Bernhard.Bodenstorfer@ECB.int">Bernhard Bodenstorfer</a>
 *
 * @version @(#)$Id: TestParser.java,v 1.1 2017/01/19 16:14:02 trainot Exp $
 */
public final class TestParser implements org.xml.sax.XMLReader {

	/* ========================= Fields ========================= */

	private ContentHandler contentHandler;
	private ErrorHandler errorHandler;
	private String namespace, prefix;

	/* ====================== Constructors ====================== */

	/* ====================== Constructors ====================== */

	public TestParser () {
		contentHandler = null;
		errorHandler = null;
		namespace = null;
		prefix = null;
	}

	/* ===================== Public Methods ===================== */

	/* ---------------------- SAX2 Methods ---------------------- */

	public void setContentHandler ( final ContentHandler handler ) {
		this.contentHandler = handler;
	}

	public ContentHandler getContentHandler () {
		return contentHandler;
	}

	public void setDTDHandler ( final DTDHandler handler ) {
		throwUp();
	}

	public DTDHandler getDTDHandler () {
		throwUp();
		return null;
	}

	public void setEntityResolver ( final EntityResolver resolver ) {
		throwUp();
	}

	public EntityResolver getEntityResolver () {
		throwUp();
		return null;
	}

	public void setErrorHandler ( final ErrorHandler handler ) {
		this.errorHandler = handler;
	}

	public ErrorHandler getErrorHandler () {
		return errorHandler;
	}

	public void setFeature ( final String name, final boolean value ) {
		throwUp();
	}

	public boolean getFeature ( final String name ) {
		throwUp();
		return false;
	}

	public void setProperty ( final String name, final Object value ) {
		throwUp();
	}

	public Object getProperty ( final String name ) {
		throwUp();
		return null;
	}

	public void parse ( final String systemId ) {
		throwUp();
	}

	public void parse ( final InputSource input ) {
		throwUp();
	}

	/* -------------------- Utility Methods --------------------- */

	void setNamespace ( final String uri, final String prefix ) {
		this.namespace = uri;
		this.prefix = prefix;
	}

	void openElement ( final String name, final String attributes ) throws SAXException {
		getContentHandler().startElement(
			namespace,
			name,
			prefix != null ? prefix + ":" + name : name,
			new TestAttributes( attributes )
		);
	}

	void characters ( final String string ) throws SAXException {
		getContentHandler().characters(
			string.toCharArray(), 0, string.length()
		);
	}

	void closeElement ( final String name ) throws SAXException {
		getContentHandler().endElement(
			namespace,
			name,
			prefix != null ? prefix + ":" + name : name
		);
	}

	void writeElement ( final String name, final String value ) throws SAXException {
		openElement( name, "" );
		characters( value );
		closeElement( name );
	}

	/* ==================== Private Methods ===================== */

	private void throwUp () {
		throw new UnsupportedOperationException(
			"TestParser does not implement all XMLReader methods."
		);
	}
}
