<?xml version="1.0" encoding="UTF-8"?>

<!-- @(#)$Id: gesmes.xsl,v 1.1 2017/01/19 16:14:02 trainot Exp $

ECB/SIS Public License, version 1.0, document reference SIS/2001/116

Copyright (c) 2002, European Central Bank. All rights reserved.

Redistribution and use in source and binary forms,
with or without modification, are permitted
provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.
Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.
Neither the name of the European Central Bank
nor the names of its contributors may be used to endorse or promote products
derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-->

<xsl:stylesheet
	xmlns="http://www.w3.org/1999/xhtml"
	xmlns:gesmes="http://www.gesmes.org/xml/2002-08-01"
	xmlns:edifact="http://www.gesmes.org/xml/edifact/2002-08-01"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	version="1.0">

	<xsl:strip-space elements="*"/>

	<xsl:output
		method="html"
		doctype-public="-//W3C//DTD XHTML 1.0 Strict//EN"
		doctype-system="http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd"
		encoding="ISO-8859-1"
		media-type="text/html"
		indent="yes"
	/>

	<xsl:variable name="contextresource">
		<xsl:choose>
			<xsl:when test="//gesmes:CodeList[@id='CL_ORGANISATION']"/>
			<xsl:otherwise>
				<xsl:text>escb2002_08_20.html</xsl:text>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:variable>

	<xsl:variable name="debug" select="'leafcube'"/>

	<!-- ****************** Main Section ****************** -->

	<xsl:template match="/">
		<xsl:comment>Created by stylesheet: @(#)$Id: gesmes.xsl,v 1.1 2017/01/19 16:14:02 trainot Exp $</xsl:comment>
		<html>
			<head>
				<title>
					<xsl:choose>
						<xsl:when test="gesmes:Envelope/gesmes:name">
							<xsl:value-of select="gesmes:Envelope/gesmes:name"/>
						</xsl:when>
						<xsl:otherwise>
							<xsl:apply-templates select="*" mode="name"/>
						</xsl:otherwise>
					</xsl:choose>
				</title>
<style type="text/css">
	h1, h2, h3, h4, h5, h6 {
		font-family: Arial, Helvetica
	}
	h1 {
		font-weight: bold;
		font-variant: small-caps;
		text-decoration: underline
	}
	table {
		border: thin solid
	}
	td.cl { color: red }
	td.sc { color: green }
	td.kf { color: magenta }
	.mistake { background-color: red }
</style>
			</head>
			<body bgcolor="#E0F0E0" text="#000040">
				<xsl:if test="installmsxml3">
					<!-- A little notification for old MSXML-users -->
					<p><b>In case you are using IE 5.x and see this note
					and otherwise only crap here, then <blink>sorry</blink>.
					Your browser does not conform to standard
					<a href="http://www.w3.org/TR/xslt">XSLT</a>.</b></p>
					<p>You probably will need to
					<a href="http://msdn.microsoft.com/library/default.asp?url=/library/en-us/xmlsdk30/htm/xmconrunningmsxml3inreplacemode.asp">install MSXML3.dll in replace mode</a>.</p>
				</xsl:if>
				<xsl:apply-templates select="gesmes:Envelope" mode="content"/>
			</body>
		</html>
	</xsl:template>

	<!-- ****************** Help Section ****************** -->

	<xsl:template match="*" mode="type">
		<xsl:choose>
			<xsl:when test="self::gesmes:CodeList">
				<xsl:attribute name="class">cl</xsl:attribute>
				<xsl:text>Code List</xsl:text>
			</xsl:when>
			<xsl:when test="self::gesmes:Concept">
				<xsl:attribute name="class">sc</xsl:attribute>
				<xsl:text>Concept</xsl:text>
			</xsl:when>
			<xsl:when test="self::gesmes:KeyFamily">
				<xsl:attribute name="class">kf</xsl:attribute>
				<xsl:text>Key Family</xsl:text>
			</xsl:when>
			<xsl:when test="self::gesmes:Envelope">
				<xsl:text>Envelope</xsl:text>
			</xsl:when>
			<xsl:when test="self::gesmes:telephone">
				<xsl:text>Telephone</xsl:text>
			</xsl:when>
			<xsl:when test="self::gesmes:fax">
				<xsl:text>Fax</xsl:text>
			</xsl:when>
			<xsl:when test="self::gesmes:x400">
				<xsl:text>X.400</xsl:text>
			</xsl:when>
			<xsl:when test="self::gesmes:uri">
				<xsl:text>URI</xsl:text>
			</xsl:when>
			<xsl:when test="self::gesmes:TextFormat">
				<xsl:text>Text Format</xsl:text>
			</xsl:when>
			<xsl:when test="self::gesmes:TimePointSet">
				<xsl:text>Time Point Set</xsl:text>
			</xsl:when>
			<xsl:when test="self::gesmes:TimeDurationSet">
				<xsl:text>Time Duration Set</xsl:text>
			</xsl:when>
			<xsl:when test="self::gesmes:RealNumberSet">
				<xsl:text>Real Number Set</xsl:text>
			</xsl:when>
			<xsl:when test="namespace-uri()!='http://www.gesmes.org/xml/2002-08-01' and not(ancestor::gesmes:Extension)">
				<xsl:text>Cube</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="local-name()"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="*" mode="name">
		<xsl:choose>
			<xsl:when test="namespace-uri()!='http://www.gesmes.org/xml/2002-08-01'">
				<xsl:apply-templates select="." mode="cubename"/>
			</xsl:when>
			<xsl:when test="@id">
				<xsl:value-of select="@id"/>
			</xsl:when>
			<xsl:when test="@ref">
				<xsl:value-of select="@ref"/>
			</xsl:when>
			<xsl:when test="@value">
				<xsl:value-of select="@value"/>
			</xsl:when>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="*" mode="xaddress">
		<xsl:apply-templates select="parent::*" mode="xaddress"/>
		<xsl:text>/</xsl:text>
		<xsl:value-of select="count(preceding-sibling::*)+1"/>
	</xsl:template>

	<xsl:template match="*" mode="address">
		<xsl:apply-templates select="parent::*[not(self::gesmes:Envelope)]" mode="address"/>
		<xsl:if test="parent::*">
			<xsl:variable name="name">
				<xsl:apply-templates select="." mode="name"/>
			</xsl:variable>
			<xsl:if test="$name!=''">
				<xsl:text>#</xsl:text>
				<xsl:value-of select="$name"/>
			</xsl:if>
		</xsl:if>
	</xsl:template>

	<xsl:template match="*" mode="reference">
		<xsl:param name="baseref"/>
		<xsl:param name="text"/>
		<xsl:element name="a">
			<xsl:attribute name="href">
				<xsl:value-of select="$baseref"/>
				<xsl:apply-templates select="." mode="address"/>
				<xsl:if test="$text">
					<xsl:text>#</xsl:text>
					<xsl:value-of select="$text"/>
				</xsl:if>
			</xsl:attribute>
			<xsl:choose>
				<xsl:when test="$text">
					<xsl:value-of select="$text"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:apply-templates select="." mode="name"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:element>
	</xsl:template>

	<xsl:template match="*" mode="short">
		<xsl:param name="text"/>
		<xsl:param name="anchor"/>
		<xsl:element name="a">
			<xsl:if test="$anchor">
				<xsl:attribute name="name">
					<xsl:value-of select="$anchor"/>
				</xsl:attribute>
			</xsl:if>
			<xsl:choose>
				<xsl:when test="@ref">
					<xsl:attribute name="href">
						<xsl:text>#</xsl:text>
						<xsl:value-of select="@ref"/>
					</xsl:attribute>
				</xsl:when>
				<xsl:when test="(self::gesmes:Sender|self::gesmes:Receiver)[$contextresource!='-']">
					<xsl:attribute name="href">
						<xsl:if test="$contextresource!=''">
							<xsl:value-of select="$contextresource"/>
						</xsl:if>
						<xsl:text>#CL_ORGANISATION#</xsl:text>
						<xsl:value-of select="@id"/>
					</xsl:attribute>
				</xsl:when>
				<xsl:when test="namespace-uri()!='http://www.gesmes.org/xml/2002-08-01'">
					<xsl:apply-templates select="." mode="cubehref"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:apply-templates select="." mode="hrefhook"/>
				</xsl:otherwise>
			</xsl:choose>
			<xsl:choose>
				<xsl:when test="$text">
					<xsl:value-of select="$text"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:apply-templates select="." mode="name"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:element>
	</xsl:template>

	<xsl:template match="*" mode="hrefhook">
		<xsl:if test="contains($debug,'hrefhook')">
			<xsl:message>
				<xsl:text>hrefhook: </xsl:text>
				<xsl:apply-templates select="." mode="xaddress"/>
			</xsl:message>
		</xsl:if>
	</xsl:template>

	<xsl:template match="*" mode="anchor">
		<xsl:param name="text"/>
		<xsl:apply-templates select="." mode="short">
			<xsl:with-param name="text" select="$text"/>
			<xsl:with-param name="anchor">
				<xsl:variable name="address">
					<xsl:apply-templates select="." mode="address"/>
					<xsl:if test="$text">
						<xsl:text>#</xsl:text>
						<xsl:value-of select="$text"/>
					</xsl:if>
				</xsl:variable>
				<xsl:value-of select="substring-after($address,'#')"/>
			</xsl:with-param>
		</xsl:apply-templates>
	</xsl:template>

	<xsl:template match="*" mode="headline">
		<xsl:param name="text"/>
		<xsl:param name="sublevel">0</xsl:param>
		<xsl:element name="h{count(ancestor-or-self::*)+$sublevel}">
			<xsl:apply-templates select="." mode="anchor">
				<xsl:with-param name="text" select="$text"/>
			</xsl:apply-templates>
		</xsl:element>
	</xsl:template>

	<xsl:template match="*" mode="subheadinganchor">
		<xsl:param name="text"/>
		<xsl:apply-templates select="." mode="headline">
			<xsl:with-param name="sublevel" select="1"/>
			<xsl:with-param name="text">
				<xsl:apply-templates select="." mode="anchor">
					<xsl:with-param name="text" select="$text"/>
				</xsl:apply-templates>
			</xsl:with-param>
		</xsl:apply-templates>
	</xsl:template>

	<xsl:template match="*" mode="content">
		<table border="1">
			<xsl:apply-templates select="." mode="contentheader"/>
		</table>
	</xsl:template>

	<xsl:template match="*" mode="contentheader">
		<xsl:param name="colspan" select="2"/>
		<tr>
			<td>
				<xsl:apply-templates select="." mode="type"/>
			</td>
			<xsl:variable name="name">
				<xsl:apply-templates select="." mode="name"/>
			</xsl:variable>
			<xsl:if test="$name!=''">
				<td>
					<xsl:if test="$colspan&gt;2">
						<xsl:attribute name="colspan">
							<xsl:value-of select="$colspan -1"/>
						</xsl:attribute>
					</xsl:if>
					<xsl:apply-templates select="." mode="anchor"/>
				</td>
			</xsl:if>
		</tr>
	</xsl:template>

	<!-- ***************** Types Section ****************** -->

	<xsl:template match="gesmes:Envelope" mode="content">
		<xsl:apply-templates select="." mode="headline"/>
		<table border="1">
			<xsl:if test="@id">
				<tr>
					<td><xsl:apply-templates select="." mode="type"/></td>
					<td><xsl:apply-templates select="." mode="name"/></td>
				</tr>
			</xsl:if>
			<xsl:if test="gesmes:name">
				<tr>
					<th colspan="2">
						<xsl:value-of select="gesmes:name"/>
					</th>
				</tr>
			</xsl:if>
			<xsl:if test="gesmes:prepared">
				<tr>
					<td>Prepared</td>
					<td><xsl:value-of select="gesmes:prepared"/></td>
				</tr>
			</xsl:if>
			<xsl:if test="gesmes:test">
				<tr>
					<td colspan="2" align="center">
						<xsl:text>Contains </xsl:text>
						<blink>test</blink>
						<xsl:text> data - not for production!</xsl:text>
					</td>
				</tr>
			</xsl:if>
			<xsl:if test="gesmes:Sender">
				<tr>
					<td>
						<xsl:apply-templates select="." mode="reference">
							<xsl:with-param name="text" select="'Sender'"/>
						</xsl:apply-templates>
					</td>
					<td>
						<xsl:for-each select="gesmes:Sender">
							<xsl:sort select="@id"/>
							<xsl:if test="position()&gt;1">
								<xsl:text>, </xsl:text>
							</xsl:if>
							<xsl:apply-templates select="." mode="reference"/>
						</xsl:for-each>
					</td>
				</tr>
			</xsl:if>
			<xsl:if test="gesmes:Receiver">
				<tr>
					<td>
						<xsl:apply-templates select="." mode="reference">
							<xsl:with-param name="text" select="'Receiver'"/>
						</xsl:apply-templates>
					</td>
					<td>
						<xsl:for-each select="gesmes:Receiver">
							<xsl:sort select="@id"/>
							<xsl:if test="position()&gt;1">
								<xsl:text>, </xsl:text>
							</xsl:if>
							<xsl:apply-templates select="." mode="reference"/>
						</xsl:for-each>
					</td>
				</tr>
			</xsl:if>
			<xsl:if test="gesmes:reportingBegin">
				<tr>
					<td>Reporting from</td>
					<td><xsl:value-of select="gesmes:reportingBegin"/></td>
				</tr>
			</xsl:if>
			<xsl:if test="gesmes:reportingEnd">
				<tr>
					<td>Reporting until before</td>
					<td><xsl:value-of select="gesmes:reportingEnd"/></td>
				</tr>
			</xsl:if>
			<xsl:if test="gesmes:Extension/edifact:maintenanceAgencyId">
				<tr>
					<td>Maintenance Agency Identifier</td>
					<td><xsl:value-of select="gesmes:Extension/edifact:maintenanceAgencyId"/></td>
				</tr>
			</xsl:if>
			<xsl:if test="gesmes:Extension/edifact:keyFamilyId">
				<tr>
					<td>Key Family Identifier</td>
					<td><xsl:value-of select="gesmes:Extension/edifact:keyFamilyId"/></td>
				</tr>
			</xsl:if>
			<xsl:if test="gesmes:action">
				<tr>
					<td>Action</td>
					<td><xsl:value-of select="gesmes:action"/></td>
				</tr>
			</xsl:if>
			<xsl:if test="gesmes:dataSetId">
				<tr>
					<td>Data Set Identifier</td>
					<td><xsl:value-of select="gesmes:dataSetId"/></td>
				</tr>
			</xsl:if>
			<xsl:if test="gesmes:CodeList">
				<tr>
					<td>
						<xsl:apply-templates select="." mode="reference">
							<xsl:with-param name="text" select="'Code Lists'"/>
						</xsl:apply-templates>
					</td>
					<td>
						<xsl:for-each select="gesmes:CodeList">
							<xsl:sort select="@id"/>
							<xsl:if test="position()&gt;1">
								<xsl:text>, </xsl:text>
							</xsl:if>
							<xsl:apply-templates select="." mode="reference"/>
						</xsl:for-each>
					</td>
				</tr>
			</xsl:if>
			<xsl:if test="gesmes:Concept">
				<tr>
					<td>
						<xsl:apply-templates select="." mode="reference">
							<xsl:with-param name="text" select="'Concepts'"/>
						</xsl:apply-templates>
					</td>
					<td>
						<xsl:for-each select="gesmes:Concept">
							<xsl:sort select="@id"/>
							<xsl:if test="position()&gt;1">
								<xsl:text>, </xsl:text>
							</xsl:if>
							<xsl:apply-templates select="." mode="reference"/>
						</xsl:for-each>
					</td>
				</tr>
			</xsl:if>
			<xsl:if test="gesmes:KeyFamily">
				<tr>
					<td>
						<xsl:apply-templates select="." mode="reference">
							<xsl:with-param name="text" select="'Key Families'"/>
						</xsl:apply-templates>
					</td>
					<td>
						<xsl:for-each select="gesmes:KeyFamily">
							<xsl:sort select="@id"/>
							<xsl:if test="position()&gt;1">
								<xsl:text>, </xsl:text>
							</xsl:if>
							<xsl:apply-templates select="." mode="reference"/>
						</xsl:for-each>
					</td>
				</tr>
			</xsl:if>
			<xsl:if test="*[namespace-uri()!='http://www.gesmes.org/xml/2002-08-01']">
				<tr>
					<td>
						<xsl:apply-templates select="." mode="reference">
							<xsl:with-param name="text" select="'Cubes'"/>
						</xsl:apply-templates>
					</td>
					<td>
						<xsl:for-each select="*[namespace-uri()!='http://www.gesmes.org/xml/2002-08-01']">
							<xsl:sort select="@id"/>
							<xsl:if test="position()&gt;1">
								<xsl:text>, </xsl:text>
							</xsl:if>
							<xsl:apply-templates select="." mode="reference"/>
						</xsl:for-each>
					</td>
				</tr>
			</xsl:if>
			<xsl:if test="gesmes:Envelope">
				<tr>
					<td>
						<xsl:apply-templates select="." mode="reference">
							<xsl:with-param name="text" select="'Envelopes'"/>
						</xsl:apply-templates>
					</td>
					<td>
						<xsl:for-each select="gesmes:Envelope">
							<xsl:sort select="@id"/>
							<xsl:if test="position()&gt;1">
								<xsl:text>, </xsl:text>
							</xsl:if>
							<xsl:apply-templates select="." mode="reference"/>
						</xsl:for-each>
					</td>
				</tr>
			</xsl:if>
		</table>
		<xsl:if test="gesmes:Sender">
			<xsl:apply-templates select="." mode="subheadinganchor">
				<xsl:with-param name="text" select="'Sender'"/>
			</xsl:apply-templates>
			<xsl:apply-templates select="gesmes:Sender" mode="content"/>
		</xsl:if>
		<xsl:if test="gesmes:Receiver">
			<xsl:apply-templates select="." mode="subheadinganchor">
				<xsl:with-param name="text" select="'Receiver'"/>
			</xsl:apply-templates>
			<xsl:apply-templates select="gesmes:Receiver" mode="content"/>
		</xsl:if>
		<xsl:if test="gesmes:CodeList">
			<xsl:apply-templates select="." mode="subheadinganchor">
				<xsl:with-param name="text" select="'Code Lists'"/>
			</xsl:apply-templates>
			<xsl:apply-templates select="gesmes:CodeList" mode="content"/>
		</xsl:if>
		<xsl:if test="gesmes:Concept">
			<xsl:apply-templates select="." mode="subheadinganchor">
				<xsl:with-param name="text" select="'Concepts'"/>
			</xsl:apply-templates>
			<xsl:apply-templates select="gesmes:Concept" mode="content"/>
		</xsl:if>
		<xsl:if test="gesmes:KeyFamily">
			<xsl:apply-templates select="." mode="subheadinganchor">
				<xsl:with-param name="text" select="'Key Families'"/>
			</xsl:apply-templates>
			<xsl:apply-templates select="gesmes:KeyFamily" mode="content"/>
		</xsl:if>
		<xsl:if test="*[namespace-uri()!='http://www.gesmes.org/xml/2002-08-01']">
			<xsl:apply-templates select="." mode="subheadinganchor">
				<xsl:with-param name="text" select="'Cubes'"/>
			</xsl:apply-templates>
			<xsl:apply-templates select="*[namespace-uri()!='http://www.gesmes.org/xml/2002-08-01']" mode="content"/>
		</xsl:if>
		<xsl:if test="gesmes:Envelope">
			<xsl:apply-templates select="." mode="subheadinganchor">
				<xsl:with-param name="text" select="'Envelopes'"/>
			</xsl:apply-templates>
			<xsl:apply-templates select="gesmes:Envelope" mode="content"/>
		</xsl:if>
	</xsl:template>

	<xsl:template match="gesmes:Sender|gesmes:Receiver" mode="content">
		<table border="1">
			<xsl:apply-templates select="." mode="contentheader"/>
			<xsl:apply-templates select="gesmes:Contact" mode="content"/>
		</table>
	</xsl:template>

	<xsl:template match="gesmes:Contact" mode="content">
		<xsl:if test="gesmes:name">
			<tr><th colspan="2"><xsl:value-of select="gesmes:name"/></th></tr>
		</xsl:if>
		<xsl:if test="gesmes:department">
			<tr>
				<td>Department</td>
				<td><xsl:value-of select="gesmes:department"/></td>
			</tr>
		</xsl:if>
		<xsl:if test="gesmes:role">
			<tr>
				<td>Function</td>
				<td><xsl:value-of select="gesmes:role"/></td>
			</tr>
		</xsl:if>
		<xsl:for-each select="gesmes:telephone|gesmes:fax|gesmes:x400|gesmes:uri">
			<tr>
				<td><xsl:apply-templates select="." mode="type"/></td>
				<td>
					<xsl:choose>
						<xsl:when test="self::gesmes:uri">
							<a href="{.}">
								<xsl:value-of select="."/>
							</a>
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="."/>
						</xsl:otherwise>
					</xsl:choose>
				</td>
			</tr>
		</xsl:for-each>
	</xsl:template>

	<xsl:template name="StructuralDefinition">
		<xsl:param name="colspan" select="2"/>
		<xsl:apply-templates select="." mode="contentheader">
			<xsl:with-param name="colspan" select="$colspan"/>
		</xsl:apply-templates>
		<xsl:if test="gesmes:name">
			<tr>
				<th colspan="{$colspan}">
					<xsl:value-of select="gesmes:name"/>
				</th>
			</tr>
		</xsl:if>
		<xsl:if test="gesmes:description">
			<tr>
				<td colspan="{$colspan}" align="center">
					<xsl:value-of select="gesmes:description"/>
				</td>
			</tr>
		</xsl:if>
	</xsl:template>

	<xsl:template match="gesmes:CodeList" mode="content">
		<table border="1">
			<xsl:call-template name="StructuralDefinition"/>
			<xsl:apply-templates select="gesmes:Code" mode="content"/>
		</table>
	</xsl:template>

	<xsl:template match="gesmes:Code" mode="content">
		<tr>
			<td><xsl:apply-templates select="." mode="anchor"/></td>
			<td><xsl:value-of select="gesmes:description"/></td>
		</tr>
	</xsl:template>

	<xsl:template match="gesmes:Concept" mode="listing">
		<td>
			<xsl:apply-templates select="." mode="anchor"/>
		</td>
		<xsl:for-each select="gesmes:TextFormat|gesmes:TimePointSet|gesmes:TimeDurationSet|gesmes:RealNumberSet|gesmes:CodeList">
			<td>
				<xsl:apply-templates select="." mode="short"/>
			</td>
		</xsl:for-each>
	</xsl:template>

	<xsl:template match="gesmes:Concept" mode="content">
		<table border="1">
			<xsl:call-template name="StructuralDefinition"/>
			<xsl:variable name="domain" select="gesmes:TextFormat|gesmes:TimePointSet|gesmes:TimeDurationSet|gesmes:RealNumberSet|gesmes:CodeList"/>
			<xsl:if test="$domain">
				<tr>
					<td colspan="2">
						<xsl:apply-templates select="$domain" mode="content"/>
					</td>
				</tr>
			</xsl:if>
		</table>
	</xsl:template>

	<xsl:template match="gesmes:KeyFamily" mode="content">
		<xsl:variable name="colmax">
			<xsl:for-each select="gesmes:Concept">
				<xsl:variable name="count" select="count(gesmes:TextFormat|gesmes:TimePointSet|gesmes:TimeDurationSet|gesmes:RealNumberSet|gesmes:CodeList)"/>
				<xsl:if test="not(../gesmes:Concept[$count&lt;count(gesmes:TextFormat|gesmes:TimePointSet|gesmes:TimeDurationSet|gesmes:RealNumberSet|gesmes:CodeList)])">
					<xsl:value-of select="$count+2"/>
					<xsl:text> </xsl:text>
				</xsl:if>
			</xsl:for-each>
		</xsl:variable>
		<xsl:variable name="colspan" select="substring-before($colmax,' ')"/>
		<table border="1">
			<xsl:call-template name="StructuralDefinition">
				<xsl:with-param name="colspan" select="$colspan"/>
			</xsl:call-template>
			<xsl:if test="gesmes:uri">
				<tr>
					<td>URI</td>
					<td>
						<xsl:if test="$colspan&gt;2">
							<xsl:attribute name="colspan">
								<xsl:value-of select="$colspan -1"/>
							</xsl:attribute>
						</xsl:if>
						<a href="{gesmes:uri}">
							<xsl:value-of select="gesmes:uri"/>
						</a>
					</td>
				</tr>
			</xsl:if>
			<!-- No short listings
			<xsl:call-template name="listComponents">
				<xsl:with-param name="section" select="'Dimensions'"/>
				<xsl:with-param name="components" select="gesmes:Concept[not(gesmes:Extension/edifact:attachmentLevel)]"/>
				<xsl:with-param name="colspan" select="$colspan"/>
			</xsl:call-template>
			<xsl:call-template name="listComponents">
				<xsl:with-param name="section" select="'Observation Attributes'"/>
				<xsl:with-param name="components" select="gesmes:Concept[gesmes:Extension/edifact:attachmentLevel='observation']"/>
				<xsl:with-param name="colspan" select="$colspan"/>
			</xsl:call-template>
			<xsl:call-template name="listComponents">
				<xsl:with-param name="section" select="'Time Series Attributes'"/>
				<xsl:with-param name="components" select="gesmes:Concept[gesmes:Extension/edifact:attachmentLevel='time series']"/>
				<xsl:with-param name="colspan" select="$colspan"/>
			</xsl:call-template>
			<xsl:call-template name="listComponents">
				<xsl:with-param name="section" select="'Sibling Group Attributes'"/>
				<xsl:with-param name="components" select="gesmes:Concept[gesmes:Extension/edifact:attachmentLevel='sibling group']"/>
				<xsl:with-param name="colspan" select="$colspan"/>
			</xsl:call-template>
			<xsl:call-template name="listComponents">
				<xsl:with-param name="section" select="'Data Set Attributes'"/>
				<xsl:with-param name="components" select="gesmes:Concept[gesmes:Extension/edifact:attachmentLevel='data set']"/>
				<xsl:with-param name="colspan" select="$colspan"/>
			</xsl:call-template>
			-->
			<xsl:if test="gesmes:Concept">
				<tr>
					<td/>
					<th>Concept</th>
					<xsl:if test="$colspan&gt;2">
						<th>
							<xsl:if test="$colspan&gt;3">
								<xsl:attribute name="colspan">
									<xsl:value-of select="$colspan -2"/>
								</xsl:attribute>
							</xsl:if>
							<xsl:text>Domain</xsl:text>
						</th>
					</xsl:if>
				</tr>
			</xsl:if>
			<xsl:call-template name="contentComponents">
				<xsl:with-param name="section" select="'Dimensions'"/>
				<xsl:with-param name="components" select="gesmes:Concept[not(gesmes:Extension/edifact:attachmentLevel)]"/>
			</xsl:call-template>
			<xsl:call-template name="contentComponents">
				<xsl:with-param name="section" select="'Observation Attributes'"/>
				<xsl:with-param name="components" select="gesmes:Concept[gesmes:Extension/edifact:attachmentLevel='observation']"/>
			</xsl:call-template>
			<xsl:call-template name="contentComponents">
				<xsl:with-param name="section" select="'Time Series Attributes'"/>
				<xsl:with-param name="components" select="gesmes:Concept[gesmes:Extension/edifact:attachmentLevel='time series']"/>
			</xsl:call-template>
			<xsl:call-template name="contentComponents">
				<xsl:with-param name="section" select="'Sibling Group Attributes'"/>
				<xsl:with-param name="components" select="gesmes:Concept[gesmes:Extension/edifact:attachmentLevel='sibling group']"/>
			</xsl:call-template>
			<xsl:call-template name="contentComponents">
				<xsl:with-param name="section" select="'Data Set Attributes'"/>
				<xsl:with-param name="components" select="gesmes:Concept[gesmes:Extension/edifact:attachmentLevel='data set']"/>
			</xsl:call-template>
		</table>
	</xsl:template>

	<xsl:template name="listComponents">
		<xsl:param name="section"/>
		<xsl:param name="components"/>
		<xsl:param name="colspan" select="2"/>
		<xsl:if test="$components">
			<tr>
				<td>
					<xsl:apply-templates select="." mode="reference">
						<xsl:with-param name="text" select="$section"/>
					</xsl:apply-templates>
				</td>
				<td>
					<xsl:if test="$colspan&gt;2">
						<xsl:attribute name="colspan">
							<xsl:value-of select="$colspan -1"/>
						</xsl:attribute>
					</xsl:if>
					<xsl:for-each select="$components">
						<xsl:if test="position()&gt;1">
							<xsl:text>, </xsl:text>
						</xsl:if>
						<xsl:apply-templates select="." mode="reference"/>
					</xsl:for-each>
				</td>
			</tr>
		</xsl:if>
	</xsl:template>

	<xsl:template name="contentComponents">
		<xsl:param name="section"/>
		<xsl:param name="components"/>
		<xsl:variable name="rowspan" select="count($components)"/>
		<xsl:if test="$components">
			<tr>
				<td>
					<xsl:if test="$rowspan&gt;1">
						<xsl:attribute name="rowspan">
							<xsl:value-of select="$rowspan"/>
						</xsl:attribute>
					</xsl:if>
					<xsl:apply-templates select="." mode="anchor">
						<xsl:with-param name="text" select="$section"/>
					</xsl:apply-templates>
				</td>
				<xsl:for-each select="$components[position()=1]">
					<xsl:apply-templates select="." mode="listing"/>
				</xsl:for-each>
			</tr>
			<xsl:for-each select="$components[position()&gt;1]">
				<tr>
					<xsl:apply-templates select="." mode="listing"/>
				</tr>
			</xsl:for-each>
		</xsl:if>
	</xsl:template>

	<xsl:template match="gesmes:TextFormat" mode="short">
		<xsl:if test="gesmes:length">
			<xsl:if test="gesmes:length/@exact='false'">
				<xsl:text>up to </xsl:text>
			</xsl:if>
			<xsl:value-of select="gesmes:length"/>
			<xsl:text> </xsl:text>
		</xsl:if>
		<xsl:if test="gesmes:cType">
			<xsl:value-of select="gesmes:cType"/>
			<xsl:text> </xsl:text>
		</xsl:if>
		<xsl:text>character</xsl:text>
		<xsl:if test="gesmes:length!=1">
			<xsl:text>s</xsl:text>
		</xsl:if>
	</xsl:template>

	<xsl:template match="gesmes:RealNumberSet|gesmes:TimePointSet|gesmes:TimeDurationSet" mode="short">
		<xsl:apply-templates select="." mode="type"/>
	</xsl:template>

	<!-- ****************** Cube Section ****************** -->

	<xsl:template match="*[namespace-uri()!='http://www.gesmes.org/xml/2002-08-01']" mode="cubename">
		<xsl:variable name="namespace" select="namespace-uri()"/>
		<xsl:variable name="structure" select="document(concat($namespace,'/gesmes.xml'))"/>
		<xsl:variable name="keyfamily" select="$structure//gesmes:KeyFamily[gesmes:uri=$namespace]"/>
		<xsl:choose>
			<xsl:when test="$keyfamily">
				<xsl:value-of select="$keyfamily/@id"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="namespace-uri()"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="*[namespace-uri()!='http://www.gesmes.org/xml/2002-08-01']" mode="cubehref">
		<xsl:variable name="namespace" select="namespace-uri()"/>
		<xsl:variable name="strucdefsfile" select="concat($namespace,'/gesmes.xml')"/>
		<xsl:variable name="structure" select="document($strucdefsfile)"/>
		<xsl:variable name="keyfamily" select="$structure//gesmes:KeyFamily[gesmes:uri=$namespace]"/>
		<xsl:attribute name="href">
			<xsl:value-of select="concat($strucdefsfile,'#',$keyfamily/@id)"/>
		</xsl:attribute>
	</xsl:template>

	<xsl:template match="*[namespace-uri()!='http://www.gesmes.org/xml/2002-08-01']" mode="content">
		<xsl:variable name="namespace" select="namespace-uri()"/>
		<xsl:variable name="strucdefsfile" select="concat($namespace,'/gesmes.xml')"/>
		<xsl:variable name="structure" select="document($strucdefsfile)"/>
		<xsl:variable name="codelists" select="$structure//gesmes:CodeList"/>
		<xsl:variable name="keyfamily" select="$structure//gesmes:KeyFamily[gesmes:uri=$namespace]"/>
		<xsl:variable name="concepts" select="$keyfamily/gesmes:Concept"/>
		<table border="1">
			<tr>
				<td>
					<xsl:apply-templates select="." mode="type"/>
				</td>
				<td>
					<xsl:if test="count($concepts)&gt;2">
						<xsl:attribute name="colspan">
							<xsl:value-of select="count($concepts)-1"/>
						</xsl:attribute>
					</xsl:if>
					<xsl:apply-templates select="." mode="anchor"/>
				</td>
			</tr>
			<tr>
				<xsl:for-each select="$concepts">
					<td>
						<a href="{concat($strucdefsfile,'#',$keyfamily/@id,'#',@ref)}">
							<xsl:value-of select="@ref"/>
						</a>
					</td>
				</xsl:for-each>
			</tr>
			<xsl:apply-templates select="." mode="cubecontent">
				<xsl:with-param name="strucdefsfile" select="$strucdefsfile"/>
				<xsl:with-param name="structure" select="$structure"/>
				<xsl:with-param name="keyfamily" select="$keyfamily"/>
				<xsl:with-param name="concepts" select="$concepts"/>
				<xsl:with-param name="codelists" select="$codelists"/>
			</xsl:apply-templates>
		</table>
	</xsl:template>

	<xsl:template match="*[namespace-uri()!='http://www.gesmes.org/xml/2002-08-01']" mode="cubecontent">
		<xsl:param name="strucdefsfile"/>
		<xsl:param name="codelists"/>
		<xsl:param name="concepts"/>
		<xsl:param name="unassigned" select="$concepts"/>
		<xsl:param name="rowspan" select="''"/>
		<xsl:if test="contains($debug,'cubecontent')">
			<xsl:message>
				<xsl:text>cubecontent </xsl:text>
				<xsl:apply-templates select="." mode="xaddress"/>
			</xsl:message>
		</xsl:if>
		<xsl:if test="contains($debug,'unassigned')">
			<xsl:message>
				<xsl:apply-templates select="$unassigned/@ref"/>
			</xsl:message>
		</xsl:if>
		<xsl:if test="contains($debug,'rowspan')">
			<xsl:message>
				<xsl:text>rowspan: </xsl:text>
				<xsl:value-of select="$rowspan"/>
			</xsl:message>
		</xsl:if>
		<xsl:variable name="cube" select="current()"/>
		<xsl:variable name="subcubes" select="*[namespace-uri()!='http://www.gesmes.org/xml/2002-08-01']"/>
		<xsl:variable name="newlyspanned">
			<xsl:text> </xsl:text>
			<xsl:for-each select="$unassigned">
				<xsl:variable name="conceptid" select="@ref"/>
				<xsl:if test="not($cube/descendant::*[namespace-uri()!='http://www.gesmes.org/xml/2002-08-01' and @*[name()=$conceptid]])">
					<xsl:value-of select="$conceptid"/>
					<xsl:text> </xsl:text>
				</xsl:if>
			</xsl:for-each>
		</xsl:variable>
		<xsl:variable name="newrowspan">
			<xsl:value-of select="$rowspan"/>
			<xsl:if test="$newlyspanned!=' '">
				<xsl:value-of select="$newlyspanned"/>
				<xsl:text>*</xsl:text>
				<xsl:value-of select="count($cube/descendant-or-self::*[namespace-uri()!='http://www.gesmes.org/xml/2002-08-01' and not(*[namespace-uri()!='http://www.gesmes.org/xml/2002-08-01'])])"/>
				<xsl:text> |</xsl:text>
			</xsl:if>
		</xsl:variable>
		<xsl:if test="contains($debug,'rowspan')">
			<xsl:message>
				<xsl:text>newrowspan: </xsl:text>
				<xsl:value-of select="$newrowspan"/>
			</xsl:message>
		</xsl:if>
		<xsl:choose>
			<xsl:when test="$subcubes">
				<xsl:for-each select="$subcubes">
					<xsl:apply-templates select="." mode="cubecontent">
						<xsl:with-param name="strucdefsfile" select="$strucdefsfile"/>
						<xsl:with-param name="codelists" select="$codelists"/>
						<xsl:with-param name="concepts" select="$concepts"/>
						<xsl:with-param name="rowspan">
							<xsl:if test="not(preceding-sibling::*[namespace-uri()!='http://www.gesmes.org/xml/2002-08-01'])">
								<xsl:value-of select="$newrowspan"/>
							</xsl:if>
						</xsl:with-param>
						<xsl:with-param name="unassigned" select="$unassigned[not(contains($newlyspanned,concat(' ',@ref,' ')))]"/>
					</xsl:apply-templates>
				</xsl:for-each>
			</xsl:when>
			<xsl:otherwise>
				<xsl:if test="contains($debug,'leafcube')">
					<xsl:message>
						<xsl:text>leafcube:</xsl:text>
						<xsl:apply-templates select="." mode="xaddress"/>
					</xsl:message>
				</xsl:if>
				<tr>
					<xsl:for-each select="$concepts">
						<xsl:variable name="concept" select="."/>
						<xsl:variable name="conceptid" select="@ref"/>
						<xsl:variable name="span" select="substring-before(substring-after(substring-after($newrowspan,concat(' ',$conceptid,' ')),'*'),' ')"/>
						<xsl:if test="$span&gt;0">
							<td>
								<xsl:if test="$span&gt;1">
									<xsl:attribute name="rowspan">
										<xsl:value-of select="$span"/>
									</xsl:attribute>
								</xsl:if>
								<xsl:variable name="value" select="$cube/ancestor-or-self::*[namespace-uri()!='http://www.gesmes.org/xml/2002-08-01']/@*[name()=$conceptid]"/>
								<xsl:if test="contains($debug,'values')">
									<xsl:message>
										<xsl:text>value:</xsl:text>
										<xsl:value-of select="$conceptid"/>
										<xsl:text> = </xsl:text>
										<xsl:value-of select="$value"/>
										<xsl:text> * </xsl:text>
										<xsl:value-of select="$span"/>
									</xsl:message>
								</xsl:if>
								<xsl:if test="$value">
									<xsl:variable name="codelist" select="$codelists[@id=$concept/gesmes:CodeList/@ref]"/>
									<xsl:choose>
										<xsl:when test="$codelist and $codelist/gesmes:Code[@value=$value]">
											<a href="{concat($strucdefsfile,'#',$codelist/@id,'#',$value)}">
												<xsl:value-of select="$value"/>
											</a>
										</xsl:when>
										<xsl:when test="$codelist">
											<xsl:attribute name="class">mistake</xsl:attribute>
											<a href="{concat($strucdefsfile,'#',$codelist/@id)}">
												<xsl:value-of select="$value"/>
											</a>
										</xsl:when>
										<xsl:otherwise>
											<xsl:value-of select="$value"/>
										</xsl:otherwise>
									</xsl:choose>
								</xsl:if>
							</td>
						</xsl:if>
					</xsl:for-each>
				</tr>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
</xsl:stylesheet>
