package Int.ecb.gesmescb.xml.sdmx.v2;

import java.util.ArrayList;

import Int.ecb.gesmescb.Attribute;
import Int.ecb.gesmescb.KeyFamily;
import Int.ecb.gesmescb.MessageBegin;
import Int.ecb.gesmescb.MessageEnd;
import Int.ecb.gesmescb.TimeSeries;
import Int.ecb.gesmescb.event.GESMESCBEventBroker;
import Int.ecb.gesmescb.metabase.Metabase;

/**
 * Broker to arrange group attributes correctly.
 * Only for use with SDMX-ML 2.1 messages. 
 * 
 * @author Moritz Korte-Stapff
 */

public class GroupLevelSorter extends GESMESCBEventBroker {
	
	ArrayList<Object> objectList;
	static KeyFamily currKeyFamily;
	
    /**
     * Constructs a new GroupLevelSorter.
     * 
     */
	public GroupLevelSorter() {
		super("", false);
		objectList = new ArrayList<Object>();
	}
	
	/* ------------------------ Listener ------------------------ */
	
	public void handle ( final MessageBegin messageBegin ){
		// Used later on to obtain attribute attachment levels
		currKeyFamily = Metabase.getMetabase().getMaintenanceAgency(
							messageBegin.getMaintenanceAgencyId()
						).getKeyFamily(
							messageBegin.getKeyFamilyId());
		dispatch(messageBegin);
	}
	
	public void handle ( final TimeSeries timeSeries ){
		this.objectList.add(timeSeries);
	}
	
	public void handle ( final Attribute attribute ){
		if ( currKeyFamily.getComponent(attribute.getName()).getAttachmentLevel().equals("9")){
			/* Dispatch Group Level Attributes immediately, 
			 * store all other attributes in a list and dispatch
			 * after the attributes in the order they were received.
			 */
			dispatch(attribute);
		} else {
			this.objectList.add(attribute);
		}
	}
	
	public void handle ( final MessageEnd messageEnd ){
		for ( Object obj : objectList ){
			// Dipsatch everything.
			if ( obj instanceof TimeSeries ){
				dispatch((TimeSeries)obj);
			} else if ( obj instanceof Attribute ){
				dispatch((Attribute)obj);
			}
		}
		objectList.clear();
		dispatch(messageEnd);
	}
}
