package Int.ecb.gesmescb.xml.sdmx.v2;

import java.util.Date;
import java.util.Iterator;

import Int.ecb.gesmescb.Attribute;
import Int.ecb.gesmescb.Contact;
import Int.ecb.gesmescb.KeyFamily;
import Int.ecb.gesmescb.KeyFamilyComponent;
import Int.ecb.gesmescb.MessageBegin;
import Int.ecb.gesmescb.MessageEnd;
import Int.ecb.gesmescb.Observation;
import Int.ecb.gesmescb.TimeSeries;
import Int.ecb.gesmescb.event.TestEventListener;
import Int.ecb.gesmescb.event.TestEventSource;
import Int.ecb.gesmescb.metabase.Metabase;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

public class GroupLevelSorterTest extends TestCase {
	
	/** Some date */
	final static Date versionDate = new Date();
	
	/** The Key Family needed for some data tests. */
	private static final KeyFamily SIS_FOOB = new KeyFamily(
		"SIS_FOOB", "Secret Intelligence Service: Foobar stuff", "",
		versionDate,
		new KeyFamilyComponent[] {
			new KeyFamilyComponent(
				"WHO", KeyFamilyComponent.KEY_STRUCTURE_DIMENSION,
				"1", "AN3", "", "", "CL_OBJ"
			),
			new KeyFamilyComponent(
				"SECURITY", KeyFamilyComponent.KEY_STRUCTURE_DIMENSION,
				"2", "AN1", "", "", "CL_CONF"
			),
			new KeyFamilyComponent(
				"TIME_PERIOD", KeyFamilyComponent.TIME_DIMENSION,
				"3", "AN..35", "", "", ""
			),
			new KeyFamilyComponent(
				"TIME_FORMAT", KeyFamilyComponent.TIME_DIMENSION,
				"4", "AN3", "", "", ""
			),
			new KeyFamilyComponent(
				"OBS_VALUE", KeyFamilyComponent.ARRAY_CELL,
				"5", "AN..15", "", "", ""
			),
			new KeyFamilyComponent(
				"OBS_STATUS", KeyFamilyComponent.ARRAY_CELL,
				"6", "AN1", "2", "5", "CL_OBS_STATUS"
			),
			new KeyFamilyComponent(
				"OBS_CONF", KeyFamilyComponent.ARRAY_CELL,
				"7", "AN1", "1", "5", "CL_OBS_CONF"
			),
			new KeyFamilyComponent(
				"OBS_PRE_BREAK", KeyFamilyComponent.ARRAY_CELL,
				"8", "AN..15", "1", "5", ""
			),
			new KeyFamilyComponent(
				"UNIT", KeyFamilyComponent.ATTRIBUTE,
				"", "AN..12", KeyFamilyComponent.MANDATORY,
				KeyFamilyComponent.ATTACHED_TO_SIBLING_GROUP,
				"CL_UNIT"
			),
			new KeyFamilyComponent(
				"ORGAN", KeyFamilyComponent.ATTRIBUTE,
				"", "AN..70", KeyFamilyComponent.MANDATORY,
				KeyFamilyComponent.ATTACHED_TO_TIME_SERIES,
				"CL_UNIT"
			)
		}
	);
	
	/** Array containing objects to be dispatched 
	 *  */
	Object[] inputNoGroupAtt = {
		new MessageBegin (
			"MREF000001","4F","4F","ECB","",
			"Hi",new Contact[]{},"SuperDSI","full",
			"Tmrw","Yesterday","DDMMYY","SIS_FOOB"
		), 
		new TimeSeries(
			"SOX.B", "20102020", "702",
			new Observation[] {
				new Observation( "12", "S", "N", "" ),
				new Observation( "19911991", "A", "", "" ),
				new Observation( "0", "A", "C", "13" ),
				new Observation( "3.14", "C", "", "" )
			}
		),
		new TimeSeries(
			"SOX.S", "20102020", "702",
			new Observation[] {
				new Observation( "13.6", "S", "N", "" ),
				new Observation( "13.5", "A", "", "" ),
				new Observation( "1", "A", "C", "13" ),
				new Observation( "1.2", "C", "", "" )
			}
		),
		new Attribute(
			"SOX.C.ORGAN", "", "", "ORGAN", "EUR"
		),
		new MessageEnd("MREF000001")
	};
	
	/** Array containing objects to be dispatched 
	 *  */
	Object[] inputWithGroupAtt = {
		new MessageBegin (
			"MREF000001","4F","4F","ECB","",
			"Hi",new Contact[]{},"SuperDSI","full",
			"Tmrw","Yesterday","DDMMYY","SIS_FOOB"
		), 
		new TimeSeries(
			"SOX.B", "20102020", "702",
			new Observation[] {
				new Observation( "12", "S", "N", "" ),
				new Observation( "19911991", "A", "", "" ),
				new Observation( "0", "A", "C", "13" ),
				new Observation( "3.14", "C", "", "" )
			}
		),
		new TimeSeries(
			"SOX.S", "20102020", "702",
			new Observation[] {
				new Observation( "13.6", "S", "N", "" ),
				new Observation( "13.5", "A", "", "" ),
				new Observation( "1", "A", "C", "13" ),
				new Observation( "1.2", "C", "", "" )
			}
		),
		new Attribute(
			"SOX.C.ORGAN", "", "", "ORGAN", "EUR"
		),
		new Attribute(
			"C.UNIT", "", "", "UNIT", "EUR"
		),
		new MessageEnd("MREF000001")
	};
	
	/** Array containing objects to be dispatched 
	 *  */
	Object[] sortedOutput = {
		new MessageBegin (
			"MREF000001","4F","4F","ECB","",
			"Hi",new Contact[]{},"SuperDSI","full",
			"Tmrw","Yesterday","DDMMYY","SIS_FOOB"
		), 
		new Attribute(
			"C.UNIT", "", "", "UNIT", "EUR"
		),
		new TimeSeries(
			"SOX.B", "20102020", "702",
			new Observation[] {
				new Observation( "12", "S", "N", "" ),
				new Observation( "19911991", "A", "", "" ),
				new Observation( "0", "A", "C", "13" ),
				new Observation( "3.14", "C", "", "" )
			}
		),
		new TimeSeries(
			"SOX.S", "20102020", "702",
			new Observation[] {
				new Observation( "13.6", "S", "N", "" ),
				new Observation( "13.5", "A", "", "" ),
				new Observation( "1", "A", "C", "13" ),
				new Observation( "1.2", "C", "", "" )
			}
		),
		new Attribute(
			"SOX.C.ORGAN", "", "", "ORGAN", "EUR"
		),
		new MessageEnd("MREF000001")
	};
	
	/** The listener to receive test output. */
	protected final TestEventSource source;
	
	/** The listener to receive test output. */
	protected final TestEventListener listener;
	
	public GroupLevelSorterTest ( String name ) {
		super( name );
		source = new TestEventSource();
		listener = new TestEventListener();
	}
	
	protected void setUp () {
		Metabase metabase = Metabase.getMetabase();
		source.addListener(metabase, true);
		source.dispatch(inputNoGroupAtt[0]);
		source.dispatch(SIS_FOOB);
		source.removeAllListeners();
	}
	
	/* ===================== Public Methods ===================== */

	/**
	 * Assembles and returns a test suite for all
	 * the test methods of this test case.
	 *
	 * @return a non-null test suite
	 */
	public static Test suite () {
		final TestSuite suite = new TestSuite( GroupLevelSorterTest.class );
		return suite;
	}

	/**
	 * @param arg the array of argument {@link String}s
	 */
	public static void main ( final String[] arg ) {
		final String[] testCaseName = {
			GroupLevelSorterTest.class.getName()
		};
		junit.textui.TestRunner.main( testCaseName );
	}
	
	/* ---------------------- Test Methods ---------------------- */
	
	public void testNoGroupAtt () {
		GroupLevelSorter sorter = new GroupLevelSorter();
		source.addListener(sorter, false);
		sorter.addListener(listener, false);
		source.dispatch(inputNoGroupAtt);
		final Iterator it = listener.iterator();
		int counter = 0;
		while (it.hasNext()){
			assertEquals(it.next(), inputNoGroupAtt[counter]);
			counter++;
		}
		assertEquals(counter, inputNoGroupAtt.length);
		listener.clearHistory();
		source.removeAllListeners();
	}
	
	public void testGroupAtt () {
		GroupLevelSorter sorter = new GroupLevelSorter();
		source.addListener(sorter, false);
		sorter.addListener(listener, false);
		source.dispatch(inputWithGroupAtt);
		final Iterator it = listener.iterator();
		int counter = 0;
		while (it.hasNext()){
			assertEquals(it.next(), sortedOutput[counter]);
			counter++;
		}
		assertEquals(counter, inputWithGroupAtt.length);
		source.removeAllListeners();
		listener.clearHistory();
	}
}
