package Int.ecb.gesmescb.xml.sdmx.v2;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import com.sun.org.apache.bcel.internal.Constants;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import static org.junit.Assert.*;

import Int.ecb.gesmescb.Attribute;
import Int.ecb.gesmescb.CodeList;
import Int.ecb.gesmescb.Contact;
import Int.ecb.gesmescb.KeyFamily;
import Int.ecb.gesmescb.KeyFamilyComponent;
import Int.ecb.gesmescb.MessageBegin;
import Int.ecb.gesmescb.StatisticalConcept;
import Int.ecb.gesmescb.event.TestEventListener;

import Int.ecb.gesmescb.metabase.Metabase;

public class SDMX21StructureReaderTest extends TestCase implements Constants {
	
	/* ====================== Constructors ====================== */
	public SDMX21StructureReaderTest(String name) {
		super(name);
		// TODO Auto-generated constructor stub
	}

	/* ======================= Constants ======================== */
	
	private static int actualTestOutputLength = 6097; 
	
	

	/* ==================== Test Methods ======================== */
	
	public void testAgainstRegistry() throws FileNotFoundException, IOException {
		
		SDMX21StructureReader reader
		= new SDMX21StructureReader( "Zampano");
		
		Object[] filteredTestOutput = output( reader );
		doTestRun( reader, filteredTestOutput );
		
	}
	
	
	/* ==================== Private Methods ===================== */
	
	private void doTestRun (
			final SDMX21StructureReader reader,
			final Object[] testOutput
		)
			throws IOException
		{
			TestEventListener listener
			= new TestEventListener();
			Metabase metabase = Metabase.getMetabase();
			reader.addListener( metabase, false );
			reader.addListener( listener, true );

			// Do the run.
			reader.run();
			
			// System.out.println( metabase );

			// Compare the event contents.
			Object[] readOutput = filteredReadOutput(listener, "ECB",512, "ECB_EXR1", 1);
			for(int i = 0; i < 4; i++){
				if (!readOutput[i].equals(testOutput[i])){
					System.out.println("Expected:" + testOutput[i]);
					System.out.println("Received:" + readOutput[i]);
				} 
				assertEquals("Output", testOutput[i], readOutput[i]);
			}
			if ((int)readOutput[4]< actualTestOutputLength){
				System.out.println("Reader read too little Objects");
			} else if ((int)readOutput[4]> actualTestOutputLength){
				System.out.println("Reader read too many Objects");
			}
			assertEquals((int)readOutput[4], actualTestOutputLength);
		}
	
	/**
	 * Used so that not all the output of the reader has to generated
	 * written by hand but instead only one of each type. But all 
	 * objects can be tested. E.g. if one wants to check whether the
	 * n-th codelist dispatched to the listener was read correctly
	 * set the parameter codeListNr to n. To test if right number 
	 * of objects were read, the last entry in the array contains
	 * the number of objects dispatched to the listener. 
	 * 
	 * @param listener The listener that the reader dispatches to.
	 * @param messageBeginNr The number of the MessageBegin to be tested.
	 * @param codeListNr The number of the Codelist to be tested.
	 * @param KeyFamilyNr The number of the KeyFamily to be tested.
	 * @param conceptNr The number of the StatisticalConcept  to be tested.
	 * @return
	 */
	private Object[] filteredReadOutput (TestEventListener listener, String maintainanceAgencyID, int codeListNr, String keyFamilyID, int conceptNr){
		
		Object[] output = new Object[5];
		int counter = 0;
		Object currentObject;
		
		final Iterator iterator = listener.iterator();
		int codeListOccurence = 0;
		int conceptOccurence = 0;
		
		output[0] = new MessageBegin(
				"MREF000001","","",maintainanceAgencyID,"","",new Contact[]{},
				"","","","","",""
		);

		while ( iterator.hasNext() ) {
			
			counter++;
			currentObject = iterator.next();
			//System.out.println(currentObject);
			
			if (currentObject instanceof CodeList ){
				codeListOccurence++;
				if ( codeListOccurence == codeListNr ){
					output[1] = currentObject;
				}
			}
			if (currentObject instanceof KeyFamily){
				if (((KeyFamily) currentObject).getId().equals(keyFamilyID)){
					output[2] = currentObject;
				}
			}
			if (currentObject instanceof StatisticalConcept){
				conceptOccurence++;
				if ( conceptOccurence == codeListNr ){
					output[3] = currentObject;
				}
			}
		}
		output[4] = counter;
		return output;
	}
	
	private static Object[] output (SDMX21StructureReader reader){
		
		Date versionDate = reader.getDate();
		Object[] filteredTestOutput = {
				new MessageBegin(
						"MREF000001","","","ECB","","",new Contact[]{},
						"","","","","",""
				),
				/*new CodeList(
						"CL_EXR_SUFFIX","Exch. rate series variation code list","",
						versionDate, 
						new String[]{"A","E","P","R","S","T"},
						new String[]{"Average", "End-of-period","Growth rate to previous period",
								"Annual rate of change",
								"Percentage change since December 1998 (1998Q4 for quarterly data)",
								"3-year percentage change"}
				),*/
				new CodeList(
						"CL_FREQ","Frequency code list","",
						versionDate, 
						new String[]{"A","B","D","E","H","M","N","Q","S","W"},
						new String[]{"Annual",
								"Business",
								"Daily",
								"Event (not supported)",
								"Half-yearly",
								"Monthly",
								"Minutely",
								"Quarterly",
								"Half Yearly, semester (value H exists but change to S in 2009, move from H to this new value to be agreed in ESCB context)",
								"Weekly"}
				),
				new KeyFamily(
						"ECB_EXR1",
						"Exchange Rates",
						"",
						versionDate,
						new KeyFamilyComponent[]{
								new KeyFamilyComponent(
									"FREQ", "4","1","","",
									"","CL_FREQ"
								),
								new KeyFamilyComponent(
									"CURRENCY", "4","2","","",
									"","CL_CURRENCY"
								),
								new KeyFamilyComponent(
									"CURRENCY_DENOM", "4","3","","",
									"","CL_CURRENCY"
								),
								new KeyFamilyComponent(
									"EXR_TYPE", "4","4","","",
									"","CL_EXR_TYPE"
								),
								new KeyFamilyComponent(
									"EXR_SUFFIX", "4","5","","",
									"","CL_EXR_SUFFIX"
								),
								new KeyFamilyComponent(
									"TIME_PERIOD", "1","6","","",
									"",""
								),
								new KeyFamilyComponent(
									"TIME_FORMAT", "Z09","","AN3","2",
									"4",""
								),
								new KeyFamilyComponent(
									"OBS_STATUS", "Z09","", "","2",
									"5","CL_OBS_STATUS"
								),
								new KeyFamilyComponent(
									"OBS_CONF", "Z09","", "","1",
									"5","CL_OBS_CONF"
								),
								new KeyFamilyComponent(
									"OBS_PRE_BREAK", "Z09","", "AN..15","1",
									"5",""
								),
								new KeyFamilyComponent(
									"OBS_COM", "Z09","", "AN..1050","1",
									"5",""
								),
								new KeyFamilyComponent(
									"BREAKS", "Z09","", "AN..350","1",
									"4",""
								),
								new KeyFamilyComponent(
									"COLLECTION", "Z09","", "","2",
									"4","CL_COLLECTION"
								),
								new KeyFamilyComponent(
									"COMPILING_ORG", "Z09","", "","1",
									"4","CL_ORGANISATION"
								),
								new KeyFamilyComponent(
									"DISS_ORG", "Z09","", "","1",
									"4","CL_ORGANISATION"
								),
								new KeyFamilyComponent(
									"DOM_SER_IDS", "Z09","", "AN..70","1",
									"4",""
								),
								new KeyFamilyComponent(
									"PUBL_ECB", "Z09","", "AN..1050","1",
									"4",""
								),
								new KeyFamilyComponent(
									"PUBL_MU", "Z09","", "AN..1050","1",
									"4",""
								),
								new KeyFamilyComponent(
									"PUBL_PUBLIC", "Z09","", "AN..1050","1",
									"4",""
								),
								new KeyFamilyComponent(
									"UNIT_INDEX_BASE", "Z09","", "AN..35","1",
									"4",""
								),
								new KeyFamilyComponent(
									"COMPILATION", "Z09","", "AN..1050","1",
									"4",""
								),
								new KeyFamilyComponent(
									"COVERAGE", "Z09","", "AN..350","1",
									"4",""
								),
								new KeyFamilyComponent(
									"DECIMALS", "Z09","", "","2",
									"4","CL_DECIMALS"
								),
								new KeyFamilyComponent(
									"NAT_TITLE", "Z09","", "AN..350","1",
									"4",""
								),
								new KeyFamilyComponent(
									"SOURCE_AGENCY", "Z09","", "","1",
									"4","CL_ORGANISATION"
								),
								new KeyFamilyComponent(
									"SOURCE_PUB", "Z09","", "AN..350","1",
									"4",""
								),
								new KeyFamilyComponent(
									"TITLE", "Z09","", "AN..200","1",
									"4",""
								),
								new KeyFamilyComponent(
									"TITLE_COMPL", "Z09","", "AN..1050","2",
									"4",""
								),
								new KeyFamilyComponent(
									"UNIT", "Z09","", "","2",
									"4","CL_UNIT"
								),
								new KeyFamilyComponent(
									"UNIT_MULT", "Z09","", "","2",
									"4","CL_UNIT_MULT"
								),
								new KeyFamilyComponent(
									"OBS_VALUE","3","","","",
									"",""
								),
						}	
				),
				/*new KeyFamily(
						"ECB_IFS1",
						"IFS (IMF)",
						"",
						versionDate,
						new KeyFamilyComponent[]{
								new KeyFamilyComponent(
										"FREQ", "4","1","","",
										"","CL_FREQ"
								),
								new KeyFamilyComponent(
										"IFS_CODE", "4","2","","",
										"",""
								),
								new KeyFamilyComponent(
										"TIME_PERIOD", "1","3","","",
										"",""
								),
								new KeyFamilyComponent(
										"TIME_FORMAT", "Z09","","AN3","2",
										"4",""
								),
								new KeyFamilyComponent(
										"OBS_STATUS", "Z09","","","2",
										"5","CL_OBS_STATUS"
								),
								new KeyFamilyComponent(
										"OBS_CONF", "Z09","","","1",
										"5","CL_OBS_CONF"
								),
								new KeyFamilyComponent(
										"OBS_PRE_BREAK", "Z09","","AN..15","1",
										"5",""
								),
								new KeyFamilyComponent(
										"EXT_REF_AREA", "Z09","","AN..70","1",
										"4",""
								),
								new KeyFamilyComponent(
										"EXT_TITLE", "Z09","","AN..350","1",
										"4",""
								),
								new KeyFamilyComponent(
										"EXT_UNIT", "Z09","","AN..70","1",
										"4",""
								),
								new KeyFamilyComponent(
										"TITLE_COMPL", "Z09","","AN..1050","1",
										"4",""
								),
								new KeyFamilyComponent(
										"OBS_VALUE", "3","","","",
										"",""
								)
						}
				),*/
				new StatisticalConcept(
						"COVERAGE",
						"Coverage",
						"",
						versionDate
				)
		};
		return filteredTestOutput;
	}
}
