// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2002, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
package Int.ecb.gesmescb.xml.sdmx.v2.compact;

import java.util.Iterator;

import junit.framework.Test;
import junit.framework.TestSuite;

import org.xml.sax.SAXException;

import Int.ecb.gesmescb.Attribute;
import Int.ecb.gesmescb.metabase.Metabase;
import Int.ecb.gesmescb.time.GregorianRegularInterval;
import Int.ecb.gesmescb.xml.SAX2Reader;
import Int.ecb.gesmescb.xml.SAX2ReaderTest;

public class GroupReaderTest extends SAX2ReaderTest {
    
    /* ====================== Constructors ====================== */
    
    public GroupReaderTest(String name) {
        super(name);
    }

    /* ================== Protected Methods ===================== */

    /** Sets up the test fixture. */
    protected void setUp () {
        super.setUp();
        //setNamespace( XMLNS_GESMES, null );
        final Metabase metabase = Metabase.getMetabase();
        metabase.handle( KF );
        /*System.setProperty(
            KeyFamilyURIMapping.class.getName() + ".uri.ECB.KF",
            "http://www.ecb.int/vocabulary/2003-11-10/kf"
        );*/
    }

    /** Tears down the test fixture. */
    protected void tearDown () {
        /*System.getProperties().remove(
            KeyFamilyURIMapping.class.getName() + ".uri.ECB.KF"
        );*/
    }

    protected SAX2Reader factory (
        final org.xml.sax.XMLReader parser,
        final SAX2Reader parent
    ) {
        final GroupReader reader
            = new GroupReader( parser, parent );
        reader.setKeyFamily( KF );
        return reader;
    }

    /* ===================== Public Methods ===================== */    

    /**
     * Assembles and returns a test suite for all
     * the test methods of this test case.
     *
     * @return a non-null test suite
     */
    public static Test suite () {
        final TestSuite suite = new TestSuite( GroupReaderTest.class );
        return suite;
    }

    /**
     * @param arg the array of argument {@link String}s
     */
    public static void main ( final String[] arg ) {
        final String[] testCaseName = {
            GroupReaderTest.class.getName()
        };
        junit.textui.TestRunner.main( testCaseName );
    }

    /* ---------------------- Test Methods ---------------------- */    
    
    public void testGetGroup() throws SAXException
    {
        openElement("Group", "ITEM=AUD");
        closeElement("Group");
        final Iterator iterator = listener.iterator();
        // As there are no real group object in the suite, we don't expect
        // to receive a group object
        assertTrue("No object expected", !iterator.hasNext());
    }
    
    public void testGetGroupWithAttributes() throws SAXException
    {
        openElement("Group", "ITEM=AUD SG_ATT=TEST");
        closeElement("Group");
        final Iterator iterator = listener.iterator();
        assertTrue("At least one object", iterator.hasNext());
        final Attribute attribute = (Attribute) iterator.next();
        final Attribute expectedAttribute = new Attribute(".AUD", 
                (GregorianRegularInterval) null, "SG_ATT", "TEST");
        assertEquals("The attributes should be equal", expectedAttribute, 
                attribute);
        assertTrue("No more object expected at this stage", 
                !iterator.hasNext());
    }
    
}
