// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2002, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
package Int.ecb.gesmescb.xml.sdmx.v2.compact;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Reader;
import java.io.StringReader;
import java.io.Writer;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Iterator;

import junit.framework.Test;
import junit.framework.TestSuite;

import org.xml.sax.SAXException;

import Int.ecb.gesmescb.Contact;
import Int.ecb.gesmescb.InterchangeBegin;
import Int.ecb.gesmescb.InterchangeEnd;
import Int.ecb.gesmescb.MessageBegin;
import Int.ecb.gesmescb.MessageEnd;
import Int.ecb.gesmescb.metabase.Metabase;
import Int.ecb.gesmescb.xml.SAX2Reader;
import Int.ecb.gesmescb.xml.SAX2ReaderTest;

public class HeaderReaderTest extends SAX2ReaderTest {

	private File structureFile;
	private static final String structureInput
	= "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>\n"
	+ "<message:Structure xsi:schemaLocation=\"http://www.SDMX.org/resources/SDMXML/schemas/v2_0/message http://ollie:8080/scorpio-external/vocabulary/sdmx/2.0/SDMXMessage.xsd http://www.SDMX.org/resources/SDMXML/schemas/v2_0/structure http://ollie:8080/scorpio-external/vocabulary/sdmx/2.0/SDMXStructure.xsd\" xmlns:generic=\"http://www.SDMX.org/resources/SDMXML/schemas/v2_0/generic\" xmlns:message=\"http://www.SDMX.org/resources/SDMXML/schemas/v2_0/message\" xmlns:structure=\"http://www.SDMX.org/resources/SDMXML/schemas/v2_0/structure\" xmlns:compact=\"http://www.SDMX.org/resources/SDMXML/schemas/v2_0/compact\" xmlns:metadatareport=\"http://www.SDMX.org/resources/SDMXML/schemas/v2_0/metadatareport\" xmlns:utility=\"http://www.SDMX.org/resources/SDMXML/schemas/v2_0/utility\" xmlns:common=\"http://www.SDMX.org/resources/SDMXML/schemas/v2_0/common\" xmlns:query=\"http://www.SDMX.org/resources/SDMXML/schemas/v2_0/query\" xmlns:genericmetadata=\"http://www.SDMX.org/resources/SDMXML/schemas/v2_0/genericmetadata\" xmlns:registry=\"http://www.SDMX.org/resources/SDMXML/schemas/v2_0/registry\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:cross=\"http://www.SDMX.org/resources/SDMXML/schemas/v2_0/cross\">\n"
	+ "    <message:Header>\n"
	+ "        <message:ID>45759fe8-56b1-48a0-a42b-3e2e53389315</message:ID>\n"
	+ "        <message:Test>false</message:Test>\n"
	+ "        <message:Prepared>2009-05-27T10:19:25+02:00</message:Prepared>\n"
	+ "        <message:Sender id=\"ECB\">\n"
	+ "            <message:Name>European Central Bank</message:Name>\n"
	+ "            <message:Contact>\n"
	+ "                <message:Email>statistics@ecb.europa.eu</message:Email>\n"
	+ "            </message:Contact>\n"
	+ "        </message:Sender>\n"
	+ "    </message:Header>\n"
	+ "    <message:CodeLists>\n"
	+ "        <structure:CodeList isFinal=\"true\" version=\"1.0\" agencyID=\"ECB\" id=\"CL_COLLECTION\">\n"
	+ "            <structure:Name>Collection indicator</structure:Name>\n"
	+ "            <structure:Code value=\"A\">\n"
	+ "                <structure:Description>Average of observations through period</structure:Description>\n"
	+ "            </structure:Code>\n"
	+ "        </structure:CodeList>\n"
	+ "        <structure:CodeList isFinal=\"true\" version=\"1.0\" agencyID=\"ECB\" id=\"CL_CURRENCY\">\n"
	+ "            <structure:Name>Currency sale</structure:Name>\n"
	+ "            <structure:Code value=\"EUR\">\n"
	+ "                <structure:Description>Euro</structure:Description>\n"
	+ "            </structure:Code>\n"
	+ "            <structure:Code value=\"USD\">\n"
	+ "                <structure:Description>US dollar</structure:Description>\n"
	+ "            </structure:Code>\n"
	+ "            <structure:Code value=\"ZWN\">\n"
	+ "                <structure:Description>Zimbabwe, Zimbabwe Dollars</structure:Description>\n"
	+ "            </structure:Code>\n"
	+ "        </structure:CodeList>\n"
	+ "        <structure:CodeList isFinal=\"true\" version=\"1.0\" agencyID=\"ECB\" id=\"CL_DECIMALS\">\n"
	+ "            <structure:Name>Decimals</structure:Name>\n"
	+ "            <structure:Code value=\"0\">\n"
	+ "                <structure:Description>Zero</structure:Description>\n"
	+ "            </structure:Code>\n"
	+ "            <structure:Code value=\"1\">\n"
	+ "                <structure:Description>One</structure:Description>\n"
	+ "            </structure:Code>\n"
	+ "        </structure:CodeList>\n"
	+ "        <structure:CodeList isFinal=\"true\" version=\"1.0\" agencyID=\"ECB\" id=\"CL_EXR_SUFFIX\">\n"
	+ "            <structure:Name>Series variation - EXR context</structure:Name>\n"
	+ "            <structure:Code value=\"A\">\n"
	+ "                <structure:Description>Average or standardised measure for given frequency</structure:Description>\n"
	+ "            </structure:Code>\n"
	+ "            <structure:Code value=\"E\">\n"
	+ "                <structure:Description>End-of-period</structure:Description>\n"
	+ "            </structure:Code>\n"
	+ "        </structure:CodeList>\n"
	+ "        <structure:CodeList isFinal=\"true\" version=\"1.0\" agencyID=\"ECB\" id=\"CL_EXR_TYPE\">\n"
	+ "            <structure:Name>Exchange rate type</structure:Name>\n"
	+ "            <structure:Code value=\"BRC0\">\n"
	+ "                <structure:Description>Real bilateral exchange rate, CPI deflated</structure:Description>\n"
	+ "            </structure:Code>\n"
	+ "        </structure:CodeList>\n"
	+ "        <structure:CodeList isFinal=\"true\" version=\"1.0\" agencyID=\"ECB\" id=\"CL_FREQ\">\n"
	+ "            <structure:Name>Frequency of the survey</structure:Name>\n"
	+ "            <structure:Code value=\"A\">\n"
	+ "                <structure:Description>Annual</structure:Description>\n"
	+ "            </structure:Code>\n"
	+ "            <structure:Code value=\"B\">\n"
	+ "                <structure:Description>Business</structure:Description>\n"
	+ "            </structure:Code>\n"
	+ "            <structure:Code value=\"D\">\n"
	+ "                <structure:Description>Daily</structure:Description>\n"
	+ "            </structure:Code>\n"
	+ "        </structure:CodeList>\n"
	+ "        <structure:CodeList isFinal=\"true\" version=\"1.0\" agencyID=\"ECB\" id=\"CL_OBS_CONF\">\n"
	+ "            <structure:Name>Observation confidentiality</structure:Name>\n"
	+ "            <structure:Code value=\"C\">\n"
	+ "                <structure:Description>Confidential statistical information</structure:Description>\n"
	+ "            </structure:Code>\n"
	+ "            <structure:Code value=\"D\">\n"
	+ "                <structure:Description>Secondary confidentiality set by the sender, not for publication</structure:Description>\n"
	+ "            </structure:Code>\n"
	+ "            <structure:Code value=\"F\">\n"
	+ "                <structure:Description>Free</structure:Description>\n"
	+ "            </structure:Code>\n"
	+ "            <structure:Code value=\"N\">\n"
	+ "                <structure:Description>Not for publication, restricted for internal use only</structure:Description>\n"
	+ "            </structure:Code>\n"
	+ "            <structure:Code value=\"S\">\n"
	+ "                <structure:Description>Secondary confidentiality set and managed by the receiver, not for publication</structure:Description>\n"
	+ "            </structure:Code>\n"
	+ "        </structure:CodeList>\n"
	+ "        <structure:CodeList isFinal=\"true\" version=\"1.0\" agencyID=\"ECB\" id=\"CL_OBS_STATUS\">\n"
	+ "            <structure:Name>Observation status</structure:Name>\n"
	+ "            <structure:Code value=\"A\">\n"
	+ "                <structure:Description>Normal value</structure:Description>\n"
	+ "            </structure:Code>\n"
	+ "            <structure:Code value=\"B\">\n"
	+ "                <structure:Description>Break</structure:Description>\n"
	+ "            </structure:Code>\n"
	+ "            <structure:Code value=\"E\">\n"
	+ "                <structure:Description>Estimated value</structure:Description>\n"
	+ "            </structure:Code>\n"
	+ "            <structure:Code value=\"F\">\n"
	+ "                <structure:Description>Forecast value</structure:Description>\n"
	+ "            </structure:Code>\n"
	+ "        </structure:CodeList>\n"
	+ "        <structure:CodeList isFinal=\"true\" version=\"1.0\" agencyID=\"ECB\" id=\"CL_ORGANISATION\">\n"
	+ "            <structure:Name>Source agency</structure:Name>\n"
	+ "            <structure:Code value=\"4D0\">\n"
	+ "                <structure:Description>Statistical Office of the European Commission (Eurostat)</structure:Description>\n"
	+ "            </structure:Code>\n"
	+ "            <structure:Code value=\"4F0\">\n"
	+ "                <structure:Description>European Central Bank (ECB)</structure:Description>\n"
	+ "            </structure:Code>\n"
	+ "            <structure:Code value=\"5B0\">\n"
	+ "                <structure:Description>Bank for International Settlements</structure:Description>\n"
	+ "            </structure:Code>\n"
	+ "        </structure:CodeList>\n"
	+ "        <structure:CodeList isFinal=\"true\" version=\"1.0\" agencyID=\"ECB\" id=\"CL_UNIT\">\n"
	+ "            <structure:Name>Unit</structure:Name>\n"
	+ "            <structure:Code value=\"EUR\">\n"
	+ "                <structure:Description>Euro</structure:Description>\n"
	+ "            </structure:Code>\n"
	+ "            <structure:Code value=\"UNITS\">\n"
	+ "                <structure:Description>Unit described in title</structure:Description>\n"
	+ "            </structure:Code>\n"
	+ "            <structure:Code value=\"USD\">\n"
	+ "                <structure:Description>US dollar</structure:Description>\n"
	+ "            </structure:Code>\n"
	+ "            <structure:Code value=\"ZWD\">\n"
	+ "                <structure:Description>Zimbabwe dollar</structure:Description>\n"
	+ "            </structure:Code>\n"
	+ "            <structure:Code value=\"ZWN\">\n"
	+ "                <structure:Description>Zimbabwe, Zimbabwe Dollars</structure:Description>\n"
	+ "            </structure:Code>\n"
	+ "        </structure:CodeList>\n"
	+ "        <structure:CodeList isFinal=\"true\" version=\"1.0\" agencyID=\"ECB\" id=\"CL_UNIT_MULT\">\n"
	+ "            <structure:Name>Unit multiplier</structure:Name>\n"
	+ "            <structure:Code value=\"-2\">\n"
	+ "                <structure:Description>Hundredth</structure:Description>\n"
	+ "            </structure:Code>\n"
	+ "            <structure:Code value=\"0\">\n"
	+ "                <structure:Description>Units</structure:Description>\n"
	+ "            </structure:Code>\n"
	+ "            <structure:Code value=\"1\">\n"
	+ "                <structure:Description>Tens</structure:Description>\n"
	+ "            </structure:Code>\n"
	+ "            <structure:Code value=\"2\">\n"
	+ "                <structure:Description>Hundreds</structure:Description>\n"
	+ "            </structure:Code>\n"
	+ "        </structure:CodeList>\n"
	+ "    </message:CodeLists>\n"
	+ "    <message:Concepts>\n"
	+ "        <structure:Concept version=\"1.0\" agencyID=\"ECB\" id=\"BREAKS\">\n"
	+ "            <structure:Name>Breaks</structure:Name>\n"
	+ "        </structure:Concept>\n"
	+ "        <structure:Concept version=\"1.0\" agencyID=\"ECB\" id=\"COLLECTION\">\n"
	+ "            <structure:Name>Collection indicator</structure:Name>\n"
	+ "        </structure:Concept>\n"
	+ "        <structure:Concept version=\"1.0\" agencyID=\"ECB\" id=\"COMPILATION\">\n"
	+ "            <structure:Name>Compilation</structure:Name>\n"
	+ "        </structure:Concept>\n"
	+ "        <structure:Concept version=\"1.0\" agencyID=\"ECB\" id=\"COVERAGE\">\n"
	+ "            <structure:Name>Coverage</structure:Name>\n"
	+ "        </structure:Concept>\n"
	+ "        <structure:Concept version=\"1.0\" agencyID=\"ECB\" id=\"CURRENCY\">\n"
	+ "            <structure:Name>Currency</structure:Name>\n"
	+ "        </structure:Concept>\n"
	+ "        <structure:Concept version=\"1.0\" agencyID=\"ECB\" id=\"CURRENCY_DENOM\">\n"
	+ "            <structure:Name>Currency denominator</structure:Name>\n"
	+ "        </structure:Concept>\n"
	+ "        <structure:Concept version=\"1.0\" agencyID=\"ECB\" id=\"DECIMALS\">\n"
	+ "            <structure:Name>Decimals</structure:Name>\n"
	+ "        </structure:Concept>\n"
	+ "        <structure:Concept version=\"1.0\" agencyID=\"ECB\" id=\"DOM_SER_IDS\">\n"
	+ "            <structure:Name>Domestic series ids</structure:Name>\n"
	+ "        </structure:Concept>\n"
	+ "        <structure:Concept version=\"1.0\" agencyID=\"ECB\" id=\"EXR_SUFFIX\">\n"
	+ "            <structure:Name>Series variation - EXR context</structure:Name>\n"
	+ "        </structure:Concept>\n"
	+ "        <structure:Concept version=\"1.0\" agencyID=\"ECB\" id=\"EXR_TYPE\">\n"
	+ "            <structure:Name>Exchange rate type</structure:Name>\n"
	+ "        </structure:Concept>\n"
	+ "        <structure:Concept version=\"1.0\" agencyID=\"ECB\" id=\"FREQ\">\n"
	+ "            <structure:Name>Frequency</structure:Name>\n"
	+ "        </structure:Concept>\n"
	+ "        <structure:Concept version=\"1.0\" agencyID=\"ECB\" id=\"NAT_TITLE\">\n"
	+ "            <structure:Name>National language title</structure:Name>\n"
	+ "        </structure:Concept>\n"
	+ "        <structure:Concept version=\"1.0\" agencyID=\"ECB\" id=\"OBS_COM\">\n"
	+ "            <structure:Name>Observation comment</structure:Name>\n"
	+ "        </structure:Concept>\n"
	+ "        <structure:Concept version=\"1.0\" agencyID=\"ECB\" id=\"OBS_CONF\">\n"
	+ "            <structure:Name>Observation confidentiality</structure:Name>\n"
	+ "        </structure:Concept>\n"
	+ "        <structure:Concept version=\"1.0\" agencyID=\"ECB\" id=\"OBS_PRE_BREAK\">\n"
	+ "            <structure:Name>Pre-break observation value</structure:Name>\n"
	+ "        </structure:Concept>\n"
	+ "        <structure:Concept version=\"1.0\" agencyID=\"ECB\" id=\"OBS_STATUS\">\n"
	+ "            <structure:Name>Observation status</structure:Name>\n"
	+ "        </structure:Concept>\n"
	+ "        <structure:Concept version=\"1.0\" agencyID=\"ECB\" id=\"OBS_VALUE\">\n"
	+ "            <structure:Name>Observation value</structure:Name>\n"
	+ "        </structure:Concept>\n"
	+ "        <structure:Concept version=\"1.0\" agencyID=\"ECB\" id=\"PUBL_ECB\">\n"
	+ "            <structure:Name>Source publication (ECB only)</structure:Name>\n"
	+ "        </structure:Concept>\n"
	+ "        <structure:Concept version=\"1.0\" agencyID=\"ECB\" id=\"PUBL_MU\">\n"
	+ "            <structure:Name>Source publication (Euro area only)</structure:Name>\n"
	+ "        </structure:Concept>\n"
	+ "        <structure:Concept version=\"1.0\" agencyID=\"ECB\" id=\"PUBL_PUBLIC\">\n"
	+ "            <structure:Name>Source publication (public)</structure:Name>\n"
	+ "        </structure:Concept>\n"
	+ "        <structure:Concept version=\"1.0\" agencyID=\"ECB\" id=\"SOURCE_AGENCY\">\n"
	+ "            <structure:Name>Source agency</structure:Name>\n"
	+ "        </structure:Concept>\n"
	+ "        <structure:Concept version=\"1.0\" agencyID=\"ECB\" id=\"SOURCE_PUB\">\n"
	+ "            <structure:Name>Source publication</structure:Name>\n"
	+ "        </structure:Concept>\n"
	+ "        <structure:Concept version=\"1.0\" agencyID=\"ECB\" id=\"TIME_FORMAT\">\n"
	+ "            <structure:Name>Time format code</structure:Name>\n"
	+ "        </structure:Concept>\n"
	+ "        <structure:Concept version=\"1.0\" agencyID=\"ECB\" id=\"TIME_PERIOD\">\n"
	+ "            <structure:Name>Time period or range</structure:Name>\n"
	+ "        </structure:Concept>\n"
	+ "        <structure:Concept version=\"1.0\" agencyID=\"ECB\" id=\"TITLE\">\n"
	+ "            <structure:Name>Title</structure:Name>\n"
	+ "        </structure:Concept>\n"
	+ "        <structure:Concept version=\"1.0\" agencyID=\"ECB\" id=\"TITLE_COMPL\">\n"
	+ "            <structure:Name>Title complement</structure:Name>\n"
	+ "        </structure:Concept>\n"
	+ "        <structure:Concept version=\"1.0\" agencyID=\"ECB\" id=\"UNIT\">\n"
	+ "            <structure:Name>Unit</structure:Name>\n"
	+ "        </structure:Concept>\n"
	+ "        <structure:Concept version=\"1.0\" agencyID=\"ECB\" id=\"UNIT_INDEX_BASE\">\n"
	+ "            <structure:Name>Unit index base</structure:Name>\n"
	+ "        </structure:Concept>\n"
	+ "        <structure:Concept version=\"1.0\" agencyID=\"ECB\" id=\"UNIT_MULT\">\n"
	+ "            <structure:Name>Unit multiplier</structure:Name>\n"
	+ "        </structure:Concept>\n"
	+ "    </message:Concepts>\n"
	+ "    <message:KeyFamilies>\n"
	+ "        <structure:KeyFamily isFinal=\"true\" urn=\"urn:sdmx:org.sdmx.infomodel.keyfamily.KeyFamily=ECB:ECB_EXR1\" version=\"1.0\" agencyID=\"ECB\" id=\"ECB_EXR1\">\n"
	+ "            <structure:Name>Exchange Rates</structure:Name>\n"
	+ "            <structure:Components>\n"
	+ "                <structure:Dimension isFrequencyDimension=\"true\" codelistAgency=\"ECB\" codelistVersion=\"1.0\" codelist=\"CL_FREQ\" conceptAgency=\"ECB\" conceptVersion=\"1.0\" conceptRef=\"FREQ\"/>\n"
	+ "                <structure:Dimension codelistAgency=\"ECB\" codelistVersion=\"1.0\" codelist=\"CL_CURRENCY\" conceptAgency=\"ECB\" conceptVersion=\"1.0\" conceptRef=\"CURRENCY\"/>\n"
	+ "                <structure:Dimension codelistAgency=\"ECB\" codelistVersion=\"1.0\" codelist=\"CL_CURRENCY\" conceptAgency=\"ECB\" conceptVersion=\"1.0\" conceptRef=\"CURRENCY_DENOM\"/>\n"
	+ "                <structure:Dimension codelistAgency=\"ECB\" codelistVersion=\"1.0\" codelist=\"CL_EXR_TYPE\" conceptAgency=\"ECB\" conceptVersion=\"1.0\" conceptRef=\"EXR_TYPE\"/>\n"
	+ "                <structure:Dimension codelistAgency=\"ECB\" codelistVersion=\"1.0\" codelist=\"CL_EXR_SUFFIX\" conceptAgency=\"ECB\" conceptVersion=\"1.0\" conceptRef=\"EXR_SUFFIX\"/>\n"
	+ "                <structure:TimeDimension conceptAgency=\"ECB\" conceptVersion=\"1.0\" conceptRef=\"TIME_PERIOD\"/>\n"
	+ "                <structure:Group id=\"Group\">\n"
	+ "                    <structure:DimensionRef>CURRENCY</structure:DimensionRef>\n"
	+ "                    <structure:DimensionRef>CURRENCY_DENOM</structure:DimensionRef>\n"
	+ "                    <structure:DimensionRef>EXR_TYPE</structure:DimensionRef>\n"
	+ "                    <structure:DimensionRef>EXR_SUFFIX</structure:DimensionRef>\n"
	+ "                </structure:Group>\n"
	+ "                <structure:PrimaryMeasure conceptAgency=\"ECB\" conceptVersion=\"1.0\" conceptRef=\"OBS_VALUE\"/>\n"
	+ "                <structure:Attribute assignmentStatus=\"Conditional\" attachmentLevel=\"Group\" conceptAgency=\"ECB\" conceptVersion=\"1.0\" conceptRef=\"COMPILATION\">\n"
	+ "                    <structure:TextFormat maxLength=\"1050\" minLength=\"0\" textType=\"String\"/>\n"
	+ "                    <structure:AttachmentGroup>Group</structure:AttachmentGroup>\n"
	+ "                </structure:Attribute>\n"
	+ "                <structure:Attribute assignmentStatus=\"Conditional\" attachmentLevel=\"Group\" conceptAgency=\"ECB\" conceptVersion=\"1.0\" conceptRef=\"COVERAGE\">\n"
	+ "                    <structure:TextFormat maxLength=\"350\" minLength=\"0\" textType=\"String\"/>\n"
	+ "                    <structure:AttachmentGroup>Group</structure:AttachmentGroup>\n"
	+ "                </structure:Attribute>\n"
	+ "                <structure:Attribute assignmentStatus=\"Mandatory\" attachmentLevel=\"Group\" codelistAgency=\"ECB\" codelistVersion=\"1.0\" codelist=\"CL_DECIMALS\" conceptAgency=\"ECB\" conceptVersion=\"1.0\" conceptRef=\"DECIMALS\">\n"
	+ "                    <structure:AttachmentGroup>Group</structure:AttachmentGroup>\n"
	+ "                </structure:Attribute>\n"
	+ "                <structure:Attribute assignmentStatus=\"Conditional\" attachmentLevel=\"Group\" conceptAgency=\"ECB\" conceptVersion=\"1.0\" conceptRef=\"NAT_TITLE\">\n"
	+ "                    <structure:TextFormat maxLength=\"350\" minLength=\"0\" textType=\"String\"/>\n"
	+ "                    <structure:AttachmentGroup>Group</structure:AttachmentGroup>\n"
	+ "                </structure:Attribute>\n"
	+ "                <structure:Attribute assignmentStatus=\"Conditional\" attachmentLevel=\"Group\" codelistAgency=\"ECB\" codelistVersion=\"1.0\" codelist=\"CL_ORGANISATION\" conceptAgency=\"ECB\" conceptVersion=\"1.0\" conceptRef=\"SOURCE_AGENCY\">\n"
	+ "                    <structure:AttachmentGroup>Group</structure:AttachmentGroup>\n"
	+ "                </structure:Attribute>\n"
	+ "                <structure:Attribute assignmentStatus=\"Conditional\" attachmentLevel=\"Group\" conceptAgency=\"ECB\" conceptVersion=\"1.0\" conceptRef=\"SOURCE_PUB\">\n"
	+ "                    <structure:TextFormat maxLength=\"350\" minLength=\"0\" textType=\"String\"/>\n"
	+ "                    <structure:AttachmentGroup>Group</structure:AttachmentGroup>\n"
	+ "                </structure:Attribute>\n"
	+ "                <structure:Attribute assignmentStatus=\"Conditional\" attachmentLevel=\"Group\" conceptAgency=\"ECB\" conceptVersion=\"1.0\" conceptRef=\"TITLE\">\n"
	+ "                    <structure:TextFormat maxLength=\"70\" minLength=\"0\" textType=\"String\"/>\n"
	+ "                    <structure:AttachmentGroup>Group</structure:AttachmentGroup>\n"
	+ "                </structure:Attribute>\n"
	+ "                <structure:Attribute assignmentStatus=\"Mandatory\" attachmentLevel=\"Group\" conceptAgency=\"ECB\" conceptVersion=\"1.0\" conceptRef=\"TITLE_COMPL\">\n"
	+ "                    <structure:TextFormat maxLength=\"1050\" minLength=\"1\" textType=\"String\"/>\n"
	+ "                    <structure:AttachmentGroup>Group</structure:AttachmentGroup>\n"
	+ "                </structure:Attribute>\n"
	+ "                <structure:Attribute assignmentStatus=\"Mandatory\" attachmentLevel=\"Group\" codelistAgency=\"ECB\" codelistVersion=\"1.0\" codelist=\"CL_UNIT\" conceptAgency=\"ECB\" conceptVersion=\"1.0\" conceptRef=\"UNIT\">\n"
	+ "                    <structure:AttachmentGroup>Group</structure:AttachmentGroup>\n"
	+ "                </structure:Attribute>\n"
	+ "                <structure:Attribute assignmentStatus=\"Mandatory\" attachmentLevel=\"Group\" codelistAgency=\"ECB\" codelistVersion=\"1.0\" codelist=\"CL_UNIT_MULT\" conceptAgency=\"ECB\" conceptVersion=\"1.0\" conceptRef=\"UNIT_MULT\">\n"
	+ "                    <structure:AttachmentGroup>Group</structure:AttachmentGroup>\n"
	+ "                </structure:Attribute>\n"
	+ "                <structure:Attribute assignmentStatus=\"Conditional\" attachmentLevel=\"Series\" conceptAgency=\"ECB\" conceptVersion=\"1.0\" conceptRef=\"BREAKS\">\n"
	+ "                    <structure:TextFormat maxLength=\"350\" minLength=\"0\" textType=\"String\"/>\n"
	+ "                </structure:Attribute>\n"
	+ "                <structure:Attribute assignmentStatus=\"Mandatory\" attachmentLevel=\"Series\" codelistAgency=\"ECB\" codelistVersion=\"1.0\" codelist=\"CL_COLLECTION\" conceptAgency=\"ECB\" conceptVersion=\"1.0\" conceptRef=\"COLLECTION\"/>\n"
	+ "                <structure:Attribute assignmentStatus=\"Conditional\" attachmentLevel=\"Series\" conceptAgency=\"ECB\" conceptVersion=\"1.0\" conceptRef=\"DOM_SER_IDS\">\n"
	+ "                    <structure:TextFormat maxLength=\"70\" minLength=\"0\" textType=\"String\"/>\n"
	+ "                </structure:Attribute>\n"
	+ "                <structure:Attribute assignmentStatus=\"Conditional\" attachmentLevel=\"Series\" conceptAgency=\"ECB\" conceptVersion=\"1.0\" conceptRef=\"PUBL_ECB\">\n"
	+ "                    <structure:TextFormat maxLength=\"1050\" minLength=\"0\" textType=\"String\"/>\n"
	+ "                </structure:Attribute>\n"
	+ "                <structure:Attribute assignmentStatus=\"Conditional\" attachmentLevel=\"Series\" conceptAgency=\"ECB\" conceptVersion=\"1.0\" conceptRef=\"PUBL_MU\">\n"
	+ "                    <structure:TextFormat maxLength=\"1050\" minLength=\"0\" textType=\"String\"/>\n"
	+ "                </structure:Attribute>\n"
	+ "                <structure:Attribute assignmentStatus=\"Conditional\" attachmentLevel=\"Series\" conceptAgency=\"ECB\" conceptVersion=\"1.0\" conceptRef=\"PUBL_PUBLIC\">\n"
	+ "                    <structure:TextFormat maxLength=\"1050\" minLength=\"0\" textType=\"String\"/>\n"
	+ "                </structure:Attribute>\n"
	+ "                <structure:Attribute isTimeFormat=\"true\" assignmentStatus=\"Mandatory\" attachmentLevel=\"Series\" conceptAgency=\"ECB\" conceptVersion=\"1.0\" conceptRef=\"TIME_FORMAT\">\n"
	+ "                    <structure:TextFormat maxLength=\"3\" minLength=\"1\" textType=\"String\"/>\n"
	+ "                </structure:Attribute>\n"
	+ "                <structure:Attribute assignmentStatus=\"Conditional\" attachmentLevel=\"Series\" conceptAgency=\"ECB\" conceptVersion=\"1.0\" conceptRef=\"UNIT_INDEX_BASE\">\n"
	+ "                    <structure:TextFormat maxLength=\"35\" minLength=\"0\" textType=\"String\"/>\n"
	+ "                </structure:Attribute>\n"
	+ "                <structure:Attribute assignmentStatus=\"Conditional\" attachmentLevel=\"Observation\" conceptAgency=\"ECB\" conceptVersion=\"1.0\" conceptRef=\"OBS_COM\">\n"
	+ "                    <structure:TextFormat maxLength=\"350\" minLength=\"0\" textType=\"String\"/>\n"
	+ "                </structure:Attribute>\n"
	+ "                <structure:Attribute assignmentStatus=\"Conditional\" attachmentLevel=\"Observation\" codelistAgency=\"ECB\" codelistVersion=\"1.0\" codelist=\"CL_OBS_CONF\" conceptAgency=\"ECB\" conceptVersion=\"1.0\" conceptRef=\"OBS_CONF\"/>\n"
	+ "                <structure:Attribute assignmentStatus=\"Conditional\" attachmentLevel=\"Observation\" conceptAgency=\"ECB\" conceptVersion=\"1.0\" conceptRef=\"OBS_PRE_BREAK\">\n"
	+ "                    <structure:TextFormat maxLength=\"15\" minLength=\"0\" textType=\"String\"/>\n"
	+ "                </structure:Attribute>\n"
	+ "                <structure:Attribute assignmentStatus=\"Mandatory\" attachmentLevel=\"Observation\" codelistAgency=\"ECB\" codelistVersion=\"1.0\" codelist=\"CL_OBS_STATUS\" conceptAgency=\"ECB\" conceptVersion=\"1.0\" conceptRef=\"OBS_STATUS\"/>\n"
	+ "            </structure:Components>\n"
	+ "        </structure:KeyFamily>\n"
	+ "    </message:KeyFamilies>\n"
	+ "</message:Structure>\n";

    public HeaderReaderTest(final String name) {
        super(name);
    }

    /* ================== Protected Methods ===================== */

    protected SAX2Reader factory (
        final org.xml.sax.XMLReader parser,
        final SAX2Reader parent
    ) {
        final HeaderReader reader = new HeaderReader( parser, parent );
        reader.setKeyFamily( ECB_EXR1 );
        return reader;
    }

    /** Sets up the test fixture. */
    protected void setUp () {
        super.setUp();
        final Metabase metabase = Metabase.getMetabase();
        
		try {
			structureFile = File.createTempFile( "structureFile.", null );
			structureFile.deleteOnExit();

			Reader input = new StringReader( structureInput );
			Writer data = new BufferedWriter(
				new OutputStreamWriter(
					new FileOutputStream( structureFile ),
					"ISO8859-1"
				)
			);
			int c = 0;
			while ( ( c = input.read() ) != -1 ) {
				data.write( c );
			}
			data.close();
			input.close();
		}
		catch ( IOException ioe ) {
			fail(
				"Write structureFile failed: " + ioe
			);
		}
		
        try {
            final MessageBegin expectedMB = new MessageBegin(
                    "MREF000001",
                    "4F0",
                    "DE2",
                    "ECB",
                    MessageBegin.STATISTICAL_DATA,
                    "Some test",
                    new Contact[] {},
                    "EXR",
                    MessageBegin.UPDATE_OR_REPLACE_ACTION,
                    new SimpleDateFormat( "yyyy-MM-dd'T'HH:mm:ssZ" ).parse( "2009-01-27T12:34:56+0100" ),
                    "2008010120081231",
                    "711",
                    "ECB_EXR1",
                    "EXR" // the original external dsi
                    );
            metabase.handle( expectedMB );
            metabase.handle( ECB_EXR1 );
            
        } catch (NullPointerException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (ParseException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
       
    }
	
    /**
	 * Tears down the test fixture.
	 */
	protected void tearDown () {
		structureFile.delete();
	}

    /* ===================== Public Methods ===================== */    

    /**
     * Assembles and returns a test suite for all
     * the test methods of this test case.
     *
     * @return a non-null test suite
     */
    public static Test suite () {
        return new TestSuite( HeaderReaderTest.class );
    }

    /**
     * @param arg the array of argument {@link String}s
     */
    public static void main ( final String[] arg ) {
        final String[] testCaseName = {
            HeaderReaderTest.class.getName()
        };
        junit.textui.TestRunner.main( testCaseName );
    }

    /* ---------------------- Test Methods ---------------------- */ 
    public void testGetMinimalHeader() throws SAXException, ParseException
    {
        openElement("CompactData", "");
        openElement("Header", "");
        openElement("ID", "");
        characters ("IREF000638");
        closeElement("ID");
        openElement("Test", "");
        characters("false");
        closeElement("Test");
        openElement("Name", "");
        characters("Some test");
        closeElement("Name");
        openElement("Prepared", "");
        characters("2008-12-03T09:38:52+01:00");
        closeElement("Prepared");
        openElement("Sender", "id=4F0");
        closeElement("Sender");
        openElement("Receiver", "id=DE2");
        closeElement("Receiver");
        
        openElement("KeyFamilyRef", "");
        characters("ECB_EXR1");
        closeElement("KeyFamilyRef");
        openElement("KeyFamilyAgency", "");
        characters("ECB");
        closeElement("KeyFamilyAgency");
        openElement("DataSetAgency", "");
        characters("ECB");
        closeElement("DataSetAgency");
        openElement("DataSetID", "");
        characters("EXR");
        closeElement("DataSetID");
        openElement("DataSetAction", "");
        characters("Append");
        closeElement("DataSetAction");
        openElement("Extracted", "");
        characters("2009-01-27T12:34:56+01:00");
        closeElement("Extracted");
        openElement("ReportingBegin", "");
        characters("2008-01-01");
        closeElement("ReportingBegin");
        openElement("ReportingEnd", "");
        characters("2008-12-31");
        closeElement("ReportingEnd");
        openElement("Source", "");
        characters("Eurostat");
        closeElement("Source");
        closeElement("Header");
        openElement("DataSet", "");
        closeElement("DataSet");
        
        final Iterator iteratorBegin = listener.iterator();
        
        assertTrue("There should be an InterchangeBegin", iteratorBegin.hasNext());
        final Date pDate = new SimpleDateFormat( "yyyy-MM-dd'T'HH:mm:ssZ" ).parse( "2008-12-03T09:38:52+0100" );
        final InterchangeBegin expectedIB = new InterchangeBegin("IREF000638", 
                "4F0", "DE2", pDate , false);
        assertEquals("Did not receive the expected InterchangeBegin", expectedIB, 
                iteratorBegin.next());
        
        assertTrue("There should be a MessageBegin", iteratorBegin.hasNext());        
        final Date eDate = new SimpleDateFormat( "yyyy-MM-dd'T'HH:mm:ssZ" ).parse( "2009-01-27T12:34:56+0100" );
        final MessageBegin expectedMB = new MessageBegin(
                "MREF000001",
                "4F0",
                "DE2",
                "ECB",
                MessageBegin.STATISTICAL_DATA,
                "Some test",
                new Contact[] {},
                "EXR",
                MessageBegin.OBJECT_LEVEL_ACTION,
                eDate,
                "2008010120081231",
                "711",
                "ECB_EXR1",
                "EXR" // the original external dsi
                );
        assertEquals("Did not receive the expected MessageBegin", expectedMB, 
                iteratorBegin.next());
        
        assertTrue("Expecting MessageEnd", iteratorBegin.hasNext());
        final MessageEnd expectedME = new MessageEnd("MREF000001");
        assertEquals("Did not receive the expected MessageEnd", expectedME, 
                iteratorBegin.next());
        assertTrue("No object expected", !iteratorBegin.hasNext());
        
        closeElement("CompactData");
        final Iterator iteratorIE = listener.iterator();
        iteratorIE.next();
        iteratorIE.next();
        iteratorIE.next();
        assertTrue("There should be another object", iteratorIE.hasNext());
        final InterchangeEnd expectedIE = new InterchangeEnd("IREF000638");
        assertEquals("It should be a InterchangeEnd", expectedIE, 
                iteratorIE.next());
        assertTrue("No object expected", !iteratorIE.hasNext());
    }
    
    public void testGetDataSetVSHeader() throws SAXException, ParseException
    {
        openElement("CompactData", "");
        openElement("Header", "");
        openElement("ID", "");
        characters ("IREF000638");
        closeElement("ID");
        openElement("Test", "");
        characters("false");
        closeElement("Test");
        openElement("Name", "");
        characters("Some test");
        closeElement("Name");
        openElement("Prepared", "");
        characters("2008-12-03T09:38:52+01:00");
        closeElement("Prepared");
        openElement("Sender", "id=4F0");
        openElement("Name", "");
        characters("Name of the sender");
        closeElement("Name");
        closeElement("Sender");
        openElement("Receiver", "id=DE2");
        closeElement("Receiver");
        
        openElement("KeyFamilyRef", "");
        characters("ECB_EXR1");
        closeElement("KeyFamilyRef");
        openElement("KeyFamilyAgency", "");
        characters("ECB");
        closeElement("KeyFamilyAgency");
        /* */
		openElement("DataSetAgency", "");
        characters("4F0");
        closeElement("DataSetAgency");
        /* */
		openElement("DataSetID", "");
        characters("EXR");
        closeElement("DataSetID");
        openElement("DataSetAction", "");
        characters("Append");
        closeElement("DataSetAction");
        openElement("Extracted", "");
        characters("2009-01-27T12:34:56+01:00");
        closeElement("Extracted");
        openElement("ReportingBegin", "");
        characters("2008-Q1");
        closeElement("ReportingBegin");
        openElement("ReportingEnd", "");
        characters("2008-Q4");
        closeElement("ReportingEnd");
        openElement("Source", "");
        characters("Eurostat");
        closeElement("Source");
        closeElement("Header");
        openElement("DataSet", "reportingBeginDate=2007-Q1 reportingEndDate=2007-Q2 dataProviderID=5D0 datasetID=ICP action=Delete");
        closeElement("DataSet");
        openElement("DataSet", "");
        closeElement("DataSet");
        
        final Iterator iteratorBegin = listener.iterator();
        
        assertTrue("There should be an InterchangeBegin", iteratorBegin.hasNext());
        final Date pDate = new SimpleDateFormat( "yyyy-MM-dd'T'HH:mm:ssZ" ).parse( "2008-12-03T09:38:52+0100" );
        final InterchangeBegin expectedIB = new InterchangeBegin("IREF000638", 
                "4F0", "DE2", pDate, false);
        assertEquals("Did not receive the expected InterchangeBegin", expectedIB, 
                iteratorBegin.next());
        
        assertTrue("There should be a MessageBegin", iteratorBegin.hasNext());
        final Date eDate = new SimpleDateFormat( "yyyy-MM-dd'T'HH:mm:ssZ" ).parse( "2009-01-27T12:34:56+0100" );
        final MessageBegin expectedMB = new MessageBegin(
                "MREF000001",
                "5D0",
                "DE2",
                "ECB",
                MessageBegin.STATISTICAL_DATA,
                "Some test",
                new Contact[] {},
                "ICP",
                MessageBegin.DELETE_ACTION,
                eDate,
                "2007120072",
                "708",
                "ECB_EXR1",
                "ICP" // the original external dsi
                );
        assertEquals("Did not receive the expected MessageBegin", expectedMB, 
                iteratorBegin.next());
        
        assertTrue("Expecting MessageEnd", iteratorBegin.hasNext());
        final MessageEnd expectedME = new MessageEnd("MREF000001");
        assertEquals("Did not receive the expected MessageEnd", expectedME, 
                iteratorBegin.next());
        
        assertEquals("Did not receive the second MessageBegin",
                new MessageBegin(
                        "MREF000002",
                        "4F0",
                        "DE2",
                        "ECB",
                        MessageBegin.STATISTICAL_DATA,
                        "Some test",
                        new Contact[] {},
                        "ICP",
                        MessageBegin.OBJECT_LEVEL_ACTION,
                        eDate,
                        "2008120084",
                        "708",
                        "ECB_EXR1",
                        "ICP" // the original external dsi
                ), 
                iteratorBegin.next()
        );
        assertEquals("Did not receive the expected MessageEnd",
                new MessageEnd("MREF000002"), 
                iteratorBegin.next()
        );
        assertTrue("No object expected", !iteratorBegin.hasNext());
        
        closeElement("CompactData");
        final Iterator iteratorIE = listener.iterator();
        iteratorIE.next();
        iteratorIE.next();
        iteratorIE.next();
        iteratorIE.next();
        iteratorIE.next();
        assertTrue("There should be another object", iteratorIE.hasNext());
        final InterchangeEnd expectedIE = new InterchangeEnd("IREF000638");
        assertEquals("It should be a InterchangeEnd", expectedIE, 
                iteratorIE.next());
        assertTrue("No object expected", !iteratorIE.hasNext());
    }
    
    public void testGetDataWithKeyFamilyURI() throws SAXException, ParseException
    {
        openElement("CompactData", "");
        openElement("Header", "");
        openElement("ID", "");
        characters ("IREF000638");
        closeElement("ID");
        openElement("Test", "");
        characters("false");
        closeElement("Test");
        openElement("Name", "");
        characters("Some test");
        closeElement("Name");
        openElement("Prepared", "");
        characters("2008-12-03T09:38:52+01:00");
        closeElement("Prepared");
        openElement("Sender", "id=4F0");
        openElement("Name", "");
        characters("Name of the sender");
        closeElement("Name");
        closeElement("Sender");
        openElement("Receiver", "id=DE2");
        closeElement("Receiver");
        
        /* */
        openElement("DataSetAgency", "");
        characters("4F0");
        closeElement("DataSetAgency");
        /* */
        openElement("DataSetID", "");
        characters("EXR");
        closeElement("DataSetID");
        openElement("DataSetAction", "");
        characters("Append");
        closeElement("DataSetAction");
        openElement("Extracted", "");
        characters("2009-01-27T12:34:56+01:00");
        closeElement("Extracted");
        openElement("ReportingBegin", "");
        characters("2008-Q1");
        closeElement("ReportingBegin");
        openElement("ReportingEnd", "");
        characters("2008-Q4");
        closeElement("ReportingEnd");
        openElement("Source", "");
        characters("Eurostat");
        closeElement("Source");
        closeElement("Header");
        String path = structureFile.getPath();
        openElement("DataSet", "keyFamilyURI=" + path );
        closeElement("DataSet");
        
        final Iterator iteratorBegin = listener.iterator();
        
        assertTrue("There should be an InterchangeBegin", iteratorBegin.hasNext());
        final Date pDate = new SimpleDateFormat( "yyyy-MM-dd'T'HH:mm:ssZ" ).parse( "2008-12-03T09:38:52+0100" );
        final InterchangeBegin expectedIB = new InterchangeBegin("IREF000638", 
                "4F0", "DE2", pDate, false);
        assertEquals("Did not receive the expected InterchangeBegin", expectedIB, 
                iteratorBegin.next());
        
        assertTrue("There should be a MessageBegin", iteratorBegin.hasNext());
        final Date eDate = new SimpleDateFormat( "yyyy-MM-dd'T'HH:mm:ssZ" ).parse( "2009-01-27T12:34:56+0100" );
        assertEquals("Did not receive the expected MessageBegin",
                new MessageBegin(
                        "MREF000001",
                        "4F0",
                        "DE2",
                        "ECB",
                        MessageBegin.STATISTICAL_DATA,
                        "Some test",
                        new Contact[] {},
                        "EXR",
                        MessageBegin.OBJECT_LEVEL_ACTION,
                        eDate,
                        "2008120084",
                        "708",
                        "ECB_EXR1",
                        "EXR" // the original external dsi
                ), 
                iteratorBegin.next()
        );
        assertEquals("Did not receive the expected MessageEnd",
                new MessageEnd("MREF000001"), 
                iteratorBegin.next()
        );
        assertTrue("No object expected", !iteratorBegin.hasNext());
        
        closeElement("CompactData");
        final Iterator iteratorIE = listener.iterator();
        iteratorIE.next();
        iteratorIE.next();
        iteratorIE.next();
        assertTrue("There should be another object", iteratorIE.hasNext());
        final InterchangeEnd expectedIE = new InterchangeEnd("IREF000638");
        assertEquals("It should be a InterchangeEnd", expectedIE, 
                iteratorIE.next());
        assertTrue("No object expected", !iteratorIE.hasNext());
    }
}
