// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2002, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
package Int.ecb.gesmescb.xml.sdmx.v2.compact;

import java.util.Iterator;

import org.xml.sax.Attributes;
import org.xml.sax.XMLReader;

import Int.ecb.gesmescb.KeyFamilyComponent;
import Int.ecb.gesmescb.TimeSeries;
import Int.ecb.gesmescb.xml.MapStack;
import Int.ecb.gesmescb.xml.SAX2Reader;
import Int.ecb.gesmescb.xml.sdmx.v2.KeyFamilyDependentReader;

/**
 * Base implementation of key-dependant content handlers (i.e.: series and
 * sibling groups).  
 * 
 * @author Francois Hoeltzli
 * @author Xavier Sosnovsky
 */
class KeyReader extends KeyFamilyDependentReader {
    
    /* ========================= Fields ========================= */
    
    protected String key;

    
    /* ====================== Constructors ====================== */
    
    /**
     * Constructs a new KeyReader.
     * 
     * @param parser The parser of the XML document
     * @param parent The class responsible for handling content
     */
    protected KeyReader(final XMLReader parser, final SAX2Reader parent) {
        super(parser, parent);
    }
    
    /* ===================== Protected Methods ================== */
    
    /* --------------------- ContentHandler --------------------- */
    
    /**
     * @return Whether or not the current content handler is a sibling group.
     */
    protected boolean isSiblingGroup() {
        return false;        
    }
    
    /**
     * Creates a map of dimensions and attributes id, and their values out of 
     * a list of XML attributes.
     * 
     * @param attributes
     * 
     * @return a map of dimensions and attributes id, and their values
     */
    protected static MapStack readXMLAttributes(final Attributes attributes) {
        final MapStack xmlAttributes = new MapStack();
        xmlAttributes.push();
        final int length = attributes.getLength();
        for ( int i = 0; i < length; ++i ) {
            xmlAttributes.put(
                attributes.getLocalName( i ), 
                attributes.getValue( i )
            );
        }
        return xmlAttributes;
    }
    
    /**
     * Constructs the series or group key out of a collection of XML attributes.
     *  
     * @param xmlAttributes The collection containing the dimensions and
     * attributes attached to the XML element.
     * 
     * @return The series or group key
     */
    protected String distilKey (final MapStack xmlAttributes) {
        final StringBuffer buffer = new StringBuffer();
        for (
            final Iterator iterator = keyFamily.keyStructureIterator();
            iterator.hasNext();
        ) {
            final KeyFamilyComponent component
            = (KeyFamilyComponent) iterator.next();
            final String value = xmlAttributes.get( component.getConceptId() );
            if ( !isSiblingGroup()
                || 
                keyFamily.getFrequency().getConceptId()  
                != 
                component.getConceptId()
            ) {
                buffer.append( value );
                if ( iterator.hasNext() ) {
                    buffer.append( TimeSeries.KEY_SEPARATOR );
                }
            }
        }
        return buffer.toString();
    }
}
