// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2004, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.gesmescb.xml.sdmx.v2.compact;

import org.xml.sax.Attributes;
import org.xml.sax.XMLReader;

import Int.ecb.gesmescb.event.GESMESCBEventListener;
import Int.ecb.gesmescb.xml.SAX2Reader;
import Int.ecb.gesmescb.xml.sdmx.v2.KeyFamilyDependentReader;
import Int.ecb.logging.Level;

/* ***************************** Class ****************************** */

/**
 * Content handler for an SDMX-ML StructureSpecific Data and TimeSeries
 * messages.  
 * 
 * @author Maria Stahl
 * @author Francois Hoeltzli
 */
public class StructureSpecificReader extends KeyFamilyDependentReader {
        
    private final HeaderReader21 headerReader;

    /* ====================== Constructors ====================== */
    
    /**
     * Constructs a new StructureSpecificReader.
     * 
     * @param parser The parser of the XML document
     * @param parent The class responsible for handling content
     */
    public StructureSpecificReader(
        final XMLReader parser, final SAX2Reader parent) {
        super(parser, parent);
        headerReader = new HeaderReader21( parser, this );
    }
    
    /* ===================== Public Methods ===================== */

    /**
     * {@inheritDoc}
     */
    public void addListener (
        final GESMESCBEventListener listener,
        final boolean isImportant
    ) {
        super.addListener( listener, isImportant );
        headerReader.addListener( listener, isImportant );
    }

    /**
     * {@inheritDoc}
     */
    public void removeAllListeners () {
        super.removeAllListeners();
        headerReader.removeAllListeners();
    }
    
    /**
     * {@inheritDoc}
     */
    public void startElement (
        final String namespaceURI,
        final String localName,
        final String qName,
        final Attributes atts
    ) {
        super.startElement( namespaceURI, localName, qName, atts );
        
        if ("Header".equals(localName)) {
            delegate(
                headerReader,
                namespaceURI, localName, qName, atts
            );
        } else if (
        	"StructureSpecificData".equals(localName)
        	||
        	"StructureSpecificTimeSeries".equals(localName)
        ) {
        } else {        	
        	componentHelper.logger.log(
                Level.SEVERE,
                componentHelper.logName,
                "StructureSpecificReader",
                "Starting unexpected element : \""
                + localName + "\""
            );
        }
    }    
}
