// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.logging;

import Int.ecb.logging.StreamHandler;
import Int.ecb.logging.LogRecord;
import Int.ecb.logging.LogManager;

/**
 * Publishes log records on {@link java.lang.System#err}.
 * By default the {@link SimpleFormatter} is used to generate brief summaries.
 *
 * @author
 * <A HREF="mailto:Sami.Airo@ECB.int">Sami Airo</A>
 *
 * @version @(#)$Id: ConsoleHandler.java,v 1.1 2017/01/19 16:14:05 trainot Exp $
 */
public class ConsoleHandler extends StreamHandler {

	private final String className = getClass().getName();

	/**
	 * Create a ConsoleHandler for System.err.
	 */
	public ConsoleHandler() {
		super();

		setOutputStream(System.err);

		LogManager manager = LogManager.getLogManager();
		String prt;

		prt = manager.getProperty(className + ".level");
		if (prt == null) {
			setLevel( Level.INFO );
		} else {
			setLevel( Level.parse( prt ) );
		}

		prt = manager.getProperty(className + ".filter");
		if (prt != null) {
			try {
				setFilter( (Filter) Class.forName(prt).newInstance() );
			} catch (ClassNotFoundException e) {
				manager.log( Level.SEVERE, className, "readConfiguration", "cannot process configuration", e);
			} catch (InstantiationException e) {
				manager.log( Level.SEVERE, className, "readConfiguration", "cannot process configuration", e);
			} catch (IllegalAccessException e) {
				manager.log( Level.SEVERE, className, "readConfiguration", "cannot process configuration", e);
			}
		}

		prt = manager.getProperty(className + ".formatter");
		if (prt != null) {
			try {
				setFormatter( (Formatter) Class.forName(prt).newInstance() );
			} catch (ClassNotFoundException e) {
				manager.log( Level.SEVERE, className, "readConfiguration", "cannot process configuration", e);
			} catch (InstantiationException e) {
				manager.log( Level.SEVERE, className, "readConfiguration", "cannot process configuration", e);
			} catch (IllegalAccessException e) {
				manager.log( Level.SEVERE, className, "readConfiguration", "cannot process configuration", e);
			}
		}

	}

	/**
	 * Override "StreamHandler.close" to do a flush but not to close the output
	 * stream. That is, we do not close System.err.
	 */
	public void close() {
		flush();
	}

	/**
	 * Publish a LogRecord.
	 * The logging request was made initially to a Logger object,
	 * which initialized the LogRecord and forwarded it here.
	 *
	 * param record description of the log event
	 */
	public void publish(LogRecord record) {
		super.publish(record);
		flush();
	}
}
