// ECB/SIS Public License, version 1.0, document reference SIS/2001/116
//
// Copyright (c) 2001, European Central Bank. All rights reserved.
//
// Redistribution and use in source and binary forms,
// with or without modification, are permitted
// provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// Neither the name of the European Central Bank
// nor the names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package Int.ecb.logging;

import Int.ecb.logging.LogRecord;

/*
 * Provides support for formatting LogRecords.
 * Typically each logging Handler will have a Formatter associated with it.
 * The Formatter takes a LogRecord and converts it to a string.
 * Some formatters (such as the XMLFormatter) need to wrap head and tail strings
 * around a set of formatted records. The getHeader and getTail methods can be
 * used to obtain these strings.
 *
 * @author
 * <A HREF="mailto:Sami.Airo@ECB.int">Sami Airo</A>
 *
 * @version @(#)$Id: Formatter.java,v 1.1 2017/01/19 16:14:05 trainot Exp $
 */
public abstract class Formatter {

	/**
	 * Construct a new Formatter.
	 */
	protected Formatter() {}

	/**
	 * Format the given log record and return the formatted string.
	 * The resulting formatted String will normally include a localized and
	 * formated version of the LogRecord's message field.
	 * The LogRecord.formatMessage
	 * convenience method can (optionally) be used to localize and
	 * format the message field.
	 *
	 * @param record	the log record to be formatted.
	 * @return			the formatted log record
	 */
	public abstract java.lang.String format(LogRecord record);


	/**
	 * Return the header string for a set of formatted records.
	 * This base class returns an empty string,
	 * but this may be overriden by subclasses.
	 *
	 * @param h the target handler.
	 * @return header string
	 */
	public java.lang.String getHead(Handler h) {
		return "";
	}


	/**
	 * Return the tail string for a set of formatted records.
	 * This base class returns an empty string,
	 * but this may be overriden by subclasses.
	 *
	 * @param h the target handler.
	 * @return tail string
	 */
	public java.lang.String getTail(Handler h) {
		return "";
	}
}
